/*
 * Copyright (C) 2013 by I.Kato @ BS21 Lab
 * BS21 Lab library for Arduino
 *    ver.1.00 (14/aug/2013)
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1,
 * both as published by the Free Software Foundation.
 */

#ifndef __BS21LABLIB_H__
#define __BS21LABLIB_H__

#ifndef __arm__
// Arduino Uno, and etc.

#include <avr/pgmspace.h>

#else // __arm__
// Arduino Due

#define __PGMSPACE_H_ 1
#define PROGMEM
#define PGM_P  const char *
#define PSTR(str) (str)
#define _SFR_BYTE(n) (n)
typedef void prog_void;
typedef char prog_char;
typedef unsigned char prog_uchar;
typedef int8_t   prog_int8_t;
typedef uint8_t  prog_uint8_t;
typedef int16_t  prog_int16_t;
typedef uint16_t prog_uint16_t;
typedef int32_t  prog_int32_t;
typedef uint32_t prog_uint32_t;
#define memcpy_P(dest, src, num) memcpy((dest), (src), (num))
#define strcpy_P(dest, src) strcpy((dest), (src))
#define strcat_P(dest, src) strcat((dest), (src))
#define strcmp_P(a, b) strcmp((a), (b))
#define strstr_P(a, b) strstr((a), (b))
#define strlen_P(a) strlen((a))
#define sprintf_P(s, f, ...) sprintf((s), (f), __VA_ARGS__)
#define pgm_read_byte(addr)  (*(const unsigned char *)(addr))
#define pgm_read_word(addr)  (*(const unsigned short *)(addr))
#define pgm_read_dword(addr) (*(const unsigned long *)(addr))
#define pgm_read_float(addr) (*(const float *)(addr))
#define pgm_read_byte_near(addr)  pgm_read_byte(addr)
#define pgm_read_word_near(addr)  pgm_read_word(addr)
#define pgm_read_dword_near(addr) pgm_read_dword(addr)
#define pgm_read_float_near(addr) pgm_read_float(addr)
#define pgm_read_byte_far(addr)   pgm_read_byte(addr)
#define pgm_read_word_far(addr)   pgm_read_word(addr)
#define pgm_read_dword_far(addr)  pgm_read_dword(addr)
#define pgm_read_float_far(addr)  pgm_read_float(addr)

#endif // __arm__


typedef uint8_t errCode_t; // エラーコードの型

typedef wchar_t PROGMEM prog_wchar; // プログラムメモリ上のワイド文字

// avr/pgmspace.h の PSTR 定義のワイド文字版
#ifndef PWSTR
#ifndef __arm__
#define PWSTR(s) (__extension__({static const wchar_t __c[] PROGMEM = (s); &__c[0];}))
#else // __arm__
#define PWSTR(s) (s)
#endif // __arm__
#endif // PWSTR


// 文字コード変換サービス関数
// ワイド文字(UTF-16)に対応した文字列操作関数が存在しないので、これらのサービス関数にて、
// ワイド文字列から通常の文字列(UTF-8)に変換してから、文字列操作すると良いでしょう。
// buffer は、変換後の UTF-8 文字列を格納するのに十分なサイズである必要があります。だいたい UTF-16 文字列の総バイト数の1.5倍＋αが目安です。
// 注意：バッファあふれのチェックはしていませんので、注意してください。

size_t convertWCharToChar   (char * buffer, const wchar_t    * wstr);  // SRAM 上の UTF-16 文字列を UTF-8 文字列に変換します
size_t convertWCharToChar_P (char * buffer, const prog_wchar * pwstr); // プログラムメモリ上の UTF-16 文字列を UTF-8 文字列に変換します

#endif // __BS21LABLIB_H__

