/*
  Color Kanji LCD Shield - Arduino Logo Demo#1

  本スケッチは "カラー漢字 LCD シールド" で、Arduino のロゴマークを表示させるデモの例です。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 15/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが、上記の CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(160×256) を背景面に設定します。仮想画面２(160×128) は使用しません。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_B1);

  // 文字の重ね書きができるように、ブレンドモードを ON にします
  cklcd.v1.setBlendMode (CKLCD_BLEND_ADD);

  // Arduino のロゴマークを表示させます（仮想画面１を使用）。
  // ※ 最大フォントサイズで '∞' を表示し、最小フォントサイズで '-' と '+' を重ね書きします。
  cklcd.v1.setCursorMin (66, 40);
  cklcd.v1.print (CKLCD_CSI_FNT_28px_MS "∞" CKLCD_CSI_FNT_RUBY_P "\x1b[15B" "\x1b[21D" "-" "\x1b[6C" "+");
  cklcd.v1.print (CKLCD_CSI_FNT_RUBY_MS "\x1b[12B" "\x1b[24D" "ARDUINO");
}


// ループ関数
void loop()
{
}

