/*
  Color Kanji LCD Shield - Demonstration #1

  本スケッチは "カラー漢字 LCD シールド" でデモ表示するスケッチ例です。
  左/右スイッチを押すと、表示されるメッセージが切り替わります。
  上スイッチを押すと、背景表示が ON/OFF されます。下スイッチを押すと、背景スクロールが ON/OFF されます。
  コンパイルすると 14kB 弱になるので、未確認ですが ATmega168 搭載の Arduino でも大丈夫だと思います。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 15/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


size_t msg_no; // 表示させるメッセージの番号


uint8_t left_sw, right_sw, up_sw, down_sw; // カラー漢字 LCD シールドの４つのスイッチの状態


uint8_t yDispPos   = 0;  // LCD 表示開始y座標
uint8_t scrollFlag = 1;  // 背景面をスクロールさせるフラグ
uint8_t backScreen = 1;  // 背景面を表示させるフラグ


// デモ表示用メッセージ
// ※巨大なので、プログラムメモリに格納します。PROGMEM 指定が必須です。
// 本当は PROGMEM const prog_char * demo_msg_list[] = { msg, msg, msg, ... }; という風にしたかったのですが、それだとコンパイラが受付けなかったので、以下の様な面倒くさい書き方になってしまいました。

PROGMEM const prog_char demo_msg00[] = "♫" CKLCD_CSI_RED "カ" CKLCD_CSI_GREEN "ラ" CKLCD_CSI_BLUE "ー" CKLCD_CSI_YELLOW "漢字" CKLCD_CSI_CYAN "LCD" CKLCD_CSI_WHITE "ｼｰﾙﾄﾞ\n" // 色の指定
                                       CKLCD_CSI_INVERTED "全角１０文字×６行" CKLCD_CSI_DEFAULT "\n⇨JIS 第" CKLCD_CSI_UNDERLINE "1～3" CKLCD_CSI_DEFAULT "水準字\n" // 下線の指定
                                       CKLCD_CSI_GREEN "utf-8&16," CKLCD_CSI_MAGENTA "SJIS," CKLCD_CSI_CYAN "EUC" CKLCD_CSI_DEFAULT "\nJPG/PNG/GIF/BMP\n⁂i2c(TWI)通信方式"; // オープニング・メッセージ
PROGMEM const prog_char demo_msg01[] = CKLCD_CSI_GREEN "仮想画面1" CKLCD_CSI_DEFAULT ":160×256\n" CKLCD_CSI_BLUE "仮想画面2" CKLCD_CSI_DEFAULT ":160×128\n"
                                       "ⓐ前景面:" CKLCD_CSI_GREEN "V1" CKLCD_CSI_DEFAULT "と\n　背景面:" CKLCD_CSI_BLUE  "V2" CKLCD_CSI_DEFAULT "の重合せ\n" // 'V1'は仮想画面1のこと
                                       "ⓑ前景面:" CKLCD_CSI_BLUE  "V2" CKLCD_CSI_DEFAULT "と\n　背景面:" CKLCD_CSI_GREEN "V1" CKLCD_CSI_DEFAULT "の重合せ";  // 'V2'は仮想画面2のこと
PROGMEM const prog_char demo_msg02[] = "サポートしている画像\n①" CKLCD_CSI_RED "JPG" CKLCD_CSI_DEFAULT "\n②" CKLCD_CSI_GREEN "PNG" CKLCD_CSI_DEFAULT ":透過色対応\n"
                                       "③" CKLCD_CSI_BLUE "GIF" CKLCD_CSI_DEFAULT ":ｱﾆﾒ､透過対応\n④" CKLCD_CSI_YELLOW "BMP" CKLCD_CSI_DEFAULT ":透過色対応\n※但し、各種制限有り";
PROGMEM const prog_char demo_msg03[] = "⓵" CKLCD_CSI_CYAN  "完成版"       CKLCD_CSI_DEFAULT "･･･\n ✓価格" CKLCD_CSI_RED     "￥7,980-" CKLCD_CSI_DEFAULT " 也\n"
                                       "⓶" CKLCD_CSI_GREEN "組立キット版" CKLCD_CSI_DEFAULT "･･･\n ✓価格" CKLCD_CSI_MAGENTA "￥7,480-" CKLCD_CSI_DEFAULT " 也\n ※但し、要半田付け";
PROGMEM const prog_char demo_msg04[] = "半角157字、及び" "\n" CKLCD_CSI_RED "全角8844字" CKLCD_CSI_DEFAULT "に対応\n"     // サポート字数 ASCII:94字＋半角カナ:63字、
                                       "❶" CKLCD_CSI_GREEN "JIS X 0213:2004\n"  CKLCD_CSI_DEFAULT "第1～3水準漢字対応\n" // 第一水準非漢字:524字＋第一水準漢字:2965字＋第二水準漢字:3390字
                                       "❷" CKLCD_CSI_BLUE  "JIS X 0201片仮名\n" CKLCD_CSI_DEFAULT "所謂、半角ｶﾅも対応";  //   ＋第三水準非漢字:659字＋第三水準漢字:1259字＋非JIS記号:47字
PROGMEM const prog_char demo_msg05[] = "▶▷表示例◁◀\n2013年07月05日\n12時34分56秒\n▼温度＝78.9℃▲\nπ≒3.14159265…\nz² = x² + y²"; // 表示例：オーソドックスな使用方法
PROGMEM const prog_char demo_msg06[] = "JIS第三水準にも対応\n€½¼ℓ2²⊕3³ⓩ㊿⓴⓾\n☎©♠♡♢♣☀☁☂☃\n☞↗↘↖↙⇄⏎ 〽ヷ\n☗３三龍、王手！\n☖同桂 ☗待った！";
PROGMEM const prog_char demo_msg07[] = "㋐NEC特殊文字もOK\n№㎡㍼㍻㈱①②③\n㋑アイヌ語も表示可能\nアイヌ・イタㇰ\n"  // アイヌ語は未勉強なので、Wikipedia からコピペ。
                                       "㋒半濁点付き仮名\nカ゚～コ゚,セ゚ツ゚ト゚ㇷ゚,か゚";   // Unicodeで要文字合成にも対応。
PROGMEM const prog_char demo_msg08[] = "Latin-1(ISO8859-1)\n文字もutf-8にて対応\nFrançais / Español\nÖsterreich\nHäagen-Dazs\nRinôçérôse, BOØWY";  // アクセント記号付きアルファベットの例は Wikipedia から拾ってきた
PROGMEM const prog_char demo_msg09[] = "ⓐ足らなかった漢字：\n禸疒辵\xF0\xA5\x9D\xB1秭沪\xF0\xA0\xAE\x9F\n"  // 𥝱(U+25771)と𠮟(U+20B9F)は、直接記述するとエラー(not a valid universal character)になるので、\xF0\xA5\x9D\xB1 という風に記述にする必要有り。
                                       "ⓑ異体字：\n掴↔摑,鴎↔鷗,涜↔瀆\nⓒ簡単な中国文：\n你好,深圳,卡拉OK";
PROGMEM const prog_char demo_msg10[] = "JIS X0213に無い47字\nH₂SO₄⇄2H⁺+SO₄²⁻\n100℉=37.8℃\nxⁿ+yⁿ=zⁿ (n≧3)\n☺☻✓✔✗✘☐☑☒\n↕⇅⇇⇈⇉⇊↰↱↲↳";
PROGMEM const prog_char demo_msg11[] = "Vio はジャンパーにて\nIOREF/3.3V/5V の\nいずれかを選択可能\n\n"  // Vio はジャンパーで設定
                                       "i2cアドレスは、0x3C～\n0x3F から選択可能";    // i2c(TWI)アドレスは DIPSW で設定しておく

// 以降は「組見本用創作文『問わずがたりの洋酒外史』フルバージョン」です。
//   ⇒ http://www.type-labo.jp/KumimihonREAD.html
// ※表示領域の狭さから、読み易いように空白・改行等を調整していますが、文章そのものは改変していません。
PROGMEM const prog_char demo_msg20[] = "※次㌻以降の創作文で\nは、ひらがな・カタカナ\nの全字種と基本的な約\n物(句読記号)がひと通\nり使われて〼。";
PROGMEM const prog_char demo_msg21[] = "『問わずがたりの洋酒\n外史』　佐藤　章\nCopyright © 1990\nby Akira Satoh\n[Directed by TYPE-\nLABO and NAVEL]";

PROGMEM const prog_char demo_msg22[] = "　洋酒といえば、誰で\nも最初に思い浮かべる\nのがウイスキー。いわ\nば洋酒のシンボル的な\n存在なのだが、英語表\n記が［一般に〔米〕では";
PROGMEM const prog_char demo_msg23[] = "Whiskey,〔英〕では\nWhisky.］であること\nはあまり知られていな\nい。米英両国では、この\nスペルの差で自国産と\n輸入品を区別している";
PROGMEM const prog_char demo_msg24[] = "という。わが和製ウイス\nキーの“Whisky”という\n英国式表示は、手本に\nしたスコッチのフォル\nムに倣ったものであり、\nそれ以上の意味はない";
PROGMEM const prog_char demo_msg25[] = "ようだ。カナ表記にして\nも、ごくまれに〈ウヰス\nキー〉という書き方を\n見かけるが、これとて\n差別化を意図したもの\nではなく単にカナづか";
PROGMEM const prog_char demo_msg26[] = "いの時代性にすぎな\nい。\n　現在、カナ表記は〈ウ\nイスキー〉に一本化して\nおり、そこに国産・舶来\nの区別はない。最近は";
PROGMEM const prog_char demo_msg27[] = "価格面での差もちぢま\nり、20年前に￥10,000\nだった本場のスコッチ\nが半値近くになるなど、\nウイスキーは一層ポ\nピュラーなものになっ";
PROGMEM const prog_char demo_msg28[] = "てきた。しかし、消費量\nについていえば、洋酒\nのトップは何といって\nもビールだろう。よっぽ\nどの下戸はともかく、ま\nず誰でもタシナミ程度";
PROGMEM const prog_char demo_msg29[] = "には口にする。\n　そのビールも明治中\n期には、せいぜい好事\n家が味わう程度だった\nらしい。当時流行した川\n上音二郎の《おっぺけ";
PROGMEM const prog_char demo_msg30[] = "ぺ節》では、まるでゲテ\nモノ扱い。「むやみに西\n洋を鼻にかけ／日本酒\nなんぞは飲まれない／\nビールにブランデーベ\nルモット／腹にも馴れ";
PROGMEM const prog_char demo_msg31[] = "ない洋食を／やたらに\n喰うのも負け惜しみ／\nないしょで後架でヘド\nついて／まじめな顔し\nてコーヒのむ」と、特権\n階級をギューの目にあ";
PROGMEM const prog_char demo_msg32[] = "わせるネタにしている。\n観客も「ざまぁみやが\nれ！」とばかり同調し\nて西洋カブレをオチョ\nクり、日々のうっぷんを\n晴らしていた。";
PROGMEM const prog_char demo_msg33[] = "　もっとも、オッペケ\nペッポーの攻勢に、ぐぅ\nのネを上げた紳士や淑\n女がいたかどうか…西\n洋カブレとは、要するに\n欧米文化への信奉者の";
PROGMEM const prog_char demo_msg34[] = "こと。〈鰯ノ頭ヲ信心〉す\nる者があるならヨー\nロッパやアメリカの文\n物を信仰する者がいて\nも一向に不思議はな\nい。信ずる者の強さで、";
PROGMEM const prog_char demo_msg35[] = "依然グラス片手に舞踏\n会、アン・ドゥ・トロヮと\nやっていたように思わ\nれる。\n　憶測になるが、むし\nろ辛かったのは庶民の";
PROGMEM const prog_char demo_msg36[] = "中の飲んべぇ連中では\nなかったか。ハイカラ族\nにユサぶりをかけた手\n前、泰西の酒には手を\n出しづらい。「なに、ビー\nルだとぉ、べらぼーめ、";
PROGMEM const prog_char demo_msg37[] = "こちとら人間サマでぃ、\nカニじゃあるめぇしア\nブクなんか飲めるかっ\nてんだ」などと強がり\nながらも、サゾ飲みた\nかったろうという気が";
PROGMEM const prog_char demo_msg38[] = "する。\n　しかし明治32年７月\n４日、恵比寿ビールの\nＰＲを目的にしたビア\nホールが新橋に登場。\nジャムパン１ヶ２銭の";
PROGMEM const prog_char demo_msg39[] = "時代に 0.25 リットル\n５銭でビールの１ぱい\n売りを始めたことから\n様相は一転する。レン\nガづくりのモダンな店\nながら、はっぴ姿の車夫";
PROGMEM const prog_char demo_msg40[] = "も金バッヂの紳士もゼ\nニを払えば平等の客。\n誰はゞかることなく\nビールをグィとやって\n舌ツヾミを打てる。当初\nから評判になって人々";
PROGMEM const prog_char demo_msg41[] = "がワッと集まり、ファン\nの数もふえた。２ヵ月\n後の９月には中央新聞\nがその盛況ぶりを図入\nりで伝えたほどだった。";
PROGMEM const prog_char demo_msg42[] = "　この年は西暦1899\n年。まさに19世紀の〆\nくくりの年にビールの\nヌーヴェルバーグが\nぼっこうしたことにな\nる。恵比寿は文字通り";
PROGMEM const prog_char demo_msg43[] = "“福の神”となったわけ\nだが、そのころの表記\nはすべて〈惠比壽〉で\nあり、仮名書きの場合\nも〈えびす〉・〈エビス〉\nならぬ〈ゑびす〉・〈ヱ";
PROGMEM const prog_char demo_msg44[] = "ビス〉だった。いかにも\nゴリヤクのありそうな\n字ヅラである。\n　20世紀に入って\n1906年になると、夏目\n漱石は小説『二百十";
PROGMEM const prog_char demo_msg45[] = "日』（中央公論・明治\n39年10月号）の中で\n次のようなエピソード\nを創作した。舞台は阿\n蘇のひなびた温泉宿。";
PROGMEM const prog_char demo_msg46[] = "　　「ビールは御坐り\nまつせん」\n　　「ビールがない？\n―――君ビールはない\nとさ。何だか日本の領\n地でない樣な氣がす";
PROGMEM const prog_char demo_msg47[] = "る。情ない所だ」\n　　「なければ、飮まな\nくつても、いゝさ」と圭\nさんは又泰然たる挨拶\nをする。\n　　「ビールは御坐り";
PROGMEM const prog_char demo_msg48[] = "ませんばつてん、惠比\n壽なら御坐ります」\n　　「ハヽヽヽ愈妙にな\nつて來た。おい君ビー\nルでない惠比壽がある\nつて云ふんだが、その";
PROGMEM const prog_char demo_msg49[] = "惠比壽でも飮んで見る\nかね」\n　　「うん、飮んでも\nいゝ。―――その惠比壽\nは矢つ張り瓶に這入つ\nてるんだらうね、姉さ";
PROGMEM const prog_char demo_msg50[] = "ん」と圭さんは此時漸\nく下女に話かけた。\n\n　ゆ上がりの男２人と\n「絣の筒袖をつんつる\nてんに着てゐる」下女";
PROGMEM const prog_char demo_msg51[] = "のやりとりは、いささか\nマンザイのボケとツッ\nコミのセリフを思わせ\nるが、ビールもこの時\n期には日本のスミズミ\nまでじゅうぶんに出";
PROGMEM const prog_char demo_msg52[] = "回っていたのがわかる。\n　こうして19～20世\n紀初頭に日本人に受け\n入れられたビールは、\nいまでは日常生活に\nすっかりとけこんでい";
PROGMEM const prog_char demo_msg53[] = "る。それだけに洋酒と\nいう感じがうすい。豚カ\nツが〈和食〉でカステラ\nが〈わぐゎし〉なら、ビー\nルが〈日本の味〉であっ\nても何ら不都合はない";
PROGMEM const prog_char demo_msg54[] = "はずだ。実際、国産ビー\nルの製造には米が一役\nかっている。";

PROGMEM const prog_char demo_msg99[] = "\n\n\n　▷▶▷糸冬◁◀◁"; // おしまいメッセージ


typedef PROGMEM const prog_char * prog_str;

// メッセージ char 配列のリスト：こいつは SRAM 上に置いています。もし PROGMEM に置く場合は、pgm_read_word 関数を使用する必要があります。
const prog_str demo_msg_list[] = {
  demo_msg00, demo_msg01, demo_msg02, demo_msg03, demo_msg04, demo_msg05, demo_msg06, demo_msg07, demo_msg08, demo_msg09,
  demo_msg10, demo_msg11,
  demo_msg20, demo_msg21, demo_msg22, demo_msg23, demo_msg24, demo_msg25, demo_msg26, demo_msg27, demo_msg28, demo_msg29,
  demo_msg30, demo_msg31, demo_msg32, demo_msg33, demo_msg34, demo_msg35, demo_msg36, demo_msg37, demo_msg38, demo_msg39,
  demo_msg40, demo_msg41, demo_msg42, demo_msg43, demo_msg44, demo_msg45, demo_msg46, demo_msg47, demo_msg48, demo_msg49,
  demo_msg50, demo_msg51, demo_msg52, demo_msg53, demo_msg54,
  demo_msg99,
};

// デモ用メッセージの個数 ＝ メッセージの末尾番号 + 1
// こうすれば、一々数えなくてもコンパイラが計算してくれるので楽チンです
size_t msg_list_size = sizeof(demo_msg_list) / sizeof(*demo_msg_list);


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

#if 1
  // 注意：ここで、強制的に TWI 通信を高速化させています。カラー漢字 LCD シールドは 400kHz の速度に対応していますが、
  //       標準 100kHz の速度にしか対応していない別の TWI 機器を同時に使用する場合、そちらの TWI 機器が正常に TWI 通信できなくなる可能性があります。
  //       その場合は、ここの「#if 1」の "1" を "0" に書き換えて、TWI 通信の高速化を行わない(つまり標準 100kHz のまま)ようにしてください。
  //       当然ですが、そうした場合は、カラー漢字 LCD シールドとの TWI 通信速度も遅くなってしまいます。
  CKLCD_Wire_setHiSpeed();
#endif

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが、上記の CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(160×256) を背景面に、仮想画面２(160×128) を前景面に設定します。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_F2_B1);

  // 自動フリップモードを OFF にします。flip() で一括して表示させます。
  cklcd.setFlipMode (CKLCD_FLIP_MODE_MANU);

  // 仮想画面１に適当な背景画を描く：邪魔ならコメントアウトしてください
  cklcd.v1.setDrawColor (0, 96, 0); // 暗い緑
  cklcd.v1.rectFill (0, 80, 159, 200); // 矩形描画(塗りつぶし)

  cklcd.v1.setDrawColor (96, 0, 0); // 暗い赤
  cklcd.v1.circleFill (60, 70, 50); // 円描画(塗りつぶし)

  cklcd.v1.setDrawColor (0, 0, 96); // 暗い青
  cklcd.v1.circleFill (120, 190, 60); // 円描画(塗りつぶし)

  // フォントタイプをプロポーショナル(可変幅)にします。
  cklcd.v2.setFontType (CKLCD_FONT_TYPE_PROPORTIONAL);

  // オープニングのメッセージを表示させます（仮想画面２を使用）。
  // プログラムメモリ上の文字列を表示させる時は、ColorKanjiLCDVPlane::print_P 関数を呼び出して下さい。
  cklcd.v2.print_P (demo_msg_list[0]);
  msg_no = 0;

  // LCD 全領域を描画させます
  cklcd.flip();

  // スイッチは押されていない状態にする
  left_sw = right_sw = up_sw = down_sw = false;
}


// ループ関数
void loop()
{
  // 最初にカラー漢字 LCD の現在のステータスを取得します。
  // これを呼び出しておかないと、以下の getRightSW 関数, getLeftSW 関数が
  // 何時までも更新されない事になってしまいます。
  // カラー漢字 LCD がコマンド処理中で忙しい時は、ステータス取得に失敗して CKLCD_ERR_BUSY が返って来ます。
  // ステータス取得成功を意味する CKLCD_ERR_NONE が返ってきた時だけ、処理を進めるようにして下さい。
  if (cklcd.getStatus() == CKLCD_ERR_NONE) {
    // ステータス取得成功した

    // 右スイッチの処理
    if (cklcd.getRightSW()) { // 右スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      right_sw = true;
    } else { // 右スイッチが離された
      if (right_sw == true) { // さっきまで右スイッチが押されていた
        // 次のメッセージを表示させる
        msg_no++; // 1加算
        if (msg_no >= msg_list_size) { // メッセージの末尾に達した
          msg_no = 0; // 最初のメッセージ番号に戻る
        }
        // 仮想画面２をクリアし、カーソル位置を(0,0)にします。
        cklcd.v2.clear ();
        // 該当するメッセージを表示させます
        cklcd.v2.print_P (demo_msg_list[msg_no]);
        // LCD 全領域を描画させます
//      cklcd.flip(); // 下で一括して flip させます
        // 右スイッチ・フラグをクリア
        right_sw = false;
      }
    }

    // 左スイッチの処理
    if (cklcd.getLeftSW()) { // 左スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      left_sw = true;
    } else { // 左スイッチが離された
      if (left_sw == true) { // さっきまで左スイッチが押されていた
        // 前のメッセージを表示させる
        if (msg_no == 0) { // メッセージの先頭の時
          msg_no = msg_list_size; // メッセージの末尾番号+1 に移動する(+1するのは直後で-1するから)
        }
        --msg_no; // 1減算
        // 仮想画面２をクリアし、カーソル位置を(0,0)にします。
        cklcd.v2.clear ();
        // 該当するメッセージを表示させます
        cklcd.v2.print_P (demo_msg_list[msg_no]);
        // LCD 全領域を描画させます
//      cklcd.flip(); // 下で一括して flip させます
        // 左スイッチ・フラグをクリア
        left_sw = false;
      }
    }

    // 上スイッチの処理
    if (cklcd.getUpSW()) { // 上スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      up_sw = true;
    } else { // 上スイッチが離された
      if (up_sw == true) { // さっきまで上スイッチが押されていた
        backScreen ^= 1;  // 背景面を表示させるフラグ
        // 背景面の仮想画面１(160×256) を ON/OFF させます
        cklcd.setScreenMode (backScreen ? CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_F2_B1
                                        : CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_F2);
        // 上スイッチ・フラグをクリア
        up_sw = false;
      }
    }

    // 下スイッチの処理
    if (cklcd.getDownSW()) { // 下スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      down_sw = true;
    } else { // 下スイッチが離された
      if (down_sw == true) { // さっきまで下スイッチが押されていた
        // 背景面をスクロールさせるフラグをトグルさせる
        scrollFlag ^= 1;
        // 下スイッチ・フラグをクリア
        down_sw = false;
      }
    }
  }

  // 10ms 待ちを入れる。
  // カラー漢字 LCD シールドのスイッチのチェック間隔が約 10ms (変更不可) なので、
  // これよりも短い間隔で getStatus 関数を呼び出しても、前と同じスイッチ状態が読み出されるだけです。
  // なので、10ms 以上の間をおいてから、getStatus 関数を呼び出すようにしてください。
//  delay(10); // 下記の flip() で 25ms 弱かかるので、待ち時間としてはそれで十分なので、わざわざ delay() を呼び出さなくても OK です

  if (scrollFlag) { // 背景面をスクロールさせるフラグが立っている時のみスクロールさせます
    // 仮想画面１の LCD 表示開始座標をずらします
    cklcd.v1.setDispStartPos (0, yDispPos++);
  }

  // LCD 全領域を描画させます：25ms 弱かかります
  cklcd.flip();
}

