/*
  Color Kanji LCD Shield - Draw Demo #1 (Portrait)

  本スケッチは "カラー漢字LCDシールド" で、描画関係関数をデモ表示するスケッチ例です。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 14/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


// 描画のパターン
#define DRAW_PATT_LINE1     0 // ライン表示デモ1
#define DRAW_PATT_LINE2     1 // ライン表示デモ2
#define DRAW_PATT_LINE3     2 // ライン表示デモ3
#define DRAW_PATT_RECT      3 // 矩形(線のみ)表示デモ
#define DRAW_PATT_RECT_F    4 // 矩形(塗りつぶし)表示デモ
#define DRAW_PATT_CIRCLE    5 // 円(線のみ)表示デモ
#define DRAW_PATT_CIRCLE_F  6 // 円(塗りつぶし)表示デモ
#define DRAW_PATT_PSET      7 // 点表示デモ
#define MAX_DRAW_PATT_NO    8

uint8_t patt_no; // 表示させるグラフィックのパターンの番号


uint8_t left_sw, right_sw; // カラー漢字 LCD シールドの２つのスイッチの状態


// 32bit論理色のテーブル
const cklcdCol32_t col32Table[7] = {
  CKLCD_COL32_RED,
  CKLCD_COL32_GREEN,
  CKLCD_COL32_YELLOW,
  CKLCD_COL32_BLUE,
  CKLCD_COL32_MAGENTA,
  CKLCD_COL32_CYAN,
  CKLCD_COL32_WHITE,
};


// デモ描画関数
void draw_demo()
{
  uint8_t n, x, y, r, w, h;

  // 仮想画面１をクリアします。
  cklcd.v1.clear ();
  // 仮想画面２をクリアします。
  cklcd.v2.clear ();

  // 仮想画面２のサイズを取得します
  w = (uint8_t)(cklcd.v2.getWidth () - 1);
  h = (uint8_t)(cklcd.v2.getHeight() - 1);

  n = 0;

  // 各種描画関数を呼び出します（仮想画面２を使用）。
  switch (patt_no) {

  case DRAW_PATT_LINE1: // ライン表示デモ1：縦糸巻きパターン
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "ライン表示デモ1");
    for (x = 0; x <= w; x += 4) { // ４画素きざみ
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      cklcd.v2.line (x, 0, w - x, h); // x座標を変化させながら、線を描画させる
    }
    break;

  case DRAW_PATT_LINE2: // ライン表示デモ2：横糸巻きパターン
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "ライン表示デモ2");
    for (y = 0; y <= h; y += 4) { // ４画素きざみ
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      cklcd.v2.line (0, y, w, h - y); // y座標を変化させながら、線を描画させる
    }
    break;

  case DRAW_PATT_LINE3: // ライン表示デモ3：格子パターン
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "ライン表示デモ3");
    for (x = 0; x <= w; x += 4) { // ４画素きざみ
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      cklcd.v2.line (x, 0, x, h); // 縦線を描画させる
    }
    for (y = 0; y <= h; y += 4) { // ４画素きざみ
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      cklcd.v2.line (0, y, w, y); // 横線を描画させる
    }
    break;

  case DRAW_PATT_RECT: // 矩形(線のみ)表示デモ
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "矩形(line)\n表示デモ   ");
    for (x = 0; x < w / 2; x += 3) { // x座標
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      y = x; // y座標の計算
      cklcd.v2.rect (x, y, w - x, h - y); // 矩形(線のみ)を描画させる
    }
    break;

  case DRAW_PATT_RECT_F: // 矩形(塗りつぶし)表示デモ
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "矩形(fill)\n表示デモ  ");
    for (x = 0; x < w / 2; x += 3) { // x座標
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      y = x; // y座標の計算
      cklcd.v2.rectFill (x, y, w - x, h - y); // 矩形(塗りつぶし)を描画させる
    }
    break;

  case DRAW_PATT_CIRCLE: // 円(線のみ)表示デモ
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "円(line)表示デモ");
    for (r = w / 2; r > 2; r -= 3) { // 半径：3～画面横方向の半分
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      cklcd.v2.circle (w / 2, h / 2, r); // 円(線のみ)を描画させる
    }
    break;

  case DRAW_PATT_CIRCLE_F: // 円(塗りつぶし)表示デモ
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "円(fill)表示デモ");
    for (r = w / 2; r > 2; r -= 3) { // 半径：3～画面横方向の半分
      cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
      n = (n + 1) % 7; // 0～6 の範囲にする
      cklcd.v2.circleFill (w / 2, h / 2, r); // 円(塗りつぶし)を描画させる
    }
    break;

  case DRAW_PATT_PSET: // 点表示デモ
    cklcd.v1.glowPrint (0, 0, CKLCD_COL32_WHITE, CKLCD_COL32_BLACK, "点表示デモ\n(時間かかります)");
    // 23×26＝598点の描画なので、やや時間が掛かります
    for (y = 0; y <= h; y += 5) { // ５画素きざみ
      for (x = 0; x <= w; x += 7) { // ７画素きざみ
        cklcd.v2.setDrawColor (col32Table[n]); // 32bit論理色のテーブルから論理色を設定
        n = (n + 1) % 7; // 0～6 の範囲にする
        cklcd.v2.pset (x, y); // 仮想画面２に点を描画
      }
    }
    break;
  }

  // LCD 全領域を描画させます
  cklcd.flip();
}


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

#if 1
  // 注意：ここで、強制的に TWI 通信を高速化させています。カラー漢字 LCD シールドは 400kHz の速度に対応していますが、
  //       標準 100kHz の速度にしか対応していない別の TWI 機器を同時に使用する場合、そちらの TWI 機器が正常に TWI 通信できなくなる可能性があります。
  //       その場合は、ここの「#if 1」の "1" を "0" に書き換えて、TWI 通信の高速化を行わない(つまり標準 100kHz のまま)ようにしてください。
  //       当然ですが、そうした場合は、カラー漢字 LCD シールドとの TWI 通信速度も遅くなってしまいます。
  CKLCD_Wire_setHiSpeed();
#endif

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はポートレート(縦向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(256×160) を背景面に、仮想画面２(128×160) を前景面に設定します。
  cklcd.begin (CKLCD_ORIENT_PORTRAIT + CKLCD_DISP_SCREEN_F1_B2);

  // フォントタイプをプロポーショナル(可変幅)にします。
  cklcd.v1.setFontType (CKLCD_FONT_TYPE_PROPORTIONAL);

  // 描画パターン番号をクリア
  patt_no = 0;

  // デモ描画関数を呼び出します
  draw_demo ();
}


// ループ関数
void loop()
{
  // 最初にカラー漢字 LCD の現在のステータスを取得します。
  // これを呼び出しておかないと、以下の getRightSW 関数, getLeftSW 関数が
  // 何時までも更新されない事になってしまいます。
  // カラー漢字 LCD がコマンド処理中で忙しい時は、ステータス取得に失敗して CKLCD_ERR_BUSY が返って来ます。
  // ステータス取得成功を意味する CKLCD_ERR_NONE が返ってきた時だけ、処理を進めるようにして下さい。
  if (cklcd.getStatus() == CKLCD_ERR_NONE) {
    // ステータス取得成功した

    // 右スイッチの処理
    if (cklcd.getRightSW()) { // 右スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      right_sw = true;
    } else { // 右スイッチが離された
      if (right_sw == true) { // さっきまで右スイッチが押されていた

        // 次の描画パターンを表示させる
        patt_no++; // 1 加算
        if (patt_no >= MAX_DRAW_PATT_NO) { // パターン番号の末尾に達した
          patt_no = 0; // 最初のパターン番号に戻る
        }

        // デモ描画関数を呼び出します
        draw_demo ();

        // 右スイッチ・フラグをクリア
        right_sw = false;
      }
    }

    // 左スイッチの処理
    if (cklcd.getLeftSW()) { // 左スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      left_sw = true;
    } else { // 右スイッチが離された
      if (left_sw == true) { // さっきまで左スイッチが押されていた

        // 前の描画パターンを表示させる
        if (patt_no == 0) { // パターン番号の先頭の時
          patt_no = MAX_DRAW_PATT_NO; // パターン番号の末尾番号+1 に移動する (+1 するのは直後で -1 するから)
        }
        --patt_no; // 1 減算

        // デモ描画関数を呼び出します
        draw_demo ();

        // 左スイッチ・フラグをクリア
        left_sw = false;
      }
    }
  }

  // 10ms 待ちを入れる。
  // カラー漢字 LCD シールドのスイッチのチェック間隔が約10ms(変更不可) なので、
  // これよりも短い間隔で getStatus 関数を呼び出しても、前と同じスイッチ状態が読み出されるだけです。
  // なので、10ms 以上の間をおいてから、getStatus 関数を呼び出すようにしてください。
  delay(10);
}

