/*
  Color Kanji LCD Shield - Font (Monospaced/Proportional) Demo #1

  本スケッチは "カラー漢字 LCD シールド" の、モノスペースとプロポーショナルのデモスケッチ例です。
  ASCII/ISO8859-1 の範囲をメインに表示します。
  左右スイッチで表示文字を切り替えます。
  上スイッチでフォントのサイズを切り替えます。
  下スイッチでフォントの属性を切り替えます。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 14/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


// ※巨大なので、プログラムメモリに格納します。PROGMEM 指定が必須です。
// このリストの全部の文字が表示できる訳ではありません。
// カラー漢字 LCD シールドで表示できない文字は、代わりに '〓' が表示されます。
PROGMEM const prog_char char_list[][32] = {
  "020: !\"#<",
  "024:$%&'<",
  "028:()*+<",
  "02C:,-./<",
  "030:0123<",
  "034:4567<",
  "038:89:;<",
  "03C:<=>?<",

  "040:@ABC<",
  "044:DEFG<",
  "048:HIJK<",
  "04C:LMNO<",
  "050:PQRS<",
  "054:TUVW<",
  "058:XYZ[<",
  "05C:\\]^_<",
  "060:`abc<",
  "064:defg<",
  "068:hijk<",
  "06C:lmno<",
  "070:pqrs<",
  "074:tuvw<",
  "078:xyz{<",
  "07C:|}~<",

  "0A0: ¡¢£<",
  "0A4:¤¥¦§<",
  "0A8:¨©ª«<",
  "0AC:¬­®¯<",
  "0B0:°±²³<",
  "0B4:´µ¶·<",
  "0B8:¸¹º»<",
  "0BC:¼½¾¿<",

  "0C0:ÀÁÂÃ<",
  "0C4:ÄÅÆÇ<",
  "0C8:ÈÉÊË<",
  "0CC:ÌÍÎÏ<",
  "0D0:ÐÑÒÓ<",
  "0D4:ÔÕÖ×<",
  "0D8:ØÙÚÛ<",
  "0DC:ÜÝÞß<",
  "0E0:àáâã<",
  "0E4:äåæç<",
  "0E8:èéêë<",
  "0EC:ìíîï<",
  "0F0:ðñòó<",
  "0F4:ôõö÷<",
  "0F8:øùúû<",
  "0FC:üýþÿ<",

  "100:ĀāĂă<",
  "104:ĄąĆć<",
  "108:ĈĉĊċ<",
  "10C:ČčĎď<",
  "110:ĐđĒē<",
  "114:ĔĕĖė<",
  "118:ĘęĚě<",
  "11C:ĜĝĞğ<",
  "120:ĠġĢģ<",
  "124:ĤĥĦħ<",
  "128:ĨĩĪī<",
  "12C:ĬĭĮį<",
  "130:İıĲĳ<",
  "134:ĴĵĶķ<",
  "138:ĸĹĺĻ<",
  "13C:ļĽľĿ<",

  "140:ŀŁłŃ<",
  "144:ńŅņŇ<",
  "148:ňŉŊŋ<",
  "14C:ŌōŎŏ<",
  "150:ŐőŒœ<",
  "154:ŔŕŖŗ<",
  "158:ŘřŚś<",
  "15C:ŜŝŞş<",
  "160:ŠšŢţ<",
  "164:ŤťŦŧ<",
  "168:ŨũŪū<",
  "16C:ŬŭŮů<",
  "170:ŰűŲų<",
  "174:ŴŵŶŷ<",
  "178:ŸŹźŻ<",
  "17C:żŽžſ<",

  "180:ƀƁƂƃ<",
  "184:ƄƅƆƇ<",
  "188:ƈƉƊƋ<",
  "18C:ƌƍƎƏ<",
  "190:ƐƑƒƓ<",
  "194:ƔƕƖƗ<",
  "198:Ƙƙƚƛ<",
  "19C:ƜƝƞƟ<",
  "1A0:ƠơƢƣ<",
  "1A4:ƤƥƦƧ<",
  "1A8:ƨƩƪƫ<",
  "1AC:ƬƭƮƯ<",
  "1B0:ưƱƲƳ<",
  "1B4:ƴƵƶƷ<",
  "1B8:Ƹƹƺƻ<",
  "1BC:Ƽƽƾƿ<",

  "1C0:ǀǁǂǃ<",
  "1C4:ǄǅǆǇ<",
  "1C8:ǈǉǊǋ<",
  "1CC:ǌǍǎǏ<",
  "1D0:ǐǑǒǓ<",
  "1D4:ǔǕǖǗ<",
  "1D8:ǘǙǚǛ<",
  "1DC:ǜǝǞǟ<",
  "1E0:ǠǡǢǣ<",
  "1E4:ǤǥǦǧ<",
  "1E8:ǨǩǪǫ<",
  "1EC:ǬǭǮǯ<",
  "1F0:ǰǱǲǳ<",
  "1F4:ǴǵǶǷ<",
  "1F8:ǸǹǺǻ<",
  "1FC:ǼǽǾǿ<",

  "200:ȀȁȂȃ<",
  "204:ȄȅȆȇ<",
  "208:ȈȉȊȋ<",
  "20C:ȌȍȎȏ<",
  "210:ȐȑȒȓ<",
  "214:ȔȕȖȗ<",
  "218:ȘșȚț<",
  "21C:ȜȝȞȟ<",
  "220:ȠȡȢȣ<",
  "224:ȤȥȦȧ<",
  "228:ȨȩȪȫ<",
  "22C:ȬȭȮȯ<",
  "230:ȰȱȲȳ<",
  "234:ȴȵȶȷ<",
  "238:ȸȹȺȻ<",
  "23C:ȼȽȾȿ<",

  "X1:⁰⁴⁵⁶⁷⁸⁹<",
  "X2:ⁱⁿ⁺⁻⁼⁽⁾<",
  "X3:₀₁₂₃₄₅₆<",
  "X4:₇₈₉⅓⅔⅕<",
  "X5:€™℉∛<",
  "X6:∓⁑⁂<",
  "Y1:↖↗↘↙<",
  "Y2:⇦⇧⇨⇩<",
  "Y3:↰↱↲↳<",
  "Y4:⇇⇈⇉⇊<",
  "Y5:↕⇅☞<",
  "Z1:☀☁☂☃<",
  "Z2:☎☺☻☼<",
  "Z3:♠♥♦♣<",
  "Z4:♤♡♢♧<",
  "Z5:☖☗⏎⌘<",
  "Z6:✓✔✗✘<",
  "Z7:☐☑☒⏏<",
};

// char_list 配列のエントリー数の計算
#define MAX_DISP_NO  ((sizeof(char_list) / sizeof(*char_list)))

int16_t disp_no; // 表示させたい画面の番号
int16_t curr_no; // 現在、表示している画面の番号

uint8_t font_type_no;  // フォントタイプの番号
uint8_t font_attr_no;  // フォントの属性の番号

uint8_t left_sw, right_sw, up_sw, down_sw; // カラー漢字 LCD シールドの４つのスイッチの状態

// フォントの属性のリスト
const uint8_t font_attr_list[] = {
  0,
  CKLCD_FONT_ATTR_UNDERLINE, // アンダーラインを表示
  CKLCD_FONT_ATTR_STRIKE,    // 取消し線を表示
  CKLCD_FONT_ATTR_INVERTED,  // 文字色を反転
};


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが、上記の CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(160×256) を背景面に設定します。仮想画面２(160×128) は使用しません。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_B1);


  disp_no =  0; // 表示させたい画面の番号：とりあえず 1番目を表示させる
  curr_no = -1; // 現在、表示している画面の番号：無しを意味する -1 に設定しておく

  font_type_no = 0;  // フォントタイプの番号
  font_attr_no = 0;  // フォントの属性の番号
  cklcd.v1.setFontAttr (font_attr_list[font_attr_no]); // フォントの属性を設定

  // スイッチは押されていない状態にする
  left_sw = right_sw = up_sw = down_sw = false;
}


// ループ関数
void loop()
{
  // 最初にカラー漢字 LCD の現在のステータスを取得します。
  // これを呼び出しておかないと、以下の getLeft/Up/Down/RightSW 関数が
  // 何時までも更新されない事になってしまいます。
  // カラー漢字 LCD がコマンド処理中で忙しい時は、ステータス取得に失敗して CKLCD_ERR_BUSY が返って来ます。
  // ステータス取得成功を意味する CKLCD_ERR_NONE が返ってきた時だけ、処理を進めるようにして下さい。
  if (cklcd.getStatus() == CKLCD_ERR_NONE) {
    // ステータス取得成功した

    // 左スイッチの処理
    if (cklcd.getLeftSW()) { // 左スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      left_sw = true;
    } else { // 左スイッチが離された
      if (left_sw == true) { // さっきまで左スイッチが押されていた
        if (disp_no == 0) {
          disp_no = MAX_DISP_NO;
        }
        disp_no--;
        // 左スイッチ・フラグをクリア
        left_sw = false;
      }
    }

    // 上スイッチの処理
    if (cklcd.getUpSW()) { // 上スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      up_sw = true;
    } else { // 上スイッチが離された
      if (up_sw == true) { // さっきまで上スイッチが押されていた
        // フォントタイプを変更します
        font_type_no++;
        font_type_no &= 3; // 0～3の間を巡回させます
        curr_no = -1; // 強制的に表示を更新させます
        // 上スイッチ・フラグをクリア
        up_sw = false;
      }
    }

    // 下スイッチの処理
    if (cklcd.getDownSW()) { // 下スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      down_sw = true;
    } else { // 下スイッチが離された
      if (down_sw == true) { // さっきまで下スイッチが押されていた
        // フォントの属性を変更します
        font_attr_no++;
        font_attr_no &= 3; // 0～3の間を巡回させます
        cklcd.v1.setFontAttr (font_attr_list[font_attr_no]); // フォントの属性を設定
        curr_no = -1; // 強制的に表示を更新させます
        // 下スイッチ・フラグをクリア
        down_sw = false;
      }
    }

    // 右スイッチの処理
    if (cklcd.getRightSW()) { // 右スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      right_sw = true;
    } else { // 右スイッチが離された
      if (right_sw == true) { // さっきまで右スイッチが押されていた
        disp_no++;
        if (disp_no >= MAX_DISP_NO) {
          disp_no = 0;
        }
        // 右スイッチ・フラグをクリア
        right_sw = false;
      }
    }
  }

  if (curr_no != disp_no) { // 現在表示している画面と表示させたい画面の番号が一致していない
    cklcd.v1.clear(); // 一旦、仮想画面をクリアする

    if (font_type_no == 0) {
      // フォントタイプをルビ文字のモノスペース(固定幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_RUBY_MS);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");

      // フォントタイプをルビ文字のプロポーショナル(可変幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_RUBY_P);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");

      // フォントタイプを 12px 文字のモノスペース(固定幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_12px_MS);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");

      // フォントタイプを 12px 文字のプロポーショナル(可変幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_12px_P);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");
    }

    // フォントタイプをモノスペース(固定幅)にします。
    cklcd.v1.setFontType (CKLCD_FONT_TYPE_MONOSPACED);
    cklcd.v1.print_P (char_list[disp_no]);
    cklcd.v1.print ("\n");

    // フォントタイプをプロポーショナル(可変幅)にします。
    cklcd.v1.setFontType (CKLCD_FONT_TYPE_PROPORTIONAL);
    cklcd.v1.print_P (char_list[disp_no]);
    cklcd.v1.print ("\n");

    if (font_type_no == 1) {
      // フォントタイプを 20px 文字のモノスペース(固定幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_20px_MS);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");

      // フォントタイプを 20px 文字のプロポーショナル(可変幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_20px_P);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");
    }

    if (font_type_no == 2) {
      // フォントタイプを 24px 文字のモノスペース(固定幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_24px_MS);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");

      // フォントタイプを 24px 文字のプロポーショナル(可変幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_24px_P);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");
    }

    if (font_type_no == 3) {
      // フォントタイプを 28px 文字のモノスペース(固定幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_28px_MS);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");

      // フォントタイプを 24px 文字のプロポーショナル(可変幅)にします。
      cklcd.v1.setFontType (CKLCD_FONT_TYPE_28px_P);
      cklcd.v1.print_P (char_list[disp_no]);
      cklcd.v1.print ("\n");
    }

    curr_no = disp_no; // 現在、表示している画面の番号を更新
  }
}

