/*
  Color Kanji LCD Shield - Sine Wave demo #1

  本スケッチは "カラー漢字 LCD シールド" で、サイン波を表示させるデモです。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 27/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが、上記の CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(160×256) を背景面に設定します。仮想画面２(160×128) は使用しません。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_B1);

  // 仮想画面１にサイン波を描画します。
  cklcd.v1.setDrawColor (CKLCD_COL32_WHITE); // 白色
  uint8_t xPos;
  for (xPos = 0; xPos < CKLCD_MAX_L_WIDTH; xPos++) {
    float rad = (xPos * TWO_PI / CKLCD_MAX_L_WIDTH); // 0°～359°に相当するラジアン値の計算
    float val = sin(rad) * (CKLCD_MAX_L_HEIGHT / 3); // サイン値を計算。そのままでは -1～0～+1 の範囲なので、42倍して -42～0～+42 になるようにする
    uint8_t yPos = (uint8_t)(val + (CKLCD_MAX_L_HEIGHT / 2) + 0.5); // (0,64)-(159,64) を中心線として y 座標を計算
    cklcd.v1.pset (xPos, yPos); // 点を打つ
  }
}


uint8_t xStart; // サイン波の開始位置


// ループ関数
void loop()
{
  // 仮想画面１の LCD 表示開始座標をずらします
  cklcd.v1.setDispStartPos (xStart, 0);

  // LCD 全領域を描画させます：25ms 弱かかります
  cklcd.flip();

  // 次の開始位置にします
  xStart++;
  if (xStart >= CKLCD_MAX_L_WIDTH) {
    xStart = 0;
  }
}

