/*
  Color Kanji LCD Shield - Unicode Supplementary Ideographic Plane test sketch example

  本スケッチは "カラー漢字 LCD シールド" で、Unicode 追加漢字面(SIP)の文字を表示するスケッチ例です。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 15/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが、上記の CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(160×256) を背景面に設定します。仮想画面２(160×128) は使用しません。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_B1);

  // 日本語のメッセージを表示させます（仮想画面１を使用）。
  // Arduino IDE の符号化方式は UTF-8 方式なのですが、Unicode 追加漢字面(SIP)の文字は直接記述できない（コンパイルエラーになる）為、
  // カラー漢字 LCD シールドに表示させるには、面倒ですが、以下の様に記述する必要があります。
  cklcd.v1.print ("追加漢字面の文字一覧\n");

  cklcd.v1.print ("\xF0\xA0\x80\x8B");  // 1-14-02  U+2000B  F0 A0 80 8B  𠀋
  cklcd.v1.print ("\xF0\xA1\x88\xBD");  // 1-15-34  U+2123D  F0 A1 88 BD  𡈽
  cklcd.v1.print ("\xF0\xA1\x8C\x9B");  // 1-15-44  U+2131B  F0 A1 8C 9B  𡌛
  cklcd.v1.print ("\xF0\xA1\x91\xAE");  // 1-15-64  U+2146E  F0 A1 91 AE  𡑮
  cklcd.v1.print ("\xF0\xA1\xA2\xBD");  // 1-15-91  U+218BD  F0 A1 A2 BD  𡢽
  cklcd.v1.print ("\xF0\xA0\xAE\x9F");  // 1-47-52  U+20B9F  F0 A0 AE 9F  𠮟  ※2010年版の改定常用漢字：叱の異体字
  cklcd.v1.print ("\xF0\xA1\x9A\xB4");  // 1-47-67  U+216B4  F0 A1 9A B4  𡚴
  cklcd.v1.print ("\xF0\xA1\xB8\xB4");  // 1-47-78  U+21E34  F0 A1 B8 B4  𡸴
  cklcd.v1.print ("\n");

  cklcd.v1.print ("\xF0\xA3\x87\x84");  // 1-85-26  U+231C4  F0 A3 87 84  𣇄
  cklcd.v1.print ("\xF0\xA3\x97\x84");  // 1-85-82  U+235C4  F0 A3 97 84  𣗄
  cklcd.v1.print ("\xF0\xA3\x9C\xBF");  // 1-86-09  U+2373F  F0 A3 9C BF  𣜿
  cklcd.v1.print ("\xF0\xA3\x9D\xA3");  // 1-86-18  U+23763  F0 A3 9D A3  𣝣
  cklcd.v1.print ("\xF0\xA3\xB3\xBE");  // 1-86-64  U+23CFE  F0 A3 B3 BE  𣳾
  cklcd.v1.print ("\xF0\xA4\x9F\xB1");  // 1-87-76  U+247F1  F0 A4 9F B1  𤟱
  cklcd.v1.print ("\xF0\xA5\x92\x8E");  // 1-88-94  U+2548E  F0 A5 92 8E  𥒎
  cklcd.v1.print ("\xF0\xA5\x94\x8E");  // 1-89-09  U+2550E  F0 A5 94 8E  𥔎
  cklcd.v1.print ("\n");

  cklcd.v1.print ("\xF0\xA5\x9D\xB1");  // 1-89-39  U+25771  F0 A5 9D B1  𥝱  ※数の単位：10²⁴：垓の上、穣の下
  cklcd.v1.print ("\xF0\xA5\xA7\x84");  // 1-89-52  U+259C4  F0 A5 A7 84  𥧄
  cklcd.v1.print ("\xF0\xA5\xB6\xA1");  // 1-89-78  U+25DA1  F0 A5 B6 A1  𥶡
  cklcd.v1.print ("\xF0\xA6\xAB\xBF");  // 1-90-61  U+26AFF  F0 A6 AB BF  𦫿
  cklcd.v1.print ("\xF0\xA6\xB9\x80");  // 1-91-19  U+26E40  F0 A6 B9 80  𦹀
  cklcd.v1.print ("\xF0\xA7\x83\xB4");  // 1-91-41  U+270F4  F0 A7 83 B4  𧃴
  cklcd.v1.print ("\xF0\xA7\x9A\x84");  // 1-91-76  U+27684  F0 A7 9A 84  𧚄
  cklcd.v1.print ("\xF0\xA8\x89\xB7");  // 1-92-41  U+28277  F0 A8 89 B7  𨉷
  cklcd.v1.print ("\n");

  cklcd.v1.print ("\xF0\xA8\x8F\x8D");  // 1-92-49  U+283CD  F0 A8 8F 8D  𨏍
  cklcd.v1.print ("\xF0\xAA\x86\x90");  // 1-94-70  U+2A190  F0 AA 86 90  𪆐
}


// ループ関数
void loop()
{
}

