/*
  Color Kanji LCD Shield - Rotation Sphere GIF Demo #1

  本スケッチは "カラー漢字LCDシールド" で、Rotation Sphere アニメーション GIF をデモ表示するスケッチ例です。
  アニメーション GIF は PROGMEM 上で完結していますので、SD カードに保存する必要はありません。
  また、右のタクトSW が押されている間は、アニメーションを停止します。離すとアニメーションを再開します。

  コンパイルすると 29kB 強になるので、Arduino Uno (ATmega328P) 等のフラッシュメモリが 32kB 以上でないと動作しません。
  また、Uno だとコンパイルできるのですが、Leonardo では「スケッチが大きすぎます」エラーになってしまいますので、
  SD を使用する cklcd_rot_sphere_gif_from_sd_demo1_sketch の方を試してください。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 16/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


const int16_t imgLeft = 30; // GIF 画像開始 x 座標
const int16_t imgTop  =  4; // GIF 画像開始 y 座標

// Rotation Sphere GIF パターン : 100×121 画素, 22kB, 6 frames
//   Made by BorisFromStockdale, from http://en.wikipedia.org/wiki/File:Rotating_Sphere.gif
//     La bildo estas kopiita de wikipedia:en. La originala priskribo estas:
//     made with mathematica
//   This file is licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported license.
//   このファイルはクリエイティブ・コモンズ 表示-継承 3.0 (CC BY-SA 3.0) 非移植ライセンスのもとに利用を許諾されています。
// ※巨大なので、プログラムメモリに格納します。PROGMEM 指定が必須です。
PROGMEM const uint8_t RotSphereGif[] = {
	0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x64, 0x00, 0x79, 0x00, 0xD5, 0xFF, 0x00, 0xD6, 0xAB, 0xB0, 
	0x56, 0x8E, 0xEA, 0xB1, 0x6B, 0x84, 0xF3, 0xA3, 0x6C, 0x8E, 0x8D, 0xCF, 0xF0, 0xD0, 0xB4, 0x6B, 
	0x61, 0xB6, 0xD2, 0x90, 0x8C, 0xCA, 0xB5, 0xC8, 0x8B, 0x72, 0xB4, 0xD4, 0xC9, 0xD3, 0xB7, 0xA7, 
	0xCB, 0xAA, 0x99, 0xCA, 0x01, 0x00, 0x01, 0xA4, 0x4E, 0x63, 0xB1, 0xB2, 0xDE, 0x61, 0x1E, 0x66, 
	0x68, 0x79, 0xD2, 0x8B, 0x54, 0x8B, 0xCA, 0xA0, 0xB0, 0xCC, 0x6C, 0x5E, 0xEF, 0xEE, 0xD0, 0xA1, 
	0xC7, 0xF7, 0xAA, 0xEB, 0xF7, 0x98, 0x29, 0x31, 0xEE, 0xB4, 0x93, 0x17, 0x1D, 0x9C, 0xB9, 0x97, 
	0xB5, 0xAA, 0x8C, 0xB8, 0x26, 0x5E, 0xD3, 0xB8, 0x89, 0xA5, 0x19, 0x02, 0x5E, 0xD1, 0xEF, 0xEB, 
	0x63, 0x49, 0xA1, 0xC5, 0xD4, 0xEA, 0x9E, 0x67, 0x92, 0x81, 0xAA, 0xF0, 0x67, 0x06, 0x27, 0xC9, 
	0x4C, 0x0E, 0xFA, 0xD3, 0x71, 0xDF, 0xB9, 0xB5, 0xB4, 0x40, 0x2D, 0xFF, 0xC8, 0x8A, 0xD6, 0xFF, 
	0xD9, 0xEB, 0x83, 0x3C, 0xFF, 0xFF, 0xFF, 0xA5, 0x77, 0xA2, 0xBA, 0x80, 0x9C, 0xC5, 0x7B, 0x84, 
	0xAD, 0x84, 0xA5, 0xA1, 0x84, 0xB5, 0x87, 0x37, 0x63, 0xDC, 0xA0, 0x94, 0x45, 0x34, 0x9B, 0xFF, 
	0x00, 0x00, 0xC4, 0x92, 0xA3, 0x71, 0x3B, 0x7B, 0xC0, 0xBF, 0xDB, 0xA1, 0xAC, 0xDF, 0xDC, 0xD9, 
	0xD8, 0xF0, 0xF1, 0x9D, 0xF4, 0xF5, 0xB5, 0x1E, 0x45, 0xC1, 0xA5, 0xED, 0xA5, 0x21, 0xFF, 0x0B, 
	0x41, 0x44, 0x4F, 0x42, 0x45, 0x3A, 0x49, 0x52, 0x31, 0x2E, 0x30, 0x02, 0xDE, 0xED, 0x00, 0x21, 
	0xFF, 0x0B, 0x4E, 0x45, 0x54, 0x53, 0x43, 0x41, 0x50, 0x45, 0x32, 0x2E, 0x30, 0x03, 0x01, 0x00, 
	0x00, 0x00, 0x21, 0xF9, 0x04, 0x04, 0x0A, 0x00, 0x00, 0x00, 0x2C, 0x00, 0x00, 0x00, 0x00, 0x64, 
	0x00, 0x79, 0x00, 0x00, 0x06, 0xFF, 0xC0, 0x96, 0x70, 0x48, 0x2C, 0x1A, 0x8F, 0xC8, 0x24, 0xD1, 
	0xA6, 0x6C, 0x3A, 0x9F, 0xD0, 0xA8, 0x14, 0xC9, 0x9C, 0x5A, 0xAF, 0xD8, 0xEC, 0xB0, 0xAA, 0xED, 
	0x7A, 0xBF, 0x5B, 0xB0, 0x78, 0x6C, 0xE5, 0x92, 0xCF, 0x68, 0xA3, 0x39, 0xCD, 0x3E, 0xAF, 0xDB, 
	0xF0, 0xEF, 0x3B, 0x4E, 0xC7, 0xCE, 0xEB, 0xF8, 0xE8, 0x3D, 0xCF, 0x57, 0xEE, 0xFB, 0x80, 0x45, 
	0x7F, 0x81, 0x84, 0x83, 0x84, 0x80, 0x86, 0x87, 0x7C, 0x89, 0x8A, 0x78, 0x8C, 0x8D, 0x68, 0x0D, 
	0x92, 0x93, 0x36, 0x93, 0x93, 0x01, 0x16, 0x0D, 0x90, 0x64, 0x96, 0x96, 0x24, 0x16, 0x24, 0x0D, 
	0x36, 0x16, 0x99, 0x17, 0x16, 0x17, 0xA6, 0x17, 0x0D, 0x99, 0x92, 0x9B, 0x58, 0x92, 0x01, 0x01, 
	0x93, 0xA1, 0xAB, 0xA4, 0x99, 0x95, 0x92, 0xA8, 0xA7, 0xB9, 0x93, 0xA4, 0xAD, 0xAE, 0x4E, 0x9D, 
	0xB3, 0x24, 0xC4, 0xB4, 0x92, 0x16, 0xA3, 0xBE, 0xB9, 0xA8, 0xCC, 0xC2, 0x99, 0xC0, 0x47, 0xC2, 
	0xC5, 0xC6, 0xBD, 0xB6, 0xC9, 0xB6, 0xAC, 0xA8, 0xCB, 0xAA, 0xDA, 0x93, 0xD0, 0x43, 0x0D, 0xB1, 
	0xC2, 0x9D, 0xD8, 0xD6, 0xE5, 0xB6, 0xC2, 0xDA, 0xCC, 0xDA, 0xD0, 0x9D, 0xD3, 0xD3, 0xB5, 0xE7, 
	0xC8, 0xF2, 0xE8, 0xCD, 0x9D, 0xCD, 0x3F, 0x90, 0x96, 0xE2, 0xEF, 0xC4, 0xC7, 0xF4, 0xD7, 0xE4, 
	0x4D, 0x02, 0xB1, 0x4E, 0x95, 0xA5, 0x0B, 0x3F, 0x34, 0x11, 0x82, 0x15, 0x4B, 0x56, 0x31, 0x77, 
	0xF1, 0xE8, 0x49, 0x6C, 0x20, 0x42, 0xC4, 0x3D, 0x55, 0x2B, 0x40, 0xAC, 0x98, 0x94, 0x0F, 0x50, 
	0xB8, 0x86, 0xE2, 0x86, 0x19, 0x0B, 0x25, 0xB1, 0x24, 0x45, 0x11, 0x20, 0x2A, 0xA2, 0x1C, 0x78, 
	0x21, 0xE3, 0x0A, 0x83, 0x7C, 0x18, 0x86, 0x94, 0xF4, 0x90, 0x26, 0x09, 0x1D, 0x24, 0x4B, 0x0A, 
	0xAC, 0x28, 0x09, 0xA5, 0x4A, 0xFF, 0x10, 0x0D, 0x80, 0x6A, 0xBC, 0x00, 0x74, 0x63, 0xC7, 0x38, 
	0x0C, 0x2F, 0x39, 0x1C, 0x86, 0x93, 0x96, 0x8E, 0x88, 0x3A, 0x33, 0x59, 0x54, 0xD9, 0xD3, 0x67, 
	0x50, 0xA0, 0x92, 0x32, 0x36, 0xD8, 0xA8, 0x90, 0x8D, 0xA4, 0x0E, 0x4A, 0x1B, 0xD2, 0xC4, 0xD9, 
	0x54, 0x12, 0xD9, 0x9C, 0x51, 0x4F, 0x5A, 0x54, 0x3B, 0xC9, 0x22, 0x88, 0xB7, 0x6F, 0xB3, 0x6E, 
	0xE5, 0x9A, 0xA6, 0x41, 0x07, 0xB0, 0x1F, 0x03, 0x10, 0x90, 0xD5, 0x80, 0x40, 0x03, 0x78, 0x0D, 
	0xC8, 0x32, 0x08, 0xF5, 0x74, 0x9C, 0xB0, 0x1C, 0x14, 0x73, 0xF0, 0x54, 0xC9, 0xF3, 0x2A, 0xDC, 
	0xB8, 0x73, 0x57, 0x6C, 0x3C, 0x63, 0x17, 0x2C, 0x58, 0x90, 0x7D, 0xF7, 0xFA, 0x7D, 0xAA, 0xA3, 
	0xE9, 0x60, 0xC2, 0x9C, 0x3B, 0x37, 0x78, 0x40, 0xBA, 0x41, 0x8E, 0xD3, 0xA6, 0x51, 0xAB, 0x44, 
	0xCC, 0x76, 0xC7, 0x0E, 0xC7, 0x70, 0x23, 0x1B, 0x1D, 0x63, 0xF7, 0x6B, 0xB8, 0xBD, 0x92, 0x34, 
	0x93, 0xF0, 0x7B, 0x93, 0x18, 0x03, 0x9C, 0x66, 0x43, 0x8B, 0x26, 0xFD, 0x20, 0x35, 0xEA, 0xD3, 
	0x88, 0x2D, 0x30, 0xA6, 0xD8, 0x60, 0x87, 0x08, 0xD7, 0x92, 0x1E, 0x37, 0xA8, 0x10, 0xF9, 0x97, 
	0x97, 0xDA, 0x77, 0x2F, 0xE7, 0xD6, 0x2D, 0x8B, 0xF7, 0x67, 0xD0, 0xA2, 0x3B, 0x0F, 0x2F, 0x8D, 
	0x1C, 0x71, 0xF9, 0x93, 0x8A, 0xD7, 0xBA, 0x76, 0x0E, 0xDD, 0xF1, 0xD6, 0xAC, 0x74, 0xBB, 0x60, 
	0xBF, 0x1B, 0x21, 0x77, 0x80, 0xDD, 0xBB, 0xFD, 0x12, 0xD8, 0xFF, 0x57, 0xC7, 0x6F, 0xF1, 0xC2, 
	0x8D, 0x57, 0xDC, 0x69, 0x16, 0x3C, 0x60, 0x1E, 0x72, 0x6A, 0x59, 0xA4, 0x40, 0x73, 0xEB, 0x3D, 
	0xF7, 0x5A, 0x50, 0xD3, 0xAD, 0x40, 0x9D, 0x51, 0x5D, 0x5D, 0x31, 0x49, 0x76, 0x60, 0xD5, 0x97, 
	0x9F, 0x5E, 0xDD, 0xED, 0xE7, 0xFF, 0x61, 0x60, 0x00, 0x3E, 0xF5, 0x80, 0x78, 0xA3, 0x8D, 0x58, 
	0x9A, 0x81, 0xC7, 0x9D, 0x27, 0x02, 0x62, 0x0A, 0xE4, 0xB0, 0x20, 0x7B, 0xEB, 0xB5, 0x27, 0xA1, 
	0x84, 0xEF, 0x1D, 0x65, 0xC5, 0x85, 0x1D, 0x44, 0x90, 0x61, 0x7D, 0xFB, 0xE9, 0x25, 0x09, 0x03, 
	0x1E, 0xEE, 0x16, 0x18, 0x88, 0xE1, 0x01, 0x58, 0x22, 0x71, 0xC6, 0xA5, 0x88, 0x60, 0x03, 0x0A, 
	0x34, 0xD9, 0x24, 0x83, 0xEC, 0x3D, 0xD7, 0x5C, 0x84, 0x15, 0x70, 0xF5, 0x83, 0x8D, 0x52, 0x34, 
	0xA0, 0x41, 0x65, 0x97, 0x45, 0x50, 0xDF, 0x7D, 0xFA, 0x05, 0xC9, 0x1F, 0x03, 0x64, 0x9A, 0x35, 
	0x5A, 0x88, 0x47, 0x92, 0x97, 0x83, 0x0E, 0x08, 0xB2, 0x66, 0x1A, 0x93, 0x2D, 0xC6, 0x29, 0x49, 
	0x8C, 0xAF, 0xED, 0x40, 0x5D, 0x05, 0x2D, 0x54, 0x29, 0x49, 0x42, 0x37, 0x6E, 0x49, 0x5F, 0x03, 
	0x5E, 0x06, 0xDA, 0x57, 0x66, 0xF9, 0x8D, 0x39, 0x18, 0x99, 0x85, 0x4D, 0x22, 0x1E, 0x69, 0x6C, 
	0xEA, 0x30, 0xE0, 0x92, 0x92, 0x94, 0xC7, 0xA2, 0x93, 0x4D, 0x22, 0xC6, 0x1E, 0x94, 0xAF, 0x49, 
	0x48, 0x5D, 0x0F, 0x0D, 0x60, 0x19, 0x8C, 0x06, 0x3E, 0x54, 0x16, 0x4E, 0xA0, 0x5E, 0x12, 0x2A, 
	0xA6, 0x77, 0x40, 0x96, 0xB9, 0x00, 0x03, 0xAB, 0x1A, 0x26, 0xCC, 0x02, 0xB0, 0x26, 0x79, 0x5C, 
	0x9C, 0x4E, 0x4E, 0xE9, 0x5A, 0x05, 0x76, 0x66, 0xBA, 0x51, 0x0F, 0x9C, 0x7A, 0x9A, 0x44, 0x03, 
	0x3E, 0x80, 0x5A, 0x1B, 0xA9, 0xA5, 0x86, 0xE9, 0xA1, 0xA1, 0x88, 0x12, 0xC0, 0x40, 0x03, 0x64, 
	0x36, 0xAB, 0x2A, 0xAC, 0xB1, 0x42, 0xBB, 0x00, 0x02, 0x0F, 0x20, 0x20, 0x2B, 0x6A, 0x4E, 0xBA, 
	0xE8, 0xE2, 0x9C, 0xB8, 0xC6, 0x38, 0x5D, 0x03, 0x3D, 0x54, 0xD0, 0x2B, 0x14, 0x5A, 0x06, 0x0B, 
	0x0B, 0xB1, 0x7D, 0x45, 0x10, 0xFF, 0x66, 0x7E, 0x81, 0x91, 0x49, 0x02, 0xA2, 0xCC, 0x3A, 0xDB, 
	0x6C, 0x03, 0xD2, 0x2E, 0x40, 0x2F, 0xB5, 0x08, 0x40, 0x7B, 0x2D, 0xB6, 0x94, 0x2E, 0xF8, 0x62, 
	0xAE, 0xDD, 0x4A, 0x22, 0x2E, 0xAF, 0xD6, 0xFD, 0x0A, 0xAC, 0x9F, 0x80, 0x92, 0x9A, 0xD9, 0x87, 
	0x1F, 0xBA, 0x4B, 0xA6, 0xB2, 0xCB, 0xCA, 0xFB, 0x6C, 0xBD, 0xF7, 0x2E, 0x50, 0xED, 0xB4, 0xFB, 
	0xB2, 0xB8, 0x2D, 0x9C, 0xFF, 0xDE, 0x6A, 0x27, 0xB8, 0x3D, 0x48, 0xC8, 0x69, 0x30, 0xE6, 0x76, 
	0x60, 0x40, 0xC2, 0xC5, 0xAA, 0xAB, 0xB2, 0x7E, 0x81, 0x41, 0xEC, 0x70, 0xBC, 0x12, 0xD3, 0x5B, 
	0xAF, 0xBD, 0xD4, 0x4E, 0x6B, 0xB1, 0xBD, 0x92, 0x9E, 0xE7, 0x6F, 0xBF, 0xCD, 0x75, 0xFB, 0x5A, 
	0x01, 0x20, 0x87, 0x5B, 0xB0, 0x11, 0x92, 0xF8, 0x69, 0xF2, 0x8E, 0xE9, 0x7A, 0xA8, 0x2E, 0x7F, 
	0x7D, 0xF9, 0x77, 0x68, 0xC4, 0x12, 0x4F, 0x2C, 0xAD, 0xCC, 0xF9, 0xDA, 0x6C, 0x6D, 0xCE, 0xA8, 
	0x71, 0xCC, 0xB3, 0xC7, 0xD3, 0x01, 0xBD, 0x02, 0xC1, 0x15, 0x12, 0xAD, 0x65, 0x03, 0x06, 0x18, 
	0x60, 0x32, 0xA0, 0x0B, 0xAF, 0x4C, 0x80, 0xBA, 0x83, 0x32, 0xFB, 0x2E, 0x90, 0x30, 0xC7, 0x3C, 
	0xB3, 0xBD, 0x36, 0x5F, 0x6C, 0xF1, 0xB5, 0x92, 0x58, 0xDB, 0x6F, 0x93, 0x28, 0xFC, 0xDB, 0x40, 
	0x01, 0x05, 0x54, 0x00, 0xB4, 0xD0, 0x61, 0x13, 0x31, 0x76, 0x65, 0x66, 0x9F, 0xBD, 0xF6, 0xE2, 
	0x11, 0xC8, 0xB0, 0xF8, 0xC2, 0x7E, 0xFD, 0x18, 0xB7, 0xDC, 0x33, 0xD3, 0x6B, 0x77, 0xBE, 0x57, 
	0x2F, 0x89, 0xC0, 0xE6, 0x5A, 0xEF, 0x3D, 0xA5, 0xE0, 0x81, 0x4F, 0xC7, 0x29, 0x0F, 0x85, 0x0B, 
	0x71, 0x61, 0xD9, 0x1D, 0x84, 0x60, 0x76, 0x66, 0x4B, 0xEF, 0xB7, 0x74, 0x03, 0x1C, 0x10, 0x10, 
	0xFB, 0x87, 0x93, 0x44, 0x2D, 0xFF, 0xF3, 0xDC, 0x54, 0x3F, 0x70, 0xF3, 0xD5, 0x79, 0x6F, 0xEE, 
	0x7B, 0xE7, 0x3C, 0x03, 0x1E, 0x38, 0xE0, 0xDF, 0x92, 0x1E, 0xCD, 0x96, 0x66, 0x87, 0x70, 0xD7, 
	0xEA, 0x27, 0x2F, 0xAD, 0x36, 0xE4, 0xB2, 0x23, 0xDB, 0x09, 0x99, 0x73, 0xEB, 0x7B, 0x79, 0x27, 
	0xBE, 0x67, 0xCF, 0xF9, 0xDE, 0xB5, 0x06, 0x0E, 0xBA, 0xE0, 0x92, 0x18, 0x4F, 0xF4, 0x07, 0x76, 
	0x99, 0x6D, 0x80, 0xF2, 0x27, 0x27, 0x70, 0x32, 0xE3, 0xFC, 0x79, 0x38, 0xFB, 0x7E, 0x1C, 0xC0, 
	0xCE, 0x00, 0x07, 0xF2, 0xBA, 0xEA, 0xEA, 0xD4, 0xDA, 0x67, 0x0F, 0xFC, 0x93, 0x40, 0x03, 0x2E, 
	0xF8, 0xFF, 0x0D, 0xE0, 0xC1, 0x09, 0x0A, 0x57, 0xB4, 0xF2, 0x2D, 0x8F, 0x6C, 0x11, 0x30, 0x40, 
	0x02, 0xD2, 0xA6, 0x9F, 0xF7, 0xBD, 0x8F, 0x59, 0xCA, 0xA2, 0x9F, 0x04, 0x55, 0xC5, 0xAA, 0x0A, 
	0xAE, 0x8A, 0x6E, 0xD5, 0x6B, 0x40, 0xFE, 0xF4, 0xE7, 0xB9, 0xBE, 0x75, 0x6D, 0x78, 0xE0, 0xE3, 
	0x81, 0xF8, 0x0C, 0xD7, 0x80, 0x1A, 0x34, 0x00, 0x7D, 0xAB, 0x53, 0x9F, 0xFA, 0xD6, 0xC7, 0xB6, 
	0x63, 0xCD, 0x2E, 0x76, 0xCC, 0xA2, 0x1F, 0x99, 0xE8, 0x07, 0xB1, 0xF8, 0x31, 0x60, 0x03, 0x37, 
	0xCC, 0x21, 0x0E, 0x6F, 0x57, 0xB9, 0x0D, 0xFE, 0x8E, 0x67, 0x0A, 0x00, 0x1C, 0x0A, 0xFE, 0x06, 
	0xBA, 0xFE, 0x05, 0x90, 0x80, 0x5A, 0xF2, 0x93, 0xF2, 0x1A, 0xA0, 0xBE, 0x04, 0x26, 0x70, 0x81, 
	0xC6, 0x7A, 0xDF, 0x87, 0x26, 0x18, 0xC1, 0x19, 0x32, 0xEB, 0x82, 0x16, 0x64, 0x15, 0xBD, 0x26, 
	0x00, 0x2B, 0x2E, 0x2E, 0xC0, 0x8B, 0x1A, 0xF4, 0xA1, 0xDE, 0xF8, 0x87, 0x82, 0x02, 0xF4, 0xAD, 
	0x7F, 0x00, 0x24, 0x1D, 0xE9, 0x2A, 0x54, 0x34, 0x0D, 0xD4, 0x00, 0x61, 0x4D, 0x54, 0xA0, 0x1C, 
	0x4F, 0xD6, 0x17, 0x07, 0x46, 0xFF, 0xCF, 0x2F, 0xCD, 0x92, 0xE1, 0xFC, 0x54, 0x85, 0xC3, 0x0D, 
	0x2C, 0xA0, 0x8F, 0x7F, 0xE4, 0x61, 0x0F, 0xC5, 0xA8, 0xB7, 0xBF, 0x95, 0xB1, 0x8C, 0x66, 0xEC, 
	0x1A, 0xF8, 0x88, 0x57, 0xB0, 0xA2, 0xBD, 0x31, 0x54, 0x27, 0x93, 0x63, 0x1C, 0x13, 0x50, 0x9F, 
	0x05, 0xD6, 0xD1, 0x85, 0x31, 0x4C, 0xD5, 0x04, 0xE9, 0x77, 0x45, 0x56, 0xF9, 0xD1, 0x93, 0x18, 
	0x94, 0x96, 0x17, 0x63, 0x45, 0x48, 0x49, 0x9C, 0xF1, 0x94, 0x89, 0x24, 0x9E, 0x08, 0x81, 0x36, 
	0xC2, 0x16, 0x48, 0x22, 0x58, 0x1A, 0x88, 0x25, 0x02, 0x0D, 0xF0, 0x44, 0x5A, 0x96, 0xAD, 0x01, 
	0x32, 0x48, 0x00, 0x14, 0xE1, 0xD7, 0xBE, 0x3D, 0xCE, 0x30, 0x8F, 0x0D, 0xE8, 0x63, 0xB3, 0x70, 
	0x78, 0xC3, 0x50, 0x8E, 0x52, 0x5F, 0x1B, 0x9C, 0xC4, 0x10, 0x85, 0x68, 0xC6, 0x02, 0x04, 0x71, 
	0x41, 0xA1, 0x03, 0x1C, 0x0F, 0x80, 0xC6, 0xC6, 0x58, 0x82, 0xEA, 0x8D, 0x64, 0x53, 0x61, 0xD9, 
	0x54, 0xC8, 0x44, 0x5D, 0xE6, 0x52, 0x12, 0x0F, 0xCC, 0xE3, 0x1E, 0xAB, 0xD8, 0xC9, 0x3F, 0x5E, 
	0xF0, 0x93, 0x82, 0xEC, 0x22, 0x29, 0xF5, 0xD7, 0x00, 0x00, 0xA0, 0x00, 0x00, 0x00, 0x60, 0x52, 
	0x19, 0x83, 0x78, 0x48, 0xE2, 0xB1, 0x72, 0x9A, 0x6B, 0xEC, 0x8A, 0x96, 0xDE, 0x28, 0xCB, 0x48, 
	0x3E, 0x31, 0x81, 0xB7, 0x4C, 0x40, 0x2E, 0x75, 0xA9, 0xCB, 0xDC, 0x64, 0x92, 0x86, 0xE2, 0xA4, 
	0x97, 0x30, 0xFF, 0x58, 0xCC, 0x1D, 0xAA, 0x13, 0x77, 0xD6, 0x32, 0x25, 0x3C, 0x27, 0xFA, 0x37, 
	0x05, 0x1C, 0xF2, 0x99, 0x67, 0x0C, 0xA0, 0x34, 0x19, 0x09, 0x8E, 0x58, 0xBE, 0xD1, 0x84, 0x21, 
	0x08, 0x41, 0x36, 0xB7, 0x59, 0x49, 0x6F, 0x0A, 0xD4, 0xA4, 0x96, 0xF8, 0xE5, 0x26, 0xCB, 0x89, 
	0xC3, 0x73, 0x06, 0xB2, 0x5E, 0xFF, 0xA3, 0xB4, 0xC4, 0x44, 0x67, 0x4A, 0x51, 0x67, 0xD6, 0xB3, 
	0x9E, 0x7F, 0xC3, 0xE7, 0x34, 0x7F, 0xD1, 0x00, 0xF2, 0x05, 0xCB, 0x5C, 0xE7, 0x8B, 0x24, 0x25, 
	0x6F, 0x99, 0xCB, 0x81, 0x9E, 0x94, 0xA0, 0xB8, 0x94, 0x81, 0x30, 0xE6, 0xC5, 0x50, 0x3F, 0x0A, 
	0xB3, 0x98, 0xB7, 0x13, 0xC6, 0x04, 0x26, 0xD0, 0x4E, 0x9A, 0xD2, 0xB4, 0x01, 0xF3, 0x74, 0xA6, 
	0x56, 0x17, 0xA4, 0xD1, 0x69, 0x52, 0x53, 0x21, 0x3D, 0x4D, 0x22, 0xF2, 0x54, 0x17, 0x52, 0x40, 
	0x29, 0x10, 0x97, 0x04, 0x2D, 0xEA, 0x49, 0x95, 0x2A, 0x83, 0xB6, 0xBA, 0xD5, 0xAD, 0xF6, 0xB3, 
	0xDF, 0x54, 0xE7, 0x3A, 0xD5, 0x0D, 0x50, 0xD5, 0xAA, 0x57, 0x6D, 0xE6, 0x21, 0x4D, 0xD9, 0x3F, 
	0x7C, 0x36, 0x60, 0x80, 0xA6, 0x23, 0x9F, 0x1B, 0xB7, 0x14, 0x52, 0xB2, 0x46, 0x32, 0x7D, 0x47, 
	0x35, 0xEA, 0x37, 0xDF, 0xCA, 0x58, 0xB8, 0x6E, 0xE0, 0xB1, 0x90, 0x8D, 0xEC, 0x63, 0x1B, 0x30, 
	0x57, 0xBB, 0x5A, 0x76, 0x02, 0x76, 0xBD, 0x2B, 0x5E, 0x29, 0x7A, 0x53, 0xAC, 0xA2, 0x20, 0x03, 
	0x43, 0x9C, 0xA6, 0x0A, 0xFE, 0x0A, 0x58, 0x57, 0x7E, 0x40, 0xB0, 0x64, 0x23, 0xAB, 0x6A, 0x25, 
	0x61, 0xD2, 0x81, 0xAA, 0x75, 0x81, 0x8D, 0x6D, 0x6C, 0x30, 0x25, 0x4B, 0xDB, 0x60, 0xD2, 0x15, 
	0xB3, 0x95, 0xD5, 0xEC, 0x66, 0xE3, 0xF9, 0x4C, 0xBE, 0xA2, 0xE0, 0xB3, 0x39, 0x25, 0x9D, 0x0A, 
	0x4A, 0xDB, 0xD3, 0x2D, 0x11, 0xF6, 0x7C, 0xAA, 0x3D, 0x1F, 0x13, 0x15, 0x4B, 0xD0, 0x93, 0x26, 
	0x35, 0xB6, 0x8E, 0xA5, 0xAD, 0x64, 0x29, 0x7B, 0xD9, 0xCC, 0x66, 0x56, 0xB7, 0xBB, 0xC5, 0xAA, 
	0x67, 0x3F, 0x6B, 0xC6, 0x32, 0xFE, 0x0D, 0x68, 0x27, 0x18, 0xAD, 0x24, 0x4E, 0x8B, 0x5A, 0xE4, 
	0x06, 0x95, 0xAC, 0x93, 0x78, 0xFF, 0xAD, 0x5A, 0x9F, 0x2B, 0x83, 0x18, 0xB4, 0xD5, 0xBD, 0xED, 
	0x6D, 0xEB, 0x6C, 0x37, 0xC0, 0x81, 0xC7, 0xD6, 0x97, 0xBE, 0x93, 0xBD, 0x6E, 0x6E, 0x2F, 0x5B, 
	0xD5, 0xEC, 0x62, 0xD5, 0x8C, 0xA0, 0xFD, 0x2C, 0x70, 0x85, 0x7B, 0x82, 0x01, 0x8E, 0x97, 0xBC, 
	0xA9, 0x55, 0x2D, 0x7A, 0x91, 0xDA, 0x4D, 0xA3, 0xC2, 0x16, 0xBA, 0x6F, 0x9D, 0xAF, 0x74, 0x27, 
	0x8B, 0xDB, 0xCB, 0xDE, 0x76, 0x03, 0xFD, 0xB5, 0xAA, 0x32, 0x41, 0x1B, 0xE0, 0x00, 0x2F, 0xF3, 
	0xAF, 0xC3, 0x35, 0xF0, 0x07, 0x34, 0x70, 0xDA, 0x13, 0x9A, 0x77, 0xB5, 0x09, 0x18, 0x01, 0x52, 
	0x5B, 0xFB, 0x60, 0xF8, 0xC6, 0xF7, 0xC5, 0xB3, 0xBD, 0xAF, 0x8C, 0xED, 0x4B, 0x61, 0xBA, 0x56, 
	0x77, 0xAA, 0x19, 0xA6, 0x68, 0x7F, 0xB7, 0xDB, 0xDD, 0x02, 0x70, 0x98, 0xB4, 0x05, 0x16, 0x2F, 
	0x79, 0xC9, 0x77, 0x42, 0x05, 0x2B, 0x97, 0xA0, 0x2A, 0xB6, 0x24, 0x52, 0x21, 0x1C, 0xE1, 0x09, 
	0xD7, 0xB6, 0xC2, 0xB7, 0xAD, 0xAB, 0x6E, 0x27, 0x01, 0x80, 0x09, 0x00, 0x80, 0x06, 0x54, 0x05, 
	0xF0, 0x6F, 0x03, 0x9C, 0xC8, 0x20, 0x17, 0x78, 0x12, 0x25, 0xAE, 0x81, 0x09, 0xCF, 0x7B, 0xE4, 
	0x24, 0x37, 0x57, 0xC9, 0xEC, 0x7D, 0x31, 0x63, 0x25, 0x3C, 0xE3, 0xE9, 0x5E, 0xF7, 0xC6, 0x99, 
	0x6D, 0xA7, 0x24, 0xAC, 0x5C, 0xE5, 0x3A, 0xC7, 0xF3, 0xB7, 0x5A, 0x16, 0xB0, 0x24, 0x0A, 0xFC, 
	0x65, 0x30, 0x93, 0x4F, 0xCC, 0x20, 0x45, 0xB1, 0x2E, 0x47, 0x60, 0x00, 0x15, 0x9B, 0xB9, 0x13, 
	0x8C, 0x85, 0xAF, 0x7B, 0x63, 0x4C, 0xDB, 0xFB, 0xD6, 0x58, 0xBF, 0xB7, 0xA5, 0x32, 0x3C, 0xA7, 
	0x7A, 0xE5, 0x3A, 0x53, 0x95, 0xCB, 0xBF, 0x1D, 0x62, 0x06, 0x32, 0x00, 0x62, 0x03, 0x9B, 0x76, 
	0x4B, 0x80, 0x0E, 0xF4, 0x91, 0xFF, 0xCF, 0x8C, 0xE4, 0x82, 0xBA, 0xE0, 0xD4, 0xE3, 0x88, 0xAE, 
	0x93, 0xEF, 0x6B, 0x5B, 0x28, 0x77, 0xE2, 0x06, 0xED, 0x9C, 0x2A, 0x0D, 0x26, 0x5D, 0xE7, 0x59, 
	0x63, 0xB5, 0xC3, 0x9A, 0xDE, 0x34, 0x69, 0x1B, 0x30, 0x00, 0x4D, 0x8C, 0xB7, 0x84, 0x38, 0x00, 
	0x74, 0x6A, 0xBB, 0x29, 0x81, 0x41, 0xA7, 0xB8, 0xD4, 0x0D, 0x38, 0xB5, 0xB2, 0x97, 0xED, 0x82, 
	0xB8, 0x4A, 0x97, 0xBA, 0xAF, 0xBE, 0x81, 0xB4, 0xA5, 0x4D, 0x59, 0x5A, 0xCF, 0x7A, 0xD6, 0x74, 
	0xFE, 0x2F, 0x68, 0x25, 0xB1, 0x69, 0x5D, 0x0F, 0x97, 0xA7, 0x61, 0x0D, 0xB5, 0x98, 0x4F, 0xC8, 
	0x44, 0x33, 0xEB, 0x52, 0x81, 0x48, 0x3D, 0x75, 0x0C, 0x98, 0xCD, 0xEE, 0x06, 0x78, 0x20, 0x06, 
	0x31, 0x78, 0xC1, 0xBB, 0x63, 0x60, 0xBF, 0x69, 0xDB, 0xFB, 0xDE, 0x0D, 0xC0, 0xB2, 0xA5, 0xE1, 
	0x89, 0xED, 0x3B, 0xE7, 0xBA, 0xDB, 0x9C, 0xEE, 0xB3, 0xE9, 0x24, 0x21, 0x6E, 0x61, 0x77, 0xF3, 
	0xD8, 0xC7, 0x36, 0xF4, 0x02, 0x95, 0x2D, 0x03, 0x75, 0x37, 0xDC, 0x05, 0xEB, 0x46, 0x75, 0x0C, 
	0x3C, 0xF0, 0x82, 0x78, 0x4F, 0xBC, 0xE2, 0x17, 0x87, 0x77, 0x03, 0xEE, 0xCD, 0xF1, 0x69, 0x57, 
	0xB5, 0xDF, 0x56, 0x0E, 0x39, 0x0D, 0xB4, 0x0B, 0xF0, 0x6E, 0x37, 0x60, 0xB4, 0xBD, 0x0E, 0x6C, 
	0x09, 0x0B, 0x3E, 0x6E, 0xD6, 0x92, 0x9A, 0x9B, 0xEC, 0x5E, 0xF6, 0xC3, 0x9B, 0x6D, 0xF1, 0x77, 
	0x53, 0x3C, 0xE3, 0x17, 0xDF, 0x78, 0xC7, 0x39, 0x5E, 0xED, 0x5A, 0xFB, 0x3C, 0x9E, 0x0D, 0x28, 
	0x39, 0xC0, 0x25, 0xC1, 0x02, 0xB0, 0x12, 0x5C, 0xCC, 0xC1, 0x0E, 0x75, 0x03, 0x24, 0x20, 0x01, 
	0x49, 0x14, 0xDB, 0xD8, 0x2A, 0x4E, 0x36, 0xC4, 0xD5, 0xCD, 0xEE, 0x75, 0x37, 0x40, 0xDE, 0x16, 
	0xC7, 0x7A, 0xC5, 0xDF, 0x5D, 0xFF, 0x71, 0x9D, 0xEF, 0xDC, 0xDE, 0xD5, 0xEE, 0x77, 0xBF, 0x27, 
	0x21, 0xF4, 0xA1, 0x67, 0x20, 0xE5, 0x03, 0x5F, 0x39, 0xA0, 0x93, 0xBE, 0xF4, 0x10, 0x30, 0xDD, 
	0xE9, 0x67, 0x96, 0xFA, 0xCC, 0x5D, 0xD0, 0x5E, 0x66, 0x37, 0x20, 0xDE, 0x36, 0x87, 0xB7, 0xD6, 
	0x6D, 0xEE, 0xF5, 0xAF, 0x53, 0x7B, 0xDF, 0x75, 0x96, 0x04, 0x0D, 0x46, 0x5E, 0x76, 0x93, 0x17, 
	0x9D, 0x84, 0x7F, 0x2E, 0xF8, 0xD2, 0x25, 0xE0, 0x76, 0xB7, 0xB3, 0x36, 0xEA, 0x31, 0x5F, 0xB6, 
	0x7B, 0x69, 0x7E, 0xF3, 0xCA, 0xD7, 0xBC, 0xE2, 0x2F, 0xE8, 0xBB, 0xDF, 0x61, 0xED, 0x73, 0xC1, 
	0x0F, 0x7E, 0xF0, 0x41, 0x2F, 0xBB, 0x24, 0xD0, 0x3E, 0xF0, 0x30, 0x27, 0xDD, 0x84, 0x8C, 0x67, 
	0x3A, 0xD3, 0xDD, 0xDE, 0x74, 0x26, 0x4A, 0x3D, 0xE2, 0x93, 0x3F, 0xF5, 0xCC, 0xEF, 0xBE, 0xF5, 
	0xDA, 0xE3, 0xFC, 0xDD, 0x9A, 0xF7, 0x7B, 0x3B, 0x6D, 0x9D, 0xEF, 0xCF, 0x7F, 0x3E, 0xF4, 0x25, 
	0xE7, 0xB5, 0xF0, 0x49, 0xD8, 0x00, 0x08, 0x24, 0x7E, 0xDC, 0xAB, 0x4F, 0x7D, 0xF2, 0x5B, 0x2F, 
	0x89, 0xC8, 0x2F, 0xDB, 0xDD, 0x7A, 0xD7, 0x3B, 0xD7, 0xF1, 0xDE, 0xF5, 0xCD, 0x83, 0x9D, 0xAA, 
	0xBD, 0xF7, 0xFD, 0xEF, 0x85, 0xCE, 0xEB, 0x5E, 0x23, 0x11, 0xC1, 0xC2, 0x56, 0x7E, 0xEA, 0x59, 
	0xEF, 0xF8, 0x11, 0x8C, 0xA0, 0x13, 0x53, 0x7F, 0x3E, 0xDE, 0xD7, 0xBF, 0x77, 0xBC, 0xE7, 0xBE, 
	0xE3, 0x96, 0xD0, 0xBE, 0xFC, 0x81, 0xAF, 0xEB, 0x01, 0xD8, 0x7F, 0x68, 0xAE, 0xEC, 0xE9, 0x07, 
	0x8C, 0xBF, 0xF2, 0xC5, 0x37, 0x5E, 0xF5, 0xE4, 0xD7, 0x00, 0xE6, 0x37, 0x80, 0xE6, 0x87, 0x7E, 
	0xF4, 0x96, 0x71, 0x37, 0x77, 0x79, 0x7A, 0xF7, 0x7E, 0xB0, 0x66, 0x09, 0x37, 0xE0, 0x01, 0xD9, 
	0x27, 0x7F, 0xDB, 0x67, 0x72, 0xFF, 0xF6, 0xE7, 0x7D, 0xDF, 0x37, 0x64, 0x25, 0xD4, 0x76, 0x00, 
	0x38, 0x7E, 0x6F, 0x67, 0x7E, 0x2E, 0xE0, 0x81, 0x04, 0x58, 0x80, 0xAE, 0x72, 0x7B, 0xD5, 0x47, 
	0x6D, 0x0E, 0xE8, 0x01, 0x07, 0x70, 0x03, 0x07, 0xB0, 0x82, 0x11, 0x28, 0x81, 0xA0, 0x57, 0x7F, 
	0x15, 0x88, 0x7F, 0x81, 0x65, 0x7C, 0xFB, 0x47, 0x64, 0xE4, 0xB7, 0x7C, 0xA9, 0x27, 0x80, 0x21, 
	0xB8, 0x83, 0x05, 0xF8, 0x02, 0x3E, 0xF8, 0x83, 0x99, 0x17, 0x57, 0x07, 0xE0, 0x01, 0x2A, 0x88, 
	0x82, 0x46, 0x38, 0x84, 0x10, 0xE8, 0x82, 0xDA, 0x17, 0x74, 0xDD, 0x17, 0x83, 0xA5, 0x93, 0x7F, 
	0x43, 0x56, 0x62, 0x92, 0x70, 0x83, 0xFF, 0xA7, 0x83, 0x20, 0x38, 0x02, 0x1F, 0x88, 0x85, 0x5A, 
	0x78, 0x7E, 0x2E, 0xF0, 0x02, 0x5D, 0xF8, 0x85, 0x40, 0x08, 0x84, 0x0D, 0x30, 0x84, 0x29, 0x88, 
	0x82, 0x64, 0x48, 0x86, 0x66, 0xD8, 0x82, 0x2E, 0x38, 0x7A, 0x15, 0x78, 0x7F, 0x4F, 0x08, 0x85, 
	0x34, 0x08, 0x01, 0x0D, 0x80, 0x03, 0x38, 0x30, 0x85, 0x1B, 0xD8, 0x74, 0x57, 0xC8, 0x83, 0x03, 
	0x78, 0x75, 0x3E, 0xE8, 0x02, 0x30, 0xD0, 0x87, 0x5E, 0x18, 0x88, 0x5D, 0xB8, 0x71, 0x67, 0xE8, 
	0x01, 0x47, 0x68, 0x86, 0x28, 0xA8, 0x86, 0x4B, 0xC8, 0x86, 0x6D, 0x48, 0x7A, 0xD1, 0x40, 0x3E, 
	0x18, 0x48, 0x87, 0x74, 0x68, 0x87, 0x39, 0xB8, 0x83, 0x59, 0x98, 0x85, 0x7B, 0x18, 0x88, 0x82, 
	0xF8, 0x87, 0x5E, 0xC8, 0x89, 0xCD, 0x86, 0x88, 0x2B, 0x88, 0x86, 0xA2, 0xA8, 0x88, 0x23, 0x27, 
	0x09, 0x2C, 0xD8, 0x88, 0xDE, 0xA7, 0x04, 0xBF, 0x46, 0x83, 0x73, 0x28, 0x89, 0x93, 0xB8, 0x78, 
	0x78, 0x78, 0x89, 0x7A, 0xF8, 0x81, 0x57, 0x07, 0x86, 0x3F, 0xD8, 0x85, 0x80, 0x98, 0x79, 0x48, 
	0x28, 0x8A, 0x87, 0x38, 0x84, 0xFF, 0x6A, 0x68, 0x8A, 0x2B, 0xC8, 0x02, 0x07, 0xD0, 0x84, 0x6E, 
	0xF8, 0x86, 0xA5, 0xA7, 0x7F, 0xAD, 0xE8, 0x8A, 0xAF, 0x28, 0x52, 0x3C, 0x88, 0x89, 0x98, 0x78, 
	0x75, 0x30, 0x80, 0x8B, 0x5F, 0x18, 0x8D, 0x9A, 0x38, 0x88, 0x2B, 0xD8, 0x8B, 0x47, 0x98, 0x82, 
	0x11, 0x08, 0x8C, 0xC2, 0x28, 0x8C, 0xC3, 0xD8, 0x86, 0x0D, 0x70, 0x78, 0x4D, 0x10, 0x56, 0x90, 
	0x98, 0x8C, 0xCA, 0xF8, 0x8A, 0x56, 0xA8, 0x87, 0x99, 0x28, 0x8D, 0x9C, 0xD8, 0x87, 0x7F, 0xE8, 
	0x87, 0x99, 0x07, 0x8A, 0xD8, 0xE8, 0x8B, 0xA5, 0x38, 0x86, 0x07, 0xE0, 0x8D, 0xA1, 0x48, 0x8C, 
	0xE1, 0x28, 0x8E, 0xE3, 0x18, 0x56, 0x92, 0x70, 0x8E, 0xCA, 0xB8, 0x74, 0x96, 0xB0, 0x85, 0xCE, 
	0x78, 0x7E, 0xB9, 0x08, 0x86, 0xB6, 0x38, 0x88, 0x0F, 0x78, 0x8D, 0xBC, 0xE8, 0x8B, 0xDC, 0x18, 
	0x8A, 0xC2, 0x38, 0x00, 0xC2, 0xD8, 0x84, 0xFB, 0x28, 0x83, 0x44, 0x53, 0x02, 0xC8, 0x08, 0x90, 
	0xAE, 0x08, 0x8B, 0x6F, 0x37, 0x09, 0x3B, 0x98, 0x6C, 0xB7, 0xE8, 0x83, 0xD4, 0x18, 0x92, 0xBA, 
	0xC8, 0x90, 0x28, 0xD8, 0x80, 0x93, 0x00, 0x03, 0x63, 0xE8, 0x8D, 0x2C, 0x30, 0x00, 0xC1, 0x98, 
	0x8F, 0xF7, 0xC7, 0x02, 0x32, 0x69, 0x8C, 0x45, 0xE0, 0x8F, 0x93, 0xA0, 0x91, 0x75, 0xE8, 0x00, 
	0x4C, 0xA7, 0x93, 0xAA, 0xC7, 0x7C, 0x3A, 0xD8, 0x00, 0xD4, 0xE8, 0x87, 0x09, 0xD9, 0x89, 0xD6, 
	0x68, 0x88, 0x9D, 0x00, 0x03, 0x48, 0x99, 0x94, 0xF6, 0x18, 0x8A, 0xF7, 0x08, 0x91, 0xDF, 0xB8, 
	0x8F, 0x33, 0x99, 0x25, 0xE5, 0x88, 0x91, 0xE6, 0x28, 0x89, 0x1C, 0x29, 0x01, 0x3C, 0xC9, 0x93, 
	0x1D, 0x19, 0x57, 0xF6, 0x93, 0x94, 0x5E, 0xE9, 0x95, 0x63, 0x28, 0x91, 0x4D, 0xC9, 0x94, 0x2C, 
	0x08, 0x95, 0xFC, 0x48, 0x2E, 0xFF, 0x0D, 0x40, 0x95, 0x1F, 0x50, 0x02, 0xE4, 0x03, 0x90, 0x4B, 
	0xA7, 0x95, 0x3D, 0x09, 0x97, 0x0D, 0x20, 0x00, 0x74, 0x59, 0x97, 0x76, 0x79, 0x97, 0x02, 0x00, 
	0x94, 0x5F, 0xB9, 0x97, 0x48, 0x19, 0x8E, 0x64, 0xD9, 0x8D, 0x96, 0x20, 0x93, 0x51, 0x79, 0x23, 
	0x61, 0x55, 0x02, 0x10, 0x50, 0x02, 0x54, 0x19, 0x90, 0xAA, 0x97, 0x95, 0x3D, 0x89, 0x95, 0x4D, 
	0x87, 0x97, 0x90, 0x59, 0x97, 0x7A, 0xC9, 0x97, 0x5F, 0xB9, 0x92, 0x2F, 0x39, 0x8C, 0x15, 0x29, 
	0x98, 0x67, 0x39, 0x05, 0x93, 0xC0, 0x96, 0x87, 0xB9, 0x96, 0xFF, 0x38, 0x89, 0x8E, 0xB9, 0x93, 
	0xA4, 0xA9, 0x95, 0x73, 0x19, 0x99, 0x90, 0x39, 0x99, 0x94, 0xA9, 0x94, 0x4C, 0x39, 0x09, 0x14, 
	0xC0, 0x02, 0x14, 0x40, 0x01, 0x50, 0x49, 0x93, 0xFD, 0x28, 0x09, 0x6C, 0x69, 0x98, 0x6B, 0x49, 
	0x64, 0x75, 0xD8, 0x98, 0xBC, 0xA9, 0x7A, 0xA7, 0x89, 0x9A, 0x76, 0xA9, 0x9A, 0xAB, 0xD9, 0x97, 
	0x98, 0x19, 0x8E, 0xAF, 0x09, 0x9B, 0x33, 0x39, 0x98, 0x5A, 0xD0, 0x99, 0xB7, 0x69, 0x98, 0x88, 
	0x49, 0x64, 0x4E, 0xC7, 0x98, 0x8E, 0xC9, 0x93, 0xBF, 0x09, 0x9C, 0x74, 0x29, 0x9C, 0x94, 0x19, 
	0x98, 0xC8, 0xB9, 0x9D, 0x2A, 0x39, 0x93, 0xB4, 0xF9, 0x04, 0xE3, 0x75, 0x98, 0xB7, 0xB9, 0x7F, 
	0x88, 0xD9, 0x00, 0x33, 0x30, 0x03, 0x9D, 0x00, 0x97, 0x3A, 0x59, 0x9D, 0xD6, 0x89, 0x9D, 0x7D, 
	0x69, 0x09, 0xB1, 0x69, 0x9C, 0xC8, 0xF9, 0x9A, 0xB1, 0x59, 0x74, 0x45, 0xF7, 0x9D, 0x68, 0xD9, 
	0x53, 0x87, 0xF9, 0x99, 0x6C, 0x69, 0x9E, 0xE7, 0xF9, 0x9F, 0xE8, 0x69, 0x09, 0x6F, 0x67, 0x9D, 
	0xC1, 0xA9, 0x94, 0xF0, 0x19, 0x9B, 0x08, 0x2A, 0x9B, 0xF4, 0x39, 0x9F, 0x82, 0xE9, 0x0D, 0xB4, 
	0x01, 0x66, 0x9E, 0x59, 0x9E, 0xFF, 0x0E, 0x80, 0x03, 0x00, 0x5A, 0xA1, 0x01, 0xCA, 0x95, 0x86, 
	0x91, 0xA0, 0xB1, 0xE9, 0x00, 0x14, 0xC0, 0xA1, 0x1C, 0x2A, 0x9F, 0x32, 0xF9, 0x9A, 0x93, 0x60, 
	0x02, 0xF8, 0xC9, 0x99, 0x96, 0xF0, 0x9C, 0xFD, 0xF9, 0x9F, 0x14, 0x3A, 0x03, 0x2B, 0x0A, 0xA0, 
	0x0D, 0xE0, 0x00, 0x30, 0x1A, 0xA3, 0x32, 0x3A, 0xA3, 0x0D, 0x90, 0xA0, 0x1E, 0xAA, 0xA1, 0xF1, 
	0x79, 0x9C, 0x30, 0x20, 0xA2, 0x0D, 0x60, 0x02, 0x24, 0x5A, 0xA2, 0x84, 0x39, 0x95, 0x87, 0x69, 
	0x9E, 0x2D, 0x0A, 0xA0, 0x2D, 0xFA, 0xA2, 0x33, 0x9A, 0xA4, 0x31, 0x5A, 0xA3, 0x1D, 0x6A, 0xA3, 
	0x1B, 0x8A, 0xA0, 0xFB, 0xC8, 0xA3, 0x3E, 0xFA, 0xA3, 0x70, 0x60, 0x9B, 0x96, 0x40, 0x3E, 0x00, 
	0x0A, 0x01, 0x15, 0xBA, 0xA2, 0x48, 0xAA, 0xA4, 0x49, 0xFA, 0xA2, 0x38, 0xAA, 0xA1, 0x1F, 0x2A, 
	0x9B, 0x92, 0x30, 0xA5, 0x54, 0x8A, 0x14, 0x56, 0x0A, 0x9D, 0x45, 0xCA, 0xA2, 0x46, 0xDA, 0xA5, 
	0x5E, 0x2A, 0xA3, 0x4C, 0xDA, 0xA4, 0x1C, 0xFA, 0xA4, 0xF1, 0x59, 0xA6, 0x66, 0xEA, 0xA3, 0x40, 
	0x2A, 0x1F, 0x9D, 0x69, 0x18, 0x16, 0x8A, 0x9E, 0x6F, 0xFA, 0xA5, 0x4D, 0x6A, 0xA3, 0xC2, 0x70, 
	0xA7, 0x78, 0x9A, 0xA7, 0xD7, 0x71, 0xA2, 0x69, 0x89, 0x98, 0xE5, 0x69, 0x09, 0x14, 0xEA, 0xA6, 
	0x7F, 0xEA, 0x00, 0x71, 0xDA, 0x09, 0x29, 0x30, 0xA9, 0x93, 0xDA, 0xA3, 0x66, 0xEA, 0xA0, 0x31, 
	0x21, 0x0C, 0x8A, 0xBA, 0xA9, 0x8B, 0x8A, 0xA1, 0xE3, 0x40, 0xA9, 0xA0, 0x4A, 0xA9, 0x96, 0x6A, 
	0x09, 0x8A, 0xA0, 0xA9, 0x9C, 0xBA, 0xA9, 0x0D, 0x80, 0x01, 0xAA, 0xBA, 0xAA, 0xAC, 0xDA, 0xAA, 
	0x0D, 0x10, 0xAA, 0xB0, 0x9A, 0x02, 0x3D, 0x4A, 0xAA, 0x9B, 0x30, 0x0E, 0xA7, 0x8A, 0x91, 0xAD, 
	0x9A, 0xAB, 0xAC, 0xFA, 0xAA, 0x5C, 0xB1, 0x2A, 0xAA, 0x56, 0xFA, 0x0D, 0x69, 0x47, 0x95, 0x89, 
	0xAA, 0xA8, 0xA9, 0xAA, 0xAB, 0xAE, 0x1A, 0xAB, 0x9A, 0x6A, 0x91, 0xAE, 0xB0, 0xA7, 0xCC, 0x59, 
	0xAC, 0xC6, 0xBA, 0xAA, 0xBC, 0xDA, 0x09, 0xA8, 0xAA, 0xAC, 0xC0, 0x8A, 0xA8, 0x9E, 0x7A, 0xA2, 
	0x8A, 0x8A, 0x01, 0xC2, 0x6A, 0xA8, 0xCB, 0x2A, 0xAD, 0xB7, 0x4A, 0xAC, 0x88, 0x89, 0x01, 0xC2, 
	0x00, 0xAC, 0xCB, 0x79, 0xAD, 0xDC, 0x4A, 0xAE, 0x52, 0xF0, 0x08, 0xE8, 0x2A, 0x06, 0xEA, 0xBA, 
	0xAE, 0x72, 0xE0, 0xAE, 0x79, 0xD0, 0xAE, 0xF0, 0xAA, 0x05, 0xF2, 0x3A, 0xAF, 0x76, 0xB0, 0x09, 
	0x41, 0x00, 0x00, 0x21, 0xF9, 0x04, 0x04, 0x0A, 0x00, 0x00, 0x00, 0x2C, 0x00, 0x00, 0x10, 0x00, 
	0x64, 0x00, 0x63, 0x00, 0x00, 0x06, 0xFF, 0xC0, 0x96, 0x70, 0x48, 0x2C, 0x1A, 0x8F, 0x48, 0x64, 
	0x63, 0xB9, 0xB4, 0x31, 0x9F, 0x8D, 0xA4, 0x74, 0x4A, 0xAD, 0x5A, 0xA5, 0xD0, 0xA7, 0x85, 0x64, 
	0xB1, 0x59, 0xBE, 0x0D, 0xCB, 0xE5, 0x22, 0xBE, 0x84, 0xA3, 0xD7, 0xB4, 0x7A, 0x3D, 0xCC, 0x36, 
	0x48, 0x70, 0x12, 0xD4, 0xC9, 0x24, 0x9B, 0x97, 0xF7, 0x33, 0x7B, 0xCF, 0x27, 0x3E, 0x03, 0x6F, 
	0x71, 0x50, 0x5F, 0x5F, 0x5E, 0x84, 0x75, 0x63, 0x89, 0x5A, 0x7D, 0x8C, 0x69, 0x0D, 0x01, 0x90, 
	0x80, 0x71, 0x5B, 0x70, 0x61, 0x84, 0x85, 0x97, 0x87, 0x0D, 0x63, 0x78, 0x9C, 0x77, 0x16, 0x8D, 
	0xA1, 0x47, 0x4B, 0x90, 0x6E, 0x5A, 0x99, 0x5D, 0xA8, 0x87, 0x89, 0xAC, 0x9B, 0x79, 0xA2, 0xB0, 
	0x59, 0x71, 0x82, 0x96, 0xAA, 0x86, 0xAA, 0x9B, 0xA6, 0xAE, 0x66, 0xB0, 0x8C, 0x4C, 0x91, 0x81, 
	0x50, 0x72, 0xAA, 0x98, 0xC4, 0x16, 0x9D, 0xAC, 0x9C, 0x78, 0x0D, 0x3F, 0xBD, 0x6C, 0x8F, 0x91, 
	0x80, 0x4C, 0xB3, 0x72, 0xB5, 0xC6, 0xC4, 0x0D, 0x22, 0x4C, 0x20, 0x17, 0xDC, 0x20, 0x0D, 0x2B, 
	0xDF, 0x4B, 0x3F, 0x68, 0xCE, 0x54, 0xA4, 0xA5, 0xD3, 0xB3, 0xD3, 0xD6, 0xD7, 0xA8, 0xD9, 0x22, 
	0xF1, 0x4F, 0x9C, 0xE1, 0xE1, 0xE3, 0xE6, 0x53, 0xA4, 0xBF, 0x92, 0xD5, 0x4B, 0xEB, 0xEE, 0xD8, 
	0xE4, 0x35, 0x00, 0x11, 0x4F, 0x20, 0x38, 0x10, 0xF5, 0xC4, 0xE1, 0x1B, 0xF5, 0x27, 0x9A, 0xBA, 
	0x75, 0x0D, 0x74, 0x2C, 0x01, 0x98, 0x09, 0x9E, 0x40, 0x82, 0x22, 0x08, 0x0E, 0xDC, 0x54, 0x0F, 
	0x1C, 0xB3, 0x85, 0x6D, 0x3A, 0xA0, 0x03, 0xF6, 0x28, 0x90, 0x0E, 0x38, 0x12, 0x4F, 0xCA, 0x91, 
	0x48, 0xF1, 0x58, 0xB6, 0x1C, 0x05, 0x0D, 0x6A, 0x03, 0x41, 0x73, 0xDB, 0x0A, 0x8F, 0xE5, 0xCC, 
	0x35, 0x10, 0x29, 0xD2, 0x61, 0x00, 0xFF, 0x02, 0x72, 0xD6, 0x91, 0xD0, 0xC1, 0x60, 0xA5, 0x0E, 
	0x89, 0xBA, 0x9E, 0xE4, 0x78, 0xB9, 0x24, 0xE6, 0xC5, 0x1D, 0x34, 0x6B, 0x6E, 0x5C, 0x71, 0x33, 
	0xA7, 0x28, 0x26, 0x3D, 0xD3, 0x01, 0x85, 0x44, 0xE0, 0x0D, 0x83, 0x93, 0x45, 0x8D, 0x1E, 0x8D, 
	0x38, 0xF6, 0xC1, 0x83, 0x06, 0x66, 0xCD, 0x36, 0xC8, 0xC1, 0x16, 0xDE, 0x52, 0xA7, 0x0D, 0x76, 
	0x0C, 0x04, 0x01, 0x15, 0xAA, 0x38, 0xAA, 0x54, 0xAD, 0xFA, 0xDA, 0xD9, 0xA1, 0xA7, 0xBF, 0x00, 
	0x24, 0x00, 0x13, 0x00, 0xA4, 0xF2, 0x64, 0xCA, 0xA3, 0x87, 0x25, 0xA6, 0x45, 0xBB, 0x98, 0x6D, 
	0x5B, 0x98, 0xDA, 0x2C, 0x66, 0x93, 0x3B, 0xD0, 0x2E, 0xD4, 0x25, 0x78, 0x97, 0x5C, 0xE5, 0x1B, 
	0x41, 0xE4, 0xA3, 0xC0, 0x83, 0x07, 0x03, 0xEA, 0x4A, 0x74, 0x28, 0xD9, 0xB2, 0x88, 0x19, 0x9F, 
	0x35, 0x9B, 0x43, 0xAD, 0xE3, 0xA5, 0x6C, 0x45, 0xBC, 0x8D, 0xBB, 0xA3, 0xB6, 0x5C, 0x9A, 0x76, 
	0x1B, 0x54, 0x00, 0x77, 0x93, 0x5C, 0xA3, 0x25, 0x7D, 0xFB, 0x92, 0x1A, 0x1C, 0x18, 0x74, 0x03, 
	0x02, 0x0C, 0xBA, 0x92, 0xF8, 0x8A, 0x38, 0x71, 0x59, 0xB5, 0x69, 0x5D, 0xBF, 0x7E, 0x29, 0x70, 
	0x87, 0x08, 0xEB, 0xB5, 0x97, 0xD0, 0xD5, 0x5D, 0x61, 0x77, 0x5E, 0xDF, 0x7C, 0x80, 0x77, 0x16, 
	0x19, 0x21, 0xC2, 0xA3, 0xD0, 0x80, 0x8F, 0x03, 0x25, 0xA0, 0xBC, 0x39, 0x5A, 0xF7, 0x66, 0x75, 
	0x40, 0x6F, 0x2D, 0xDD, 0x31, 0xD3, 0x06, 0x0A, 0x68, 0xD7, 0xBE, 0x7E, 0x7B, 0x60, 0x85, 0x15, 
	0xDE, 0xAD, 0xC0, 0x03, 0x78, 0x6B, 0x34, 0xA0, 0x01, 0x5F, 0xE4, 0x95, 0x77, 0x5C, 0x71, 0x4B, 
	0xB0, 0x47, 0x02, 0x7B, 0x11, 0x31, 0xE1, 0xDC, 0x73, 0x8B, 0x45, 0x77, 0xD6, 0x6B, 0x6D, 0xE1, 
	0xA7, 0xC0, 0x86, 0x71, 0xF1, 0x87, 0xFF, 0xDD, 0x12, 0xDD, 0x85, 0xE8, 0x11, 0x0F, 0x7A, 0x55, 
	0x61, 0xA0, 0x0F, 0x7C, 0x3D, 0x52, 0x9E, 0x79, 0x5C, 0xA9, 0xB7, 0x5E, 0x57, 0x0C, 0xC4, 0x48, 
	0x94, 0x84, 0xCD, 0x59, 0xB8, 0xDA, 0x03, 0xF4, 0x5D, 0xB8, 0xD4, 0x12, 0x4B, 0x6D, 0xA8, 0x40, 
	0x0E, 0xF9, 0xED, 0x67, 0x5B, 0x5C, 0xBA, 0x81, 0x20, 0x22, 0x33, 0x24, 0xAA, 0x71, 0x62, 0x8A, 
	0x2B, 0x76, 0xA0, 0xA0, 0x1C, 0x0E, 0xB2, 0x07, 0xE3, 0x72, 0x31, 0x32, 0xD0, 0xC0, 0x02, 0x0B, 
	0x48, 0x18, 0x9D, 0x0E, 0x39, 0x28, 0xB6, 0x98, 0x52, 0xD3, 0xF9, 0xC8, 0xA1, 0x6D, 0xD7, 0xD1, 
	0xC6, 0x9D, 0x91, 0xBB, 0x31, 0x53, 0x62, 0x12, 0x27, 0x1E, 0x98, 0xE0, 0x8A, 0x0A, 0x2E, 0x28, 
	0x25, 0x8C, 0x31, 0x2E, 0xB7, 0x5C, 0x03, 0x55, 0xC6, 0x98, 0x14, 0x13, 0x08, 0x60, 0xB9, 0x16, 
	0x86, 0x6B, 0xFD, 0x28, 0xE8, 0x64, 0x72, 0xD9, 0x26, 0x57, 0x77, 0x46, 0x36, 0xD0, 0x83, 0x9A, 
	0x57, 0x34, 0xE0, 0x83, 0x06, 0x07, 0x36, 0xD0, 0xE4, 0x8A, 0x92, 0xBA, 0x08, 0x14, 0x9E, 0x45, 
	0xE5, 0x89, 0x69, 0x9E, 0x59, 0x62, 0xE9, 0x69, 0xA7, 0x0B, 0xF4, 0xB9, 0xC0, 0x03, 0x08, 0xFC, 
	0x09, 0xA8, 0x98, 0x63, 0xEA, 0x57, 0x40, 0x76, 0xDD, 0xDD, 0xD4, 0xC3, 0xA2, 0x6B, 0x1A, 0xB1, 
	0x84, 0x06, 0x28, 0x4A, 0x0A, 0xA7, 0x82, 0x94, 0xAE, 0x87, 0x29, 0x95, 0x04, 0xCC, 0x98, 0x67, 
	0x95, 0x57, 0x7E, 0xDA, 0x69, 0x9F, 0xA4, 0xFA, 0x89, 0xA1, 0x7D, 0x1B, 0x02, 0x09, 0xE4, 0x12, 
	0x86, 0x0E, 0xD9, 0x9D, 0x6E, 0x3D, 0xA4, 0x69, 0x22, 0xAD, 0x4C, 0x76, 0x86, 0x2B, 0x01, 0x11, 
	0x24, 0x60, 0xDE, 0x94, 0x55, 0x52, 0xB9, 0xA9, 0xA6, 0xC2, 0xFA, 0x39, 0xAA, 0xA8, 0x08, 0x90, 
	0x6A, 0x6A, 0x98, 0xA8, 0xE2, 0x47, 0xFF, 0xDB, 0xAA, 0xAB, 0x1E, 0xAA, 0x68, 0x05, 0xAF, 0xC6, 
	0xEA, 0xC7, 0x81, 0x3E, 0x74, 0x80, 0x22, 0x9C, 0xB6, 0x26, 0x80, 0x6D, 0x9C, 0xDD, 0x6A, 0xFA, 
	0xAB, 0x9E, 0xE1, 0x66, 0x49, 0xEE, 0x03, 0x9E, 0x9E, 0x1B, 0xE6, 0xB2, 0x1A, 0x06, 0xD9, 0x2C, 
	0x65, 0xD1, 0x36, 0x2C, 0x6F, 0x0B, 0x58, 0x75, 0x10, 0xC2, 0x9B, 0x92, 0xEE, 0x9B, 0x2D, 0xB6, 
	0xC7, 0x2D, 0xD1, 0xEF, 0xB7, 0xE0, 0x06, 0x8C, 0x56, 0x9F, 0x20, 0x9B, 0x7B, 0x6C, 0x5B, 0xF9, 
	0xA1, 0xAA, 0x70, 0xBB, 0x72, 0x15, 0x00, 0x2D, 0xBC, 0x0F, 0x03, 0xE7, 0x83, 0x01, 0xF6, 0x92, 
	0x67, 0x6B, 0xB6, 0x11, 0xEC, 0xDB, 0x00, 0x07, 0x52, 0xE2, 0x89, 0x9C, 0x95, 0xFF, 0x02, 0xEC, 
	0x31, 0xB1, 0x9E, 0xF6, 0x69, 0xB0, 0x7D, 0x09, 0xA7, 0x6B, 0x9B, 0xCA, 0x05, 0x14, 0xB0, 0x9B, 
	0xC3, 0xB1, 0xBA, 0x1C, 0xB3, 0x01, 0xB5, 0xD6, 0x5C, 0x1E, 0xB6, 0xDA, 0x5A, 0x0A, 0x21, 0x9E, 
	0x1C, 0x77, 0x1C, 0xEE, 0x95, 0x21, 0x8B, 0x6A, 0x30, 0x9F, 0x45, 0x1B, 0x9D, 0x5D, 0xD2, 0x49, 
	0x43, 0xAB, 0x99, 0x12, 0x0D, 0x4C, 0xAC, 0xB6, 0x48, 0x06, 0xE4, 0x4B, 0xB3, 0x82, 0xEC, 0xE1, 
	0x8C, 0xF3, 0xD5, 0x59, 0x5B, 0x19, 0xB0, 0xB8, 0x04, 0x07, 0x2D, 0x32, 0x9F, 0x08, 0xF4, 0x1D, 
	0xB6, 0xD1, 0x0D, 0x24, 0xBD, 0x43, 0xD9, 0xD1, 0x36, 0x90, 0x24, 0x43, 0x8E, 0xF6, 0x55, 0x6F, 
	0x03, 0x06, 0x24, 0xD0, 0x80, 0xB6, 0xFB, 0x3A, 0x4E, 0xC0, 0xDC, 0x73, 0xD2, 0x09, 0x45, 0x8C, 
	0x77, 0xFB, 0xD9, 0x75, 0x16, 0x7D, 0x77, 0xDE, 0xF9, 0xDF, 0x62, 0x06, 0x3E, 0x38, 0xD9, 0x2A, 
	0x2F, 0xCA, 0xC3, 0xE1, 0x7E, 0x18, 0xB8, 0x93, 0xDA, 0x8C, 0x1B, 0xE0, 0xFA, 0xCC, 0x5D, 0xCD, 
	0x39, 0xB7, 0xDC, 0x37, 0x33, 0xC0, 0xFF, 0x81, 0xA6, 0x7B, 0x66, 0xF1, 0x69, 0x03, 0x9E, 0xF7, 
	0x5E, 0xAA, 0xC9, 0x3E, 0xEA, 0x46, 0x7A, 0x05, 0x4A, 0x2F, 0x71, 0xBA, 0x5E, 0xB3, 0xF2, 0xB5, 
	0xB8, 0x01, 0x11, 0x30, 0xDF, 0x76, 0xD5, 0x93, 0xCB, 0x9E, 0x73, 0x8C, 0xC8, 0xDD, 0x6E, 0xBD, 
	0xDD, 0x0C, 0x2C, 0x90, 0xFD, 0xF6, 0xA0, 0xDE, 0xCD, 0xBB, 0xEF, 0x7E, 0x03, 0x9F, 0x1F, 0x0A, 
	0xA2, 0xB3, 0x4B, 0xB6, 0xE1, 0xC7, 0x17, 0x31, 0xEB, 0x81, 0x13, 0x33, 0x9E, 0x80, 0xEB, 0x8D, 
	0xB7, 0x6D, 0xF5, 0xEC, 0x10, 0xDA, 0x5E, 0xE5, 0xED, 0x3B, 0xE3, 0xB9, 0x01, 0x03, 0xFB, 0xF7, 
	0x6F, 0x77, 0xE6, 0xDF, 0x03, 0xDF, 0xEF, 0xD2, 0x85, 0x02, 0xF2, 0x91, 0x4E, 0x70, 0x2A, 0x3B, 
	0x1D, 0xEA, 0x84, 0xD0, 0xA6, 0x9D, 0xB4, 0xCE, 0x75, 0xCD, 0x73, 0x9D, 0xE4, 0xE8, 0x37, 0x27, 
	0x3C, 0x5D, 0xAF, 0x7A, 0x7A, 0xE2, 0x9E, 0x06, 0x3B, 0x35, 0x01, 0x2C, 0x75, 0x70, 0x01, 0x1F, 
	0x0C, 0x20, 0xF8, 0xC2, 0xD6, 0x80, 0x02, 0x16, 0x30, 0x70, 0x08, 0x44, 0x1A, 0xFA, 0x72, 0xB2, 
	0x3E, 0x37, 0x3D, 0xCE, 0x79, 0x10, 0x7C, 0x5E, 0x83, 0xE8, 0x87, 0x33, 0x4C, 0x61, 0x30, 0x46, 
	0xB7, 0xBB, 0x52, 0xFF, 0x16, 0xB0, 0xC3, 0x0D, 0x04, 0xCB, 0x7B, 0x02, 0xFC, 0x5C, 0xF0, 0x4C, 
	0x78, 0xC2, 0xD1, 0x95, 0xAD, 0x6C, 0xE9, 0x6B, 0x43, 0x9B, 0x0E, 0x14, 0x43, 0xF8, 0xBD, 0x2E, 
	0x01, 0x50, 0xB4, 0x1A, 0x8C, 0x6E, 0x67, 0x3F, 0xFC, 0xD9, 0x0E, 0x4F, 0xDA, 0xE3, 0x61, 0x16, 
	0x37, 0xD0, 0x3D, 0x4F, 0x7D, 0x50, 0x73, 0x41, 0xFC, 0x5D, 0x09, 0x89, 0x78, 0x42, 0xD2, 0xA1, 
	0x30, 0x81, 0x2C, 0xA4, 0xD5, 0x07, 0x20, 0x25, 0x29, 0xF8, 0x35, 0xEF, 0x7D, 0x0D, 0x90, 0x01, 
	0x01, 0xF4, 0xE5, 0xB8, 0xE3, 0xCC, 0xFF, 0xAD, 0x76, 0xF6, 0xC3, 0x21, 0xB0, 0xFC, 0x17, 0xA3, 
	0xFD, 0xF1, 0x0F, 0x54, 0x5F, 0x04, 0x21, 0x18, 0x83, 0xB8, 0x04, 0x32, 0x2A, 0xA0, 0x8C, 0xAB, 
	0x3A, 0x23, 0x1A, 0xE7, 0x05, 0x29, 0x14, 0x85, 0xC0, 0x7D, 0x4E, 0x34, 0x0F, 0x14, 0x27, 0x19, 
	0xC5, 0x8C, 0x51, 0xD1, 0x8A, 0xD7, 0xE3, 0x00, 0x16, 0xB3, 0xC7, 0x45, 0x4E, 0x6A, 0xEF, 0x87, 
	0x3F, 0x1B, 0xE1, 0x12, 0x00, 0x30, 0xC6, 0x43, 0x92, 0x11, 0x85, 0x81, 0xE3, 0x01, 0x12, 0xCF, 
	0xD6, 0x42, 0xF6, 0x3D, 0x12, 0x86, 0xE6, 0x91, 0x23, 0x1D, 0xE7, 0x38, 0xC7, 0x3A, 0x5A, 0xF0, 
	0x7E, 0xF7, 0xD3, 0x21, 0xFF, 0x38, 0xF9, 0xC7, 0xFD, 0x81, 0x32, 0x90, 0x83, 0x2C, 0xD5, 0x28, 
	0x01, 0x40, 0xCC, 0x31, 0x1A, 0x92, 0x7C, 0x29, 0x3B, 0x20, 0x13, 0x18, 0xF8, 0x01, 0x03, 0xB9, 
	0x32, 0x04, 0x6D, 0x73, 0xE2, 0xE3, 0x28, 0x39, 0xC9, 0x5A, 0xC6, 0xF1, 0x09, 0x7A, 0x44, 0xCE, 
	0x26, 0x79, 0xC8, 0x3D, 0x2E, 0xFA, 0xD0, 0x83, 0x9F, 0xFA, 0x22, 0xEF, 0x98, 0x40, 0xCC, 0x72, 
	0x16, 0x93, 0x88, 0xA6, 0x34, 0xA1, 0x22, 0xCF, 0xC7, 0xCA, 0x66, 0x52, 0x2B, 0x04, 0xF0, 0x84, 
	0x67, 0x1B, 0xE3, 0x48, 0xCB, 0x59, 0x42, 0xB1, 0x2B, 0x32, 0xC8, 0xA7, 0x3E, 0xB1, 0x69, 0xB7, 
	0x1E, 0xEE, 0xF2, 0x8F, 0xA0, 0xF4, 0x53, 0x16, 0xCC, 0x69, 0x4E, 0x04, 0x9C, 0x93, 0x8C, 0xEA, 
	0x24, 0x9D, 0x2A, 0xCB, 0x86, 0x86, 0x06, 0xB8, 0x93, 0x7D, 0x50, 0x8B, 0x27, 0xE3, 0xA6, 0x59, 
	0x4B, 0x6A, 0x3A, 0x4E, 0x9F, 0x18, 0xDD, 0x67, 0xEE, 0x74, 0x31, 0x81, 0x8E, 0x76, 0xB4, 0x01, 
	0x04, 0x05, 0x80, 0x41, 0xCD, 0x59, 0xC2, 0x74, 0xA2, 0xE0, 0x90, 0x25, 0x34, 0xDC, 0x01, 0x55, 
	0xD6, 0x50, 0x77, 0xA6, 0x2D, 0x9E, 0xFF, 0xF1, 0x6C, 0x9B, 0x0C, 0xE8, 0x68, 0x4F, 0x5A, 0xC6, 
	0x31, 0xA3, 0x38, 0x6D, 0xC0, 0x06, 0x76, 0xCA, 0xD3, 0x9E, 0xEE, 0xB4, 0x01, 0x1E, 0xF5, 0xE8, 
	0x06, 0x26, 0x30, 0xD4, 0xA1, 0x82, 0x74, 0xA4, 0x21, 0x35, 0xA8, 0x31, 0x4D, 0x59, 0xC2, 0x0C, 
	0xAC, 0x93, 0xA5, 0x51, 0x70, 0xE8, 0x07, 0x9A, 0xF9, 0xD2, 0x78, 0xFA, 0x40, 0x9E, 0xD5, 0xB4, 
	0x68, 0x14, 0xF3, 0x19, 0x03, 0x9C, 0xEE, 0xD3, 0xA7, 0x60, 0xF5, 0x61, 0x51, 0x89, 0x1A, 0xD4, 
	0xA2, 0x82, 0xD4, 0x9C, 0x0B, 0x20, 0x66, 0x5A, 0x0F, 0x7A, 0xC2, 0x31, 0x16, 0xC0, 0x80, 0xE7, 
	0x3B, 0x81, 0x66, 0xA4, 0xDA, 0x4C, 0x78, 0xBE, 0x0C, 0x9A, 0x12, 0x5D, 0xC2, 0x3D, 0x6B, 0x7A, 
	0x53, 0x8C, 0x76, 0x55, 0x06, 0x7F, 0xD5, 0x69, 0x58, 0x7D, 0x0A, 0xD4, 0xA0, 0x92, 0x75, 0xA8, 
	0x1F, 0x25, 0xE8, 0x48, 0x91, 0x6A, 0xCC, 0x31, 0x66, 0xA0, 0x80, 0x4E, 0x2D, 0xC0, 0x42, 0x1B, 
	0xA0, 0x82, 0xB9, 0x4E, 0xB5, 0xAE, 0x30, 0x95, 0xE8, 0xFB, 0xEA, 0xA8, 0xD5, 0xBE, 0x02, 0xD6, 
	0xAB, 0x82, 0xE5, 0xC0, 0x4E, 0x45, 0xBB, 0x01, 0xD2, 0x8A, 0x16, 0xA8, 0x63, 0x45, 0xAC, 0x50, 
	0x8F, 0xBA, 0xD6, 0xA4, 0x92, 0x92, 0x7C, 0x28, 0x78, 0xEC, 0x5B, 0x1F, 0x7B, 0xC6, 0x06, 0x9C, 
	0xA0, 0xB2, 0x51, 0xBD, 0xEC, 0x12, 0xAC, 0x0A, 0xCF, 0x68, 0x8E, 0x20, 0x7E, 0xC7, 0xB1, 0xE7, 
	0x4D, 0xFF, 0xCA, 0x55, 0x8C, 0x0A, 0x76, 0xB0, 0x3C, 0x2D, 0xAC, 0x61, 0xC7, 0x9A, 0xD8, 0x90, 
	0x12, 0xB4, 0x90, 0x44, 0x94, 0x2D, 0x5C, 0x29, 0x7B, 0xDB, 0xA8, 0x3A, 0xB4, 0x01, 0x35, 0xA8, 
	0x41, 0x55, 0x23, 0xFA, 0xC2, 0xDF, 0xFE, 0x56, 0xAF, 0x5B, 0xF5, 0x6A, 0x71, 0x65, 0x70, 0x5C, 
	0xD3, 0x8E, 0x36, 0xB9, 0xA9, 0x25, 0xFF, 0x6B, 0x47, 0xCD, 0x8A, 0xD6, 0x91, 0x66, 0x89, 0x09, 
	0xB1, 0x9D, 0x2D, 0x0A, 0xDE, 0x1A, 0xDB, 0xC0, 0xD9, 0x56, 0x05, 0x2A, 0x90, 0x2B, 0xC4, 0xAE, 
	0xAB, 0x81, 0x1A, 0x3C, 0xF3, 0x91, 0xEF, 0xFB, 0x6D, 0x80, 0x1B, 0xA7, 0x57, 0xCF, 0xEA, 0x93, 
	0xB8, 0xF9, 0xD4, 0x29, 0x69, 0x91, 0xAB, 0xDC, 0xE5, 0x06, 0xF5, 0xAC, 0xCF, 0x85, 0xB0, 0x63, 
	0xA3, 0xAB, 0xCE, 0xDB, 0xE6, 0xB7, 0xB2, 0xCC, 0xC4, 0x6E, 0x7F, 0xB5, 0x2B, 0x51, 0x03, 0xFC, 
	0x56, 0x02, 0x09, 0x00, 0xB1, 0x87, 0x09, 0xBC, 0x84, 0xCF, 0x9A, 0xD8, 0xB8, 0x60, 0x35, 0x6F, 
	0x69, 0x7D, 0x78, 0xD8, 0x16, 0xB7, 0x98, 0x77, 0x6B, 0x1D, 0xE5, 0x04, 0x00, 0xD0, 0xD1, 0xD7, 
	0x16, 0x50, 0xBE, 0xF3, 0x75, 0x2A, 0x7E, 0x2D, 0xDC, 0x52, 0xEC, 0x42, 0x00, 0x02, 0xFD, 0x7D, 
	0xE9, 0x0B, 0xA1, 0x18, 0x3F, 0x01, 0x4B, 0xA0, 0x6D, 0x2E, 0x78, 0xC2, 0x81, 0xBF, 0x8A, 0xDC, 
	0x9E, 0x36, 0xD8, 0xC1, 0xEB, 0x05, 0x29, 0x39, 0x69, 0x4C, 0x63, 0x1A, 0xD0, 0xB8, 0x01, 0x8F, 
	0xCD, 0xB2, 0x7C, 0x97, 0x60, 0xE1, 0xEA, 0x66, 0x78, 0xC3, 0xFE, 0xD5, 0x2E, 0xE3, 0x40, 0x7C, 
	0x64, 0xEF, 0x8E, 0x38, 0x8A, 0x2E, 0x48, 0x73, 0x9A, 0xA1, 0x90, 0xE0, 0xF3, 0xF2, 0x54, 0xC5, 
	0x62, 0x6D, 0x71, 0x6A, 0xA1, 0x40, 0xCC, 0x19, 0x5B, 0x79, 0xC6, 0x54, 0x2E, 0x21, 0x7D, 0xD5, 
	0x89, 0x65, 0xEA, 0xE2, 0x97, 0x95, 0xD8, 0xAD, 0x01, 0x90, 0x6B, 0x80, 0x83, 0x20, 0x43, 0xF1, 
	0xC8, 0x03, 0x1E, 0x70, 0x03, 0x5C, 0x90, 0x00, 0x46, 0x3B, 0xFA, 0xD1, 0x49, 0x09, 0x6B, 0x83, 
	0xDD, 0x70, 0x83, 0x4A, 0x03, 0x95, 0xCA, 0x76, 0xA6, 0xB2, 0x95, 0xB1, 0x9C, 0xE3, 0xFA, 0x66, 
	0xE0, 0xD3, 0xF7, 0x95, 0xEB, 0x00, 0xFF, 0x1A, 0xAA, 0xE1, 0xEC, 0x6A, 0x60, 0xD0, 0x63, 0x6E, 
	0x5C, 0x88, 0xDF, 0x27, 0x01, 0x01, 0x3F, 0x4E, 0xCD, 0xB0, 0x4E, 0x73, 0xA3, 0x93, 0xEC, 0x81, 
	0x18, 0xC4, 0xE0, 0x05, 0xB5, 0x8E, 0x41, 0xEE, 0x2A, 0xCD, 0xEB, 0x5E, 0xDF, 0x00, 0xA4, 0x99, 
	0xC6, 0x33, 0x0D, 0xF0, 0xDC, 0x54, 0x75, 0x7E, 0x1A, 0xD4, 0xF8, 0xAD, 0xEC, 0xA8, 0x19, 0xE8, 
	0x63, 0xFF, 0xE2, 0xC0, 0xD4, 0x4C, 0x08, 0x71, 0xE3, 0x44, 0x3C, 0xE2, 0x45, 0x3B, 0xBA, 0xD1, 
	0xD8, 0x56, 0xB3, 0xE3, 0x5E, 0x10, 0x83, 0x5A, 0x73, 0xDB, 0x03, 0xDC, 0xEE, 0x76, 0xB8, 0x1B, 
	0xE0, 0xEB, 0x72, 0xF3, 0x1A, 0xA4, 0x56, 0xC6, 0x74, 0x39, 0x67, 0x3C, 0x81, 0x31, 0x2E, 0xE1, 
	0xD8, 0xC7, 0xB6, 0x6D, 0x03, 0x06, 0xE0, 0x54, 0x66, 0x37, 0x60, 0xD0, 0x10, 0x70, 0x76, 0x03, 
	0x24, 0x20, 0x01, 0xBD, 0x22, 0xDA, 0xC8, 0xD6, 0x8E, 0xB5, 0xAC, 0xD5, 0xDC, 0x00, 0x71, 0x1B, 
	0x1C, 0xDC, 0xB7, 0x16, 0x37, 0xB9, 0xCD, 0x5D, 0xEE, 0x4B, 0x67, 0xBA, 0xCA, 0x75, 0x7E, 0x2D, 
	0x96, 0xE1, 0x1D, 0x6F, 0x65, 0x2F, 0x3B, 0xC3, 0xA6, 0x16, 0xB4, 0x76, 0x25, 0x10, 0x02, 0x8E, 
	0x2F, 0x41, 0xC4, 0xAE, 0xBE, 0xB6, 0xC8, 0x07, 0x5E, 0x70, 0x5C, 0xDF, 0xDA, 0xDB, 0xDD, 0xB6, 
	0x35, 0xB7, 0x17, 0xCE, 0xF0, 0x5E, 0x03, 0x5B, 0xD3, 0x78, 0xCE, 0xF4, 0xBB, 0x29, 0x1E, 0xEF, 
	0x79, 0x67, 0xE0, 0xE2, 0x10, 0x9B, 0xAA, 0x81, 0x9E, 0x1D, 0xE4, 0x8E, 0xF3, 0xDB, 0xE3, 0x0D, 
	0x08, 0xF9, 0xAC, 0x05, 0x0E, 0x69, 0x6F, 0xE7, 0xDA, 0xE4, 0xDC, 0x36, 0x39, 0xCB, 0x5B, 0x7E, 
	0x6E, 0x00, 0xDC, 0xD9, 0xE9, 0x98, 0x66, 0x02, 0xCD, 0xE1, 0x6D, 0xF3, 0x01, 0xE0, 0x9C, 0xAE, 
	0x1A, 0xC6, 0xC1, 0xBE, 0xF9, 0xDD, 0xFF, 0x71, 0x9F, 0xF7, 0x7B, 0xDF, 0x01, 0xCF, 0x76, 0xB6, 
	0xD7, 0x9C, 0xF0, 0x84, 0xA3, 0x1C, 0xE1, 0x2B, 0x67, 0xBA, 0xAF, 0x5F, 0xAE, 0xEE, 0x25, 0xD0, 
	0x80, 0x06, 0x13, 0x9F, 0xBA, 0x53, 0xAD, 0x0E, 0x6A, 0x25, 0xEA, 0x36, 0xC8, 0x1C, 0xE7, 0xBA, 
	0xDE, 0x3B, 0xCE, 0x84, 0x47, 0xC7, 0x1A, 0xDB, 0x0D, 0xA8, 0x35, 0xC2, 0x07, 0xFF, 0xED, 0xC2, 
	0x2F, 0x9D, 0xE9, 0x0E, 0x77, 0x3A, 0x13, 0xDE, 0x7E, 0x80, 0xB7, 0xC7, 0x9D, 0xE2, 0xEF, 0xB6, 
	0x3A, 0x0B, 0xE9, 0xDA, 0x4C, 0x0E, 0xEB, 0x3D, 0xEF, 0x7A, 0x0F, 0x7A, 0xB4, 0x47, 0xBE, 0xE6, 
	0xA4, 0x1B, 0xFD, 0xE4, 0xA0, 0x4F, 0xBB, 0xDA, 0x2D, 0x0D, 0x75, 0xB7, 0x33, 0xFE, 0xF4, 0x8F, 
	0x8F, 0xF7, 0xA8, 0xE9, 0x8D, 0x73, 0x66, 0x5E, 0x96, 0xAA, 0x97, 0xEF, 0x3A, 0xE6, 0x83, 0x3E, 
	0x82, 0xDA, 0x3F, 0x41, 0xDB, 0x49, 0x4E, 0x39, 0xD2, 0x3F, 0x0F, 0xFA, 0xC3, 0xB7, 0x5C, 0xCA, 
	0x0D, 0x38, 0x7D, 0xE3, 0x69, 0x30, 0xFC, 0xD4, 0xCF, 0x7D, 0xD4, 0x37, 0x3F, 0x5B, 0x1B, 0x5E, 
	0xAF, 0x73, 0xCC, 0x7B, 0xDD, 0xE7, 0x8F, 0xAC, 0xBD, 0xF4, 0xA5, 0xDF, 0xF7, 0xDC, 0x87, 0xFB, 
	0xFA, 0x2A, 0xD7, 0xBD, 0xAE, 0xD5, 0x0E, 0x85, 0xD3, 0xBF, 0xFD, 0xFB, 0x8C, 0x4F, 0xFD, 0xBC, 
	0x25, 0x2F, 0x79, 0x59, 0x31, 0xBF, 0x99, 0xB1, 0xCF, 0xBB, 0xCF, 0x83, 0xDE, 0xEA, 0x11, 0xB4, 
	0x7F, 0xFA, 0x7B, 0x22, 0xBC, 0xC2, 0x5D, 0x9E, 0x85, 0x1B, 0x78, 0xE0, 0xD7, 0xC4, 0xFF, 0x7E, 
	0xE3, 0xF7, 0xFF, 0xFD, 0xC7, 0x8F, 0x9F, 0xFC, 0xCA, 0xE7, 0x07, 0xCC, 0xC7, 0x04, 0x97, 0xF7, 
	0x73, 0xFD, 0x36, 0x7D, 0x08, 0xF8, 0x7E, 0x23, 0x10, 0x70, 0xB1, 0xB6, 0x51, 0xF6, 0xF7, 0x80, 
	0xF7, 0xE7, 0x01, 0x07, 0xE0, 0x01, 0xFF, 0xC1, 0xE7, 0x7D, 0xE0, 0xC7, 0x7F, 0x13, 0xF7, 0x7F, 
	0x92, 0xD7, 0x00, 0x26, 0x50, 0x22, 0x4B, 0xF0, 0x7A, 0x0D, 0x80, 0x03, 0x5A, 0xB7, 0x75, 0xB2, 
	0x17, 0x7D, 0xB5, 0xF7, 0x7E, 0x28, 0x68, 0x7B, 0x2F, 0xE0, 0x02, 0x2B, 0xD8, 0x82, 0x2C, 0x48, 
	0x74, 0x81, 0x07, 0x81, 0x13, 0x38, 0x83, 0x12, 0x78, 0x7F, 0xC1, 0x37, 0x7C, 0xC2, 0x07, 0x7E, 
	0x70, 0x77, 0x7C, 0x56, 0xB7, 0x81, 0x6B, 0xC2, 0x04, 0xBA, 0x25, 0x82, 0x22, 0xF8, 0x71, 0x5E, 
	0x47, 0x7B, 0x08, 0x78, 0x84, 0x0B, 0xA8, 0x66, 0x2E, 0xB8, 0x82, 0x2F, 0xE8, 0x82, 0xE4, 0x46, 
	0x83, 0x13, 0x78, 0x03, 0x50, 0x48, 0x81, 0xFA, 0x97, 0x7F, 0x55, 0xD8, 0x78, 0x4B, 0xD0, 0x83, 
	0x5A, 0xC8, 0x81, 0x4D, 0xF3, 0x81, 0xCD, 0x84, 0x03, 0x10, 0x00, 0x86, 0x43, 0x98, 0x36, 0x1E, 
	0xE7, 0x7E, 0xD2, 0x97, 0x82, 0x27, 0xD8, 0x00, 0x4C, 0x18, 0x6B, 0x4C, 0xB8, 0x84, 0x81, 0x37, 
	0x81, 0x35, 0x08, 0x87, 0x51, 0x58, 0x83, 0x0D, 0x80, 0x83, 0x16, 0x08, 0x77, 0x59, 0xA8, 0x85, 
	0x3D, 0x18, 0x80, 0xB2, 0x72, 0x5D, 0x5F, 0x48, 0x68, 0xCF, 0x16, 0x86, 0x23, 0x78, 0x80, 0x48, 
	0x78, 0x84, 0x8B, 0xB6, 0x84, 0x4D, 0x28, 0x70, 0x75, 0x78, 0x7F, 0x73, 0x78, 0x00, 0xF6, 0x47, 
	0x83, 0xF8, 0x97, 0x7F, 0xC3, 0x77, 0x00, 0x4C, 0x70, 0x00, 0x94, 0xA8, 0x87, 0x1B, 0x78, 0x0E, 
	0x4F, 0x20, 0x86, 0x82, 0x28, 0x86, 0x23, 0x48, 0x7B, 0x68, 0x68, 0x86, 0x0B, 0x88, 0x88, 0x6B, 
	0x98, 0x88, 0x2F, 0x40, 0x6E, 0x71, 0x58, 0x83, 0x35, 0xF8, 0x88, 0x54, 0xA8, 0x83, 0x4B, 0x60, 
	0x89, 0xB0, 0xA8, 0x81, 0x3E, 0xA8, 0x89, 0x54, 0x25, 0x84, 0x84, 0x06, 0x01, 0x21, 0x20, 0x82, 
	0x61, 0x08, 0x76, 0x4C, 0x50, 0x88, 0xFF, 0x49, 0x48, 0x74, 0xA4, 0x48, 0x6B, 0x8D, 0xC8, 0x88, 
	0xC4, 0x28, 0x85, 0x37, 0x48, 0x7C, 0xAF, 0x08, 0x8B, 0xB1, 0xA8, 0x87, 0x0D, 0xC0, 0x02, 0x0F, 
	0x63, 0x77, 0x21, 0x28, 0x88, 0xF9, 0x66, 0x8B, 0x5A, 0x67, 0x80, 0x5F, 0xC7, 0x7E, 0xF0, 0x17, 
	0x8C, 0x2F, 0x08, 0x6B, 0xA7, 0xC8, 0x8A, 0x34, 0xA8, 0x8A, 0x70, 0x18, 0x7C, 0x95, 0xA8, 0x8C, 
	0x2C, 0x70, 0x00, 0xCE, 0xB8, 0x85, 0xCD, 0xC8, 0x87, 0x58, 0x70, 0x5D, 0xD8, 0x25, 0x84, 0x61, 
	0xF8, 0x8E, 0x5B, 0x67, 0x8D, 0xD7, 0xB8, 0x6F, 0x49, 0xE8, 0x82, 0x2F, 0x08, 0x03, 0x69, 0x66, 
	0x8F, 0x75, 0xC8, 0x8A, 0x0F, 0x38, 0x83, 0xBF, 0xF6, 0x04, 0xCA, 0x68, 0x89, 0xE5, 0xB8, 0x8C, 
	0x92, 0xC7, 0x02, 0x06, 0xF9, 0x8C, 0xA9, 0xB3, 0x04, 0x25, 0x70, 0x6F, 0x82, 0xF6, 0x6C, 0x42, 
	0x18, 0x8F, 0xF2, 0xF8, 0x73, 0x7B, 0x92, 0x8F, 0xDB, 0x78, 0x8A, 0x1E, 0x20, 0x81, 0xFF, 0x98, 
	0x05, 0x30, 0xB0, 0x91, 0x75, 0x48, 0x8E, 0x01, 0x79, 0x89, 0xA3, 0x66, 0x90, 0xCE, 0x58, 0x20, 
	0x1F, 0xB8, 0x90, 0x21, 0xC8, 0x89, 0x1B, 0x17, 0x91, 0xF2, 0xD8, 0x00, 0x02, 0xD0, 0x92, 0x2E, 
	0xD9, 0x92, 0x1B, 0xB5, 0x91, 0x32, 0x39, 0x93, 0x1C, 0x29, 0x90, 0x01, 0x59, 0x8E, 0xE5, 0x38, 
	0x6F, 0xCD, 0x28, 0x92, 0x08, 0xD9, 0x87, 0x0E, 0x55, 0x02, 0x26, 0x29, 0x84, 0xD5, 0xA8, 0x92, 
	0xD6, 0xC8, 0x92, 0x2F, 0x79, 0x94, 0x48, 0xD9, 0x00, 0x34, 0xB9, 0x94, 0x33, 0xD9, 0x91, 0xE6, 
	0x68, 0x93, 0xCA, 0xB8, 0x04, 0x22, 0x39, 0x92, 0xCF, 0xA0, 0x90, 0x1F, 0x50, 0x02, 0x57, 0xE9, 
	0x63, 0x43, 0x49, 0x94, 0x12, 0xE9, 0x92, 0x12, 0xD0, 0x92, 0x5F, 0x89, 0x94, 0x30, 0xC9, 0x94, 
	0x64, 0xD9, 0x8C, 0x1F, 0x69, 0x89, 0xFF, 0x4F, 0x30, 0x95, 0x3D, 0x89, 0x36, 0x0D, 0x80, 0x95, 
	0x6E, 0xB9, 0x04, 0x5B, 0xC9, 0x95, 0xFD, 0x26, 0x00, 0x61, 0x89, 0x94, 0x61, 0x19, 0x96, 0x4A, 
	0x49, 0x96, 0x4B, 0xD9, 0x91, 0x03, 0x99, 0x93, 0x0D, 0x40, 0x01, 0x14, 0xB0, 0x93, 0x07, 0xB9, 
	0x96, 0x68, 0x73, 0x95, 0x86, 0x89, 0x95, 0x04, 0x28, 0x97, 0x73, 0xE9, 0x95, 0x60, 0x29, 0x96, 
	0x02, 0x90, 0x97, 0x7A, 0xD9, 0x94, 0x51, 0xB9, 0x04, 0x80, 0x49, 0x01, 0xCE, 0xC8, 0x93, 0x84, 
	0xC9, 0x26, 0x6D, 0x09, 0x94, 0x57, 0x89, 0x98, 0x33, 0xF0, 0x04, 0x44, 0x69, 0x94, 0x75, 0xC9, 
	0x98, 0x74, 0xE9, 0x92, 0x90, 0x19, 0x99, 0x35, 0xC9, 0x04, 0x95, 0xC9, 0x02, 0x95, 0xB9, 0x93, 
	0xEA, 0xB8, 0x07, 0x4F, 0x70, 0x98, 0x40, 0xD9, 0x00, 0x33, 0x50, 0x9B, 0x9F, 0xF9, 0x71, 0x45, 
	0x69, 0x97, 0x62, 0xF9, 0x95, 0xA7, 0xA9, 0x97, 0x4F, 0xB0, 0x9A, 0xC0, 0xE9, 0x9A, 0x1D, 0x18, 
	0x0B, 0x4C, 0xC0, 0x99, 0x9E, 0x69, 0x9B, 0xB6, 0x09, 0x9A, 0x73, 0x79, 0x97, 0xA4, 0x39, 0x9A, 
	0xBD, 0x29, 0x93, 0x50, 0x20, 0x00, 0x81, 0x59, 0x99, 0xC0, 0x09, 0x98, 0xE9, 0x38, 0x9C, 0xBD, 
	0x00, 0x05, 0x58, 0xD9, 0x4C, 0xB5, 0x09, 0x01, 0xC8, 0x89, 0x9C, 0x7B, 0x72, 0x94, 0x78, 0x09, 
	0x9D, 0xBF, 0x29, 0x9D, 0x14, 0x20, 0x9D, 0x7F, 0x69, 0x99, 0xEA, 0x49, 0x9D, 0xCE, 0xC8, 0x85, 
	0xF8, 0xF0, 0x81, 0xC5, 0xB9, 0x90, 0xC8, 0xE9, 0x9D, 0xDE, 0x09, 0x9E, 0x0E, 0x70, 0x9F, 0xF8, 
	0x79, 0x9F, 0xB9, 0x73, 0x9E, 0x80, 0x29, 0x9D, 0x0E, 0x40, 0x01, 0xFF, 0xE9, 0x00, 0xE9, 0xC9, 
	0x9E, 0x81, 0xB9, 0x04, 0x26, 0x80, 0x9D, 0x3A, 0x91, 0x05, 0xDC, 0x59, 0x9F, 0xF3, 0x99, 0x9C, 
	0xF9, 0xF9, 0xA0, 0x10, 0x2A, 0xA0, 0xFE, 0xFC, 0x19, 0xA0, 0x00, 0xDA, 0x9F, 0xAD, 0xA9, 0x9E, 
	0xAC, 0x59, 0xA0, 0x07, 0xEA, 0x9E, 0x20, 0xA1, 0x0B, 0xDF, 0x69, 0x9B, 0xF5, 0xD9, 0x00, 0x11, 
	0x3A, 0xA2, 0x02, 0x4A, 0xA1, 0xD4, 0x49, 0xA1, 0x8F, 0x49, 0x9D, 0x4C, 0x70, 0xA0, 0x1D, 0x88, 
	0xA0, 0x1D, 0x9A, 0x14, 0x33, 0xC0, 0xA0, 0xF6, 0x49, 0xA2, 0xF9, 0x39, 0xA0, 0xE7, 0xF9, 0x9F, 
	0xE6, 0x69, 0x9E, 0x12, 0xBA, 0xA2, 0x2C, 0xDA, 0xA2, 0x99, 0xD9, 0x07, 0x59, 0x00, 0x94, 0x26, 
	0xB9, 0x04, 0x20, 0xFA, 0x99, 0x34, 0x5A, 0xA3, 0xFE, 0x59, 0x99, 0x26, 0x0A, 0x05, 0x3D, 0xDA, 
	0xA3, 0xAF, 0x09, 0x12, 0xFB, 0xF5, 0x04, 0x42, 0x2A, 0xA4, 0x4C, 0x50, 0x9B, 0x22, 0x4A, 0xA3, 
	0x33, 0xA0, 0x9F, 0x15, 0xFA, 0x98, 0x59, 0x90, 0x02, 0x29, 0xC0, 0x81, 0x4D, 0xDA, 0x81, 0x4F, 
	0x0A, 0xA5, 0x4A, 0x14, 0x9F, 0x53, 0x3A, 0x9B, 0x1B, 0x65, 0x0A, 0x5E, 0xBA, 0xA6, 0x6C, 0x0A, 
	0xA6, 0x2C, 0xFA, 0x04, 0x64, 0xAA, 0x89, 0x66, 0x7A, 0xA6, 0x0B, 0x89, 0x01, 0x76, 0x7A, 0xA7, 
	0x78, 0x8A, 0xA7, 0x0D, 0xC0, 0xA6, 0x29, 0x80, 0x01, 0x7D, 0xFA, 0xA7, 0x7E, 0x0A, 0xA6, 0x5F, 
	0xBA, 0x4C, 0x71, 0x6A, 0x05, 0x6E, 0x40, 0xA7, 0x0D, 0x90, 0xA7, 0x8A, 0x6A, 0xA7, 0x7B, 0xEA, 
	0xA7, 0x7E, 0xCA, 0xA7, 0x5E, 0xCA, 0x85, 0x70, 0x5A, 0xA8, 0x4A, 0x62, 0x0A, 0xB3, 0xB9, 0xA8, 
	0x8A, 0xBA, 0xA7, 0x7C, 0xEA, 0xA8, 0x83, 0x0A, 0x05, 0x94, 0x0A, 0x9B, 0x69, 0xBA, 0x04, 0x8B, 
	0xDA, 0xA8, 0x9D, 0x2A, 0xA5, 0x84, 0xFA, 0xA9, 0x7B, 0x71, 0xA8, 0x53, 0xBA, 0x27, 0x67, 0x3A, 
	0xA9, 0xA8, 0x4A, 0x9C, 0xDA, 0x49, 0xA7, 0x54, 0x4A, 0xA5, 0x6E, 0xF0, 0xAA, 0x50, 0x1A, 0xAA, 
	0xAE, 0xFA, 0xA9, 0x41, 0x00, 0x00, 0x21, 0xF9, 0x04, 0x04, 0x0A, 0x00, 0x00, 0x00, 0x2C, 0x00, 
	0x00, 0x10, 0x00, 0x64, 0x00, 0x63, 0x00, 0x00, 0x06, 0xFF, 0xC0, 0x96, 0x70, 0x48, 0x2C, 0x1A, 
	0x8F, 0x48, 0x62, 0x63, 0xC9, 0x64, 0xDA, 0x9A, 0x4C, 0xCB, 0x32, 0x49, 0xAD, 0x5A, 0xAF, 0x58, 
	0xE4, 0x32, 0xC0, 0x24, 0x35, 0x2C, 0x16, 0x92, 0xC5, 0x06, 0x06, 0x37, 0x2E, 0x97, 0x68, 0x40, 
	0x9A, 0x6D, 0xBB, 0xDF, 0x4A, 0x28, 0x49, 0x3C, 0x6F, 0x3E, 0x9B, 0xE8, 0xFC, 0x32, 0xCD, 0x84, 
	0xFB, 0xFF, 0x45, 0x72, 0x75, 0x50, 0x65, 0x64, 0x65, 0x52, 0x67, 0x79, 0x7C, 0x5F, 0x7C, 0x80, 
	0x8E, 0x59, 0x50, 0x91, 0x51, 0x87, 0x63, 0x94, 0x66, 0x67, 0x92, 0x98, 0x0D, 0x8F, 0x9C, 0x47, 
	0x82, 0x73, 0xA0, 0x5E, 0x96, 0x60, 0x86, 0xA3, 0x89, 0x8A, 0x69, 0x7C, 0x17, 0x9D, 0xAC, 0x4C, 
	0x01, 0x5C, 0xA1, 0x84, 0xA3, 0x95, 0xB3, 0x88, 0x4C, 0x7A, 0x68, 0x7D, 0xAC, 0x80, 0x0D, 0xAF, 
	0xAF, 0x91, 0xB1, 0xB5, 0xA5, 0xA6, 0x8A, 0x91, 0x7A, 0x9B, 0xBB, 0x6E, 0x5B, 0xBF, 0x4B, 0xA1, 
	0xB1, 0x5F, 0xB5, 0xD1, 0x4C, 0x20, 0x17, 0xD4, 0x68, 0x20, 0x78, 0x0D, 0x3F, 0xC8, 0xC9, 0x56, 
	0xCB, 0x5B, 0x0D, 0xCE, 0x5E, 0xCD, 0xD0, 0xD1, 0xB3, 0x0D, 0x22, 0x22, 0x50, 0x7A, 0x2B, 0xD8, 
	0x4B, 0xDB, 0xDC, 0xDD, 0x46, 0xAE, 0xCB, 0xB0, 0xE3, 0xCD, 0xA0, 0xE5, 0xE6, 0x94, 0xE8, 0xEA, 
	0xE9, 0x20, 0x22, 0x76, 0x34, 0x01, 0xD1, 0xAE, 0xDD, 0xBB, 0x78, 0xF2, 0x84, 0x34, 0xF0, 0xD1, 
	0xA1, 0x1E, 0xB3, 0x70, 0xCF, 0xC2, 0xE9, 0xD0, 0xB7, 0x4F, 0x4A, 0x3A, 0x75, 0xFD, 0x44, 0x00, 
	0x54, 0x87, 0x8D, 0xA0, 0x47, 0x5D, 0x09, 0x5B, 0x34, 0x68, 0xD8, 0xCB, 0x17, 0x17, 0x7C, 0xF9, 
	0x1A, 0xE8, 0x20, 0xA1, 0x63, 0xE2, 0xC4, 0x25, 0xE6, 0x98, 0xE4, 0xC8, 0x88, 0xF1, 0x62, 0x03, 
	0x10, 0x38, 0x6F, 0x16, 0x6C, 0xB0, 0x42, 0x5B, 0xC8, 0xFF, 0x91, 0x40, 0x4D, 0x96, 0x24, 0x10, 
	0x4E, 0x25, 0x4B, 0x96, 0x0C, 0xBC, 0xB4, 0x54, 0xF9, 0x32, 0x53, 0x83, 0x99, 0x39, 0xA2, 0xD2, 
	0xAC, 0x79, 0x73, 0x47, 0x4E, 0x9E, 0xD8, 0x56, 0xF4, 0x44, 0xD8, 0x89, 0x49, 0x87, 0x86, 0xCC, 
	0x02, 0x10, 0x20, 0x01, 0x6B, 0x65, 0xD2, 0xA4, 0x4A, 0x5D, 0xB6, 0x9C, 0xF8, 0x40, 0xC7, 0x83, 
	0x06, 0x0F, 0xE2, 0x3E, 0x95, 0x2A, 0x42, 0xEA, 0xD4, 0x25, 0x38, 0xAD, 0x5E, 0xCD, 0xAA, 0x75, 
	0x4A, 0xAB, 0x91, 0x5F, 0x7B, 0x35, 0x13, 0x4B, 0xB6, 0x01, 0x51, 0x12, 0x49, 0x57, 0x2E, 0x59, 
	0xBA, 0x96, 0xAD, 0xDB, 0xB7, 0x71, 0xDF, 0x46, 0x95, 0x9A, 0xA3, 0x2E, 0x54, 0x9B, 0x02, 0xF3, 
	0x82, 0xD0, 0xBB, 0x44, 0xAB, 0x67, 0xAE, 0x7E, 0x00, 0x7F, 0x25, 0xF9, 0x8A, 0x40, 0x80, 0xC2, 
	0x63, 0x0F, 0x2B, 0x6E, 0xCC, 0x94, 0x71, 0x5C, 0xB6, 0x0F, 0x72, 0x48, 0x9E, 0x3C, 0xB7, 0x6E, 
	0xBF, 0x06, 0x3B, 0x72, 0xE3, 0xD5, 0xEC, 0xCE, 0xF3, 0xD6, 0x47, 0x4B, 0x46, 0x93, 0x1E, 0x7B, 
	0x9A, 0x0B, 0x81, 0xE3, 0x24, 0x54, 0xB3, 0x6E, 0xEC, 0x58, 0x6E, 0xE4, 0xD9, 0xB4, 0x27, 0xCF, 
	0xC4, 0x1D, 0x30, 0xA0, 0x40, 0xBD, 0x56, 0x1B, 0x54, 0xE0, 0xD9, 0x97, 0x07, 0x68, 0x48, 0x1A, 
	0x44, 0x37, 0x88, 0xD0, 0x8B, 0x2C, 0x51, 0xC3, 0xA9, 0x8F, 0xB7, 0x5E, 0xDA, 0x76, 0x2D, 0xDC, 
	0xC7, 0xB0, 0x23, 0xCF, 0x8D, 0x6E, 0x5B, 0x01, 0xEE, 0xDC, 0x3B, 0x02, 0xEE, 0xD6, 0x5E, 0x61, 
	0x3B, 0xF7, 0x06, 0xDE, 0x85, 0x16, 0x5E, 0x60, 0xE3, 0x45, 0x40, 0x9E, 0x69, 0xE1, 0x20, 0x87, 
	0x9C, 0x61, 0x4C, 0xA8, 0xB5, 0xDC, 0x6B, 0x90, 0xC5, 0x26, 0x97, 0x74, 0x94, 0x35, 0xA0, 0xC0, 
	0x85, 0xD4, 0xE1, 0xA7, 0x5B, 0x56, 0x15, 0xAC, 0xD0, 0xFF, 0x1F, 0x77, 0x3F, 0x04, 0xA8, 0x8C, 
	0x06, 0x3E, 0x04, 0x65, 0x60, 0x07, 0xE4, 0x19, 0xA7, 0xA0, 0x58, 0x86, 0x31, 0x40, 0x80, 0x8B, 
	0x0D, 0x36, 0x06, 0x61, 0x5B, 0x70, 0x3D, 0x07, 0x5D, 0x13, 0x17, 0x2A, 0x90, 0x83, 0x7D, 0xB9, 
	0x55, 0xA7, 0x1B, 0x7F, 0xFD, 0x7D, 0xD8, 0x93, 0x88, 0xE0, 0x95, 0xD8, 0xD0, 0x89, 0x06, 0x92, 
	0x67, 0x58, 0x72, 0x62, 0xA9, 0x77, 0xDC, 0x8B, 0x2F, 0x36, 0xB0, 0xC0, 0x02, 0x0D, 0xB6, 0x15, 
	0xD7, 0x02, 0x6E, 0xD5, 0x28, 0x5B, 0x24, 0x08, 0x74, 0x69, 0xE1, 0x8E, 0x39, 0xDE, 0xD7, 0xE3, 
	0x75, 0xDA, 0x81, 0x50, 0x81, 0x99, 0xFE, 0x85, 0xF8, 0x9D, 0x16, 0x3E, 0x84, 0x37, 0x1E, 0x8A, 
	0x11, 0xA0, 0x38, 0x1E, 0x7A, 0xC9, 0x8D, 0xD5, 0xA2, 0x8B, 0x50, 0x36, 0xC0, 0xC0, 0x9E, 0x7B, 
	0x3A, 0xD5, 0xC4, 0x94, 0x0D, 0x74, 0x29, 0xA8, 0x97, 0x17, 0xEE, 0xB8, 0x63, 0x86, 0xD7, 0xE9, 
	0x16, 0x64, 0x90, 0x4B, 0x10, 0xE9, 0x8D, 0x06, 0x6E, 0x26, 0x99, 0x64, 0x81, 0xE4, 0xD5, 0xE9, 
	0x05, 0x94, 0xC7, 0xC1, 0xC8, 0x27, 0x9F, 0x54, 0x4E, 0xE9, 0x69, 0xA7, 0x53, 0x22, 0xF0, 0x00, 
	0x02, 0x54, 0x0E, 0x4A, 0x68, 0x8E, 0x61, 0xEA, 0xA6, 0x61, 0x66, 0xFD, 0x99, 0xA9, 0x9D, 0x5F, 
	0x57, 0x2C, 0x34, 0x52, 0x89, 0x71, 0xC6, 0x59, 0x20, 0x01, 0x06, 0x2E, 0x89, 0x1E, 0x94, 0x78, 
	0xEA, 0xB9, 0x29, 0x03, 0x52, 0x4E, 0xA9, 0x03, 0x03, 0x80, 0x2E, 0x20, 0xAA, 0xB1, 0x0B, 0xBC, 
	0xD5, 0x65, 0x0E, 0x83, 0x3E, 0x55, 0xA8, 0x8E, 0xF6, 0xDD, 0x57, 0xC0, 0x0E, 0xD3, 0x0A, 0x74, 
	0xE6, 0x76, 0x1F, 0x3A, 0x4A, 0x45, 0x70, 0x0C, 0x51, 0x3A, 0x1E, 0x01, 0x09, 0x44, 0x10, 0xAE, 
	0x92, 0xBC, 0x3E, 0x09, 0xEC, 0xAF, 0x7D, 0x62, 0x49, 0xFF, 0xAC, 0xBA, 0xC1, 0x92, 0x8A, 0x6C, 
	0xA0, 0xA6, 0x9E, 0x8A, 0x2A, 0x8F, 0xAB, 0x2A, 0x8A, 0x6D, 0x0F, 0xFE, 0xC5, 0x3A, 0x52, 0x08, 
	0xDD, 0xCA, 0x29, 0xA9, 0x81, 0xB8, 0x32, 0xA8, 0x67, 0xA6, 0x44, 0xA1, 0xDB, 0xE7, 0xBA, 0x9F, 
	0x06, 0x1A, 0xEA, 0x03, 0xC6, 0x2A, 0xDB, 0x65, 0x6C, 0x5E, 0x1A, 0x0A, 0x6D, 0x03, 0x28, 0x88, 
	0x59, 0xAD, 0xA2, 0x40, 0x6E, 0x17, 0x6B, 0x07, 0x0C, 0x75, 0x10, 0x02, 0x49, 0x00, 0x87, 0x8B, 
	0x6B, 0x02, 0x86, 0x25, 0xE0, 0x64, 0xAF, 0x06, 0x07, 0x3B, 0x25, 0x03, 0x3A, 0x14, 0x3B, 0x2A, 
	0xC3, 0xEE, 0x3E, 0x15, 0x2F, 0x02, 0x16, 0x62, 0x48, 0xB1, 0x02, 0x34, 0x5F, 0x4C, 0xED, 0x8F, 
	0x15, 0xE0, 0x8B, 0xEF, 0x9A, 0x0A, 0x71, 0xCC, 0xD0, 0xD0, 0x05, 0x8E, 0x0B, 0xEE, 0x81, 0x4F, 
	0x9A, 0xBC, 0xC4, 0xB9, 0x29, 0xAB, 0xEB, 0x29, 0xB1, 0x52, 0xBA, 0x4B, 0xEA, 0xA8, 0x81, 0x8E, 
	0x1A, 0xAF, 0x85, 0x14, 0x23, 0xA0, 0x00, 0x0A, 0x5A, 0x4B, 0xAB, 0x61, 0x03, 0x05, 0x64, 0x0C, 
	0xB4, 0x48, 0x3E, 0x18, 0xE0, 0x31, 0xC7, 0x73, 0x46, 0x70, 0xF4, 0xA4, 0xE0, 0x26, 0xAD, 0x9E, 
	0xAF, 0xE8, 0xAA, 0xEC, 0x69, 0xCB, 0xED, 0x26, 0xEB, 0x6E, 0xA9, 0x83, 0x32, 0xCB, 0x84, 0xD6, 
	0x5C, 0x6F, 0xCD, 0xB5, 0x98, 0xD4, 0x82, 0x5D, 0x40, 0xD8, 0x3D, 0xE7, 0xAB, 0xC5, 0xAC, 0x44, 
	0x1B, 0xA0, 0xF8, 0xAD, 0xE2, 0x96, 0x7C, 0x9C, 0xC9, 0x6D, 0x37, 0xC0, 0x01, 0x13, 0x9B, 0xAA, 
	0xCC, 0xF2, 0xCA, 0xA5, 0xC2, 0x3C, 0xE5, 0xA8, 0x78, 0x43, 0xE1, 0x25, 0x02, 0x28, 0xE0, 0x1C, 
	0x3A, 0xDF, 0x60, 0x2B, 0x3A, 0x6D, 0xD8, 0xF8, 0x1A, 0xEE, 0x49, 0x78, 0x1F, 0x77, 0x6B, 0x40, 
	0x04, 0x8B, 0x8B, 0x8B, 0xF4, 0xE3, 0x6D, 0xAB, 0xC7, 0xFF, 0x01, 0x01, 0x1C, 0x4C, 0xBE, 0xB4, 
	0xCA, 0x2D, 0x5F, 0x0E, 0x2A, 0xA9, 0x34, 0x73, 0x79, 0xF5, 0xE8, 0xC4, 0xF7, 0x8D, 0x9B, 0xE0, 
	0x83, 0x0F, 0xCE, 0x1F, 0x80, 0xDF, 0x35, 0x30, 0x60, 0xD9, 0x0D, 0x28, 0xAE, 0x38, 0xEC, 0x73, 
	0xCA, 0xD0, 0x36, 0xE4, 0x8F, 0x4B, 0x9E, 0xFB, 0xED, 0xDB, 0xFB, 0x29, 0x49, 0xC2, 0x33, 0x0B, 
	0x9A, 0xF5, 0xD6, 0x5A, 0x93, 0x5F, 0xF1, 0xE0, 0xD4, 0xA2, 0x8F, 0x3A, 0xF3, 0xAB, 0x3F, 0x1F, 
	0xFD, 0xEB, 0xD2, 0x2F, 0x4E, 0xBB, 0xDB, 0x4E, 0x6E, 0x9F, 0x3B, 0xEE, 0x93, 0x1B, 0x9C, 0xEE, 
	0xA7, 0xFC, 0xC3, 0x3B, 0x73, 0xA0, 0x7D, 0x43, 0x41, 0x00, 0x67, 0x92, 0xBC, 0xD3, 0x29, 0x6F, 
	0x3B, 0x3C, 0xD0, 0x96, 0x48, 0x34, 0xF0, 0x81, 0x85, 0x1C, 0x29, 0x7E, 0xF0, 0x6B, 0x00, 0xF6, 
	0x4C, 0x06, 0x39, 0x92, 0xD9, 0x0F, 0x7F, 0xDC, 0x83, 0x5B, 0xDC, 0xF8, 0xD7, 0xBF, 0xF0, 0x11, 
	0x4A, 0x80, 0x7C, 0xDB, 0x1A, 0xC5, 0x90, 0x57, 0xC0, 0xB0, 0x01, 0x48, 0x81, 0xCE, 0x83, 0xD4, 
	0x12, 0x20, 0x28, 0x3D, 0x09, 0x2E, 0xA1, 0x76, 0xB4, 0xD3, 0x1E, 0x06, 0xF1, 0xA7, 0xC1, 0x0D, 
	0x4E, 0x60, 0x4A, 0x37, 0x5C, 0x40, 0x0E, 0xFD, 0xF7, 0xBF, 0xBE, 0x91, 0x8F, 0x66, 0x02, 0xC4, 
	0x4D, 0x01, 0x2A, 0xA0, 0xBE, 0xC0, 0x25, 0xD0, 0x51, 0x29, 0x54, 0x21, 0xFC, 0x14, 0x97, 0x80, 
	0x08, 0x26, 0xE0, 0x89, 0xE8, 0xA1, 0x20, 0x51, 0xB8, 0x67, 0xBF, 0xEE, 0xE9, 0x6F, 0x7F, 0x1C, 
	0x04, 0x94, 0x07, 0xBD, 0xC4, 0xB5, 0xD0, 0x2D, 0x41, 0x80, 0x41, 0x2C, 0x61, 0xF2, 0x4E, 0x88, 
	0xC4, 0x0F, 0x40, 0x2A, 0x3C, 0x2C, 0x5C, 0xDC, 0x13, 0xD7, 0xF8, 0xC2, 0x27, 0x69, 0xEF, 0x7E, 
	0x54, 0x2C, 0xD8, 0x15, 0xE5, 0x86, 0x43, 0xF0, 0x79, 0xFF, 0xD0, 0x42, 0x28, 0x98, 0x09, 0x18, 
	0xC1, 0x58, 0xBA, 0xC1, 0x11, 0x31, 0x6C, 0x61, 0x4B, 0x60, 0x3C, 0x52, 0xD8, 0xC0, 0x10, 0xBC, 
	0x6F, 0x89, 0x06, 0x90, 0xE0, 0x1A, 0xD9, 0xC8, 0xA0, 0xED, 0x61, 0xD0, 0x7E, 0x35, 0xDC, 0x60, 
	0x1D, 0x3B, 0xB8, 0xC5, 0x11, 0xEE, 0x31, 0x74, 0x61, 0x4C, 0x1F, 0x20, 0x03, 0xE9, 0x1D, 0x64, 
	0x2C, 0xC1, 0x8C, 0x25, 0x0A, 0x81, 0x21, 0x61, 0x67, 0x80, 0x26, 0x2A, 0x32, 0x5C, 0x8B, 0x84, 
	0xA2, 0xE4, 0x1E, 0x89, 0x3F, 0xDC, 0x45, 0xF2, 0x57, 0x52, 0xCA, 0xA1, 0xA7, 0x76, 0x58, 0xC9, 
	0x2F, 0x5E, 0x92, 0x8F, 0x05, 0x44, 0xDE, 0x09, 0xFD, 0xB2, 0x04, 0x15, 0x8A, 0x52, 0x94, 0x87, 
	0x4C, 0x64, 0x2A, 0x53, 0xD9, 0x00, 0x19, 0x30, 0x81, 0x8A, 0x19, 0xBC, 0x22, 0x9F, 0xE8, 0xC8, 
	0x41, 0x1E, 0x7A, 0x69, 0x09, 0x00, 0x00, 0x00, 0xC5, 0x04, 0xE8, 0xB7, 0x3D, 0x0A, 0x4E, 0x70, 
	0x2A, 0x50, 0x5E, 0x03, 0x4E, 0xE0, 0xC9, 0x14, 0x2E, 0x84, 0x5F, 0xC0, 0x54, 0xE3, 0x30, 0xC5, 
	0x05, 0x45, 0x19, 0x98, 0xD3, 0x98, 0x4B, 0x80, 0xA4, 0x32, 0xB1, 0xA8, 0xC3, 0x4F, 0xDD, 0x10, 
	0x5E, 0x4D, 0x88, 0xA6, 0x3C, 0xA7, 0x09, 0x46, 0xBF, 0x89, 0x50, 0x20, 0x62, 0x5C, 0x02, 0x37, 
	0x15, 0xD2, 0x40, 0xD6, 0x89, 0xD2, 0x07, 0xE1, 0x3C, 0x25, 0x39, 0x89, 0x79, 0xCE, 0x82, 0x1E, 
	0xF3, 0x95, 0xB0, 0x9C, 0x24, 0x95, 0x24, 0x21, 0xCF, 0x86, 0x46, 0x93, 0x9E, 0xB7, 0x0C, 0x63, 
	0x01, 0xB3, 0xA9, 0xCF, 0x7D, 0x36, 0xA0, 0x81, 0x0D, 0xF8, 0xE7, 0x2F, 0x81, 0x39, 0xD0, 0x27, 
	0x76, 0xB4, 0x98, 0x05, 0x0D, 0x29, 0x3A, 0xBD, 0x97, 0x89, 0x0D, 0x4C, 0x60, 0x03, 0x28, 0x6D, 
	0x80, 0x43, 0xA3, 0x89, 0x00, 0x96, 0x4A, 0x93, 0x9A, 0xFF, 0x11, 0x6D, 0x40, 0x36, 0x0B, 0x30, 
	0x53, 0xE5, 0x9D, 0x80, 0x9B, 0x17, 0xC5, 0xE8, 0x46, 0xC1, 0x69, 0xC8, 0x61, 0x12, 0x54, 0xA4, 
	0x21, 0x7D, 0x63, 0x15, 0x21, 0x89, 0x52, 0x94, 0x9E, 0xF4, 0xA4, 0x26, 0xDD, 0x80, 0x4A, 0x01, 
	0xD0, 0xD2, 0x95, 0x3E, 0xF4, 0x92, 0xD5, 0x1C, 0xA1, 0x18, 0x55, 0xB0, 0x4D, 0x9C, 0x7E, 0xA0, 
	0x90, 0x3C, 0xDD, 0xE8, 0x12, 0x3C, 0xBA, 0x48, 0x72, 0x82, 0x14, 0xA8, 0xE7, 0x14, 0xEA, 0x50, 
	0x73, 0xD7, 0x80, 0xA4, 0x16, 0xD5, 0xAC, 0x27, 0x5D, 0x6A, 0x34, 0x8D, 0xB5, 0xD6, 0x79, 0xDE, 
	0xF2, 0x8B, 0x19, 0x40, 0xDE, 0x4C, 0x2B, 0xFA, 0x49, 0xAC, 0x8A, 0xD2, 0x63, 0x00, 0x7D, 0xDF, 
	0x56, 0x87, 0xF9, 0x55, 0xB0, 0x1A, 0x73, 0xAC, 0x43, 0x2D, 0x6B, 0x52, 0x8F, 0x6A, 0xD4, 0x94, 
	0x3A, 0x95, 0xA9, 0x2C, 0xBD, 0x19, 0x1F, 0x51, 0x90, 0x81, 0x30, 0xD6, 0x34, 0x6C, 0x54, 0xB5, 
	0xAA, 0x4E, 0x01, 0xAA, 0xD5, 0x52, 0x2E, 0x4E, 0x82, 0x03, 0xED, 0x2B, 0x58, 0xC5, 0x0A, 0xD8, 
	0xB2, 0x12, 0x16, 0xA9, 0x84, 0x55, 0x29, 0xA9, 0x0E, 0xFB, 0xD2, 0x20, 0xD2, 0xB3, 0xB1, 0x71, 
	0xCD, 0xA5, 0x0A, 0x22, 0xCB, 0x84, 0x7E, 0x66, 0xF4, 0x97, 0x79, 0x95, 0xDE, 0x08, 0xC4, 0x59, 
	0x4E, 0xBF, 0x9A, 0x93, 0xB3, 0x63, 0x15, 0x6C, 0x61, 0xCD, 0x6A, 0x52, 0xB5, 0x92, 0xF6, 0xA5, 
	0xD3, 0x44, 0x2D, 0x63, 0xCF, 0x47, 0x53, 0xC8, 0xB2, 0xF6, 0x93, 0x0D, 0xA8, 0x01, 0xEB, 0x28, 
	0x1B, 0xBD, 0x26, 0x32, 0xF1, 0xB2, 0x24, 0xB3, 0x6D, 0x58, 0x01, 0x5B, 0x45, 0xCF, 0x82, 0xB6, 
	0xB0, 0x69, 0x6D, 0x68, 0x4B, 0xD9, 0x9A, 0x58, 0xB8, 0x0E, 0x17, 0x8C, 0x8D, 0xBD, 0xE6, 0x6A, 
	0xA9, 0xEA, 0xC9, 0x06, 0x42, 0x40, 0x03, 0x35, 0x78, 0xFF, 0xAD, 0x21, 0x65, 0x2B, 0x3D, 0xE7, 
	0x32, 0x41, 0xBA, 0x7F, 0xA5, 0x6E, 0xF7, 0xAE, 0x8B, 0xD6, 0xA2, 0xFA, 0xB6, 0xA1, 0x4C, 0x98, 
	0x40, 0x03, 0x84, 0x3B, 0xDC, 0xC6, 0x32, 0x16, 0x6C, 0x32, 0x1D, 0x2F, 0x2F, 0x53, 0xA8, 0xDC, 
	0xF0, 0xE4, 0x75, 0xB6, 0xB3, 0x65, 0xA2, 0x04, 0x2C, 0xDB, 0x00, 0x17, 0xBC, 0x17, 0xA8, 0xB8, 
	0xCD, 0xED, 0x6E, 0xE9, 0x9B, 0xD6, 0xD1, 0x3E, 0x74, 0x09, 0x13, 0xC8, 0x30, 0x00, 0xF4, 0x0B, 
	0xDE, 0xEF, 0x86, 0xD7, 0xB8, 0x02, 0x56, 0x48, 0x72, 0x35, 0x00, 0x81, 0x02, 0xAF, 0xB0, 0xBD, 
	0x06, 0x48, 0x70, 0x0B, 0x13, 0xE0, 0x02, 0x16, 0x3F, 0x78, 0xBA, 0xF2, 0xED, 0x5E, 0x7D, 0xEB, 
	0xAB, 0xDF, 0x0B, 0x37, 0x40, 0xC3, 0x13, 0xD8, 0x70, 0x8E, 0xF7, 0xFB, 0x5D, 0x0F, 0x57, 0x6C, 
	0xBC, 0x21, 0x16, 0x49, 0x0A, 0xCF, 0x5B, 0x62, 0x34, 0x3A, 0x37, 0x7E, 0x08, 0x6E, 0x30, 0x8B, 
	0x97, 0xDC, 0xE2, 0x07, 0x47, 0x38, 0xB0, 0x13, 0xC6, 0x2E, 0x14, 0x32, 0x9C, 0x63, 0x1C, 0xEB, 
	0x98, 0x62, 0xC2, 0xE5, 0x6F, 0x5C, 0x81, 0xBC, 0xDA, 0x6E, 0x16, 0x59, 0xB9, 0xE7, 0x6D, 0x40, 
	0x82, 0x11, 0x6C, 0x59, 0x35, 0xB6, 0xF8, 0xCC, 0x4C, 0x26, 0x99, 0x53, 0x72, 0x0B, 0x5A, 0x49, 
	0xDC, 0x40, 0xA9, 0x56, 0x8E, 0xF3, 0x86, 0x4B, 0xDB, 0xE3, 0xF0, 0x06, 0x58, 0xC0, 0xDD, 0x04, 
	0x33, 0x89, 0xD1, 0xCB, 0x04, 0x15, 0xB3, 0x57, 0xC9, 0x68, 0x76, 0x41, 0xA0, 0x1D, 0x1C, 0x83, 
	0x42, 0x1B, 0x3A, 0x06, 0x24, 0x7D, 0xB3, 0xA2, 0x37, 0xA0, 0xE8, 0x1B, 0xCF, 0x59, 0xCE, 0x55, 
	0xE6, 0x71, 0x96, 0x97, 0xB0, 0x65, 0x01, 0x0F, 0xA0, 0x9B, 0xCE, 0x53, 0xAE, 0x72, 0xD5, 0xBB, 
	0x42, 0x32, 0x97, 0x52, 0x82, 0x81, 0x66, 0x72, 0x8B, 0xFF, 0x49, 0x76, 0xE8, 0x52, 0x97, 0xBA, 
	0xAC, 0x8B, 0x4E, 0x35, 0xA3, 0x19, 0xED, 0xE8, 0x2A, 0x57, 0xF9, 0xD1, 0x2F, 0xF5, 0x6F, 0xC5, 
	0xF6, 0x4B, 0x83, 0x4A, 0x77, 0xF9, 0xD2, 0x22, 0x6E, 0xA0, 0x9E, 0x1B, 0x20, 0x01, 0x1C, 0x48, 
	0x20, 0x9C, 0xCF, 0x4D, 0x64, 0xA8, 0x07, 0x4D, 0x6A, 0x53, 0x1B, 0xBB, 0x01, 0xAA, 0x4E, 0x36, 
	0x9C, 0x5F, 0x0D, 0x69, 0x0E, 0x87, 0x37, 0x03, 0xB5, 0x86, 0xF6, 0x9D, 0x1B, 0x30, 0x80, 0xB8, 
	0xE6, 0xBA, 0x9F, 0x9B, 0xFE, 0xA5, 0xAF, 0x0D, 0x79, 0x48, 0x40, 0xA7, 0x59, 0xD0, 0x4C, 0x6E, 
	0x80, 0xB1, 0x8F, 0xBD, 0x6A, 0x65, 0x37, 0x9A, 0xD9, 0xCC, 0xD6, 0x31, 0x70, 0x69, 0x9D, 0x01, 
	0x68, 0xB7, 0x5B, 0xA6, 0xD4, 0xC6, 0xF5, 0x10, 0x72, 0xEA, 0x3C, 0x1C, 0xB0, 0xEE, 0xD7, 0xBF, 
	0x0E, 0xC1, 0xAF, 0x97, 0x30, 0x02, 0x50, 0x7F, 0x3B, 0xD4, 0xE2, 0x1E, 0xF7, 0xA9, 0xCD, 0x5D, 
	0x6E, 0xA5, 0x3E, 0x3A, 0xDD, 0x38, 0xA6, 0xB4, 0xBB, 0x17, 0x4E, 0x03, 0x78, 0x57, 0x7B, 0x90, 
	0x57, 0xC5, 0x36, 0xAF, 0x43, 0x80, 0x03, 0x51, 0xE2, 0x1B, 0x98, 0xDE, 0x46, 0x73, 0x9A, 0x03, 
	0x2E, 0xF0, 0x43, 0xA3, 0xBA, 0xE0, 0xC9, 0x6E, 0xB5, 0xBA, 0x5F, 0xDD, 0x04, 0x86, 0xB7, 0x3B, 
	0xDA, 0x4B, 0xA8, 0xB6, 0xBC, 0xF9, 0x19, 0xF1, 0x25, 0x58, 0x5C, 0xDF, 0x14, 0xA7, 0x78, 0xBE, 
	0x9B, 0x40, 0xEC, 0x33, 0x73, 0xBC, 0xE3, 0x85, 0x46, 0x36, 0xC8, 0x77, 0x7E, 0x03, 0x91, 0x6B, 
	0x98, 0x09, 0x07, 0x08, 0xFA, 0x7E, 0x4D, 0xEE, 0xEE, 0x78, 0x3F, 0x5C, 0x09, 0x11, 0xBF, 0x6A, 
	0x03, 0xB6, 0x1D, 0x73, 0x98, 0xC3, 0x5C, 0xCC, 0x23, 0xE8, 0xF7, 0x12, 0xC0, 0x8D, 0xE6, 0x9B, 
	0xE3, 0xFC, 0xE3, 0x04, 0x7F, 0xB3, 0x4A, 0x23, 0xBD, 0xFF, 0x84, 0xA0, 0x7B, 0xFD, 0x00, 0xEC, 
	0x26, 0x3A, 0xB5, 0xDB, 0xBD, 0x72, 0x91, 0x24, 0xBD, 0x81, 0x17, 0xBF, 0xB8, 0xBE, 0x2B, 0x0E, 
	0xF5, 0xA8, 0x47, 0x9D, 0x09, 0xE1, 0xC6, 0xB9, 0xC7, 0xB3, 0x5E, 0xEE, 0x1B, 0x03, 0xFD, 0xEB, 
	0x07, 0xA0, 0x81, 0xD0, 0xA3, 0xCD, 0xF7, 0x77, 0xEF, 0x77, 0x00, 0x97, 0x46, 0x48, 0x4E, 0x95, 
	0xEE, 0xF2, 0x8A, 0xFF, 0x9A, 0xE9, 0x12, 0x10, 0xB3, 0x04, 0xDC, 0xCE, 0x78, 0x26, 0x10, 0x5A, 
	0xEE, 0x39, 0xE7, 0x79, 0xDD, 0x9B, 0xF0, 0x75, 0xBD, 0xE7, 0x3D, 0xE8, 0x37, 0xD0, 0x7B, 0xD8, 
	0x6B, 0x9D, 0xF2, 0xB1, 0xC3, 0x6A, 0xDE, 0x83, 0x6F, 0x00, 0x04, 0x20, 0x30, 0x71, 0xC3, 0x5B, 
	0xBC, 0xED, 0x23, 0x58, 0x7C, 0xEA, 0x19, 0x2F, 0x75, 0x49, 0x8C, 0x5B, 0xE7, 0x5A, 0x97, 0xC4, 
	0x0B, 0x66, 0xDF, 0x00, 0xAF, 0x5B, 0xFE, 0x00, 0x99, 0xBF, 0xBC, 0xD0, 0x4F, 0xFE, 0x6E, 0xC0, 
	0x5F, 0x1A, 0xF0, 0x9F, 0x17, 0x31, 0xBD, 0x4B, 0x4C, 0xFA, 0x89, 0xCB, 0xDC, 0x90, 0xAC, 0x67, 
	0xBD, 0xEA, 0xFB, 0x2D, 0xE8, 0xE6, 0x0B, 0x9A, 0xA4, 0x0D, 0x78, 0xC1, 0x0D, 0x66, 0x3F, 0xFD, 
	0xD9, 0x5B, 0xBF, 0xF6, 0x5E, 0xBF, 0x41, 0xDE, 0x6F, 0x6F, 0xF9, 0x06, 0xF0, 0xDD, 0xE8, 0x97, 
	0x8E, 0xAB, 0x09, 0x40, 0x53, 0x57, 0xD1, 0xE3, 0xA0, 0x06, 0x35, 0x28, 0xBE, 0xD3, 0xA1, 0xBE, 
	0xFC, 0xE5, 0x37, 0xDE, 0x05, 0x02, 0x80, 0xBF, 0xFC, 0x9D, 0xEF, 0x7C, 0x64, 0xC7, 0xC0, 0xFA, 
	0xF8, 0x9F, 0xBD, 0x07, 0x5E, 0xB0, 0x7F, 0xEF, 0x6B, 0x5F, 0xEF, 0x96, 0x47, 0x03, 0xDA, 0xE7, 
	0x75, 0x43, 0x97, 0x72, 0x03, 0x40, 0x03, 0xC0, 0x17, 0x78, 0x87, 0x33, 0x7C, 0x38, 0x00, 0x01, 
	0x0D, 0x48, 0x7A, 0xA7, 0x97, 0x7C, 0x51, 0xA7, 0x7A, 0xFF, 0xAA, 0xD7, 0x60, 0xF4, 0x27, 0x7F, 
	0xF1, 0x97, 0x81, 0x0E, 0x46, 0x7D, 0xD6, 0xB7, 0x7F, 0xFC, 0x97, 0x7F, 0xB5, 0x77, 0x7B, 0x78, 
	0xA7, 0x77, 0x99, 0xE7, 0x7D, 0xE0, 0x37, 0x00, 0x07, 0x10, 0x78, 0x2C, 0xD0, 0x3C, 0xAD, 0x25, 
	0x7A, 0xA3, 0x87, 0x7E, 0xA3, 0xE7, 0x72, 0x8A, 0x37, 0x81, 0x12, 0xF8, 0x76, 0x17, 0x78, 0x83, 
	0xCF, 0x87, 0x7F, 0x1E, 0xF8, 0x81, 0x1F, 0xD8, 0x7F, 0x41, 0x47, 0x82, 0x40, 0xF8, 0x83, 0x42, 
	0x17, 0x6F, 0x29, 0x88, 0x82, 0x09, 0x38, 0x7E, 0x49, 0x00, 0x05, 0x35, 0x10, 0x02, 0x0E, 0x58, 
	0x62, 0x35, 0xD0, 0x80, 0xFC, 0xA6, 0x7C, 0x6E, 0x57, 0x81, 0x18, 0x58, 0x85, 0xF1, 0x47, 0x7F, 
	0xD1, 0xC7, 0x83, 0x5A, 0xB8, 0x83, 0x2F, 0x80, 0x7D, 0xBA, 0x17, 0x80, 0x60, 0xC7, 0x04, 0xBE, 
	0x67, 0x84, 0xBE, 0xD7, 0x00, 0x48, 0x98, 0x84, 0x4D, 0xF0, 0x82, 0x4D, 0xD8, 0x80, 0xE9, 0xD7, 
	0x04, 0xEE, 0xC7, 0x7A, 0x16, 0x88, 0x83, 0x82, 0xA6, 0x81, 0x0D, 0xB0, 0x83, 0x76, 0xA8, 0x7F, 
	0x1F, 0x18, 0x82, 0x03, 0xD8, 0x7D, 0x5D, 0x27, 0x74, 0x80, 0x57, 0x84, 0x65, 0xB8, 0x82, 0xDE, 
	0xC0, 0x04, 0x25, 0xC6, 0x86, 0xA3, 0x07, 0x01, 0x4C, 0xC8, 0x6B, 0x12, 0x90, 0x78, 0xFC, 0xD6, 
	0x7E, 0xCC, 0x37, 0x87, 0x90, 0x38, 0x7F, 0xF5, 0x87, 0x87, 0x94, 0xD8, 0x81, 0xB4, 0x27, 0x84, 
	0x0D, 0xD7, 0x87, 0x5F, 0x77, 0x82, 0xC0, 0x67, 0x86, 0x63, 0x23, 0x7C, 0x4B, 0x97, 0x7E, 0xE7, 
	0xF7, 0x80, 0xE9, 0xD5, 0x6B, 0x8B, 0xB8, 0x88, 0x7D, 0xE6, 0x76, 0x0D, 0x76, 0x85, 0x37, 0xC8, 
	0x8A, 0x1B, 0x58, 0x89, 0xFB, 0x17, 0x8B, 0xD7, 0x37, 0x84, 0x5E, 0x88, 0x77, 0x60, 0x37, 0x86, 
	0x61, 0xB8, 0x82, 0x9F, 0x08, 0x8A, 0x4B, 0x90, 0x7E, 0xFF, 0x4E, 0x08, 0x85, 0x12, 0xE0, 0x00, 
	0xA6, 0xE8, 0x6B, 0xA8, 0xC8, 0x6F, 0x71, 0x08, 0x89, 0x1A, 0xD8, 0x7C, 0xF1, 0x17, 0x7D, 0x1E, 
	0x28, 0x8B, 0xF8, 0x17, 0x09, 0xB6, 0x88, 0x77, 0x44, 0x08, 0x7C, 0x2C, 0xA0, 0x8B, 0x6D, 0xD0, 
	0x5A, 0x0D, 0x34, 0x8A, 0xC4, 0xA7, 0x88, 0x8B, 0x28, 0x8C, 0x8B, 0x48, 0x8C, 0x8C, 0x98, 0x09, 
	0xCE, 0xC7, 0x8A, 0x59, 0x48, 0x7B, 0x92, 0x00, 0x03, 0xE8, 0x58, 0x8B, 0xD1, 0xE8, 0x87, 0x9D, 
	0x58, 0x8D, 0xD6, 0x78, 0x8D, 0x0D, 0x50, 0x02, 0x25, 0x80, 0x51, 0x6B, 0xC8, 0x6B, 0xBE, 0x46, 
	0x8C, 0x38, 0xE0, 0x8D, 0xC4, 0x28, 0x66, 0x02, 0x20, 0x00, 0x23, 0xE0, 0x8F, 0xFD, 0x08, 0x7D, 
	0xE8, 0x38, 0x90, 0xE8, 0x28, 0x00, 0x30, 0x10, 0x90, 0xEB, 0xC8, 0x02, 0x9B, 0x48, 0x8D, 0xEE, 
	0x28, 0x88, 0xCA, 0xF0, 0x49, 0xF3, 0x48, 0x8F, 0xA4, 0xF7, 0x8D, 0xDE, 0xF8, 0x8D, 0xC1, 0xD8, 
	0x6B, 0x0D, 0xE0, 0x8F, 0xFF, 0xF8, 0x8F, 0x12, 0x00, 0x90, 0x00, 0xB9, 0x91, 0x0D, 0x30, 0x90, 
	0x06, 0xD9, 0x8F, 0x07, 0x69, 0x90, 0x07, 0x99, 0x8E, 0xB6, 0xA8, 0x90, 0xD2, 0x78, 0x69, 0x0D, 
	0xE9, 0x90, 0x6F, 0x50, 0x57, 0xF2, 0xA8, 0x74, 0xA4, 0x07, 0x8E, 0xC1, 0x48, 0x93, 0xA8, 0x38, 
	0x02, 0x0E, 0xE0, 0x8F, 0x0E, 0xB0, 0x91, 0x1A, 0xF9, 0x91, 0xFD, 0x36, 0x92, 0x27, 0x19, 0x94, 
	0x27, 0x69, 0x90, 0xEA, 0xB8, 0x8E, 0x4B, 0xD0, 0x92, 0x2E, 0xF9, 0x92, 0x31, 0x79, 0x55, 0xF3, 
	0xC8, 0x04, 0x0E, 0x70, 0x8F, 0xDD, 0x08, 0x95, 0xBE, 0x06, 0x75, 0x3E, 0x29, 0x00, 0x1D, 0x19, 
	0x75, 0x3B, 0x29, 0x00, 0x21, 0x09, 0x94, 0x43, 0xD9, 0x95, 0x30, 0x80, 0x7D, 0x2A, 0x29, 0x8D, 
	0x4C, 0x80, 0x94, 0xBB, 0xB8, 0x2D, 0x4C, 0x79, 0x96, 0xFF, 0x0D, 0x30, 0x03, 0x6E, 0x48, 0x8C, 
	0xDE, 0x28, 0x8C, 0xFC, 0xE8, 0x76, 0x02, 0x90, 0x95, 0x38, 0xF9, 0x91, 0x21, 0xE9, 0x95, 0x40, 
	0x09, 0x94, 0x45, 0x19, 0x86, 0x0D, 0x40, 0x01, 0x14, 0xD0, 0x00, 0x2D, 0x19, 0x7C, 0x70, 0xC0, 
	0x04, 0xF3, 0x38, 0x98, 0xE6, 0x35, 0x03, 0x10, 0xA0, 0x96, 0x4B, 0x60, 0x91, 0x37, 0xF9, 0x8F, 
	0x3B, 0xB9, 0x91, 0x8E, 0x19, 0x75, 0x5A, 0x69, 0x92, 0x5C, 0x69, 0x92, 0x43, 0x59, 0x8B, 0x4C, 
	0xC0, 0x97, 0x14, 0xC0, 0x02, 0x7D, 0xD9, 0x90, 0x9E, 0xC8, 0x09, 0x82, 0x19, 0x71, 0x25, 0x90, 
	0x96, 0x86, 0x79, 0x98, 0x86, 0xD9, 0x04, 0x53, 0xE9, 0x98, 0x3D, 0x09, 0x99, 0x8E, 0x59, 0x97, 
	0x93, 0xD9, 0x95, 0x44, 0xA9, 0x97, 0x98, 0x99, 0x99, 0x98, 0xE9, 0x97, 0xD5, 0xD8, 0x99, 0x5D, 
	0xD1, 0x5A, 0x4D, 0x39, 0x7A, 0x18, 0x70, 0x98, 0xA3, 0x37, 0x03, 0x86, 0x89, 0x98, 0x4D, 0xA0, 
	0x93, 0xC2, 0x79, 0x95, 0x39, 0xB9, 0x95, 0x41, 0x79, 0x97, 0x5F, 0xD9, 0x04, 0xB1, 0x29, 0x9B, 
	0xB3, 0x59, 0x9B, 0xB6, 0xF9, 0x17, 0xB8, 0x99, 0x96, 0xBC, 0x39, 0x03, 0xBB, 0xC9, 0x9B, 0x87, 
	0xD9, 0x00, 0x0E, 0x30, 0x03, 0x0E, 0x90, 0x09, 0x1A, 0xF9, 0x93, 0x27, 0x99, 0x09, 0x14, 0x20, 
	0x00, 0x7D, 0xC9, 0x97, 0x9A, 0xA9, 0x99, 0xE4, 0x99, 0x99, 0x7E, 0x69, 0x86, 0x26, 0x70, 0x86, 
	0xD0, 0xD9, 0x82, 0x86, 0xB9, 0x9B, 0xBF, 0xF9, 0x9E, 0xD7, 0xE9, 0x00, 0xF4, 0x59, 0x9F, 0xDA, 
	0x99, 0x9D, 0xDB, 0x09, 0x9E, 0xE2, 0x19, 0x9E, 0x7C, 0x29, 0x9E, 0x0E, 0x30, 0x9E, 0xCB, 0x69, 
	0x9E, 0x7C, 0xB9, 0x04, 0xEB, 0xC9, 0x9E, 0xBB, 0x90, 0x09, 0xBE, 0x39, 0x9D, 0x0A, 0x9A, 0x96, 
	0xF6, 0x59, 0x9F, 0xF4, 0x79, 0x9F, 0xDA, 0xB9, 0x97, 0xFF, 0xFC, 0xF9, 0x9F, 0x14, 0x4A, 0x01, 
	0x15, 0x3A, 0x9E, 0xE5, 0x19, 0x9B, 0x4C, 0x50, 0xA0, 0x80, 0xD9, 0x9E, 0x91, 0x00, 0x9F, 0xA4, 
	0x39, 0x9F, 0xF8, 0xE9, 0xA0, 0x0E, 0x7A, 0x9F, 0x7B, 0x79, 0xA1, 0x98, 0xB9, 0x9F, 0x03, 0xBA, 
	0x9C, 0xE8, 0xA9, 0x9E, 0x2E, 0x5A, 0x96, 0x7F, 0x10, 0x09, 0xF2, 0x18, 0x9A, 0x4B, 0x30, 0x9A, 
	0xBB, 0x89, 0x9D, 0x25, 0xFA, 0xA0, 0x24, 0xBA, 0x9D, 0x2A, 0x1A, 0x97, 0xE1, 0x59, 0xA1, 0x71, 
	0xB9, 0x97, 0xE6, 0xB9, 0xA1, 0x29, 0x30, 0x7E, 0xE3, 0xD7, 0xA1, 0xDD, 0x20, 0xA3, 0x33, 0x4A, 
	0xA3, 0xA2, 0xA7, 0x96, 0xF7, 0x39, 0xA2, 0x39, 0x4A, 0x9F, 0x27, 0xAA, 0xA2, 0xFF, 0x89, 0x99, 
	0x55, 0xBA, 0x97, 0x1B, 0xBA, 0x9E, 0x45, 0xEA, 0x89, 0x30, 0x7A, 0x9B, 0x82, 0xB9, 0xA4, 0xF2, 
	0xE8, 0x27, 0x3A, 0xAA, 0xA3, 0xD8, 0x69, 0xA5, 0xFB, 0x99, 0x09, 0x26, 0x50, 0xA4, 0x6A, 0x9A, 
	0xA6, 0x20, 0x11, 0x12, 0x0B, 0x18, 0x8F, 0x60, 0x3A, 0xA3, 0x0D, 0x80, 0x01, 0x74, 0x5A, 0xA7, 
	0xD0, 0xD7, 0x04, 0x29, 0x90, 0xA7, 0x79, 0xAA, 0x9E, 0x05, 0xBA, 0xA5, 0x6D, 0xEA, 0xA6, 0x68, 
	0xD8, 0x04, 0x71, 0x3A, 0xA7, 0x75, 0x5A, 0xA8, 0x86, 0x8A, 0x01, 0x0D, 0xA0, 0xA7, 0x8A, 0x9A, 
	0xA7, 0x18, 0x90, 0x02, 0x8D, 0x9A, 0xA8, 0x6A, 0x2A, 0x98, 0x5D, 0x0A, 0xA8, 0x92, 0x20, 0xA7, 
	0x18, 0x50, 0x02, 0x97, 0x7A, 0xA8, 0x85, 0x9A, 0xA8, 0x8D, 0xDA, 0xA9, 0x8E, 0xAA, 0xA8, 0x2F, 
	0x1A, 0x8F, 0x88, 0x3A, 0xA9, 0x80, 0xCA, 0x8B, 0x92, 0x5A, 0xA8, 0x98, 0x9A, 0xAA, 0x99, 0x8A, 
	0xA8, 0x8B, 0xBA, 0xA8, 0x50, 0x10, 0xA6, 0x48, 0x5A, 0xAA, 0x90, 0x40, 0x52, 0x9A, 0xCA, 0xA9, 
	0x29, 0x50, 0xA9, 0x70, 0x4A, 0xA3, 0xB2, 0x7A, 0xA0, 0x21, 0x82, 0xBA, 0xA4, 0xDE, 0x13, 0xA7, 
	0xA1, 0x39, 0xAA, 0xA4, 0xBA, 0xAB, 0x58, 0x90, 0x09, 0x83, 0x2A, 0x8F, 0x97, 0x8A, 0xA9, 0x91, 
	0x40, 0xAC, 0xB2, 0x7A, 0xA7, 0x4D, 0xC0, 0xAC, 0x42, 0x10, 0x04, 0x00, 0x21, 0xF9, 0x04, 0x04, 
	0x0A, 0x00, 0x00, 0x00, 0x2C, 0x00, 0x00, 0x10, 0x00, 0x64, 0x00, 0x63, 0x00, 0x00, 0x06, 0xFF, 
	0xC0, 0x96, 0x70, 0x48, 0x2C, 0x1A, 0x8F, 0x48, 0x62, 0x63, 0xC9, 0x64, 0xDA, 0x9A, 0xD0, 0x46, 
	0x72, 0x4A, 0xAD, 0x5A, 0xAF, 0xC7, 0x66, 0x80, 0x49, 0x6A, 0x58, 0xBE, 0xCF, 0xEF, 0xB7, 0x71, 
	0xB9, 0x40, 0xB1, 0xE8, 0xB4, 0x5A, 0x19, 0x6D, 0x90, 0xDE, 0x6F, 0x27, 0xB4, 0x4C, 0x67, 0x9A, 
	0xA5, 0xEB, 0xBC, 0x9E, 0x1D, 0xD8, 0x2E, 0xE1, 0x50, 0x62, 0x36, 0x62, 0x63, 0x64, 0x74, 0x65, 
	0x76, 0x78, 0x7B, 0x8B, 0x58, 0x0D, 0x7E, 0x6D, 0x4B, 0x84, 0x60, 0x92, 0x85, 0x77, 0x6D, 0x66, 
	0x8C, 0x99, 0x48, 0x50, 0x70, 0x9D, 0x5E, 0x94, 0x16, 0x83, 0xA0, 0x16, 0x86, 0x87, 0x88, 0x76, 
	0x9A, 0xA9, 0x4C, 0x7D, 0x6E, 0x80, 0x4C, 0xA3, 0xA1, 0xB0, 0xA4, 0x4D, 0x20, 0x75, 0xA7, 0x8A, 
	0xA9, 0x6B, 0x4B, 0x7D, 0xAC, 0x9C, 0x71, 0xB0, 0xA2, 0xA3, 0xA5, 0x96, 0x76, 0x75, 0xB9, 0x6A, 
	0xAB, 0x8F, 0x9D, 0xAE, 0x9F, 0xB2, 0xC2, 0x0D, 0x20, 0xB5, 0xA6, 0x0D, 0x2B, 0xA5, 0x4B, 0xC7, 
	0x56, 0x4B, 0x3E, 0x5A, 0xAD, 0x5D, 0x4D, 0xBF, 0xCE, 0xC2, 0x22, 0x4C, 0xD1, 0x20, 0x22, 0x17, 
	0x20, 0xD4, 0x4B, 0xA7, 0xD8, 0x53, 0x0D, 0xDB, 0xBB, 0xBC, 0xDF, 0x9E, 0x0D, 0x3A, 0xCD, 0xE1, 
	0x84, 0x0D, 0x22, 0xFB, 0xE3, 0xC5, 0xD0, 0x20, 0xD5, 0xC8, 0x30, 0x69, 0x67, 0x24, 0x99, 0xBC, 
	0x3F, 0xCB, 0xFE, 0xE8, 0xF0, 0x86, 0x4F, 0xCC, 0x92, 0x7D, 0x0F, 0xF9, 0xF5, 0x83, 0xB6, 0x62, 
	0x05, 0xC0, 0x75, 0x3F, 0x70, 0xB5, 0x5B, 0xD2, 0xC1, 0x91, 0x3C, 0x79, 0xAE, 0x16, 0xEA, 0x60, 
	0x50, 0xCF, 0x5E, 0x24, 0x59, 0x4B, 0x72, 0xE8, 0xE3, 0x17, 0x91, 0x65, 0x3A, 0x80, 0x30, 0x97, 
	0x54, 0x23, 0xD8, 0x82, 0x63, 0xC7, 0x5E, 0x7D, 0x08, 0x6C, 0xE9, 0x22, 0x92, 0x04, 0x03, 0xFF, 
	0x9E, 0x3A, 0xEC, 0x05, 0x85, 0xC4, 0x44, 0x65, 0x0E, 0x95, 0x12, 0x27, 0x8E, 0x2B, 0x47, 0xF1, 
	0x62, 0xB5, 0x8C, 0xD8, 0x6C, 0xDE, 0x8C, 0xA7, 0x93, 0xC4, 0x96, 0x9E, 0x3F, 0x97, 0x04, 0x25, 
	0x19, 0xD4, 0xDE, 0x83, 0xAF, 0x0D, 0xBE, 0x3E, 0x68, 0x70, 0xF4, 0x28, 0x59, 0xA4, 0x39, 0x92, 
	0x2E, 0x01, 0xB1, 0x23, 0x9A, 0xCC, 0x74, 0x15, 0x07, 0xAA, 0xEA, 0x40, 0x97, 0x2E, 0xD5, 0x00, 
	0x56, 0x23, 0x34, 0x20, 0xD0, 0xE0, 0x67, 0xD6, 0xAE, 0x42, 0xBB, 0xD6, 0x7B, 0xE0, 0x55, 0xEC, 
	0x59, 0xB3, 0x66, 0x57, 0xB2, 0x54, 0xF0, 0xAF, 0x6D, 0x3A, 0x75, 0x15, 0xE2, 0x6A, 0xD4, 0x23, 
	0xB5, 0xA3, 0x47, 0x9D, 0x01, 0xF4, 0xEE, 0x25, 0xC1, 0x97, 0x44, 0x57, 0xBF, 0x25, 0x87, 0x7E, 
	0xD5, 0x31, 0xF6, 0x6B, 0x8E, 0xB1, 0x65, 0x8D, 0x8A, 0x30, 0xBB, 0x7A, 0x9C, 0x02, 0x11, 0x8C, 
	0xCB, 0x39, 0x96, 0xB9, 0x22, 0xF2, 0xCC, 0x45, 0x0D, 0x34, 0x34, 0xA8, 0xDB, 0x40, 0x2F, 0x2F, 
	0x56, 0x04, 0x38, 0x07, 0xE7, 0x1B, 0xD4, 0x73, 0x60, 0xC1, 0xA4, 0x09, 0x87, 0x35, 0x9C, 0xDA, 
	0x68, 0x59, 0x7D, 0x0D, 0x14, 0xEC, 0x90, 0xCE, 0xD8, 0x31, 0x53, 0x6A, 0x15, 0x2B, 0x5C, 0xA3, 
	0xAC, 0xDB, 0xAE, 0xA3, 0x08, 0x7A, 0x75, 0x2E, 0x21, 0x40, 0x5E, 0xF8, 0xDE, 0x7A, 0x0C, 0x8E, 
	0x8B, 0x1E, 0x8D, 0x5A, 0x2C, 0xEA, 0xD4, 0x64, 0x57, 0xAB, 0x8C, 0xAE, 0xA0, 0x3E, 0xEC, 0x25, 
	0x6D, 0xDB, 0x36, 0xD0, 0x5E, 0xB1, 0x5A, 0x8F, 0xC9, 0x57, 0x34, 0xF0, 0xC1, 0x36, 0x1D, 0x81, 
	0x17, 0x41, 0x07, 0xBE, 0xED, 0x55, 0x5E, 0x00, 0xC1, 0xED, 0xC5, 0x00, 0x71, 0xA1, 0x75, 0x45, 
	0x98, 0x0E, 0x0B, 0x28, 0x67, 0x1A, 0x58, 0xCD, 0x19, 0xC5, 0x04, 0x63, 0xB0, 0xC1, 0xF6, 0xFF, 
	0x5A, 0x6C, 0xE9, 0x54, 0x20, 0x22, 0x6D, 0x0D, 0xFC, 0xA7, 0x0B, 0x81, 0xDF, 0x21, 0x88, 0xE0, 
	0x78, 0x9C, 0x09, 0xC7, 0x17, 0x79, 0x04, 0x3C, 0xD8, 0x84, 0x84, 0x62, 0x05, 0x55, 0x9A, 0x7B, 
	0x87, 0x3D, 0x47, 0x9F, 0x02, 0x39, 0x30, 0x56, 0xDF, 0x74, 0xD4, 0x41, 0xB3, 0x9F, 0x88, 0x23, 
	0x56, 0xD4, 0xC3, 0x6D, 0x8D, 0x7C, 0xA0, 0x5B, 0x6F, 0x07, 0x1A, 0xD8, 0x9B, 0x82, 0x2D, 0xBE, 
	0xC8, 0xC0, 0x83, 0x31, 0xF6, 0x35, 0x65, 0x1B, 0xA3, 0x2D, 0xF7, 0xC0, 0x69, 0xEF, 0x31, 0x81, 
	0xC0, 0x8E, 0xF5, 0xF9, 0xF8, 0xE1, 0x87, 0x4B, 0x54, 0x00, 0x02, 0x91, 0xFC, 0x95, 0x68, 0x62, 
	0x23, 0xF0, 0x80, 0xA7, 0x62, 0x78, 0xE0, 0x29, 0x48, 0x9E, 0x83, 0xE4, 0xC9, 0x38, 0xE5, 0x9D, 
	0x57, 0x2E, 0xA0, 0x27, 0x51, 0x4D, 0x20, 0xE0, 0xE7, 0x97, 0x3C, 0x06, 0xDA, 0x63, 0x03, 0x40, 
	0x02, 0x49, 0xE8, 0x7E, 0x6D, 0x11, 0xB9, 0xC4, 0x91, 0x00, 0x26, 0xA1, 0x4D, 0x6F, 0x08, 0xC6, 
	0x69, 0x60, 0x9C, 0x0D, 0xC6, 0x58, 0xE7, 0x9C, 0x78, 0xE6, 0xA9, 0xE7, 0xA6, 0x0B, 0x34, 0xC0, 
	0xE9, 0x9E, 0x7F, 0x86, 0x0A, 0x66, 0x98, 0xD1, 0x79, 0x48, 0xE8, 0x0E, 0x3B, 0x0C, 0x99, 0xA8, 
	0x76, 0x0D, 0xF0, 0xB0, 0x66, 0x15, 0xB9, 0xBD, 0xB3, 0x4D, 0x93, 0x7B, 0x4D, 0x1A, 0xA7, 0x9C, 
	0x54, 0xF2, 0x95, 0x69, 0x9E, 0x23, 0x2D, 0xD0, 0xAB, 0xA7, 0xEE, 0x55, 0xD8, 0x40, 0xA8, 0x7E, 
	0xD2, 0xD7, 0xE3, 0xB1, 0x3E, 0xA6, 0x8A, 0xEA, 0xB2, 0x43, 0x56, 0x90, 0x6A, 0x91, 0x8D, 0x16, 
	0x24, 0xEB, 0xAC, 0x7A, 0x4D, 0x9A, 0x80, 0x93, 0x31, 0x52, 0x69, 0xE7, 0x48, 0x99, 0xF6, 0xF5, 
	0x69, 0xA7, 0x9F, 0x3E, 0xD0, 0xE9, 0x9F, 0xA7, 0x15, 0x1B, 0xE6, 0xB9, 0x8C, 0x9D, 0x5A, 0xFF, 
	0xC0, 0x0E, 0xEB, 0x3E, 0x3B, 0xA2, 0xA2, 0xD1, 0x16, 0x21, 0x6B, 0x07, 0xB3, 0xD6, 0x6A, 0x6B, 
	0x78, 0x0A, 0x6A, 0x6B, 0x65, 0xB7, 0x9F, 0xA6, 0xD7, 0xA9, 0xB8, 0x5F, 0x55, 0x38, 0x2E, 0x02, 
	0x0F, 0x10, 0x4B, 0x96, 0xA0, 0x4B, 0x7C, 0xB9, 0xEC, 0xC2, 0xEE, 0xEE, 0xD7, 0xC3, 0x88, 0xF1, 
	0x0A, 0x31, 0xAD, 0x0F, 0x6D, 0x5A, 0x1B, 0xA7, 0x0C, 0x73, 0xE6, 0xBB, 0x6F, 0xB7, 0xFE, 0xFA, 
	0xBB, 0xA7, 0xC0, 0x20, 0x8F, 0x45, 0xAC, 0xB9, 0xA4, 0x22, 0x80, 0x82, 0x02, 0x28, 0x34, 0xD0, 
	0x2E, 0xAA, 0xED, 0x96, 0xF9, 0x70, 0x05, 0x0F, 0x47, 0x5C, 0x93, 0x0F, 0xF4, 0x6E, 0x63, 0x00, 
	0x93, 0x11, 0x5C, 0x7B, 0x31, 0x01, 0x09, 0xC0, 0xD8, 0xD7, 0xC6, 0xFC, 0x7E, 0x0B, 0x2E, 0xC0, 
	0x20, 0x0F, 0x1B, 0x6A, 0x0E, 0xE6, 0x2E, 0x71, 0xB2, 0xC9, 0x39, 0xA4, 0xCC, 0x2E, 0xC3, 0xEE, 
	0xC2, 0xAC, 0xA8, 0x3B, 0xD3, 0x36, 0x60, 0xC0, 0xD5, 0x38, 0x13, 0x40, 0x29, 0x79, 0x3D, 0xF7, 
	0xCC, 0xC4, 0xAE, 0x24, 0x6D, 0xDA, 0xAB, 0xAF, 0x24, 0x11, 0x4D, 0xB4, 0xD1, 0x08, 0x20, 0xFD, 
	0xA7, 0xD2, 0x4D, 0x9B, 0x8C, 0x72, 0x75, 0x2C, 0x3F, 0x1D, 0x9D, 0xC3, 0x22, 0xC6, 0x4C, 0x35, 
	0xC5, 0x36, 0x63, 0xAD, 0xF3, 0xD6, 0x3D, 0xC3, 0xF8, 0x22, 0x5F, 0x4D, 0xF0, 0xCA, 0x80, 0x9E, 
	0x63, 0x03, 0x1B, 0xB2, 0xB0, 0x04, 0x17, 0x9B, 0x70, 0x74, 0x27, 0xA3, 0x80, 0x00, 0xCA, 0x26, 
	0xAB, 0xBC, 0xB0, 0xCA, 0x05, 0x14, 0x30, 0xE4, 0xCB, 0xDB, 0x15, 0xA4, 0x1B, 0xC5, 0x56, 0x47, 
	0x70, 0xF5, 0xCD, 0x4E, 0x62, 0xDC, 0x77, 0xD7, 0x7C, 0x71, 0xC0, 0x01, 0x01, 0xA7, 0x9F, 0xDE, 
	0xC6, 0xD8, 0xFF, 0x1E, 0x2E, 0xEE, 0xB0, 0x5E, 0x2E, 0x80, 0xC0, 0xB8, 0x90, 0xA3, 0xDC, 0xFF, 
	0xB4, 0x02, 0x0A, 0xA3, 0x4A, 0xB9, 0x02, 0x96, 0x4B, 0xAD, 0x9D, 0xE5, 0x9B, 0x7C, 0xF0, 0x8E, 
	0xD5, 0x9F, 0x1B, 0xE0, 0xF9, 0x93, 0x18, 0xC3, 0xD8, 0x37, 0x5F, 0xC9, 0xA7, 0x8E, 0x3A, 0xEA, 
	0x7D, 0x71, 0x70, 0x25, 0x9F, 0x51, 0xCC, 0x6E, 0x3D, 0x00, 0xC5, 0xA2, 0xD0, 0x74, 0xE3, 0xB5, 
	0x1F, 0xCA, 0x3B, 0xEF, 0x95, 0x5F, 0xDE, 0x2A, 0x80, 0xB9, 0x6D, 0x4E, 0x7C, 0xF1, 0x58, 0x73, 
	0xCD, 0xB3, 0xFA, 0x09, 0xEC, 0x75, 0xFA, 0xF3, 0xA9, 0x47, 0xCF, 0x80, 0xF4, 0xD2, 0xCF, 0x1F, 
	0xB6, 0xD0, 0xE3, 0xCA, 0x8E, 0xBD, 0xEC, 0x0B, 0x00, 0x30, 0xEC, 0xD2, 0xDA, 0x6B, 0x1C, 0x0A, 
	0x52, 0x56, 0xB9, 0xCA, 0x7D, 0xAF, 0x77, 0xDA, 0xE1, 0x01, 0xF0, 0xE4, 0xA5, 0x81, 0x25, 0x5D, 
	0xCD, 0x73, 0xE8, 0x6B, 0xC0, 0xF2, 0xD6, 0x37, 0x27, 0xF8, 0x39, 0x4F, 0x75, 0xF5, 0xC3, 0x53, 
	0xFD, 0x3C, 0x85, 0xBF, 0x06, 0xEC, 0x6F, 0x76, 0x1F, 0xFC, 0x92, 0xE3, 0x04, 0xE8, 0x27, 0xDC, 
	0x45, 0xA7, 0x00, 0xE0, 0x2B, 0x60, 0x89, 0x12, 0xC8, 0x03, 0x8D, 0x94, 0x4F, 0x78, 0xE8, 0x33, 
	0x9E, 0xDE, 0x24, 0x48, 0xC1, 0xF5, 0x35, 0xC0, 0x79, 0xF0, 0x83, 0xDE, 0x94, 0xE8, 0x67, 0xBF, 
	0xF9, 0x71, 0xB0, 0x83, 0xFC, 0xDB, 0xDF, 0xFE, 0x18, 0x87, 0xBB, 0x00, 0xA6, 0x8B, 0x80, 0x05, 
	0x3C, 0xE0, 0x12, 0x0A, 0xA0, 0x40, 0x17, 0x2A, 0xA9, 0x3B, 0x09, 0x88, 0xE1, 0xCD, 0x12, 0xD0, 
	0x3E, 0x05, 0xF5, 0xAD, 0x01, 0x18, 0xC3, 0xA1, 0xF3, 0x6E, 0x08, 0x36, 0xE9, 0x71, 0x70, 0x02, 
	0x7A, 0x02, 0xE3, 0xA6, 0x86, 0x85, 0x3D, 0xEB, 0x5D, 0x8F, 0x71, 0x8F, 0x3B, 0x59, 0x03, 0x06, 
	0x48, 0xC0, 0x03, 0x16, 0xD0, 0x72, 0x3C, 0x88, 0x63, 0x0B, 0x95, 0xD0, 0xC0, 0x06, 0x84, 0xFF, 
	0xE0, 0x7C, 0x32, 0x9C, 0x22, 0x15, 0xA9, 0x38, 0x1E, 0xAF, 0xE5, 0xF0, 0x7D, 0xAA, 0xDB, 0xA1, 
	0xFD, 0x78, 0xC8, 0x81, 0x1F, 0x7E, 0x0A, 0x8C, 0x9E, 0x02, 0xA1, 0x22, 0x65, 0xB7, 0xC6, 0xB6, 
	0x29, 0x8D, 0x8D, 0x48, 0x44, 0xA1, 0x24, 0x19, 0xC3, 0x44, 0x26, 0x2A, 0x62, 0x09, 0x1A, 0x10, 
	0x9E, 0x0F, 0xEE, 0x68, 0x80, 0x0E, 0x14, 0x4F, 0x82, 0x7B, 0xDC, 0xE3, 0x79, 0xFE, 0x88, 0xBA, 
	0xF8, 0x65, 0x70, 0x57, 0x86, 0x14, 0x9A, 0x07, 0xAD, 0x17, 0x44, 0xEB, 0xAD, 0xF1, 0x91, 0x90, 
	0x6C, 0xE3, 0x1B, 0x2D, 0x07, 0xC7, 0x26, 0x0E, 0x01, 0x93, 0xBA, 0x09, 0x41, 0x0D, 0x38, 0xF9, 
	0x39, 0x50, 0x86, 0x52, 0x94, 0xE3, 0xB9, 0x20, 0x79, 0x02, 0x49, 0xC8, 0x41, 0x7A, 0x4A, 0x8C, 
	0xC8, 0x1C, 0xA3, 0x22, 0x3F, 0xC8, 0x48, 0x58, 0xB2, 0x11, 0x65, 0x03, 0xA4, 0x9C, 0x24, 0x29, 
	0xD7, 0xAA, 0x38, 0x6E, 0x47, 0x40, 0x4B, 0xD0, 0x65, 0x08, 0x7C, 0x50, 0x83, 0xCE, 0xF9, 0xF2, 
	0x5A, 0xBF, 0xC4, 0xA2, 0x0C, 0x98, 0x90, 0x43, 0xBE, 0x6C, 0x00, 0x6C, 0x77, 0x4A, 0xA5, 0x2A, 
	0xFD, 0xC4, 0xBF, 0x2F, 0x2D, 0x61, 0x02, 0x13, 0x58, 0x63, 0x2C, 0x21, 0xA9, 0x32, 0xF0, 0x51, 
	0x4E, 0x05, 0xB4, 0x6C, 0x21, 0x1E, 0xB0, 0xD9, 0x80, 0x1A, 0x6C, 0x72, 0x97, 0x9C, 0xFC, 0xE6, 
	0x1E, 0x73, 0xD6, 0x3E, 0x19, 0x18, 0xD4, 0xA0, 0x4D, 0x30, 0x9D, 0xFC, 0x8A, 0xE9, 0xC5, 0x6F, 
	0x89, 0x71, 0x01, 0x88, 0x6C, 0xE6, 0x3B, 0x27, 0x00, 0x00, 0x8A, 0xCA, 0x53, 0x80, 0x03, 0x84, 
	0x66, 0x3D, 0xA5, 0x19, 0xBE, 0x26, 0x4A, 0x41, 0x40, 0xC2, 0xB3, 0xA3, 0x36, 0xB9, 0x79, 0xC7, 
	0x81, 0xFE, 0x92, 0x8F, 0x07, 0x4D, 0x29, 0x42, 0xDB, 0x90, 0xA9, 0x42, 0x6E, 0xEA, 0xA1, 0xFF, 
	0x90, 0x80, 0x27, 0x3C, 0x2B, 0x4A, 0xD1, 0x78, 0xC6, 0x12, 0x9A, 0x6C, 0x54, 0xD9, 0x3D, 0x55, 
	0xD8, 0xC2, 0x13, 0x7C, 0xF4, 0x03, 0xC2, 0xDB, 0xE5, 0x26, 0x47, 0x2A, 0x50, 0x82, 0xA2, 0x54, 
	0xA5, 0x48, 0xBD, 0xA1, 0xE9, 0x14, 0x4A, 0xBD, 0x25, 0x6C, 0x60, 0x02, 0x1B, 0x88, 0x6A, 0x03, 
	0x64, 0x5A, 0x51, 0x9A, 0x56, 0x35, 0x9E, 0x38, 0x9D, 0x67, 0x24, 0x0B, 0x88, 0xCF, 0x25, 0xF8, 
	0xB4, 0x26, 0x40, 0xB5, 0xE3, 0x3F, 0x37, 0xB9, 0x0D, 0x09, 0x08, 0x34, 0x94, 0xE2, 0x44, 0x6A, 
	0x4A, 0x95, 0xBA, 0xD4, 0xB6, 0x2A, 0xF4, 0xA9, 0x51, 0x85, 0x2A, 0x5C, 0xE1, 0x3A, 0x55, 0xAB, 
	0xD6, 0xB4, 0xA6, 0xFE, 0xD3, 0x2A, 0x4E, 0x55, 0x86, 0xCF, 0x59, 0x7A, 0xD5, 0xA7, 0x20, 0xCD, 
	0x26, 0x40, 0x77, 0x49, 0xBC, 0x25, 0x84, 0x32, 0x67, 0x69, 0x55, 0x2B, 0x42, 0xDD, 0xCA, 0xD8, 
	0x42, 0x46, 0xF5, 0xB1, 0x72, 0x8D, 0xEC, 0x06, 0xEA, 0x3A, 0xD3, 0xBB, 0x56, 0x54, 0x9E, 0xD0, 
	0xCC, 0xEA, 0x12, 0x2B, 0xD7, 0x57, 0x15, 0x9E, 0x00, 0xB0, 0xC2, 0xCB, 0x25, 0x59, 0x43, 0xC0, 
	0x49, 0x09, 0xE8, 0x0D, 0xAD, 0x8A, 0x5D, 0x6B, 0x63, 0xDD, 0x3A, 0x55, 0xB8, 0x4A, 0x76, 0xAE, 
	0x1E, 0xAC, 0xAC, 0x5D, 0x2F, 0xAB, 0xD5, 0x47, 0x72, 0x94, 0xB3, 0x96, 0xFB, 0xEC, 0x12, 0x42, 
	0x9B, 0x4B, 0x6D, 0x36, 0xC0, 0xB4, 0xC0, 0xBD, 0x99, 0x2F, 0x13, 0xAB, 0x58, 0xB6, 0xAE, 0xD6, 
	0xB1, 0x50, 0x7D, 0x6D, 0x72, 0x27, 0x2B, 0xDB, 0xE6, 0xE6, 0x95, 0x9E, 0x17, 0x4D, 0x59, 0x67, 
	0xFB, 0xFA, 0xD7, 0xDD, 0xE6, 0x06, 0x02, 0x62, 0xDD, 0xA6, 0xD5, 0x82, 0x6B, 0xDA, 0x29, 0x12, 
	0xB7, 0xB8, 0xC7, 0x5D, 0x6A, 0x03, 0xE2, 0x4A, 0x5E, 0xB9, 0xC6, 0x95, 0xB2, 0x57, 0xC5, 0xFF, 
	0xAB, 0x4D, 0x1B, 0x17, 0xDD, 0x68, 0xCE, 0xD2, 0x72, 0x2A, 0x68, 0x00, 0x60, 0x6B, 0x52, 0xBE, 
	0xDE, 0xFE, 0xF6, 0x6A, 0xDC, 0xC5, 0xDA, 0x12, 0x52, 0x7B, 0x50, 0xE3, 0xAE, 0x76, 0xBC, 0xCB, 
	0x8D, 0xEC, 0x72, 0x63, 0x7B, 0xD5, 0xD9, 0xDA, 0xD4, 0x99, 0x00, 0xC8, 0x40, 0x82, 0xF9, 0x1A, 
	0x3E, 0x15, 0xC4, 0x77, 0xBE, 0x98, 0x84, 0x40, 0x1D, 0xBB, 0x69, 0x00, 0x09, 0xE4, 0xD7, 0xC2, 
	0xDE, 0xDD, 0xAF, 0x5A, 0xFD, 0xFB, 0x5F, 0xF3, 0x2A, 0x57, 0xAA, 0xEA, 0x35, 0x30, 0x13, 0xB4, 
	0x9A, 0x53, 0x15, 0x3A, 0xF8, 0xC1, 0xFB, 0x6C, 0xA0, 0x8A, 0xED, 0x78, 0xDF, 0x0A, 0xBB, 0xB8, 
	0xBB, 0x66, 0x45, 0xAB, 0x86, 0xFB, 0x1B, 0x5E, 0xF1, 0xBA, 0x96, 0xBC, 0x73, 0x85, 0x2A, 0x7A, 
	0x2B, 0xCB, 0x84, 0xF5, 0xA2, 0x20, 0xC1, 0x00, 0x88, 0x25, 0x35, 0x4F, 0xEC, 0xE0, 0x6B, 0x6A, 
	0x40, 0xC2, 0x47, 0xD6, 0x0D, 0x7E, 0x97, 0x1C, 0x5C, 0x50, 0xBA, 0x20, 0x01, 0x4F, 0xE6, 0xA3, 
	0x86, 0x39, 0xDC, 0xD8, 0xF1, 0xE6, 0xF8, 0xCA, 0x3A, 0x96, 0x6D, 0x8F, 0x65, 0xDA, 0x80, 0x0C, 
	0x90, 0x38, 0xA7, 0x0D, 0x20, 0x72, 0x91, 0x53, 0x9C, 0xE4, 0x3A, 0x02, 0x17, 0xC6, 0x2F, 0x16, 
	0x68, 0x94, 0x81, 0x99, 0xD0, 0xE3, 0x02, 0xF8, 0xC6, 0x57, 0x9E, 0xEC, 0x65, 0x27, 0x2A, 0x53, 
	0x2E, 0xC7, 0x52, 0xC1, 0xF4, 0x14, 0xF3, 0x98, 0x25, 0xF6, 0x42, 0x09, 0x2F, 0x01, 0xC6, 0xF9, 
	0x95, 0xE0, 0x93, 0xD7, 0x3C, 0x68, 0x51, 0xC6, 0xE0, 0xD0, 0x6D, 0x10, 0xAF, 0x79, 0xC9, 0x4B, 
	0x94, 0x3A, 0xD7, 0xF9, 0x06, 0xF0, 0x8C, 0x6E, 0x90, 0x07, 0x48, 0x83, 0x68, 0xEA, 0x39, 0xBE, 
	0xFB, 0xE4, 0x6D, 0x1D, 0xEF, 0xD8, 0xE2, 0xE0, 0xEA, 0x31, 0x94, 0x6B, 0x86, 0x72, 0xFB, 0xFF, 
	0x0E, 0x4D, 0xEA, 0x52, 0xC7, 0xA0, 0xA9, 0x1E, 0x48, 0xB5, 0xAA, 0xA7, 0xFA, 0x68, 0x78, 0x42, 
	0xDA, 0xCE, 0x09, 0x9E, 0x27, 0x13, 0x2E, 0x7D, 0xCD, 0xD0, 0x4A, 0x18, 0xBB, 0x21, 0xC0, 0x41, 
	0x69, 0x5F, 0x1C, 0xE3, 0x42, 0x47, 0xB9, 0xD0, 0x0D, 0x30, 0xB5, 0xB0, 0x49, 0xDD, 0x00, 0x55, 
	0x1B, 0xDB, 0x03, 0x37, 0x40, 0xB6, 0xB2, 0x59, 0xFD, 0x6A, 0x57, 0xCB, 0x14, 0xD2, 0x17, 0x9D, 
	0x74, 0xCA, 0xBA, 0x4C, 0x83, 0x0C, 0x84, 0x99, 0xC8, 0x72, 0x09, 0xEC, 0x8A, 0x49, 0x9B, 0xEB, 
	0x25, 0x2C, 0x59, 0xCD, 0x27, 0x1D, 0xF5, 0xB0, 0x85, 0x5D, 0xEC, 0x64, 0x9B, 0x5B, 0xD5, 0xC9, 
	0x4E, 0xF5, 0x0D, 0x58, 0xED, 0x6C, 0x99, 0xD2, 0xC0, 0xCE, 0x90, 0xB4, 0x76, 0x06, 0xAA, 0x3D, 
	0xEF, 0x6B, 0x17, 0x79, 0x00, 0x99, 0xD3, 0xB6, 0x1D, 0x75, 0xAD, 0x6B, 0xD2, 0x7A, 0x7B, 0x8A, 
	0x85, 0xA6, 0xA2, 0xAF, 0xC5, 0x3D, 0xEE, 0x52, 0x37, 0xE0, 0xDC, 0xEA, 0x56, 0xF6, 0xB9, 0xA7, 
	0xDA, 0x6C, 0x47, 0x37, 0xFB, 0xA2, 0x4B, 0xA0, 0x37, 0xBD, 0xED, 0xDD, 0x80, 0x01, 0xE0, 0xFB, 
	0x92, 0x61, 0xD5, 0x4D, 0xBF, 0x49, 0xCB, 0xEF, 0x3B, 0xC6, 0xF8, 0x97, 0x01, 0xE7, 0x63, 0xC1, 
	0x4D, 0x5D, 0xEC, 0x84, 0x23, 0x5C, 0xD9, 0x1E, 0x60, 0x35, 0x0D, 0x36, 0xB0, 0xF2, 0x77, 0x37, 
	0x1B, 0xDA, 0xD3, 0xA6, 0xF7, 0xBC, 0x33, 0x50, 0xEF, 0x13, 0x77, 0x19, 0xDF, 0x4A, 0x00, 0x6A, 
	0x48, 0x73, 0xCD, 0xF3, 0x9E, 0x73, 0x5A, 0xD0, 0xA2, 0x06, 0x35, 0x94, 0x83, 0x3D, 0x72, 0x62, 
	0xA3, 0x3C, 0xDD, 0x0A, 0x4F, 0x38, 0xBB, 0x65, 0xCA, 0x72, 0x47, 0x2F, 0x61, 0xE6, 0x50, 0x9F, 
	0xF8, 0xBD, 0x07, 0x60, 0xED, 0x9C, 0xEB, 0x5C, 0xA4, 0x1D, 0xE7, 0xB9, 0xAE, 0x7F, 0xCB, 0xFF, 
	0x04, 0x51, 0x87, 0x9A, 0xE8, 0x45, 0x47, 0x74, 0xC2, 0x93, 0x8E, 0xF0, 0x75, 0x3B, 0xBB, 0xE1, 
	0xEB, 0x66, 0x42, 0xD4, 0xD7, 0x4E, 0x83, 0x25, 0x50, 0xDD, 0xE2, 0xB8, 0xD8, 0x6D, 0x58, 0xB3, 
	0xEE, 0xF3, 0xDF, 0x5A, 0xD8, 0xC2, 0x5D, 0x07, 0x76, 0xD8, 0x89, 0x9D, 0xEE, 0xBE, 0xA3, 0x1C, 
	0xE5, 0x4B, 0xE7, 0xF2, 0x12, 0x0E, 0x40, 0xF8, 0x06, 0x48, 0x9C, 0xE6, 0x34, 0x97, 0x79, 0xC5, 
	0x2D, 0x9E, 0x01, 0x9C, 0xDF, 0x12, 0xA4, 0x3B, 0xD7, 0x3A, 0xC7, 0x3D, 0x3E, 0x02, 0x09, 0x54, 
	0xFE, 0xEE, 0x5D, 0x27, 0x78, 0xD8, 0x4B, 0xEE, 0x77, 0x93, 0xAB, 0x9B, 0xE1, 0x91, 0x1E, 0x3C, 
	0xE1, 0x47, 0xDF, 0x65, 0xB6, 0x27, 0xDE, 0xDA, 0x54, 0x6F, 0xBC, 0xE3, 0x1F, 0x1F, 0x56, 0xEC, 
	0x62, 0x5D, 0xEB, 0x76, 0xBF, 0xFB, 0xE5, 0x31, 0x0F, 0x85, 0xCD, 0x1F, 0x9D, 0xEC, 0x4A, 0x8F, 
	0xA7, 0xE8, 0x09, 0x7F, 0x83, 0x03, 0xF4, 0x7E, 0xDD, 0x6B, 0x3F, 0x3D, 0x13, 0x2C, 0x4E, 0xF5, 
	0xC9, 0xC8, 0xBD, 0x01, 0x10, 0x48, 0xFE, 0xEB, 0xFD, 0x6D, 0xE1, 0xD9, 0x5B, 0xFE, 0xF9, 0x78, 
	0x77, 0x81, 0xF4, 0x21, 0x61, 0xF0, 0xB1, 0x7F, 0x1E, 0x12, 0xA3, 0xF7, 0x7D, 0xF6, 0xB3, 0x4F, 
	0xED, 0xD3, 0x57, 0xBB, 0xE2, 0x07, 0xB8, 0xB9, 0xB5, 0x8D, 0x6F, 0x5D, 0x08, 0xD4, 0x40, 0xF9, 
	0xFD, 0xDE, 0xBA, 0xF3, 0x9D, 0xFF, 0xFC, 0x06, 0x48, 0xFF, 0xFD, 0xF0, 0x9F, 0x7E, 0x53, 0x5F, 
	0x40, 0xFF, 0xFA, 0xBF, 0xA0, 0x01, 0xA3, 0xEF, 0xBD, 0xF6, 0x7F, 0xAF, 0x7D, 0xC3, 0x9F, 0x7E, 
	0xFC, 0x07, 0x00, 0x77, 0x8D, 0xD7, 0x00, 0x26, 0x40, 0x3E, 0xD8, 0x94, 0x7C, 0xE6, 0xE7, 0x7A, 
	0x12, 0xE0, 0x71, 0xB2, 0x77, 0x77, 0xD0, 0x17, 0x7D, 0xF1, 0x17, 0x81, 0xEF, 0xD7, 0x00, 0xFF, 
	0xF6, 0x57, 0x81, 0x16, 0x88, 0x7F, 0xDA, 0xB7, 0x7D, 0xDB, 0x57, 0x7A, 0xDF, 0x87, 0x7F, 0x16, 
	0x47, 0x03, 0xF8, 0x06, 0x77, 0x8D, 0xD2, 0x04, 0x09, 0x88, 0x80, 0xB8, 0x66, 0x77, 0xB3, 0x57, 
	0x79, 0x97, 0x77, 0x79, 0xEE, 0xE7, 0x02, 0x23, 0x20, 0x7D, 0x2F, 0x08, 0x83, 0xF1, 0x47, 0x81, 
	0x2F, 0xE0, 0x01, 0xF4, 0x67, 0x83, 0x35, 0x98, 0x83, 0x36, 0x98, 0x72, 0x19, 0xC8, 0x7F, 0xFC, 
	0xE7, 0x7B, 0xDD, 0x07, 0x7E, 0xC4, 0x17, 0x7E, 0x70, 0xB7, 0x7A, 0xF2, 0xC2, 0x04, 0x08, 0x78, 
	0x7E, 0xE7, 0xE7, 0x7A, 0xB1, 0xC7, 0x7E, 0xB2, 0xD7, 0x82, 0xF1, 0x17, 0x83, 0x2E, 0x38, 0x81, 
	0xF5, 0x87, 0x83, 0x39, 0x78, 0x83, 0xF5, 0x87, 0x81, 0x1A, 0x98, 0x7F, 0x85, 0x37, 0x7E, 0x03, 
	0x10, 0x80, 0x07, 0x00, 0x82, 0x21, 0x58, 0x71, 0x05, 0xE8, 0x28, 0x4B, 0x50, 0x82, 0x49, 0x88, 
	0x5D, 0x66, 0xF5, 0x67, 0xCD, 0xD7, 0x86, 0x10, 0x28, 0x83, 0x12, 0x38, 0x7D, 0x57, 0x38, 0x87, 
	0x56, 0x88, 0x83, 0x18, 0xE8, 0x83, 0x19, 0x58, 0x78, 0x42, 0x28, 0x86, 0x5F, 0x78, 0x71, 0x26, 
	0xC0, 0x02, 0x11, 0x83, 0x84, 0x09, 0x78, 0x64, 0xE8, 0x47, 0x7B, 0xB1, 0xD7, 0x86, 0x2D, 0x28, 
	0x85, 0xEF, 0x27, 0x85, 0x2F, 0x48, 0x83, 0x74, 0x58, 0x85, 0x55, 0x88, 0x7F, 0xFA, 0xA7, 0x81, 
	0x4D, 0xD0, 0x87, 0x16, 0x07, 0x86, 0x70, 0x67, 0x02, 0x03, 0x50, 0x86, 0x54, 0x73, 0x86, 0x26, 
	0x78, 0x64, 0x76, 0x87, 0x03, 0x0E, 0xC8, 0x04, 0xA3, 0x18, 0x85, 0x71, 0xE8, 0x02, 0x34, 0x88, 
	0x83, 0x75, 0x08, 0x89, 0xF7, 0xC7, 0x85, 0xBD, 0xC7, 0x04, 0x07, 0x00, 0x03, 0x44, 0x08, 0x82, 
	0x01, 0x38, 0x00, 0x62, 0x48, 0x84, 0x9C, 0x48, 0x05, 0xD6, 0x85, 0x7C, 0x4B, 0xD8, 0x4D, 0xFF, 
	0xA2, 0x28, 0x01, 0xA2, 0xF8, 0x8B, 0x78, 0xF7, 0x67, 0x50, 0xC8, 0x88, 0x8B, 0x48, 0x85, 0xAC, 
	0xB8, 0x8A, 0x59, 0x98, 0x81, 0xB0, 0x08, 0x03, 0xB2, 0x28, 0x8B, 0x1E, 0x78, 0x89, 0xD2, 0xB8, 
	0x04, 0x80, 0xD8, 0x08, 0x81, 0x65, 0x82, 0x6A, 0x28, 0x8C, 0xC1, 0x38, 0x8A, 0x4D, 0x70, 0x8A, 
	0x13, 0xB8, 0x83, 0xAC, 0xF8, 0x88, 0xF8, 0xC7, 0x04, 0xCF, 0x28, 0x8B, 0xB1, 0x18, 0x8B, 0xE0, 
	0x77, 0x8B, 0x17, 0x47, 0x80, 0x99, 0x03, 0x2B, 0x02, 0x52, 0x02, 0x21, 0x75, 0x64, 0xBE, 0x68, 
	0x61, 0xBF, 0x58, 0x8F, 0xC0, 0x68, 0x56, 0x23, 0x90, 0x8F, 0x90, 0x20, 0x83, 0x8E, 0x78, 0x85, 
	0x36, 0x48, 0x14, 0xCF, 0x78, 0x8E, 0xB1, 0x18, 0x90, 0x8B, 0xD7, 0x87, 0x6D, 0xC7, 0x02, 0x80, 
	0x68, 0x84, 0xD9, 0x00, 0x8F, 0x1F, 0x00, 0x8F, 0xC8, 0xA7, 0x86, 0xF7, 0x18, 0x91, 0xDA, 0xD8, 
	0x00, 0x02, 0x30, 0x02, 0x15, 0x99, 0x8F, 0x18, 0xA9, 0x8F, 0x4D, 0xD5, 0x00, 0xCE, 0xD8, 0x91, 
	0xCE, 0xC8, 0x91, 0xE7, 0x58, 0x8E, 0x21, 0x19, 0x7E, 0x98, 0x48, 0x8D, 0x08, 0xD9, 0x8E, 0x8D, 
	0xC0, 0x90, 0x0E, 0x09, 0x91, 0x12, 0x49, 0x8F, 0x98, 0x67, 0x91, 0x30, 0x59, 0x91, 0x32, 0x99, 
	0x91, 0x23, 0xC0, 0x91, 0x1E, 0xD9, 0x91, 0x02, 0x00, 0x03, 0x02, 0x90, 0x93, 0x3B, 0x09, 0x03, 
	0xF8, 0x17, 0x90, 0x40, 0x39, 0x90, 0x86, 0x77, 0x71, 0x08, 0x49, 0x01, 0x80, 0x88, 0x92, 0x01, 
	0xD2, 0x90, 0x40, 0xE5, 0x90, 0xBF, 0x65, 0x8F, 0x2D, 0x89, 0x03, 0x14, 0x99, 0x8F, 0x0E, 0x50, 
	0x91, 0x0E, 0x20, 0x95, 0x31, 0x59, 0x93, 0x3A, 0xC9, 0x93, 0x3C, 0xA9, 0x93, 0xCE, 0x98, 0x93, 
	0x3A, 0x09, 0x92, 0x41, 0x19, 0x90, 0x44, 0xD8, 0x00, 0x45, 0x89, 0x90, 0x47, 0x49, 0x19, 0xFF, 
	0xC2, 0xC3, 0x90, 0x21, 0xB5, 0x04, 0x4E, 0x29, 0x8C, 0x0D, 0x30, 0x02, 0x53, 0x99, 0x8F, 0x33, 
	0x49, 0x95, 0x17, 0x49, 0x91, 0x5D, 0x79, 0x97, 0x5E, 0x99, 0x97, 0x3E, 0x39, 0x92, 0x61, 0xC9, 
	0x04, 0x66, 0x59, 0x96, 0x32, 0xA3, 0x8B, 0x20, 0x05, 0x8F, 0x25, 0x80, 0x7C, 0x33, 0x40, 0x8A, 
	0x2D, 0xF9, 0x96, 0x17, 0x29, 0x00, 0x12, 0x70, 0x91, 0x30, 0x69, 0x91, 0x0E, 0xC0, 0x91, 0x7A, 
	0xC9, 0x95, 0x93, 0x29, 0x00, 0x60, 0xC9, 0x97, 0x4C, 0x40, 0x01, 0x64, 0xC9, 0x02, 0x46, 0xC9, 
	0x99, 0xD5, 0x88, 0x1B, 0x4C, 0xA0, 0x94, 0x0D, 0x30, 0x03, 0x10, 0x40, 0x9A, 0x87, 0xF9, 0x67, 
	0xC1, 0xF8, 0x96, 0x31, 0x29, 0x93, 0x55, 0xF9, 0x98, 0x96, 0x79, 0x97, 0x5C, 0x49, 0x99, 0xB0, 
	0xF9, 0x93, 0xE7, 0xD8, 0x04, 0x14, 0x70, 0x9B, 0x9B, 0xD9, 0x99, 0x9A, 0x89, 0x94, 0xBA, 0xB0, 
	0x04, 0x0C, 0xF9, 0x90, 0xA5, 0x99, 0x7C, 0xA4, 0x49, 0x8A, 0x8A, 0x69, 0x95, 0x72, 0x69, 0x91, 
	0x8D, 0xA9, 0x8F, 0xB2, 0xB9, 0x95, 0xCC, 0xF9, 0x95, 0xE8, 0xB8, 0x04, 0xB7, 0x19, 0x9D, 0xB8, 
	0xD9, 0x99, 0x27, 0x99, 0x8B, 0x8C, 0xD0, 0x04, 0x2B, 0x89, 0x01, 0x10, 0x50, 0x02, 0x33, 0xC0, 
	0x9D, 0xC1, 0x09, 0x05, 0x30, 0x19, 0x97, 0x72, 0x29, 0x93, 0x76, 0xD9, 0x9C, 0x7A, 0x19, 0x05, 
	0xD2, 0x99, 0x9E, 0x9A, 0x69, 0x96, 0x4B, 0x60, 0x9D, 0xD7, 0xD9, 0x04, 0xC2, 0x43, 0x9A, 0xC1, 
	0x29, 0x9C, 0xDE, 0xD9, 0x00, 0x0E, 0x30, 0x03, 0x91, 0xD9, 0x06, 0x56, 0x29, 0x99, 0x1F, 0x09, 
	0x09, 0x0E, 0x40, 0x01, 0x91, 0xA9, 0x9E, 0xE9, 0xB9, 0x99, 0xED, 0xE9, 0x9E, 0x99, 0x10, 0x05, 
	0xF4, 0x29, 0x9F, 0xC1, 0x79, 0x98, 0x0E, 0x70, 0x9F, 0x0D, 0x8A, 0x9F, 0x0E, 0x9A, 0x9F, 0xFF, 
	0x6D, 0x00, 0xA0, 0x00, 0xDA, 0xA0, 0x14, 0xFA, 0x9F, 0x01, 0x2A, 0xA0, 0xD1, 0x49, 0x96, 0x04, 
	0x68, 0x02, 0x1E, 0x1A, 0x98, 0xC8, 0xD0, 0x06, 0xC1, 0xA9, 0x9D, 0xC9, 0x57, 0x98, 0x11, 0x0A, 
	0xA1, 0x28, 0x1A, 0xA1, 0xF6, 0x59, 0xA1, 0x18, 0x7A, 0x9B, 0x16, 0xDA, 0xA2, 0x0D, 0xA0, 0xA1, 
	0xB8, 0x59, 0xA0, 0x1F, 0x4A, 0x13, 0x50, 0x50, 0x02, 0x85, 0x09, 0x9C, 0xDA, 0x39, 0x9C, 0x0F, 
	0xDA, 0xA3, 0x0E, 0x8A, 0xA2, 0x31, 0x6A, 0xA1, 0x17, 0xFA, 0x9F, 0x43, 0x1A, 0xA3, 0x02, 0xDA, 
	0x04, 0x1E, 0xFA, 0xA1, 0x20, 0x4A, 0x19, 0xD8, 0xD9, 0x90, 0x48, 0xD8, 0x9D, 0x0C, 0x8A, 0x9F, 
	0x29, 0x7A, 0xA2, 0x91, 0x49, 0xA4, 0x2F, 0x0A, 0xA0, 0x02, 0xD0, 0xA2, 0x9A, 0x39, 0xA0, 0x4B, 
	0x90, 0x02, 0x26, 0x90, 0x02, 0x1D, 0x2A, 0x17, 0x34, 0xF1, 0x78, 0xBE, 0x89, 0xA3, 0x38, 0x8A, 
	0x84, 0xF6, 0xF9, 0xA3, 0x3E, 0x0A, 0xA4, 0x58, 0x8A, 0xA1, 0x59, 0x7A, 0xA1, 0x59, 0x9A, 0xA1, 
	0x4C, 0x90, 0x02, 0x74, 0xFA, 0xA5, 0x61, 0xCA, 0x9B, 0x36, 0xCA, 0x04, 0x84, 0xD9, 0x90, 0x67, 
	0xEA, 0x9F, 0x54, 0xBA, 0xA2, 0x5A, 0x2A, 0xA1, 0x6D, 0xE0, 0xA5, 0x84, 0x6A, 0xA7, 0x05, 0x88, 
	0xA7, 0x63, 0xCA, 0x67, 0x7A, 0x8A, 0xA3, 0x7C, 0xDA, 0x00, 0x18, 0xF0, 0xA8, 0x90, 0xBA, 0x91, 
	0x73, 0xFA, 0xA8, 0x29, 0xF0, 0xA8, 0x0D, 0x40, 0xA7, 0x84, 0x8A, 0xA9, 0x74, 0x2A, 0xA6, 0x89, 
	0xDA, 0x89, 0x8B, 0x2A, 0x3C, 0x18, 0x50, 0x02, 0xA1, 0x3A, 0xAA, 0x90, 0x1A, 0xA9, 0x94, 0x0A, 
	0xA9, 0x95, 0x7A, 0xAA, 0x18, 0x00, 0xA6, 0x5F, 0xFA, 0xA5, 0x84, 0x7A, 0x06, 0x9D, 0x9A, 0x0D, 
	0x6D, 0x50, 0x98, 0xA3, 0x2A, 0xAA, 0xB5, 0x3A, 0xAA, 0x97, 0x7A, 0xAA, 0x95, 0xBA, 0xAB, 0x50, 
	0xAB, 0x6A, 0xA9, 0x98, 0x7A, 0xA8, 0x0D, 0x90, 0xA3, 0xB1, 0x9A, 0x06, 0x7C, 0xF2, 0xA8, 0xA2, 
	0x7A, 0xAC, 0xC6, 0xEA, 0xA8, 0xBD, 0x5A, 0xAA, 0xA5, 0x0A, 0xA6, 0x5E, 0xAA, 0xA7, 0x9C, 0x3A, 
	0xAC, 0xC4, 0x7A, 0xA3, 0x6D, 0x80, 0xAB, 0xCB, 0xBA, 0xAB, 0x44, 0x91, 0xA3, 0x4B, 0x2A, 0xAD, 
	0x9E, 0x1A, 0xAC, 0x66, 0x6A, 0xA6, 0xD4, 0xF3, 0xAD, 0x85, 0xA9, 0xAD, 0xDC, 0x1A, 0x15, 0x51, 
	0x20, 0xAE, 0xE3, 0x2A, 0xAE, 0x6D, 0x50, 0xAE, 0xD2, 0x2A, 0xA9, 0xD1, 0x9A, 0xA8, 0x41, 0x00, 
	0x00, 0x21, 0xF9, 0x04, 0x04, 0x0A, 0x00, 0x00, 0x00, 0x2C, 0x00, 0x00, 0x10, 0x00, 0x64, 0x00, 
	0x63, 0x00, 0x00, 0x06, 0xFF, 0xC0, 0x96, 0x70, 0x48, 0x2C, 0x1A, 0x8F, 0x48, 0x63, 0xA0, 0xC1, 
	0x6C, 0xDA, 0x9A, 0x8D, 0xC0, 0x32, 0x49, 0xAD, 0x5A, 0xAF, 0xD8, 0x2A, 0x94, 0x49, 0xB2, 0x90, 
	0x1A, 0x16, 0xCB, 0xF3, 0x62, 0xB9, 0x80, 0xCD, 0xDB, 0xAC, 0x7A, 0xCD, 0x2E, 0x6E, 0xBF, 0xCC, 
	0x0B, 0x89, 0xCC, 0x7C, 0x42, 0x2F, 0xF8, 0xFC, 0xB6, 0xD1, 0xEE, 0xFB, 0xDD, 0x6F, 0x24, 0x82, 
	0x70, 0x4C, 0x61, 0x36, 0x61, 0x88, 0x71, 0x68, 0x7A, 0x78, 0x4D, 0x7F, 0x8F, 0x58, 0x81, 0x82, 
	0x7B, 0x88, 0x86, 0x95, 0x89, 0x79, 0x99, 0x0D, 0x9A, 0x90, 0x9D, 0x47, 0x4D, 0x52, 0x0D, 0x82, 
	0x5E, 0x83, 0x60, 0x97, 0x96, 0xA7, 0x16, 0x71, 0x7B, 0x8A, 0x4C, 0x9E, 0xAF, 0x51, 0xA1, 0xAC, 
	0x85, 0xA9, 0x62, 0xB5, 0x61, 0x4D, 0x99, 0x66, 0x68, 0x9B, 0xAF, 0x8F, 0x4C, 0x4B, 0x5C, 0x83, 
	0xC3, 0xB4, 0xA9, 0x87, 0xB7, 0x9B, 0xBC, 0xAB, 0x17, 0x2B, 0xBC, 0xBE, 0x6C, 0x5B, 0xA1, 0xC3, 
	0x84, 0x5F, 0xB7, 0xD6, 0xAA, 0xC9, 0x78, 0x22, 0xCA, 0xCD, 0xCD, 0x7C, 0xCF, 0x91, 0xC0, 0x52, 
	0x4B, 0x93, 0x4D, 0xC4, 0xD7, 0xC8, 0x22, 0x4D, 0xDB, 0x20, 0xDB, 0x0D, 0x20, 0x4D, 0xDE, 0xDF, 
	0xE0, 0x49, 0xE2, 0xE3, 0xC1, 0xA2, 0xC4, 0x5C, 0xA6, 0xE8, 0x97, 0x0D, 0x22, 0x00, 0xD5, 0xE5, 
	0x52, 0x07, 0xA2, 0x60, 0x1A, 0x7A, 0x46, 0x98, 0x74, 0x88, 0x25, 0x2B, 0x5F, 0xA9, 0x7C, 0x3A, 
	0xAA, 0xF5, 0x4B, 0xF4, 0x4F, 0x5D, 0xC0, 0x80, 0x4D, 0x0A, 0x6A, 0x84, 0xE7, 0x08, 0xA1, 0x10, 
	0x85, 0xB1, 0x40, 0x49, 0x33, 0x47, 0x42, 0x47, 0xC4, 0x06, 0x3A, 0x9A, 0xF4, 0x6B, 0x90, 0xA3, 
	0xA2, 0xC5, 0x8B, 0xEA, 0x76, 0xC0, 0xDB, 0x08, 0x6F, 0xC5, 0x8A, 0x79, 0xCF, 0x14, 0x2E, 0x0C, 
	0x76, 0x8F, 0x40, 0xB0, 0xFF, 0x41, 0x11, 0xE1, 0x98, 0x44, 0x99, 0x72, 0x16, 0x94, 0x1C, 0x48, 
	0x5D, 0x32, 0xB9, 0xD8, 0x60, 0x87, 0xCC, 0x77, 0x1A, 0x1B, 0xD8, 0xBC, 0x09, 0xAE, 0x41, 0x87, 
	0xAB, 0x0B, 0x63, 0xF9, 0xF4, 0xB9, 0x84, 0x40, 0x3E, 0x06, 0x93, 0x4C, 0xA6, 0x14, 0xDB, 0xE0, 
	0xC1, 0x03, 0x1D, 0x0F, 0xCA, 0xA6, 0x45, 0xDA, 0x32, 0x87, 0x88, 0xA4, 0x22, 0x14, 0x00, 0x5C, 
	0xDA, 0xB4, 0x9D, 0x53, 0x26, 0x51, 0x6F, 0xBA, 0x82, 0x65, 0xF5, 0x2A, 0xB0, 0x08, 0x24, 0xC6, 
	0x31, 0x21, 0xE0, 0xB5, 0xE4, 0x17, 0xB0, 0x0C, 0xC6, 0x92, 0x3D, 0x6B, 0x56, 0xED, 0xDA, 0xA4, 
	0x6C, 0x5B, 0xC2, 0xC4, 0xEB, 0x54, 0xE6, 0xD3, 0x77, 0x0D, 0x2A, 0xD8, 0xDC, 0x0B, 0xA9, 0x81, 
	0x86, 0xBE, 0x0D, 0x22, 0x44, 0x21, 0x11, 0x21, 0x80, 0x57, 0xC2, 0x81, 0x0B, 0x9B, 0x24, 0x91, 
	0x98, 0xE8, 0x50, 0xB4, 0x8C, 0xD5, 0xB6, 0x65, 0xCB, 0xF2, 0xAD, 0x64, 0x8C, 0x31, 0x41, 0x38, 
	0x15, 0x51, 0x99, 0xC9, 0x54, 0xBD, 0x9D, 0x3F, 0xFB, 0x8D, 0x40, 0x3C, 0x64, 0x03, 0x02, 0xA9, 
	0x7D, 0xA2, 0x4C, 0xFC, 0x5A, 0x6C, 0x4A, 0xC6, 0x67, 0x1D, 0xB3, 0x8C, 0x5C, 0x1B, 0x6E, 0x4B, 
	0x05, 0x72, 0xEB, 0x3A, 0xD5, 0xAD, 0xDB, 0xF7, 0x0A, 0xCD, 0x9C, 0xDB, 0x78, 0x16, 0x1E, 0x3A, 
	0x42, 0x07, 0xD1, 0x5D, 0xB7, 0xA2, 0x3E, 0xCE, 0x45, 0x31, 0x59, 0xB4, 0x3A, 0x16, 0xA4, 0x35, 
	0x4B, 0x7F, 0x3A, 0x6D, 0xB7, 0x49, 0xFF, 0xC9, 0xC5, 0xFE, 0x4F, 0xF7, 0x0E, 0xDE, 0x99, 0x55, 
	0x20, 0x95, 0x4D, 0x02, 0xE2, 0xB4, 0x46, 0x03, 0x1F, 0x58, 0x15, 0xDA, 0x55, 0xC5, 0x1D, 0xE7, 
	0x13, 0x09, 0xC8, 0x75, 0xC5, 0x00, 0x61, 0x4D, 0x38, 0xF7, 0x9C, 0x59, 0x43, 0x99, 0x95, 0x43, 
	0x63, 0x91, 0x41, 0xD6, 0xFF, 0x84, 0x64, 0xD8, 0xC5, 0xC5, 0x84, 0x65, 0x3B, 0x04, 0x28, 0xE0, 
	0x54, 0x0D, 0xF4, 0x60, 0x60, 0x24, 0x1A, 0x7C, 0x56, 0x1E, 0x71, 0xA2, 0xA1, 0x07, 0xE1, 0x56, 
	0x0D, 0x4C, 0x48, 0xC0, 0x84, 0x35, 0x32, 0x50, 0x61, 0x59, 0xF0, 0x35, 0x46, 0x1F, 0x87, 0x1D, 
	0x32, 0x11, 0x64, 0x5C, 0x44, 0x2A, 0xD0, 0xD4, 0x53, 0x15, 0x80, 0x50, 0x81, 0x80, 0xE0, 0xF5, 
	0xA0, 0x22, 0x34, 0x09, 0x2E, 0x78, 0xDE, 0x42, 0xC5, 0x11, 0x66, 0xDA, 0x17, 0x84, 0xD9, 0x98, 
	0x23, 0x03, 0x5C, 0xEA, 0x08, 0x45, 0x6C, 0x1A, 0xD6, 0x97, 0x5F, 0x03, 0x08, 0xD8, 0x47, 0x1B, 
	0x76, 0xFB, 0x19, 0x59, 0xE2, 0x92, 0x3B, 0x2C, 0x79, 0xE2, 0x4D, 0x4E, 0xAE, 0x48, 0x85, 0x67, 
	0x0D, 0xF8, 0xF0, 0xE2, 0x4E, 0x30, 0x1E, 0x07, 0xA1, 0x83, 0x36, 0xEA, 0x88, 0x18, 0x6B, 0x3A, 
	0x2E, 0x20, 0xE8, 0x02, 0x46, 0x41, 0x81, 0x00, 0x02, 0x0F, 0x94, 0xD9, 0x21, 0x5C, 0x45, 0x0A, 
	0xE4, 0x66, 0x92, 0x4B, 0x4A, 0x95, 0x62, 0x9C, 0x59, 0x30, 0xE1, 0x83, 0x9D, 0xE6, 0xC5, 0x08, 
	0x63, 0x95, 0x84, 0xDD, 0x98, 0xE5, 0x96, 0x5D, 0x06, 0x3A, 0xE8, 0xA0, 0x0D, 0x0C, 0xFA, 0x80, 
	0xA0, 0x64, 0x1E, 0x9A, 0xE8, 0xA1, 0x66, 0x9E, 0x19, 0xA2, 0x91, 0x46, 0x56, 0xD0, 0x66, 0x9B, 
	0x91, 0x36, 0xC0, 0xC3, 0x0A, 0x4F, 0x5E, 0x61, 0xE9, 0xA5, 0x0B, 0x1E, 0xB7, 0x69, 0x95, 0x13, 
	0xF6, 0x59, 0x23, 0x6B, 0xC4, 0x82, 0x55, 0xAA, 0x0E, 0x0C, 0x2C, 0x80, 0x2C, 0xA1, 0xF2, 0x09, 
	0xDA, 0x18, 0xA2, 0xAA, 0x22, 0xDA, 0x2A, 0x6D, 0x00, 0xC1, 0x5A, 0x40, 0xAC, 0x8F, 0x2A, 0xC9, 
	0x44, 0x9C, 0x72, 0x26, 0x74, 0x29, 0xAF, 0x9A, 0xFE, 0xCA, 0x9E, 0xA7, 0x60, 0x79, 0x15, 0x6A, 
	0x97, 0xA5, 0x26, 0x3B, 0xFF, 0x2A, 0xB3, 0x66, 0x99, 0x4A, 0xE6, 0xAA, 0xAB, 0x4E, 0x9B, 0x94, 
	0x9A, 0x0A, 0xEC, 0xC0, 0x9F, 0xAC, 0xF8, 0x06, 0x38, 0x69, 0xB7, 0x44, 0xD4, 0xF9, 0xAD, 0x9D, 
	0x51, 0xFC, 0x2A, 0x5A, 0xA7, 0x83, 0x51, 0xC8, 0x25, 0x6B, 0x04, 0xE8, 0x70, 0x63, 0xA9, 0xA3, 
	0x26, 0x46, 0xE8, 0xA9, 0xCD, 0x9E, 0x9A, 0x2A, 0xBC, 0x8A, 0x2E, 0xDA, 0x52, 0x03, 0x68, 0x62, 
	0x67, 0xAF, 0x89, 0x02, 0x16, 0x10, 0x69, 0xAE, 0x73, 0xFE, 0xDB, 0x80, 0x01, 0x2F, 0xE6, 0x49, 
	0x58, 0x02, 0x04, 0x9B, 0x8B, 0xD8, 0xCA, 0xE9, 0xC6, 0xE7, 0xF0, 0xC3, 0x11, 0x37, 0xFB, 0x6E, 
	0xB4, 0xAC, 0x06, 0x49, 0x66, 0x03, 0xD7, 0xEE, 0x50, 0xC0, 0xB5, 0xFC, 0x29, 0xC9, 0x64, 0x05, 
	0x3D, 0x14, 0xC8, 0x6F, 0x0B, 0xBB, 0xDA, 0x69, 0x00, 0xC9, 0x02, 0x3B, 0x48, 0x58, 0x04, 0x14, 
	0x0E, 0x7B, 0xEE, 0x17, 0xEB, 0x92, 0x6A, 0x6A, 0xC4, 0x13, 0x47, 0xFB, 0x58, 0x52, 0x28, 0x20, 
	0xA0, 0x40, 0x99, 0xF6, 0x62, 0x57, 0x80, 0xBD, 0x05, 0x64, 0x16, 0xA0, 0xC7, 0xB5, 0xCE, 0xE9, 
	0xEF, 0xC8, 0x47, 0x23, 0x6D, 0x32, 0xCA, 0x11, 0xC8, 0x90, 0x72, 0x13, 0x2B, 0x07, 0xBA, 0xEC, 
	0xA8, 0x6A, 0x35, 0x2B, 0x33, 0xB4, 0x14, 0x2B, 0xCA, 0xC4, 0xA1, 0x87, 0x6E, 0x8D, 0x31, 0xCF, 
	0x0A, 0xF0, 0xCC, 0x84, 0x9B, 0x1E, 0x7B, 0x1C, 0x5E, 0xBF, 0x9F, 0xF1, 0x6A, 0x40, 0x07, 0x69, 
	0x97, 0xA7, 0x74, 0xA7, 0x04, 0xA0, 0xDC, 0x00, 0x07, 0x04, 0x50, 0x0E, 0xF7, 0x72, 0x82, 0xBA, 
	0x1C, 0x1F, 0xC3, 0x10, 0x77, 0x3E, 0x33, 0xB4, 0x65, 0xEE, 0x8D, 0xC2, 0xD6, 0x5B, 0x67, 0xDD, 
	0x54, 0xE0, 0xF5, 0xA2, 0x8E, 0x31, 0xCE, 0x8F, 0x16, 0xF8, 0x89, 0x06, 0x09, 0x1A, 0x9D, 0xF6, 
	0xD1, 0xA1, 0x29, 0xCD, 0xFF, 0xB4, 0x0C, 0x4C, 0x7B, 0xE5, 0x36, 0xE5, 0xBC, 0x7B, 0xB5, 0x85, 
	0xBA, 0x0B, 0xE8, 0x78, 0xEA, 0xF0, 0xC3, 0x13, 0xCA, 0x2A, 0x14, 0x0B, 0x94, 0xC9, 0xF7, 0xF2, 
	0x7E, 0x7B, 0x8D, 0x26, 0xCE, 0x61, 0x03, 0x5D, 0x81, 0xE1, 0x87, 0x7F, 0xF4, 0x81, 0x8B, 0xB3, 
	0x33, 0x4E, 0xFB, 0xC9, 0x4B, 0x77, 0x3A, 0x70, 0xEF, 0xBD, 0xE7, 0xC8, 0x81, 0x97, 0x85, 0xEE, 
	0x81, 0x40, 0xF2, 0xC9, 0x9F, 0x8F, 0xB1, 0xD6, 0xEC, 0x93, 0xCE, 0x75, 0xEA, 0x7F, 0xEF, 0x6C, 
	0xA2, 0x8A, 0x61, 0x1B, 0x38, 0x5E, 0x82, 0x47, 0x6B, 0x9F, 0x3F, 0xC9, 0xDE, 0xCB, 0xE0, 0x76, 
	0xDB, 0x93, 0x23, 0x0C, 0xF8, 0xCC, 0x35, 0x3E, 0x06, 0x14, 0xB0, 0x80, 0xE9, 0x0A, 0x9E, 0x02, 
	0x93, 0x45, 0xA6, 0xE4, 0x01, 0x00, 0x01, 0x00, 0x48, 0x1F, 0x00, 0x52, 0xB5, 0xBC, 0xBE, 0x95, 
	0x89, 0x3F, 0x3B, 0x53, 0xDD, 0xF4, 0x22, 0xC5, 0x83, 0xC3, 0x21, 0xE8, 0x7A, 0x68, 0x5B, 0xDC, 
	0xEC, 0x1A, 0x90, 0x80, 0xD0, 0x40, 0x2E, 0x65, 0x1C, 0x18, 0x20, 0x07, 0xC4, 0xD7, 0xA5, 0x03, 
	0xD6, 0x68, 0x03, 0x0C, 0x80, 0xA1, 0x0C, 0x03, 0xF5, 0x40, 0xF4, 0x41, 0x30, 0x79, 0xEB, 0x73, 
	0x5F, 0xE9, 0xFC, 0x86, 0xB3, 0xC0, 0xED, 0x4C, 0x7E, 0x91, 0x2A, 0x40, 0x07, 0xED, 0xD7, 0xA2, 
	0x06, 0xD4, 0x20, 0x84, 0x22, 0x24, 0x59, 0x02, 0x4A, 0xD8, 0x00, 0xDC, 0x11, 0x80, 0x69, 0xC7, 
	0xA1, 0x5C, 0xE5, 0xA6, 0x68, 0x39, 0x03, 0x5A, 0xF1, 0x80, 0x3A, 0xDA, 0x80, 0x02, 0xB5, 0x98, 
	0x2C, 0x2D, 0x36, 0xF0, 0x7C, 0x35, 0x3C, 0x9F, 0xF2, 0x2A, 0x78, 0xA8, 0x1C, 0xA0, 0xA0, 0x87, 
	0x19, 0xBC, 0x16, 0x10, 0x53, 0xC4, 0x83, 0xB0, 0xB9, 0x01, 0x76, 0x46, 0x3C, 0xE2, 0xEC, 0x90, 
	0x96, 0x80, 0x08, 0x2C, 0xFF, 0x71, 0x30, 0x6E, 0x0B, 0xA0, 0x14, 0xC1, 0x37, 0xB9, 0x73, 0x59, 
	0x31, 0x50, 0x32, 0x5C, 0x40, 0x20, 0x37, 0x50, 0x2A, 0x08, 0x1E, 0xCA, 0x81, 0xEA, 0x6B, 0x5F, 
	0xFB, 0xF6, 0x06, 0x3D, 0x1F, 0x66, 0x50, 0x6C, 0x42, 0xAC, 0x1F, 0xE2, 0x8C, 0xE8, 0x03, 0x39, 
	0xEA, 0x8F, 0x84, 0x4B, 0xB4, 0xE3, 0x1D, 0x4F, 0x43, 0x45, 0x01, 0x56, 0x8E, 0x85, 0xE3, 0xC3, 
	0x62, 0x8D, 0xB6, 0x48, 0xCA, 0x2F, 0x2E, 0x20, 0x8C, 0xE9, 0xA3, 0xA0, 0x02, 0x4C, 0x87, 0x00, 
	0xD3, 0xFD, 0x10, 0x75, 0x3F, 0xB4, 0x55, 0x0F, 0x78, 0x30, 0xC4, 0x49, 0xD6, 0xE0, 0x96, 0x72, 
	0xA4, 0x5D, 0x26, 0x77, 0x69, 0x47, 0x26, 0xA4, 0x90, 0x8A, 0xE1, 0x2B, 0x60, 0xA8, 0x10, 0x28, 
	0xC8, 0x18, 0x76, 0x09, 0x86, 0x5C, 0x2A, 0x15, 0x2A, 0x0D, 0x89, 0x43, 0xF6, 0x85, 0xAE, 0x95, 
	0xD0, 0x8C, 0xDF, 0x2B, 0xC3, 0x16, 0xC9, 0x02, 0x80, 0x8C, 0x09, 0xB1, 0xAB, 0x41, 0x25, 0xD1, 
	0x16, 0x9A, 0x3A, 0xEE, 0x72, 0x89, 0x25, 0x64, 0x62, 0x14, 0x3D, 0xA9, 0xB2, 0x50, 0xFE, 0x71, 
	0x85, 0x30, 0x24, 0x65, 0x17, 0x8D, 0x07, 0x46, 0x66, 0xB6, 0xF3, 0x66, 0x0D, 0xC8, 0x9A, 0x3C, 
	0x5B, 0x09, 0x3D, 0x35, 0x06, 0x0E, 0x67, 0x6C, 0xA4, 0xE5, 0x3C, 0x9A, 0x80, 0x4B, 0x6D, 0xCA, 
	0x11, 0x93, 0xDF, 0xB4, 0xA3, 0x68, 0x64, 0x90, 0x00, 0x82, 0xFA, 0x52, 0x80, 0x0D, 0x80, 0xA1, 
	0x30, 0x5B, 0x18, 0xA8, 0x18, 0x0A, 0xB2, 0x98, 0x5A, 0x84, 0x61, 0x21, 0xC5, 0x88, 0x48, 0x42, 
	0x31, 0x61, 0x95, 0xD0, 0x9C, 0xE7, 0x19, 0x1D, 0x49, 0xCD, 0x8E, 0xF6, 0x40, 0x92, 0x44, 0x63, 
	0x82, 0x3F, 0x6B, 0x10, 0x82, 0x5B, 0x92, 0x50, 0xA0, 0xDF, 0x0C, 0x27, 0x41, 0x0B, 0x5A, 0xD0, 
	0x26, 0xA8, 0xEC, 0x8A, 0xFF, 0xE7, 0x1C, 0x65, 0xA8, 0xD4, 0xD9, 0xC0, 0x30, 0x42, 0x61, 0x02, 
	0xF1, 0x6C, 0xE5, 0x03, 0x35, 0x5A, 0x26, 0x7B, 0xE2, 0x2C, 0x03, 0xF2, 0xD3, 0xE7, 0x09, 0xBE, 
	0x21, 0xD2, 0x4A, 0xFA, 0x73, 0x9B, 0x9A, 0xF4, 0xE6, 0x2E, 0x9B, 0x58, 0x50, 0x82, 0xAE, 0xD4, 
	0xA0, 0x7B, 0x60, 0xA8, 0x9F, 0x22, 0x0A, 0x51, 0x2E, 0x79, 0xF1, 0x81, 0x37, 0x9D, 0x80, 0x56, 
	0x01, 0x80, 0xD3, 0xAC, 0x65, 0xF4, 0xAB, 0xD0, 0xC3, 0x19, 0x0A, 0x0A, 0x00, 0x54, 0x5B, 0x85, 
	0x6D, 0xA8, 0x1F, 0x89, 0xE3, 0x2D, 0x8D, 0x3A, 0x32, 0x4C, 0x26, 0xB5, 0x97, 0x4B, 0x5C, 0x29, 
	0x4B, 0x9F, 0x3A, 0xB9, 0x14, 0xA6, 0xB0, 0x7C, 0xE5, 0xD3, 0xAA, 0x5E, 0x4F, 0x39, 0x81, 0x05, 
	0xE0, 0xB4, 0x95, 0x3C, 0x95, 0x27, 0x3E, 0xC7, 0xFA, 0xC3, 0xB0, 0xCD, 0xB2, 0x01, 0x68, 0x0D, 
	0x29, 0x25, 0x71, 0x29, 0xBB, 0x6E, 0x2A, 0xB5, 0x84, 0x4F, 0x75, 0xEA, 0x5C, 0xEB, 0x6A, 0xD7, 
	0xCA, 0xDA, 0x35, 0xA1, 0x1B, 0xC8, 0xAC, 0x66, 0x37, 0x4B, 0xC8, 0xAD, 0x4E, 0x80, 0xAB, 0x5C, 
	0xED, 0xEB, 0x67, 0x73, 0x8A, 0x82, 0x9D, 0x7E, 0xF5, 0x8C, 0x3B, 0x03, 0x2A, 0x50, 0x83, 0x8A, 
	0x58, 0xA2, 0xD2, 0x69, 0xA4, 0x23, 0x93, 0x80, 0x01, 0x24, 0xC0, 0x84, 0xA5, 0x36, 0x35, 0xAE, 
	0xB8, 0x35, 0xA8, 0x65, 0x77, 0x8B, 0x4E, 0xCE, 0xFA, 0xB6, 0x01, 0x9F, 0xF5, 0xAB, 0x70, 0x3F, 
	0x1B, 0x5C, 0x32, 0xC9, 0xB3, 0xB4, 0x3C, 0x6D, 0xC0, 0x6A, 0xC7, 0x9A, 0x81, 0xB1, 0x32, 0xE1, 
	0x04, 0xD0, 0xE5, 0x03, 0x13, 0x3E, 0xB3, 0xD6, 0x5C, 0x8E, 0x10, 0x9C, 0x4C, 0x95, 0xAC, 0xFF, 
	0x26, 0xCB, 0x5B, 0xCB, 0x62, 0xD6, 0xB7, 0x9C, 0x2D, 0x15, 0x71, 0x43, 0xCB, 0x57, 0xAD, 0x1A, 
	0xF7, 0x50, 0x1A, 0x6D, 0xFF, 0x42, 0x01, 0xC6, 0xAA, 0x02, 0xF6, 0xCA, 0x0F, 0xBA, 0x68, 0xC5, 
	0x66, 0xE2, 0x4C, 0x6A, 0x80, 0x10, 0xCC, 0xF6, 0xBE, 0xB5, 0x03, 0xE7, 0x5C, 0x73, 0x5B, 0xC2, 
	0xEE, 0x56, 0xF6, 0xBB, 0x37, 0xC8, 0x6C, 0x80, 0x37, 0x30, 0xE0, 0xCE, 0xF6, 0x95, 0xAB, 0xE5, 
	0xE5, 0x6B, 0x3C, 0x8F, 0x4B, 0xCF, 0x06, 0x00, 0xE0, 0xC1, 0xD0, 0x23, 0xEC, 0x7A, 0xCF, 0x1A, 
	0x5D, 0xA2, 0xB5, 0x28, 0x41, 0xF4, 0x95, 0xAD, 0x6C, 0xD3, 0x46, 0xDB, 0x26, 0x6E, 0x77, 0xBF, 
	0x2C, 0xA5, 0xAC, 0x7F, 0x7B, 0x0B, 0xDE, 0xCD, 0x02, 0x17, 0xC1, 0x7A, 0x8D, 0xE0, 0x56, 0x53, 
	0xE5, 0xD5, 0x33, 0x3E, 0xF8, 0xC5, 0x3F, 0x4D, 0x2D, 0x0A, 0x54, 0xF0, 0xDC, 0x0A, 0x7B, 0xE6, 
	0x7A, 0x18, 0x46, 0xA2, 0x7D, 0xED, 0x4B, 0xBB, 0x26, 0x3C, 0x95, 0xA5, 0x2D, 0x1D, 0xF1, 0x65, 
	0x05, 0x4C, 0x64, 0x02, 0x13, 0x19, 0xB8, 0x7E, 0x0D, 0x6E, 0x8A, 0x93, 0x4C, 0x5A, 0x26, 0xBC, 
	0x18, 0xC6, 0x3F, 0x24, 0xAC, 0x72, 0x69, 0x3C, 0x54, 0xE9, 0xE2, 0xB8, 0x88, 0x47, 0xD3, 0xF0, 
	0x7D, 0xF1, 0x9B, 0x80, 0x11, 0x30, 0x51, 0xBB, 0xBA, 0x15, 0x32, 0x89, 0x4B, 0x9C, 0x59, 0x07, 
	0x1F, 0x58, 0xAB, 0x7C, 0xF5, 0x2B, 0x57, 0xE1, 0xF9, 0x64, 0x00, 0x64, 0x00, 0xCA, 0xAB, 0x9D, 
	0x72, 0x06, 0x5A, 0x6B, 0xE5, 0x0B, 0x7F, 0xA6, 0xBE, 0x73, 0xE4, 0xB2, 0x97, 0xBB, 0xCC, 0x44, 
	0x20, 0x8B, 0x78, 0xC4, 0xDF, 0x25, 0x73, 0x99, 0xC9, 0x8B, 0x60, 0xD0, 0xE2, 0xD4, 0xC9, 0x6D, 
	0x6E, 0xB3, 0x72, 0xE5, 0x97, 0x01, 0x15, 0x34, 0xBA, 0xB5, 0x1F, 0x81, 0x1D, 0x8E, 0x99, 0x70, 
	0x5F, 0xFB, 0x6E, 0x58, 0x89, 0x7C, 0xDE, 0xF3, 0x08, 0x7C, 0xFC, 0x67, 0xFF, 0x62, 0x76, 0xC0, 
	0xA0, 0x0E, 0xAF, 0x92, 0xFF, 0x89, 0x7B, 0x53, 0x07, 0x3F, 0xF9, 0xCD, 0x8A, 0x5E, 0xAF, 0x9C, 
	0x1D, 0x0D, 0xE9, 0x90, 0xE2, 0x38, 0x41, 0x21, 0x88, 0xED, 0x96, 0xED, 0xDB, 0x00, 0x2F, 0xDB, 
	0x9A, 0xCF, 0x4B, 0xDC, 0x34, 0x14, 0x84, 0x1C, 0x68, 0xDF, 0x16, 0x18, 0xC9, 0x5B, 0xD0, 0xEB, 
	0x0D, 0x70, 0xFA, 0x62, 0x54, 0xBB, 0xF9, 0xC1, 0xA8, 0x66, 0x42, 0xA3, 0x33, 0xC0, 0x6C, 0x2A, 
	0xBB, 0x16, 0x76, 0x45, 0xC4, 0x41, 0xAC, 0x67, 0xB7, 0x61, 0x12, 0xDE, 0xFA, 0xDA, 0x77, 0x74, 
	0x81, 0xB6, 0x59, 0xF1, 0xDF, 0xCD, 0xFE, 0x7A, 0x16, 0x7A, 0x9D, 0xC0, 0xB0, 0x0F, 0x30, 0x01, 
	0x1A, 0xDC, 0x80, 0x06, 0xA6, 0x4E, 0xF4, 0x93, 0x57, 0x3D, 0x00, 0x15, 0xB0, 0x9A, 0xA8, 0x09, 
	0x92, 0x74, 0x03, 0x42, 0x20, 0x6D, 0x4A, 0xF3, 0x98, 0xB6, 0xD8, 0xC6, 0x76, 0x03, 0xB4, 0xCD, 
	0xEF, 0x7E, 0xBB, 0x00, 0xAF, 0x0D, 0xB8, 0x81, 0x07, 0x3C, 0x20, 0x70, 0x0F, 0x00, 0x57, 0xAB, 
	0xE4, 0x26, 0x37, 0x0D, 0xB4, 0x3A, 0x6C, 0x62, 0x27, 0xDA, 0xD8, 0x13, 0x74, 0x74, 0xA3, 0x25, 
	0x4E, 0x63, 0x78, 0x23, 0xE8, 0x33, 0xF4, 0x0E, 0x41, 0xC6, 0x65, 0x8D, 0xC9, 0x7C, 0x67, 0x7A, 
	0xDF, 0x2E, 0x88, 0x81, 0xBF, 0x47, 0xDE, 0x80, 0x81, 0x17, 0x9C, 0xE0, 0x26, 0x4F, 0x39, 0x70, 
	0xC9, 0x1D, 0xEE, 0x61, 0x2F, 0x7C, 0x02, 0x07, 0x30, 0xF5, 0x9B, 0x21, 0x3E, 0x41, 0x65, 0x3B, 
	0xFA, 0xE6, 0x8F, 0x86, 0x77, 0x82, 0x60, 0x9D, 0xF1, 0x8D, 0xEB, 0xD2, 0xE3, 0xB7, 0x06, 0xB9, 
	0x0C, 0xB4, 0x1D, 0x83, 0xA1, 0x87, 0xBC, 0xDF, 0x25, 0x17, 0xB8, 0xD2, 0x51, 0xCE, 0xF4, 0x1B, 
	0x1C, 0x3C, 0xE1, 0xE5, 0x16, 0x37, 0xCC, 0xCD, 0xAB, 0xEE, 0x9A, 0xD3, 0x00, 0xDD, 0x8D, 0x1E, 
	0xC0, 0xB2, 0x2B, 0xAE, 0xFF, 0xF3, 0x28, 0x49, 0xBB, 0xE7, 0xF5, 0x26, 0x19, 0xD0, 0x71, 0x0D, 
	0xF2, 0x91, 0xBB, 0x60, 0xE8, 0x22, 0xFF, 0xF7, 0xC9, 0xD7, 0x9E, 0x72, 0x83, 0xB3, 0x9C, 0xE1, 
	0x51, 0xD7, 0xEA, 0xC2, 0xD3, 0x0D, 0xE3, 0x06, 0x5C, 0xFD, 0xEA, 0x0D, 0xB8, 0xB9, 0xC4, 0x1B, 
	0x30, 0x80, 0x7D, 0x7E, 0xE0, 0xEF, 0x4C, 0xD0, 0x38, 0x0E, 0x48, 0x2A, 0xED, 0x7A, 0x5B, 0x3B, 
	0xD3, 0xB8, 0xF6, 0x72, 0xD9, 0x8F, 0x3E, 0x72, 0xA3, 0x27, 0x1D, 0xE5, 0x4B, 0x87, 0xBC, 0xC1, 
	0xF5, 0xAA, 0xF0, 0x71, 0x4F, 0xFD, 0xDC, 0x0E, 0x7E, 0x33, 0x13, 0xEE, 0x7E, 0x77, 0xE5, 0x6E, 
	0x9D, 0xEF, 0x7D, 0x1F, 0xC2, 0x07, 0x01, 0x0F, 0x76, 0x8D, 0x6F, 0xBC, 0x09, 0x88, 0x0F, 0x3A, 
	0xD1, 0xB5, 0x6D, 0xF4, 0xB4, 0xF3, 0x3B, 0x06, 0x25, 0x67, 0x7A, 0xD3, 0x07, 0x8E, 0x72, 0xE0, 
	0xBE, 0xBC, 0xE1, 0xE1, 0xA6, 0xFA, 0xE6, 0xEF, 0x9E, 0x01, 0xCE, 0xE7, 0xFD, 0xD1, 0x5A, 0x0F, 
	0xBD, 0xE8, 0xFF, 0xCE, 0xF3, 0xAF, 0x0F, 0xBE, 0xDE, 0x12, 0x48, 0x7E, 0x87, 0x73, 0xFD, 0x71, 
	0xD6, 0xFB, 0xBB, 0xF5, 0xDB, 0x9E, 0x7D, 0xE4, 0x97, 0xEE, 0x74, 0xCB, 0x9B, 0xFB, 0xFA, 0xE7, 
	0x76, 0xFA, 0xEE, 0x39, 0xCF, 0xFD, 0x29, 0x83, 0x7E, 0xCE, 0x38, 0x19, 0x3D, 0x13, 0xBE, 0x2E, 
	0x78, 0x8D, 0x37, 0x40, 0x02, 0x23, 0x50, 0x7E, 0x87, 0x6B, 0xCD, 0xE7, 0x06, 0xB8, 0xDE, 0xEC, 
	0xFC, 0x8E, 0x7D, 0xC1, 0xD9, 0x2E, 0xFB, 0x83, 0xE7, 0xDE, 0xBC, 0x4C, 0x88, 0x39, 0xF7, 0xF7, 
	0x8F, 0xF5, 0xEF, 0xF7, 0x3D, 0xFC, 0x1F, 0xD4, 0x00, 0x10, 0x30, 0x6F, 0x85, 0x27, 0x78, 0xE7, 
	0xA7, 0x7C, 0xE9, 0xA7, 0x7C, 0xA8, 0x57, 0x76, 0x69, 0x27, 0x03, 0x69, 0xE7, 0x7A, 0x8F, 0x27, 
	0x79, 0xF3, 0x07, 0x79, 0xFF, 0x01, 0x87, 0x7D, 0x50, 0x70, 0x00, 0x18, 0xA8, 0x7F, 0xFC, 0xE7, 
	0x7B, 0xFE, 0x27, 0x7C, 0xFD, 0x02, 0x78, 0x10, 0x30, 0x80, 0x3D, 0x67, 0x7E, 0xE8, 0x57, 0x82, 
	0x26, 0xA8, 0x80, 0x50, 0x60, 0x76, 0xAD, 0x17, 0x7B, 0xB2, 0x37, 0x81, 0x2A, 0x87, 0x7F, 0x0D, 
	0x90, 0x81, 0x18, 0x78, 0x03, 0x18, 0x68, 0x77, 0x1B, 0xD8, 0x79, 0x5A, 0x97, 0x01, 0x39, 0xB8, 
	0x22, 0xD8, 0x24, 0x80, 0x1A, 0x30, 0x80, 0x04, 0x48, 0x6F, 0x07, 0xA8, 0x7E, 0xC9, 0x97, 0x7E, 
	0x09, 0x58, 0x6B, 0x2E, 0xA0, 0x6B, 0x5B, 0x80, 0x74, 0xD2, 0x47, 0x70, 0x85, 0x22, 0x83, 0x34, 
	0x48, 0x83, 0x33, 0x68, 0x83, 0x37, 0x88, 0x6E, 0x03, 0x70, 0x85, 0x3A, 0xE8, 0x81, 0xA2, 0xD7, 
	0x04, 0x21, 0xD8, 0x85, 0xF3, 0x16, 0x6B, 0x26, 0x98, 0x80, 0x09, 0xA8, 0x7E, 0x48, 0x28, 0x00, 
	0x49, 0x78, 0x86, 0x23, 0xF0, 0x6F, 0x00, 0xF7, 0x02, 0x6C, 0xD8, 0x86, 0x2F, 0x10, 0x83, 0x19, 
	0x28, 0x85, 0x71, 0x98, 0x81, 0x54, 0xB8, 0x7F, 0x4D, 0x90, 0x83, 0x57, 0xD8, 0x2D, 0x5C, 0x08, 
	0x01, 0x3F, 0xF8, 0x83, 0x0D, 0x80, 0x03, 0x43, 0x48, 0x84, 0x45, 0x58, 0x82, 0x0D, 0x20, 0x00, 
	0x23, 0x90, 0x86, 0x69, 0x68, 0x88, 0x2E, 0x60, 0x88, 0x88, 0xD8, 0x00, 0x6E, 0xF8, 0x88, 0x90, 
	0xF8, 0x86, 0x07, 0x10, 0x85, 0x33, 0x38, 0x89, 0x32, 0xA8, 0x81, 0xDD, 0x07, 0x7A, 0x7C, 0x97, 
	0x85, 0xD5, 0x93, 0x56, 0x02, 0xF8, 0x01, 0x5D, 0x18, 0x82, 0x4C, 0x50, 0x82, 0x46, 0x28, 0x01, 
	0x09, 0x20, 0x88, 0xB5, 0x66, 0x86, 0x23, 0xA0, 0x88, 0xAB, 0x98, 0x88, 0x49, 0xE8, 0x88, 0x2F, 
	0xE0, 0x01, 0x6C, 0x28, 0x8B, 0xB1, 0x58, 0x8B, 0xB2, 0x28, 0x8B, 0x70, 0x78, 0x89, 0x73, 0x48, 
	0x83, 0x75, 0x88, 0x6E, 0xFF, 0xA0, 0x57, 0x83, 0xC1, 0xC7, 0x77, 0xFC, 0xC2, 0x04, 0x21, 0xF8, 
	0x83, 0xA0, 0xE8, 0x87, 0xA3, 0x28, 0x88, 0x62, 0x08, 0x72, 0x88, 0x98, 0x84, 0xAA, 0x88, 0x86, 
	0x02, 0x00, 0x8B, 0xB5, 0x38, 0x8B, 0x6D, 0x48, 0x8B, 0x6C, 0x18, 0x83, 0x94, 0x48, 0x89, 0x97, 
	0x48, 0x85, 0x4C, 0x70, 0x85, 0x18, 0x98, 0x87, 0xE0, 0x38, 0x34, 0x21, 0x15, 0x8A, 0x5E, 0x88, 
	0x03, 0x80, 0x38, 0x8A, 0x63, 0x78, 0x84, 0x87, 0x68, 0x88, 0xAC, 0xB8, 0x88, 0xAD, 0x98, 0x86, 
	0x25, 0x37, 0x8D, 0xF2, 0x68, 0x8D, 0xB8, 0xA8, 0x8B, 0x95, 0x48, 0x89, 0x76, 0xD7, 0x8D, 0x07, 
	0x30, 0x00, 0xFB, 0xF8, 0x8D, 0xCA, 0xC5, 0x77, 0x26, 0x20, 0x8E, 0x4D, 0x70, 0x8C, 0x7C, 0x38, 
	0x80, 0x38, 0x20, 0x01, 0xE6, 0x78, 0x8E, 0x27, 0x48, 0x5B, 0xEE, 0xA8, 0x6D, 0x89, 0x78, 0x88, 
	0xCE, 0x08, 0x8F, 0x6E, 0x48, 0x8F, 0xD4, 0x48, 0x8B, 0x70, 0x98, 0x8D, 0xBA, 0x78, 0x87, 0xFC, 
	0xC8, 0x8F, 0xDF, 0xB8, 0x8F, 0x4C, 0x60, 0x02, 0x2C, 0x20, 0x8E, 0xD6, 0x23, 0x80, 0x5D, 0x98, 
	0x20, 0x09, 0x89, 0x90, 0xE6, 0x98, 0x8C, 0xE9, 0xB7, 0x6F, 0x88, 0xD8, 0x8A, 0xCE, 0x78, 0x86, 
	0xAF, 0x38, 0x8F, 0xD5, 0x38, 0x91, 0x92, 0x28, 0x87, 0x71, 0x08, 0x05, 0x30, 0xC0, 0x77, 0x1D, 
	0x79, 0x85, 0x1B, 0x19, 0x73, 0x2C, 0x90, 0x87, 0xBA, 0x82, 0x20, 0xC4, 0x58, 0x8E, 0xE6, 0x88, 
	0x90, 0x28, 0xD9, 0x04, 0x0C, 0xB9, 0x8A, 0x68, 0xE8, 0x92, 0x87, 0xB8, 0x8A, 0xD2, 0x48, 0x91, 
	0x15, 0x79, 0x8D, 0xBB, 0xD8, 0x04, 0x30, 0x70, 0x00, 0x55, 0x19, 0x73, 0xFD, 0xB8, 0x91, 0x30, 
	0x30, 0x00, 0x2C, 0xE0, 0x91, 0x03, 0x10, 0x90, 0x95, 0x12, 0x80, 0x05, 0xF9, 0x87, 0x28, 0x79, 
	0x92, 0x09, 0xB9, 0x05, 0xFF, 0x10, 0xA9, 0x8A, 0xEC, 0x98, 0x86, 0xFF, 0x26, 0x93, 0xD4, 0x28, 
	0x8F, 0x01, 0x17, 0x73, 0x54, 0x69, 0x95, 0x74, 0x09, 0x03, 0x39, 0xD9, 0x93, 0x78, 0xC9, 0x04, 
	0x2C, 0x00, 0x96, 0x6A, 0x80, 0x4D, 0x09, 0x22, 0x8A, 0x66, 0x59, 0x96, 0x07, 0x09, 0x88, 0x45, 
	0x38, 0x0B, 0xEF, 0xF8, 0x94, 0x6F, 0xF9, 0x84, 0x55, 0x79, 0x95, 0x8B, 0x49, 0x97, 0x31, 0x38, 
	0x00, 0x5B, 0x99, 0x95, 0x0D, 0x10, 0x92, 0xFF, 0x77, 0x20, 0x09, 0x52, 0x02, 0x7F, 0x49, 0x96, 
	0x09, 0x89, 0x03, 0x33, 0x60, 0x94, 0x29, 0x39, 0x02, 0x0E, 0x80, 0x80, 0x83, 0xA8, 0x84, 0xE5, 
	0x03, 0x03, 0x02, 0x60, 0x9A, 0xA8, 0xD9, 0x00, 0x8D, 0xB9, 0x98, 0xAB, 0x69, 0x95, 0x3A, 0xD9, 
	0x93, 0x7A, 0x49, 0x01, 0x1D, 0x61, 0x99, 0x01, 0xF8, 0x87, 0x9C, 0xE9, 0x99, 0x9B, 0x49, 0x98, 
	0xE9, 0x27, 0x00, 0x12, 0xE0, 0x00, 0x09, 0xC8, 0x9B, 0x26, 0x58, 0x88, 0xA8, 0x79, 0x9A, 0xC4, 
	0x59, 0x9C, 0x02, 0x70, 0x9C, 0x31, 0xD8, 0x9A, 0xCA, 0xF9, 0x8B, 0xFF, 0xC7, 0x02, 0x2C, 0x20, 
	0x9B, 0x9D, 0x18, 0x09, 0x3B, 0xB7, 0x05, 0x9B, 0x89, 0x92, 0x08, 0x59, 0x88, 0x12, 0x00, 0x9C, 
	0xE9, 0xE7, 0x00, 0xC0, 0xC9, 0x9B, 0xBC, 0xA9, 0x9A, 0xC6, 0x79, 0x9C, 0xA6, 0x49, 0x9C, 0xA9, 
	0xA9, 0x9C, 0x75, 0xE9, 0x98, 0xFE, 0xF8, 0x9C, 0xEA, 0xD9, 0x00, 0x7C, 0x29, 0x1E, 0x0D, 0x50, 
	0x02, 0x62, 0x09, 0x84, 0x12, 0x30, 0x03, 0x45, 0x99, 0x92, 0xC9, 0x17, 0x9A, 0xBC, 0xE9, 0x9B, 
	0xC9, 0x97, 0x9F, 0xC9, 0x57, 0x88, 0xE2, 0x49, 0x9E, 0x66, 0x88, 0x9A, 0x2E, 0x60, 0x9A, 0xC9, 
	0x79, 0x9E, 0xCB, 0xF9, 0x7F, 0x14, 0xE0, 0x9C, 0xCE, 0xC9, 0x9E, 0x22, 0xA9, 0x05, 0x08, 0x82, 
	0x99, 0x24, 0x29, 0x8A, 0xFF, 0xE3, 0x77, 0x96, 0xAB, 0x98, 0x9D, 0x83, 0xC8, 0x9F, 0xBB, 0x09, 
	0x9E, 0x02, 0x3A, 0x9E, 0xE3, 0x59, 0x9C, 0xAA, 0x69, 0xA0, 0x06, 0xDA, 0x04, 0x0A, 0xAA, 0x9E, 
	0xD0, 0xD9, 0x09, 0x3D, 0x48, 0x92, 0x25, 0x10, 0x8A, 0x4D, 0x40, 0x98, 0xCA, 0xE7, 0x9D, 0xA1, 
	0xB9, 0x9B, 0x16, 0x2A, 0x9C, 0x66, 0x78, 0x9C, 0xA7, 0xB9, 0x88, 0xC3, 0x79, 0x9A, 0x05, 0xDA, 
	0x9A, 0x50, 0x20, 0x9B, 0xCF, 0x99, 0xA0, 0x3E, 0x1A, 0x9D, 0xD0, 0xE0, 0x97, 0xEF, 0x39, 0x03, 
	0x10, 0x90, 0xA2, 0x45, 0x4A, 0xA4, 0x50, 0x00, 0x9C, 0xA1, 0xB9, 0x9F, 0xF7, 0xD9, 0x9F, 0xFF, 
	0x39, 0x9C, 0x1C, 0x1A, 0xA0, 0xD1, 0x18, 0xA2, 0x4C, 0x40, 0x01, 0x56, 0xCA, 0xA3, 0x09, 0xBA, 
	0x9E, 0xB3, 0xD9, 0x19, 0x3D, 0x18, 0x82, 0x29, 0x4A, 0xA4, 0x5E, 0x3A, 0x80, 0x0E, 0xE0, 0x00, 
	0x5B, 0x60, 0xA1, 0xD9, 0x69, 0x84, 0x32, 0xFA, 0x9F, 0xC7, 0x39, 0xA0, 0x52, 0x1A, 0x8D, 0x55, 
	0xB9, 0xA3, 0x57, 0x7A, 0xA5, 0x93, 0xE9, 0xA3, 0x21, 0x19, 0x90, 0x0D, 0xDA, 0x97, 0x7B, 0x18, 
	0xA6, 0x47, 0x2A, 0x8A, 0x33, 0x30, 0xA6, 0x7E, 0x5A, 0xA6, 0xDF, 0x09, 0xA5, 0xE4, 0x59, 0x28, 
	0x71, 0x5A, 0xA8, 0x56, 0x3A, 0x99, 0xCF, 0xF9, 0x91, 0xED, 0xE9, 0x0B, 0x7B, 0xE0, 0xA5, 0x44, 
	0x9A, 0xA2, 0xF0, 0xE9, 0x00, 0x33, 0x30, 0xA9, 0x93, 0xEA, 0xA7, 0x63, 0x0A, 0x70, 0x96, 0xCA, 
	0x9D, 0x64, 0x6A, 0xA8, 0x86, 0x3A, 0xA7, 0x8A, 0xBA, 0xA8, 0x39, 0xC1, 0x04, 0xF0, 0x49, 0x8C, 
	0x46, 0x2A, 0x8A, 0x92, 0x6A, 0xA9, 0x94, 0xDA, 0xA7, 0x7D, 0x4A, 0xA6, 0x99, 0xEA, 0xA7, 0x02, 
	0xC0, 0x9D, 0xAF, 0xFA, 0xAA, 0x9B, 0xCA, 0xA9, 0x71, 0x3A, 0x99, 0xEC, 0x69, 0x02, 0xB8, 0x0A, 
	0xAA, 0xA1, 0xFA, 0x9E, 0xFF, 0xBC, 0x2A, 0xAA, 0x5E, 0x78, 0xAA, 0x95, 0xEA, 0xA7, 0xAA, 0x3A, 
	0x03, 0x0D, 0x00, 0xAB, 0xC6, 0x1A, 0xAB, 0x63, 0x2A, 0xAB, 0xD1, 0x48, 0xAB, 0x72, 0xFA, 0xA9, 
	0xB8, 0x7A, 0xA7, 0x7F, 0xD0, 0x04, 0xF0, 0x89, 0x99, 0xA3, 0x2A, 0x80, 0xC5, 0xAA, 0xAA, 0x92, 
	0x4A, 0xA9, 0xA7, 0x7A, 0xA9, 0xC9, 0xDA, 0xAD, 0xDE, 0xFA, 0xAA, 0x0D, 0xC0, 0xAC, 0x50, 0x90, 
	0xAB, 0xCF, 0x0A, 0xAD, 0xBF, 0xF0, 0xA0, 0xBC, 0x3A, 0xAD, 0x5B, 0xB0, 0xAD, 0xC0, 0x4A, 0xA9, 
	0x85, 0xF8, 0xAD, 0x9A, 0x4A, 0x01, 0xC9, 0x1A, 0xAE, 0xB5, 0xDA, 0x04, 0x29, 0x70, 0xAF, 0xB7, 
	0x6A, 0xA7, 0x1E, 0x51, 0x0F, 0xA2, 0xFA, 0x9E, 0xD3, 0x7A, 0x99, 0xD5, 0x0A, 0x05, 0x96, 0x5A, 
	0xAC, 0xF2, 0xEA, 0x00, 0x05, 0x2B, 0xAB, 0x9A, 0x3A, 0xAB, 0xF6, 0x7A, 0xAF, 0xF7, 0x6A, 0x02, 
	0x29, 0x70, 0xAB, 0x40, 0xBA, 0xAF, 0x8A, 0xF5, 0x9E, 0x1F, 0xA0, 0xAE, 0x18, 0x50, 0x02, 0x18, 
	0x9B, 0xB1, 0x18, 0x0B, 0x70, 0x46, 0xC1, 0xB0, 0x29, 0xE0, 0xB0, 0x1F, 0xFB, 0xB0, 0xFA, 0x2A, 
	0xB1, 0x56, 0x00, 0x05, 0xEA, 0x5A, 0x02, 0x17, 0x9B, 0xB2, 0x18, 0x7B, 0xB1, 0x19, 0xDB, 0x00, 
	0x18, 0x80, 0x01, 0x29, 0x00, 0xB3, 0x32, 0x1B, 0xB3, 0x31, 0x0B, 0xB3, 0x0D, 0xD0, 0xB0, 0x21, 
	0x8B, 0xB3, 0x50, 0x40, 0xB2, 0x61, 0x29, 0xAD, 0xFE, 0xCA, 0xB2, 0x40, 0x8B, 0xB2, 0xF0, 0x59, 
	0xB3, 0x2F, 0x5B, 0xB4, 0x45, 0x1B, 0xB3, 0x37, 0x0B, 0xB2, 0x0D, 0xBB, 0x07, 0x3C, 0x1B, 0xA4, 
	0x5B, 0xB0, 0xB2, 0x1A, 0xDB, 0xB2, 0x2F, 0x4B, 0xB3, 0x33, 0x2B, 0xB3, 0x18, 0x70, 0xB3, 0x0F, 
	0x6B, 0xB2, 0x17, 0x6B, 0xAE, 0x4D, 0x9B, 0x10, 0xFD, 0xBA, 0x07, 0x42, 0xEB, 0xB2, 0x54, 0x8B, 
	0xB4, 0xB3, 0x10, 0xB4, 0x24, 0x5B, 0xDA, 0xB5, 0x7D, 0x60, 0xB2, 0x1A, 0x5B, 0x3E, 0x51, 0xFB, 
	0x9E, 0x2C, 0x5B, 0xAD, 0x68, 0xBB, 0xAB, 0x6A, 0x3B, 0xAD, 0x51, 0xBB, 0xB6, 0x1B, 0x7B, 0xB6, 
	0x71, 0x8B, 0x10, 0x1C, 0xBB, 0xB3, 0x4D, 0x1B, 0x04, 0x00, 0x21, 0xF9, 0x04, 0x04, 0x0A, 0x00, 
	0x00, 0x00, 0x2C, 0x00, 0x00, 0x10, 0x00, 0x64, 0x00, 0x63, 0x00, 0x00, 0x06, 0xFF, 0xC0, 0x96, 
	0x70, 0x48, 0x2C, 0x1A, 0x8F, 0xC8, 0x62, 0xA3, 0x11, 0x58, 0x3A, 0x6D, 0xCE, 0x65, 0xA0, 0x99, 
	0xAC, 0x5A, 0xAF, 0xD8, 0x6C, 0x32, 0x1A, 0x25, 0x59, 0x48, 0x0D, 0x8B, 0xCD, 0xD2, 0xB8, 0x58, 
	0x2E, 0x61, 0x74, 0x54, 0xCB, 0x6E, 0xBB, 0x89, 0xD2, 0xE6, 0x12, 0x1C, 0x26, 0x99, 0x97, 0xD0, 
	0xE5, 0x99, 0x5C, 0x56, 0x87, 0x97, 0x6F, 0x81, 0x82, 0x70, 0x5C, 0x73, 0x24, 0x87, 0x74, 0x7A, 
	0x62, 0x16, 0x8C, 0x4E, 0x17, 0x8F, 0x90, 0x5C, 0x64, 0x83, 0x94, 0x59, 0x51, 0x4D, 0x88, 0x89, 
	0x4E, 0x8C, 0x9C, 0x63, 0x9C, 0x8D, 0x7D, 0x90, 0x8F, 0x4B, 0x7E, 0x95, 0xA6, 0x47, 0x4C, 0x53, 
	0x98, 0x5E, 0x9A, 0x61, 0x9F, 0x8C, 0x9E, 0xAF, 0x7C, 0x85, 0x8E, 0x68, 0xA7, 0xB7, 0x71, 0x72, 
	0x5C, 0x87, 0xAE, 0xB2, 0x8B, 0xBE, 0x8D, 0xA2, 0xA2, 0x5C, 0xB7, 0x83, 0xB4, 0x0D, 0x99, 0x88, 
	0x8A, 0xBE, 0xB1, 0xBE, 0xB5, 0x20, 0x91, 0x2B, 0x20, 0x6B, 0xC5, 0x6D, 0x97, 0x53, 0xC8, 0xBC, 
	0x5D, 0xBD, 0xC0, 0xDD, 0x65, 0x22, 0x5C, 0xA3, 0xD2, 0xD2, 0x80, 0xD5, 0x96, 0xA9, 0xAA, 0xBA, 
	0xD9, 0x99, 0xDC, 0xDD, 0xB2, 0x0D, 0x22, 0xE0, 0xDF, 0x17, 0xD3, 0x4E, 0x2B, 0x17, 0xE3, 0x0D, 
	0xE6, 0x57, 0xB9, 0xEA, 0xC9, 0x89, 0x60, 0xDC, 0x01, 0x83, 0x17, 0x4F, 0x1E, 0xBC, 0x0B, 0xE0, 
	0xEA, 0x35, 0x90, 0x56, 0x6F, 0x1F, 0x92, 0x5C, 0x10, 0xD9, 0xAD, 0x6B, 0xA0, 0xA3, 0x9D, 0x40, 
	0x82, 0x04, 0x0B, 0x3A, 0x6A, 0x00, 0xA2, 0xA3, 0xBD, 0x72, 0x0E, 0x87, 0x48, 0x71, 0x92, 0x8E, 
	0x00, 0x32, 0x27, 0x87, 0x2A, 0x52, 0x24, 0xA1, 0x52, 0x60, 0x23, 0x11, 0x0A, 0xE2, 0x2D, 0x29, 
	0xA8, 0x91, 0x63, 0xC7, 0x9B, 0x4B, 0x56, 0x80, 0x34, 0xB7, 0xA4, 0x43, 0x3F, 0xFF, 0x02, 0xAA, 
	0x0C, 0xB1, 0x04, 0xA3, 0x83, 0x25, 0x03, 0x8A, 0x2A, 0x8F, 0x39, 0xC9, 0x91, 0x51, 0x66, 0xD3, 
	0x06, 0x3B, 0x76, 0xD8, 0xC4, 0xB9, 0xA2, 0xAA, 0x3E, 0x9E, 0x1D, 0x7C, 0xA2, 0x03, 0x8A, 0xAD, 
	0x81, 0xC9, 0xA1, 0x4B, 0x8A, 0x32, 0x20, 0xAA, 0xA3, 0xE2, 0x03, 0x1D, 0x0F, 0x1A, 0x3C, 0x58, 
	0xDB, 0x20, 0x87, 0xDB, 0xB6, 0x4C, 0x69, 0x3A, 0x05, 0xB7, 0x03, 0x44, 0xD4, 0xA9, 0xD3, 0x2A, 
	0xE8, 0xD4, 0x59, 0xAD, 0x41, 0x56, 0xBF, 0x4C, 0xB8, 0xCA, 0x21, 0x00, 0xF4, 0x6B, 0xD9, 0xA2, 
	0x45, 0x91, 0x9A, 0x45, 0x9B, 0x76, 0x2D, 0xDB, 0x07, 0x6F, 0x45, 0x30, 0x6D, 0x2B, 0x77, 0x49, 
	0x54, 0xBB, 0x52, 0xA7, 0x2E, 0xAC, 0xE0, 0x04, 0x17, 0xE0, 0x0E, 0x11, 0xD0, 0x2D, 0x21, 0x1C, 
	0x80, 0x04, 0x01, 0xA2, 0x24, 0xC6, 0x8E, 0x55, 0x7B, 0x98, 0xF5, 0x59, 0xB5, 0x90, 0xD3, 0xBA, 
	0x65, 0x3A, 0x3B, 0x2E, 0xCD, 0x06, 0x0A, 0xA0, 0x5E, 0xBE, 0xCB, 0xB1, 0x81, 0x5E, 0xBD, 0x3B, 
	0x05, 0xF5, 0x6C, 0x10, 0xE1, 0xEF, 0xE0, 0xD3, 0xA4, 0x4D, 0x7E, 0x05, 0x7B, 0xB8, 0x22, 0xE3, 
	0xC7, 0x8E, 0xE1, 0xBE, 0xCD, 0x21, 0x39, 0x2E, 0x75, 0xDC, 0x30, 0x39, 0xEE, 0xE6, 0xBD, 0xE2, 
	0xB7, 0xCE, 0xAB, 0xC2, 0x3F, 0x67, 0x0D, 0xED, 0xF5, 0x74, 0x00, 0xA0, 0xA6, 0xCB, 0x9B, 0x4C, 
	0x7C, 0xB6, 0xF5, 0xD9, 0xD7, 0xB1, 0xD9, 0xD6, 0x86, 0x5B, 0xDD, 0x49, 0xCC, 0xFB, 0xBA, 0x77, 
	0x43, 0xD5, 0xF7, 0x9B, 0x33, 0x78, 0x37, 0x0D, 0x68, 0xD0, 0x53, 0x07, 0x01, 0xF8, 0x14, 0xDA, 
	0x79, 0xE7, 0x99, 0xA6, 0x1C, 0x72, 0x84, 0x29, 0x56, 0x96, 0x6B, 0x68, 0xC1, 0x16, 0x5B, 0x5B, 
	0x0F, 0x58, 0x30, 0xDD, 0x6C, 0x6D, 0xC1, 0xA3, 0xC0, 0x86, 0x1B, 0x2E, 0x81, 0xFF, 0x99, 0x54, 
	0x15, 0x48, 0xB5, 0x42, 0x0B, 0xC0, 0xF5, 0xF0, 0x9F, 0x16, 0x01, 0xF6, 0x54, 0x9C, 0x81, 0xA1, 
	0x31, 0x78, 0x1C, 0x61, 0x0D, 0x26, 0xB5, 0x98, 0x63, 0x66, 0x39, 0x26, 0x5F, 0x0E, 0x90, 0xD1, 
	0xB6, 0x04, 0x86, 0xF7, 0xC1, 0x94, 0x5D, 0x5D, 0x52, 0x45, 0xC5, 0xD9, 0x6F, 0x0D, 0x98, 0xF8, 
	0xC6, 0x12, 0x02, 0xAE, 0x48, 0x5C, 0x04, 0x4C, 0x7A, 0x55, 0x9A, 0x7A, 0x0A, 0x82, 0xC1, 0xC0, 
	0x94, 0x53, 0x72, 0xC1, 0x98, 0x5A, 0x38, 0x46, 0x87, 0xE1, 0x52, 0xB5, 0x31, 0xC5, 0xA1, 0x8F, 
	0xB9, 0x05, 0x19, 0xE2, 0x5D, 0x44, 0x1A, 0xC9, 0x06, 0x92, 0x3E, 0xF8, 0xC5, 0x22, 0x93, 0x4D, 
	0x2E, 0xA8, 0x60, 0x8C, 0x53, 0xA6, 0x76, 0xD4, 0x02, 0x74, 0x2E, 0xA0, 0x14, 0x17, 0x08, 0xE4, 
	0x29, 0x1D, 0x86, 0x60, 0x6E, 0x98, 0x5D, 0x7E, 0x3B, 0x84, 0xE8, 0x9B, 0x4E, 0x3D, 0x70, 0x86, 
	0x62, 0x80, 0x3E, 0xA4, 0xB9, 0x26, 0x9B, 0x2D, 0xC2, 0xE8, 0x68, 0x03, 0x53, 0xEA, 0x40, 0x00, 
	0x95, 0x0D, 0xD4, 0x69, 0x69, 0xA5, 0x96, 0xDA, 0x99, 0x27, 0x02, 0x0F, 0x70, 0xBA, 0x27, 0x8F, 
	0x5F, 0xC6, 0x84, 0x9B, 0x6F, 0x81, 0x0A, 0xE9, 0x5B, 0x91, 0x15, 0x98, 0x69, 0xC5, 0x12, 0x89, 
	0x6A, 0x90, 0xE6, 0x92, 0x8C, 0xB6, 0xE8, 0xD5, 0x9B, 0x44, 0x31, 0x30, 0xA9, 0x6A, 0x60, 0x2C, 
	0xA0, 0x03, 0x03, 0xBA, 0xF2, 0x8A, 0xE9, 0xA5, 0x9C, 0xE6, 0xD9, 0x29, 0x02, 0x9F, 0x46, 0x76, 
	0x5F, 0x6E, 0x0A, 0x14, 0x10, 0xA6, 0x90, 0xBC, 0xA5, 0x6A, 0x28, 0x3F, 0x3E, 0xB8, 0xDA, 0x41, 
	0x92, 0xB1, 0x36, 0x6A, 0xD2, 0xAC, 0x26, 0x45, 0xAA, 0x5A, 0x95, 0x0B, 0xF0, 0x7A, 0x69, 0xA6, 
	0x74, 0x36, 0x20, 0xEC, 0xB8, 0xC5, 0xBE, 0xE5, 0x27, 0x6E, 0x3B, 0x24, 0x9B, 0xFF, 0xDB, 0x98, 
	0x15, 0x0C, 0xB9, 0x90, 0x89, 0x27, 0x1A, 0x81, 0xE4, 0xB4, 0x02, 0x2E, 0xDA, 0x62, 0x04, 0x04, 
	0xE0, 0xAB, 0x1C, 0x45, 0x93, 0xDE, 0x2A, 0xE9, 0x9C, 0xE0, 0xFE, 0x7A, 0xE9, 0xB0, 0x04, 0x13, 
	0xDB, 0x25, 0x8F, 0x50, 0x6D, 0x98, 0xAE, 0xB2, 0x80, 0xFA, 0xC6, 0x59, 0x0F, 0xAA, 0x3E, 0xD4, 
	0x40, 0xA2, 0x8A, 0x1A, 0x00, 0x6B, 0x9B, 0xF9, 0x12, 0x86, 0xEF, 0x12, 0x71, 0x32, 0xF0, 0x2F, 
	0xC0, 0x1E, 0x77, 0xAB, 0x83, 0x9D, 0xE0, 0x86, 0x1B, 0xEC, 0xC9, 0x06, 0x1F, 0x6C, 0xD9, 0x86, 
	0xCA, 0xB2, 0xBC, 0x2E, 0x88, 0xCE, 0x16, 0x19, 0x9C, 0x12, 0x01, 0xFA, 0x64, 0x80, 0x01, 0x21, 
	0x5C, 0xEC, 0x95, 0xBE, 0xFA, 0x9A, 0xC4, 0x01, 0xC7, 0x54, 0xA6, 0x56, 0xE9, 0xC8, 0x96, 0x02, 
	0x1C, 0xF0, 0xB8, 0xC1, 0xCA, 0x36, 0x9F, 0xB8, 0x50, 0x29, 0xBB, 0x70, 0xBA, 0xE9, 0xEE, 0x57, 
	0x41, 0x01, 0xED, 0x9E, 0x1A, 0xAF, 0x48, 0x02, 0x46, 0xDB, 0x80, 0x01, 0x3E, 0xDC, 0xBC, 0xA4, 
	0x7A, 0x3C, 0xE7, 0xAB, 0x1E, 0xD0, 0xDC, 0x7A, 0xFB, 0x6D, 0xC9, 0xE2, 0x76, 0x5A, 0xB0, 0xC1, 
	0x4E, 0xA0, 0xB0, 0x29, 0x6E, 0x2E, 0x73, 0xC8, 0x70, 0xBB, 0x53, 0x57, 0xDD, 0xC0, 0x09, 0x57, 
	0xB7, 0xD0, 0xC0, 0x07, 0x29, 0xE2, 0xDC, 0x01, 0xCE, 0xC4, 0xED, 0x9C, 0xF1, 0xE0, 0x0D, 0x70, 
	0x40, 0x00, 0x07, 0x86, 0x5F, 0x6B, 0xB4, 0xC8, 0xE1, 0x96, 0x1C, 0xAE, 0xDA, 0x48, 0x3B, 0xB1, 
	0xE9, 0xE4, 0x4D, 0x2B, 0x00, 0xB5, 0xD3, 0xEB, 0x52, 0xED, 0xAC, 0x7F, 0x79, 0xA7, 0xB8, 0xB5, 
	0x01, 0x1D, 0x84, 0x70, 0xB3, 0x4F, 0x30, 0x86, 0x6D, 0x92, 0x0C, 0x87, 0x13, 0x66, 0xB8, 0xE1, 
	0x85, 0xD0, 0xE9, 0xAB, 0xE3, 0x9A, 0xEA, 0x19, 0xC5, 0x02, 0xC4, 0x4E, 0x4E, 0xFF, 0xB9, 0xBA, 
	0x72, 0xE7, 0x56, 0x80, 0xC3, 0x75, 0x17, 0x7A, 0x77, 0xBC, 0x01, 0xF2, 0xCD, 0xF5, 0xDF, 0xC3, 
	0x5B, 0x3C, 0x78, 0xD8, 0x5E, 0xAD, 0x9E, 0x3A, 0xEB, 0x0C, 0x70, 0x30, 0xE5, 0xCF, 0x77, 0xD2, 
	0x82, 0x00, 0xED, 0xB4, 0x4F, 0x2F, 0xAE, 0xED, 0x6F, 0x5B, 0xAE, 0x2C, 0xC3, 0x05, 0xA0, 0xE0, 
	0x9B, 0xE6, 0x74, 0xFF, 0x2E, 0xEF, 0x07, 0x02, 0x8A, 0xEE, 0xB7, 0xD7, 0xF7, 0x6A, 0xDC, 0x60, 
	0xE2, 0xCA, 0xAF, 0x0E, 0x69, 0xF3, 0x54, 0x3A, 0x3F, 0x27, 0xAF, 0xF4, 0x77, 0xAB, 0x29, 0xF5, 
	0x08, 0x00, 0x30, 0xBD, 0xFE, 0xD7, 0x63, 0x4F, 0x2C, 0x87, 0x51, 0x53, 0xD7, 0xEE, 0x9C, 0x45, 
	0xB5, 0x06, 0xF0, 0x60, 0x66, 0x29, 0xAA, 0x81, 0x5F, 0x42, 0xF0, 0xB7, 0x9C, 0x19, 0x20, 0x01, 
	0x09, 0x10, 0x5C, 0xC6, 0x92, 0xD7, 0xBE, 0xD4, 0x15, 0xEE, 0x79, 0xF0, 0x6B, 0x1E, 0xA4, 0x36, 
	0xC0, 0x00, 0x0E, 0x7A, 0x70, 0x4E, 0xF9, 0x9B, 0xDE, 0xFE, 0x44, 0xD8, 0x3F, 0xEC, 0x35, 0xA0, 
	0x65, 0x27, 0xEC, 0x5E, 0xF7, 0xBE, 0x17, 0xBE, 0xDD, 0x9D, 0x08, 0x49, 0x0D, 0x50, 0x20, 0xE8, 
	0x46, 0xB7, 0xB5, 0x04, 0x18, 0x80, 0x3C, 0x13, 0x4C, 0xDD, 0xF2, 0x2C, 0x28, 0x3F, 0xF9, 0x61, 
	0xB0, 0x01, 0x1B, 0xB0, 0x5F, 0x10, 0x79, 0x15, 0xC4, 0x4A, 0x89, 0x70, 0x01, 0xFA, 0xA3, 0x1D, 
	0x00, 0x4A, 0x68, 0x42, 0xDD, 0x29, 0x00, 0x05, 0xDD, 0x4B, 0x16, 0xEF, 0x4E, 0x78, 0xC0, 0x17, 
	0x2E, 0xA1, 0x06, 0x02, 0xC2, 0x19, 0xE0, 0x20, 0xF8, 0x40, 0x1B, 0xCA, 0x0A, 0x75, 0x15, 0x74, 
	0x1F, 0x06, 0xE1, 0x27, 0xBF, 0x4A, 0x79, 0x70, 0x01, 0x67, 0xDC, 0x80, 0xB8, 0xF4, 0xA7, 0x3F, 
	0x11, 0x02, 0x20, 0x76, 0xFE, 0x23, 0xD6, 0x09, 0xA1, 0xF8, 0xC4, 0x02, 0xA8, 0xFF, 0xF0, 0x54, 
	0x05, 0xE0, 0x81, 0x1D, 0x77, 0xE2, 0x84, 0x1A, 0xD4, 0x20, 0x4D, 0xA0, 0xFB, 0x9C, 0x0D, 0x07, 
	0x69, 0xC3, 0xD1, 0x84, 0xF1, 0x70, 0x90, 0xEA, 0x21, 0x95, 0xE0, 0x07, 0x29, 0x21, 0x3A, 0xD2, 
	0x88, 0xD4, 0x4B, 0x62, 0x9E, 0xE0, 0x48, 0xB9, 0x06, 0x78, 0x2F, 0x59, 0x74, 0xCC, 0x24, 0x1E, 
	0x79, 0xA0, 0x47, 0x3E, 0x66, 0x4D, 0x03, 0x32, 0xCC, 0x19, 0x17, 0x47, 0xF9, 0xC0, 0x06, 0x44, 
	0xD0, 0x90, 0x3B, 0x7C, 0x9F, 0x22, 0xCB, 0x88, 0xC6, 0x0E, 0x52, 0x89, 0x83, 0x55, 0x0A, 0x61, 
	0x24, 0x47, 0xC8, 0x44, 0x62, 0xA1, 0xE0, 0x96, 0x73, 0x54, 0xA1, 0x0A, 0xA5, 0x98, 0x47, 0x3B, 
	0x56, 0x51, 0x24, 0x48, 0xF2, 0xA3, 0x02, 0x6D, 0xE6, 0xC5, 0x41, 0x3E, 0xD0, 0x62, 0x32, 0x48, 
	0x40, 0x32, 0x97, 0xA0, 0xBC, 0x6C, 0x29, 0x32, 0x7E, 0x40, 0xAC, 0xDF, 0x10, 0xD1, 0x48, 0xB2, 
	0x11, 0xCE, 0x92, 0x92, 0x4B, 0xB8, 0xA5, 0xFE, 0x2E, 0x99, 0x49, 0x15, 0x7A, 0xAF, 0x97, 0x9C, 
	0xDC, 0x1D, 0x30, 0x63, 0x18, 0x2D, 0x3F, 0x9A, 0xF2, 0x98, 0xC6, 0x1C, 0xA4, 0x29, 0x93, 0xC9, 
	0x4E, 0x27, 0xFC, 0x0C, 0x96, 0x19, 0x74, 0x9E, 0xF3, 0x2A, 0xD5, 0x41, 0x6A, 0xA6, 0xF1, 0x28, 
	0x6D, 0x54, 0xE2, 0x24, 0xF5, 0x24, 0xC7, 0x5B, 0xBA, 0xCD, 0x9F, 0x29, 0x84, 0x62, 0x14, 0x77, 
	0x47, 0x50, 0x3D, 0xE6, 0x11, 0x3C, 0x57, 0x04, 0x25, 0x16, 0x65, 0x98, 0x4E, 0x74, 0x36, 0x80, 
	0x9D, 0xCA, 0x84, 0x60, 0x3B, 0xDD, 0x39, 0xC6, 0xE7, 0x35, 0x72, 0x91, 0x8E, 0x54, 0xA3, 0x35, 
	0x69, 0xD9, 0x36, 0x14, 0x00, 0xC0, 0xA3, 0x1E, 0x45, 0x80, 0x47, 0x71, 0x33, 0xD0, 0x6F, 0x16, 
	0xB4, 0x97, 0x08, 0x8D, 0xE1, 0x42, 0xB5, 0x56, 0x43, 0x74, 0x1E, 0x73, 0x9D, 0xFF, 0x11, 0x95, 
	0x01, 0x44, 0x23, 0x88, 0xB8, 0x42, 0x94, 0xB1, 0x9E, 0x41, 0xFC, 0x60, 0x3D, 0xC5, 0x45, 0xBB, 
	0x42, 0x4C, 0x60, 0x02, 0x96, 0xFC, 0xA8, 0x48, 0x45, 0x0A, 0xD2, 0x6F, 0x0A, 0x54, 0x77, 0x19, 
	0xD8, 0x9D, 0x1E, 0x3B, 0x89, 0x37, 0x21, 0x38, 0xC1, 0x07, 0xC2, 0xFC, 0xDC, 0x12, 0x06, 0x19, 
	0x01, 0x08, 0xC2, 0x74, 0xA6, 0xED, 0x44, 0x9C, 0x56, 0xB7, 0x1A, 0x3D, 0xA5, 0xFC, 0xF4, 0xAB, 
	0x3F, 0x05, 0x00, 0x50, 0x89, 0xFA, 0xD1, 0x7F, 0xFE, 0xF3, 0x84, 0x4F, 0x6C, 0x80, 0x0A, 0xA0, 
	0x08, 0x45, 0x2A, 0xEE, 0xAE, 0xA9, 0x7A, 0xE3, 0x1B, 0x16, 0xA1, 0xFA, 0x39, 0xAF, 0x19, 0x13, 
	0x99, 0x31, 0x8D, 0x68, 0x44, 0x0B, 0xB7, 0xD5, 0xBE, 0xD6, 0x74, 0x03, 0x80, 0x0D, 0xAC, 0x60, 
	0x01, 0xDB, 0x80, 0xAF, 0x8A, 0xF5, 0x8D, 0x13, 0x40, 0xEC, 0x12, 0xB5, 0xF9, 0x4F, 0xA1, 0x8E, 
	0xB4, 0xAD, 0xBA, 0x54, 0x81, 0x52, 0x0F, 0x0A, 0xD7, 0xBD, 0xC9, 0x15, 0x94, 0x5B, 0x33, 0x9F, 
	0xC5, 0x6A, 0x18, 0xC1, 0xBC, 0x62, 0x95, 0xAF, 0x7E, 0xE5, 0xEA, 0x60, 0x47, 0xAB, 0xC6, 0xC4, 
	0x2E, 0x60, 0x02, 0xA7, 0x35, 0xAD, 0x69, 0x2D, 0x49, 0x54, 0xB3, 0x96, 0xD5, 0x7B, 0x29, 0xB4, 
	0x23, 0x5B, 0x09, 0xAA, 0xD6, 0xA6, 0x2E, 0xE1, 0x03, 0x72, 0x95, 0xE1, 0xCD, 0xB4, 0xE8, 0x35, 
	0x8B, 0x79, 0x36, 0xA6, 0xCB, 0x0C, 0x6D, 0x5F, 0x81, 0x48, 0x5A, 0xC1, 0x56, 0xCA, 0xB0, 0xA8, 
	0x0D, 0xEB, 0x57, 0x1B, 0x20, 0xD4, 0x8F, 0x3A, 0x56, 0xA4, 0x27, 0x4C, 0x6A, 0x52, 0xD7, 0x2A, 
	0x59, 0x03, 0xDE, 0xCD, 0xB6, 0x96, 0xE5, 0x1B, 0x20, 0x45, 0x67, 0x3E, 0xF3, 0x2D, 0x01, 0xA2, 
	0xE0, 0xB5, 0xAA, 0x70, 0x87, 0xBB, 0x01, 0x0E, 0x00, 0xD6, 0xBC, 0xE5, 0x3D, 0xFF, 0x6F, 0x69, 
	0x91, 0x88, 0x5A, 0xB1, 0xB2, 0x97, 0xBD, 0xAC, 0xF5, 0xA7, 0x48, 0x3F, 0xDA, 0x36, 0x5D, 0x0A, 
	0xF4, 0x9B, 0x77, 0x53, 0x01, 0xDE, 0xB2, 0x2B, 0xD7, 0xCF, 0x75, 0xF7, 0x66, 0xA2, 0x73, 0xC2, 
	0x6F, 0x83, 0x3B, 0x5E, 0xAD, 0x12, 0xB7, 0xB8, 0x84, 0x45, 0x6E, 0x62, 0x95, 0x7B, 0x5A, 0x4B, 
	0x86, 0x14, 0xA0, 0xCC, 0x05, 0xC0, 0x12, 0xED, 0x58, 0x80, 0xA4, 0x56, 0x78, 0x8F, 0x27, 0xC8, 
	0xF0, 0x7E, 0x71, 0xCB, 0xE1, 0xCC, 0xEE, 0x16, 0xC0, 0x76, 0xB5, 0xAA, 0x29, 0x3D, 0x0B, 0xDA, 
	0xF1, 0x1E, 0x38, 0xB0, 0xE8, 0x45, 0x6F, 0x82, 0x4F, 0xAB, 0x58, 0x16, 0x2F, 0xD8, 0xC1, 0xB0, 
	0x8D, 0xB0, 0x84, 0x27, 0x6C, 0x61, 0x0B, 0x17, 0x40, 0x05, 0xF9, 0x3D, 0x01, 0x8E, 0x2D, 0xAB, 
	0x01, 0xDC, 0xD6, 0x95, 0xBB, 0x20, 0x6E, 0xC0, 0x08, 0x12, 0x30, 0xE4, 0x53, 0x3E, 0x54, 0x99, 
	0x25, 0x36, 0xB1, 0x60, 0x55, 0x8C, 0x62, 0xC2, 0x8A, 0x35, 0xB9, 0x50, 0x3E, 0x6D, 0x83, 0x61, 
	0x3B, 0xE3, 0x2A, 0x2F, 0x56, 0xB6, 0xDE, 0x5C, 0x82, 0x8E, 0x33, 0xEC, 0x04, 0x0E, 0xF3, 0xCD, 
	0xC3, 0x1F, 0x0E, 0x31, 0x04, 0xC7, 0x7C, 0x4A, 0x9A, 0x16, 0xF8, 0xAF, 0xE9, 0x45, 0xF0, 0x7A, 
	0x9F, 0xCC, 0xE6, 0x17, 0x2F, 0xC1, 0xCA, 0x12, 0xCE, 0xC0, 0x8C, 0xE7, 0x28, 0xD9, 0x3D, 0xAA, 
	0x00, 0xC7, 0x5C, 0xEE, 0xB2, 0x8F, 0xCD, 0x09, 0x62, 0xEE, 0x8E, 0x18, 0x82, 0x43, 0x0E, 0xB4, 
	0x88, 0x93, 0x1C, 0xDA, 0x13, 0x37, 0x79, 0xB0, 0x85, 0x0D, 0x2B, 0x8B, 0xB9, 0x00, 0x54, 0x38, 
	0xCB, 0xB9, 0xCA, 0x01, 0xAD, 0x70, 0x03, 0x32, 0x90, 0xD4, 0xFC, 0x76, 0xD9, 0xB2, 0x0D, 0x08, 
	0x01, 0x0E, 0x74, 0xFB, 0x61, 0x21, 0x13, 0x99, 0xCC, 0x63, 0x1E, 0x72, 0x03, 0xFF, 0x5C, 0xC0, 
	0x85, 0x42, 0x1F, 0xFA, 0xD0, 0xE6, 0x2D, 0xEC, 0x12, 0xA3, 0x00, 0xD6, 0x1B, 0x34, 0x3A, 0xCE, 
	0x8E, 0xA6, 0xF1, 0x1E, 0x29, 0x9D, 0x01, 0x1C, 0xEB, 0x57, 0x1F, 0x7B, 0xC3, 0xB4, 0x1F, 0x37, 
	0xBD, 0x04, 0x20, 0x8F, 0xB8, 0x8B, 0x80, 0x26, 0xF3, 0xA8, 0x5D, 0x40, 0x6C, 0x62, 0x97, 0xFA, 
	0x9D, 0x4C, 0x26, 0xEC, 0x31, 0xC0, 0x3A, 0x01, 0x57, 0xBB, 0xBA, 0xD9, 0xAF, 0x86, 0x33, 0xA4, 
	0x67, 0xAD, 0x82, 0x5A, 0x67, 0xE0, 0x77, 0x57, 0xB9, 0x2D, 0xDF, 0x36, 0x1D, 0x82, 0x1A, 0xF0, 
	0x3A, 0xC4, 0x82, 0x96, 0x40, 0xA8, 0x23, 0x58, 0xEC, 0x72, 0x9B, 0x3B, 0x7A, 0x1E, 0x48, 0xB7, 
	0xBA, 0x13, 0xED, 0x6C, 0x68, 0x7F, 0xF5, 0xD9, 0x32, 0x06, 0xC0, 0xA3, 0x1F, 0x2D, 0x61, 0x4B, 
	0xAA, 0xB5, 0xD6, 0x28, 0xA8, 0x35, 0xB6, 0x9D, 0xCA, 0x37, 0xB9, 0x76, 0x9B, 0xDB, 0x0A, 0x14, 
	0x25, 0xA8, 0xC9, 0x5C, 0x4A, 0x62, 0xC7, 0xC0, 0xE0, 0xE6, 0x2E, 0x76, 0x03, 0xD4, 0xCD, 0xF0, 
	0x86, 0xA7, 0xBB, 0xB0, 0xCF, 0x6E, 0x75, 0xAB, 0x23, 0x4C, 0xEF, 0x19, 0x3F, 0x7A, 0x09, 0xD5, 
	0xCE, 0x78, 0xAD, 0x5D, 0x98, 0xED, 0x2F, 0xD7, 0xE0, 0xDF, 0x1F, 0xF7, 0xB6, 0x02, 0x3D, 0x1D, 
	0x6C, 0x22, 0x1B, 0x20, 0xD0, 0xC3, 0x96, 0x01, 0xB1, 0x55, 0xEE, 0x82, 0x18, 0xB0, 0xFC, 0xE0, 
	0x0B, 0x77, 0xB8, 0xCC, 0xD9, 0x0D, 0xED, 0x67, 0xB7, 0x3B, 0xDA, 0x16, 0x9F, 0xF3, 0x12, 0x68, 
	0x5D, 0x6D, 0x6B, 0xE3, 0x98, 0x05, 0x1D, 0x97, 0x2B, 0xC0, 0x01, 0x9E, 0xB3, 0xA9, 0x8E, 0x00, 
	0xD8, 0xC2, 0x4E, 0xB8, 0xB9, 0x59, 0x1E, 0x73, 0x99, 0x37, 0xBC, 0x01, 0x11, 0x67, 0x36, 0xB3, 
	0x65, 0x5C, 0xF1, 0x25, 0xD0, 0x80, 0x06, 0xF7, 0xCE, 0x78, 0xCF, 0xAF, 0x6D, 0xFF, 0x82, 0x8E, 
	0xF7, 0x5A, 0xE4, 0x9B, 0x0E, 0xBB, 0x02, 0x25, 0x20, 0x81, 0xA9, 0x7E, 0x5A, 0xD0, 0xA3, 0x3E, 
	0x78, 0xB1, 0xD5, 0x7E, 0x6E, 0xA7, 0x3B, 0x3C, 0xD1, 0x35, 0x8F, 0x7B, 0xBB, 0x27, 0x0D, 0xE9, 
	0x06, 0x5C, 0xFD, 0xEA, 0x93, 0xA6, 0x6E, 0xBE, 0x51, 0xB0, 0xE3, 0xFF, 0xE4, 0x3A, 0x86, 0xDD, 
	0x0E, 0xFC, 0xC7, 0x1B, 0x40, 0xF6, 0xC2, 0x9B, 0x7D, 0xCC, 0x29, 0x5F, 0x7A, 0xC2, 0x9B, 0xEE, 
	0xF6, 0x75, 0xBB, 0x5B, 0xEA, 0x60, 0x95, 0xB1, 0xD5, 0xEF, 0x8E, 0xF7, 0xAD, 0x67, 0xBC, 0x01, 
	0x03, 0xE0, 0x63, 0xBF, 0x63, 0x28, 0x76, 0x5E, 0x4B, 0xC0, 0x00, 0x9F, 0x37, 0xFC, 0x9F, 0x87, 
	0x6D, 0x70, 0x96, 0x97, 0xBE, 0xE5, 0xA4, 0x6E, 0xFC, 0xD3, 0x7F, 0xDA, 0xEE, 0xD6, 0x43, 0x9B, 
	0xB9, 0x93, 0xA7, 0xFC, 0xD5, 0xAF, 0xCD, 0x77, 0x4A, 0xE3, 0x78, 0x00, 0x99, 0xF7, 0xFB, 0x12, 
	0x20, 0xE0, 0x6F, 0x91, 0xE7, 0xAC, 0xF0, 0xA1, 0x17, 0x3D, 0xB9, 0x95, 0xAE, 0x74, 0xC6, 0xAB, 
	0x1E, 0xEA, 0x90, 0x9F, 0x7A, 0xEC, 0x65, 0x7F, 0xF7, 0xAC, 0x7B, 0x0F, 0xF7, 0x9D, 0x21, 0xC4, 
	0x07, 0x78, 0xCF, 0x79, 0x91, 0x13, 0x1E, 0xF4, 0x23, 0x20, 0x3B, 0xE8, 0x45, 0xBF, 0x04, 0xB6, 
	0xA3, 0xDE, 0xFB, 0xA9, 0x57, 0xBD, 0xE3, 0x59, 0x5F, 0x73, 0x2E, 0x1C, 0xC0, 0xEE, 0x77, 0xCF, 
	0x00, 0xF3, 0xB1, 0x6E, 0x6D, 0xCC, 0x43, 0x1F, 0x78, 0xFD, 0xA6, 0x7E, 0xC8, 0x7F, 0x0F, 0xFC, 
	0xED, 0x6F, 0xBF, 0xEC, 0x23, 0x18, 0x01, 0x17, 0x56, 0x8E, 0xF0, 0x18, 0x18, 0xBF, 0xF1, 0x10, 
	0x07, 0x54, 0x51, 0x70, 0x00, 0x04, 0x58, 0x80, 0xE8, 0xB7, 0x7E, 0xCD, 0x57, 0x6D, 0xEE, 0xF7, 
	0x7E, 0xA8, 0xD0, 0x00, 0xBC, 0x47, 0x7D, 0x9E, 0x97, 0x7D, 0xF7, 0x97, 0x7D, 0xFF, 0xC1, 0x37, 
	0x6A, 0xF9, 0x77, 0x81, 0xA4, 0x16, 0x05, 0x0A, 0xE7, 0x74, 0xD1, 0x53, 0x80, 0x04, 0xE8, 0x01, 
	0x1F, 0x48, 0x80, 0x07, 0x88, 0x80, 0xEC, 0xB7, 0x80, 0x0C, 0x28, 0x2F, 0x7B, 0xF3, 0x80, 0xFD, 
	0x75, 0x7F, 0xC1, 0x67, 0x7F, 0xF8, 0x37, 0x02, 0x2E, 0x00, 0x83, 0x17, 0x38, 0x83, 0x5D, 0xF5, 
	0x02, 0x36, 0x78, 0x83, 0x2F, 0xD0, 0x00, 0x1E, 0x78, 0x00, 0x20, 0xB8, 0x83, 0x20, 0x38, 0x82, 
	0x08, 0xB8, 0x04, 0xB8, 0x37, 0x84, 0x99, 0x27, 0x31, 0xB8, 0xA5, 0x82, 0x4B, 0xA0, 0x7D, 0xC1, 
	0x57, 0x78, 0x14, 0xE8, 0x69, 0x33, 0x18, 0x83, 0x50, 0x28, 0x83, 0xFA, 0x87, 0x83, 0x38, 0xE8, 
	0x02, 0x2F, 0x60, 0x85, 0x58, 0xA8, 0x83, 0x07, 0xF0, 0x02, 0x3C, 0x58, 0x80, 0x20, 0xC8, 0x85, 
	0x06, 0x48, 0x82, 0x4E, 0x40, 0x84, 0x27, 0x88, 0x0A, 0xD3, 0x77, 0x84, 0xD3, 0xE7, 0x04, 0x4C, 
	0xB8, 0x84, 0x9F, 0x27, 0x64, 0xF9, 0x47, 0x64, 0x33, 0x18, 0x87, 0xA4, 0x76, 0x85, 0x74, 0x88, 
	0x85, 0x30, 0x60, 0x83, 0x2E, 0x70, 0x87, 0x39, 0xD8, 0x85, 0x05, 0x08, 0x86, 0x5E, 0x28, 0x82, 
	0xEB, 0x67, 0x75, 0xE7, 0x47, 0x86, 0x33, 0x23, 0x7D, 0xBC, 0x77, 0x86, 0x0D, 0x80, 0x03, 0x38, 
	0xA0, 0x86, 0x4A, 0xB8, 0x7D, 0x16, 0xF8, 0x86, 0x82, 0x16, 0x89, 0x53, 0x78, 0x85, 0x7A, 0x88, 
	0x85, 0x75, 0x68, 0x83, 0x5A, 0xC8, 0x87, 0x5D, 0xD8, 0x83, 0x5D, 0x08, 0x84, 0x56, 0x37, 0x00, 
	0x07, 0x90, 0x79, 0x44, 0xD8, 0x00, 0x40, 0x57, 0x05, 0x5D, 0xA6, 0x01, 0x87, 0x98, 0x88, 0x8A, 
	0xA8, 0x88, 0x49, 0xB8, 0x86, 0x6E, 0x18, 0x83, 0x71, 0xF8, 0x84, 0xFA, 0x67, 0x85, 0x97, 0xF8, 
	0x02, 0x30, 0x40, 0x8B, 0x74, 0xA8, 0x83, 0x9C, 0xE8, 0x87, 0x04, 0xE8, 0x87, 0xFF, 0x07, 0xB8, 
	0x04, 0xA1, 0xE8, 0x81, 0x0B, 0x48, 0x8A, 0x85, 0x08, 0x07, 0x7C, 0xF3, 0x80, 0x10, 0xA0, 0x8A, 
	0xAB, 0xC8, 0x8A, 0x5B, 0x63, 0x78, 0x81, 0x26, 0x83, 0x31, 0x48, 0x64, 0x51, 0x38, 0x89, 0x79, 
	0x88, 0x8B, 0x96, 0x48, 0x8B, 0x99, 0xC8, 0x87, 0x60, 0xD8, 0x83, 0x3F, 0x88, 0x75, 0x98, 0x17, 
	0x8C, 0xC1, 0x28, 0x8A, 0x99, 0xC7, 0x02, 0xA5, 0xB8, 0x2A, 0x5F, 0x76, 0x86, 0xDB, 0xB6, 0x8C, 
	0xAB, 0xA8, 0x86, 0x8F, 0x18, 0x8B, 0x71, 0x18, 0x41, 0x94, 0x48, 0x8B, 0xB8, 0x48, 0x89, 0x57, 
	0xA8, 0x8B, 0x9A, 0x18, 0x82, 0x06, 0x08, 0x8C, 0x04, 0x08, 0x8A, 0xE0, 0x38, 0x88, 0xA4, 0x48, 
	0x8E, 0x79, 0x23, 0x7D, 0xF1, 0x97, 0x8E, 0xEA, 0xB8, 0x8C, 0x1A, 0x18, 0x89, 0xD1, 0xF8, 0x8C, 
	0x0D, 0x50, 0x85, 0x75, 0x98, 0x87, 0x74, 0xB8, 0x87, 0x1E, 0x08, 0x82, 0x3D, 0x18, 0x05, 0x30, 
	0xA0, 0x83, 0xA0, 0xC8, 0x8F, 0x05, 0x08, 0x8A, 0x4B, 0x40, 0x8E, 0xE5, 0xC8, 0x0F, 0x9B, 0x97, 
	0x86, 0x05, 0x59, 0x90, 0x84, 0x57, 0x76, 0x51, 0x10, 0x8B, 0x31, 0x38, 0x6A, 0x74, 0x78, 0x8B, 
	0x36, 0x78, 0x87, 0xF2, 0x08, 0x91, 0x5B, 0x28, 0x82, 0x4E, 0x00, 0x03, 0x32, 0x29, 0x93, 0xDF, 
	0x98, 0x91, 0xFD, 0xF8, 0x8F, 0x03, 0xD0, 0x75, 0xE7, 0x80, 0x69, 0x04, 0x19, 0x92, 0xAC, 0x28, 
	0x01, 0x02, 0x40, 0x76, 0x4D, 0x78, 0x90, 0xFA, 0xA7, 0x92, 0x97, 0x68, 0x8D, 0x98, 0xF8, 0x83, 
	0x31, 0x39, 0x93, 0x4C, 0x59, 0x91, 0xFB, 0xD8, 0x8F, 0xE7, 0x27, 0x84, 0x26, 0xD0, 0x91, 0x58, 
	0x90, 0x6B, 0x5F, 0xA6, 0x8C, 0x3E, 0xB9, 0x88, 0x12, 0xE0, 0x00, 0x23, 0xE0, 0x00, 0x40, 0x29, 
	0x94, 0xDC, 0xD7, 0x55, 0xB4, 0xD0, 0x94, 0x64, 0x49, 0x93, 0x50, 0xA9, 0x91, 0xFF, 0x40, 0x47, 
	0x8E, 0x45, 0x68, 0x0D, 0xFD, 0x76, 0x95, 0x58, 0xA9, 0x8E, 0x42, 0x06, 0x96, 0x5C, 0xE9, 0x95, 
	0x41, 0x19, 0x94, 0x64, 0xD7, 0x00, 0x02, 0x90, 0x97, 0x7A, 0xB9, 0x97, 0x7C, 0xD9, 0x00, 0x65, 
	0xF9, 0x97, 0x99, 0xC8, 0x8F, 0x1B, 0x49, 0x01, 0xFF, 0x58, 0x8C, 0xE6, 0xF8, 0x77, 0x10, 0x90, 
	0x8C, 0x6F, 0xC9, 0x8A, 0x74, 0xF9, 0x95, 0x12, 0x40, 0x81, 0x5E, 0xD9, 0x95, 0xFA, 0xC7, 0x97, 
	0x94, 0xB9, 0x97, 0x7E, 0xF9, 0x97, 0x64, 0x69, 0x91, 0x30, 0x49, 0x01, 0x2C, 0xC0, 0x99, 0xC4, 
	0x18, 0x90, 0x1E, 0x99, 0x6B, 0x10, 0x50, 0x02, 0x89, 0xE9, 0x04, 0x06, 0xF9, 0x95, 0x91, 0xE9, 
	0x00, 0x75, 0xF9, 0x95, 0xD9, 0x87, 0x97, 0x95, 0x59, 0x99, 0x97, 0x89, 0x99, 0x4C, 0x59, 0x93, 
	0xFF, 0xD8, 0x99, 0x69, 0x69, 0x98, 0xE7, 0x70, 0x8C, 0xA3, 0xB9, 0x9B, 0x8A, 0x99, 0x88, 0x23, 
	0xF9, 0x98, 0x5B, 0x09, 0x99, 0x64, 0x27, 0x00, 0x5E, 0xE9, 0x9A, 0xAF, 0xD9, 0x97, 0xB2, 0xD9, 
	0x94, 0x3A, 0x38, 0x98, 0x9C, 0xD9, 0x99, 0x4B, 0xA0, 0x93, 0xC6, 0x80, 0x98, 0x89, 0xC9, 0x9B, 
	0xA5, 0xE9, 0x04, 0xAB, 0xC9, 0x9A, 0x76, 0xD9, 0x9A, 0xC7, 0x49, 0x99, 0xB1, 0x29, 0x9B, 0x51, 
	0x60, 0x9B, 0xE4, 0x08, 0x03, 0x00, 0x09, 0x9D, 0x94, 0x70, 0x5B, 0x0D, 0x40, 0x9A, 0x25, 0x40, 
	0x9A, 0xBB, 0x49, 0x9A, 0x0D, 0x30, 0x03, 0x33, 0xC0, 0x05, 0x8E, 0xB9, 0x9A, 0xC6, 0xB9, 0x9D, 
	0x79, 0xD9, 0x9D, 0xB3, 0x19, 0x05, 0x14, 0x40, 0x98, 0x9C, 0xB9, 0x9F, 0x9E, 0xD9, 0x00, 0xE4, 
	0x59, 0x09, 0x4E, 0x40, 0x9D, 0xE8, 0x59, 0x9A, 0xEE, 0x59, 0xA0, 0xEE, 0x09, 0x9F, 0xC5, 0x49, 
	0x9F, 0x96, 0x39, 0x93, 0x5C, 0x90, 0x9F, 0x0E, 0xAA, 0x9F, 0x1C, 0xD9, 0x9C, 0xFF, 0xCF, 0x09, 
	0x9A, 0x47, 0x12, 0x05, 0xBB, 0x39, 0x7D, 0xA3, 0xD9, 0x9E, 0x06, 0xBA, 0xA1, 0x07, 0x2A, 0x96, 
	0x4A, 0xF1, 0xA0, 0x20, 0x9A, 0x9F, 0xA4, 0x98, 0x9F, 0x00, 0xE9, 0x9F, 0xB8, 0x19, 0x1E, 0x1F, 
	0x50, 0x02, 0xBB, 0x97, 0x9E, 0xD3, 0xA9, 0xA1, 0x1C, 0xCA, 0xA1, 0x0D, 0xE0, 0x00, 0x32, 0x3A, 
	0xA3, 0x34, 0x5A, 0xA3, 0x0D, 0x10, 0xA2, 0x0E, 0x2A, 0x00, 0x0F, 0x7A, 0xA3, 0x25, 0x6A, 0x02, 
	0x5D, 0x47, 0xA1, 0xD1, 0x79, 0x9E, 0x5F, 0x96, 0x98, 0xEC, 0xE9, 0x9E, 0x0E, 0x60, 0xA4, 0x05, 
	0x7A, 0xA4, 0x05, 0x1A, 0xA3, 0x35, 0xDA, 0xA4, 0x33, 0x7A, 0xA3, 0x14, 0xA0, 0xA3, 0xF9, 0x29, 
	0xA5, 0x51, 0x1A, 0xA5, 0xFF, 0x48, 0x98, 0x3E, 0xFA, 0xA3, 0x21, 0xE1, 0x54, 0x4B, 0xA0, 0xA2, 
	0x2A, 0xBA, 0x7B, 0x2E, 0xFA, 0xA2, 0x33, 0xA0, 0xA4, 0x4C, 0xEA, 0xA4, 0x4D, 0x0A, 0xA5, 0x38, 
	0xBA, 0xA3, 0x84, 0xE9, 0x9F, 0x59, 0x0A, 0xA4, 0xA6, 0x70, 0x9E, 0x70, 0xEA, 0xA5, 0x51, 0x80, 
	0xA4, 0x63, 0x0A, 0xA3, 0x66, 0xEA, 0xA4, 0x37, 0x4A, 0xA5, 0x7A, 0x2A, 0xA2, 0xCF, 0xD9, 0xA6, 
	0x6E, 0x7A, 0x0A, 0x5D, 0x1A, 0xA7, 0x82, 0x3A, 0xA7, 0x4A, 0x5A, 0xA7, 0x65, 0x7A, 0xA7, 0x4F, 
	0x8A, 0xA3, 0x85, 0x90, 0xA5, 0x3F, 0xFA, 0xA7, 0x9E, 0x11, 0xA8, 0x5F, 0x9A, 0x9E, 0x92, 0xFA, 
	0xA5, 0x4E, 0x70, 0xA0, 0x88, 0x6A, 0xA3, 0x7C, 0xEA, 0x04, 0x29, 0xB0, 0xA9, 0x9B, 0xCA, 0xA6, 
	0x81, 0xBA, 0xA5, 0x5B, 0xC0, 0x05, 0x5E, 0x5A, 0x02, 0x18, 0x90, 0x9E, 0xA5, 0x3A, 0xA9, 0x1E, 
	0x4A, 0x0B, 0x9C, 0xCA, 0xA9, 0x26, 0xD0, 0xA9, 0x3F, 0xFA, 0xA5, 0xA0, 0x6A, 0x8A, 0xB4, 0x30, 
	0xA9, 0x92, 0x7A, 0xAA, 0xE9, 0xD9, 0x00, 0x18, 0x90, 0xAB, 0xBA, 0xBA, 0xAB, 0x60, 0xBC, 0xDA, 
	0x00, 0xAB, 0xFA, 0xAB, 0x29, 0xE0, 0x04, 0xB0, 0x1A, 0xAB, 0xFC, 0x50, 0x08, 0xB4, 0x3A, 0xA9, 
	0xA5, 0x8A, 0xAB, 0xBC, 0xBA, 0xAC, 0xBA, 0xEA, 0xAB, 0xC0, 0x4A, 0x0B, 0xC4, 0x7A, 0x26, 0x4A, 
	0x71, 0xAC, 0x2A, 0xCA, 0xAC, 0xCB, 0xEA, 0xAC, 0xAD, 0x1A, 0xAC, 0x5D, 0x3A, 0xAC, 0xD1, 0x5A, 
	0xA1, 0x90, 0x2A, 0xAA, 0xD5, 0x6A, 0xAD, 0xBB, 0xEA, 0xAC, 0xC2, 0x4A, 0xA9, 0xDD, 0x0A, 0xA0, 
	0x51, 0x20, 0xA7, 0x94, 0x9A, 0xAA, 0xE7, 0x69, 0xAE, 0xE7, 0xDA, 0x17, 0xA2, 0x1A, 0xA7, 0xD4, 
	0x2A, 0xA9, 0x70, 0x1A, 0x7D, 0xEF, 0x1A, 0xAB, 0xEC, 0xEA, 0xA8, 0xA7, 0x10, 0x04, 0x00, 0x3B, 
};


// ユーザー定義データ構造体：ColorKanjiLCDLoadImgSize 構造体の派生構造体にしてください
// ユーザーが自由に使えるデータです。通常は画像データに関する構造体にします。
struct UserDefData : public ColorKanjiLCDLoadImgSize {
  uint16_t nPos;  // Rotation Sphere GIF パターン画像データの位置
};


uint8_t left_sw, right_sw, up_sw, down_sw; // カラー漢字 LCD シールドの４つのスイッチの状態：以下で実際に使用しているのは右SW だけ


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

#if 1
  // 注意：ここで、強制的に TWI 通信を高速化させています。カラー漢字 LCD シールドは 400kHz の速度に対応していますが、
  //       標準 100kHz の速度にしか対応していない別の TWI 機器を同時に使用する場合、そちらの TWI 機器が正常に TWI 通信できなくなる可能性があります。
  //       その場合は、ここの「#if 1」の "1" を "0" に書き換えて、TWI 通信の高速化を行わない(つまり標準 100kHz のまま)ようにしてください。
  //       当然ですが、そうした場合は、カラー漢字 LCD シールドとの TWI 通信速度も遅くなってしまいます。
  CKLCD_Wire_setHiSpeed();
#endif


  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // 漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが CKLCD_TWI_ADDR と一致しているか確認してください。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_B1); // Back面：仮想画面１(160×256) を使用


  // スイッチは押されていない状態にする
  left_sw = right_sw = up_sw = down_sw = false;
}


// loadGif 関数から呼び出されるユーザー定義 callback 関数(Load)
// buff に buff_size 分の画像データを転送してください。正常に転送できた時は CKLCD_ERR_NONE を返してください。
// もし転送できなかったら CKLCD_ERR_NONE 以外（例えば CKLCD_ERR_UNKNOWN）を返してください。その時は loadGif 関数の処理を中断します。
errCode_t UserDefLoadGifCallback (uint8_t * buff, uint8_t buff_size, ColorKanjiLCDLoadImgSize * user_def_data)
{
  UserDefData * udd = (UserDefData *)user_def_data; // ユーザー定義データ構造体にキャストする：ダウンキャストなので要注意！

  // buff ポインタが指すメモリに buff_size バイト分の Rotation Sphere GIF パターン画像データを転送します。
  while (buff_size) { // 0バイトになるまで繰り返す
    uint8_t dat = pgm_read_byte(&RotSphereGif[udd->nPos]); // プログラムメモリから読み出す
    *buff++ = dat;
    udd->nPos++; // Rotation Sphere GIF パターン画像データの位置を1バイト進める
    --buff_size;
  }

  // 正常に転送できた
  return CKLCD_ERR_NONE;
}


// loadGif 関数から呼び出されるユーザー定義 callback 関数(Frame)
// アニメーション GIF の時のみ、各フレーム毎に呼び出されます。flip 関数で LCD に表示させると、アニメーションしている様に演出できます。
// ただし、最終フレームの時は、この callback 関数は呼ばれずに、loadGif 関数自体が正常終了しますので、最終フレームだけは、他の loadXxx 関数と同じ手順で表示させてください。
// また、アニメーション GIF でない単なる GIF 画像の場合、この callback 関数が呼び出される事はありません。他の loadXxx 関数と同じ手順で表示させてください。
// なお、この callback 関数内では、以下の '★' 印の付いた関数だけしか呼び出せませんので、注意して下さい。これ以外の関数を呼び出すと、loadGif 関数が中断される事になります。
// 特に print/write 系関数は一切、呼び出せない（CKLCD 内の作業メモリが足らないので不可能）ので、アニメーションの最中に、文字を表示させる事はできません。
// 一方、タクトSW の現在の状態は読み出せますので、タクトSW が押されたらアニメーション GIF をコマ送りする事は可能です。
errCode_t UserDefGifFrameCallback (ColorKanjiLCDLoadImgSize * user_def_data)
{
  UserDefData * udd = (UserDefData *)user_def_data; // ユーザー定義データ構造体にキャストする：ダウンキャストなので要注意！

  // LCD 全領域を描画させます
//  cklcd.flip();  ← これだとより時間が掛かる

  // アニメーション GIF 描画範囲だけを LCD に描画させます
  // x_offset, y_offset も加算しているのは、アニメーション GIF の場合、フレームによって描画位置を矩形範囲が変化する為です。
  int16_t flipLeft   = imgLeft  + udd->x_offset;
  int16_t flipTop    = imgTop   + udd->y_offset;
  int16_t flipRight  = flipLeft + udd->width  - 1;
  int16_t flipBottom = flipTop  + udd->height - 1;
  cklcd.flip (flipLeft, flipTop, flipRight, flipBottom);

  // このユーザー定義 callback 関数(Frame)で、少なくとも一度は、タクト SW のチェックを行います。
  // ただし、フレームとフレームにまたがって、タクトSW の ON/OFF の変化を検出させる場合は、
  // フレーム描画に時間が掛かる場合（通常は寸法の大きなアニメーション GIF）に、検出の反応が悪くなる場合があります。
  while(1) {
    // 最初にカラー漢字 LCD の現在のステータスを取得します。
    // これを呼び出しておかないと、以下の getRightSW 関数, getLeftSW 関数が
    // 何時までも更新されない事になってしまいます。
    // カラー漢字 LCD がコマンド処理中で忙しい時は、ステータス取得に失敗して CKLCD_ERR_BUSY が返って来ます。
    // ステータス取得成功を意味する CKLCD_ERR_NONE が返ってきた時だけ、処理を進めるようにして下さい。
    if (cklcd.getStatus() == CKLCD_ERR_NONE) {
      // ステータス取得成功した

      // 右スイッチの処理：押されている間はアニメーションを停止する
      if (cklcd.getRightSW()) { // 右スイッチが押されている
        // 押され続けている間はフラグを立てるだけの処理
        right_sw = true;
      } else { // 右スイッチが離された
        if (right_sw == true) { // さっきまで右スイッチが押されていた
          // 右スイッチ・フラグをクリア
          right_sw = false;
        }
      }
    }

    // 右スイッチが離されていたら、while 文を break させます。
    if (right_sw == false) {
      break;
    }

    // 10ms 待ちを入れる。
    // カラー漢字 LCD シールドのスイッチのチェック間隔が約 10ms (変更不可) なので、
    // これよりも短い間隔で getStatus 関数を呼び出しても、前と同じスイッチ状態が読み出されるだけです。
    // なので、10ms 以上の間をおいてから、getStatus 関数を呼び出すようにしてください。
    delay(10);
  }

  // 正常に転送できた
  return CKLCD_ERR_NONE;
}


// ループ関数
void loop()
{
  // PROGMEM 上のアニメーション GIF ファイルを、カラー漢字 LCD シールドに転送します

  // ユーザー定義データ構造体
  UserDefData udd;
  udd.nPos = 0;  // Rotation Sphere GIF 画像データの位置を0にし、先頭から開始させます

  // "読込み中"メッセージを表示させます（仮想画面１を使用）。
//  cklcd.v1.print ("GIF画像読込み中...");

  // GIF 読み込み予定位置を暗い緑で塗りつぶしておきます。こうする事で GIF の端が分かりやすくなります。
//  cklcd.v1.setBackColor (CKLCD_COL32_DARK_GREEN); // 暗い緑
//  cklcd.v1.clearRect (0, 0, 159, 127);

  // 仮想画面１の(30,4)に、Rotation Sphere GIF 画像データを読み込ませます。エラー発生時はメッセージを表示させます。
  errCode_t res = cklcd.v1.loadGif (imgLeft, imgTop, CKLCD_LOAD_IMG_DISP_ERR_MSG, sizeof(RotSphereGif), UserDefLoadGifCallback, UserDefGifFrameCallback, &udd);

//  cklcd.v1.setBackColor (CKLCD_COL32_BLACK); // 黒に戻す

  if (res != CKLCD_ERR_NONE) { // エラーの時はエラーメッセージを表示
    cklcd.v1.print ("\nErrCode=0x");
    cklcd.v1.print (res, HEX);
  }

  // LCD 全領域を描画させます
//  cklcd.flip();  ← これだとより時間が掛かる

  // アニメーション GIF 描画範囲だけを LCD に描画させます
  // x_offset, y_offset も加算しているのは、アニメーション GIF の場合、フレームによって描画位置を矩形範囲が変化する為です。
  int16_t flipLeft   = imgLeft  + udd.x_offset;
  int16_t flipTop    = imgTop   + udd.y_offset;
  int16_t flipRight  = flipLeft + udd.width  - 1;
  int16_t flipBottom = flipTop  + udd.height - 1;
  cklcd.flip (flipLeft, flipTop, flipRight, flipBottom);
}

