/*
  Color Kanji LCD Shield - Rotation Demo #2

  本スケッチは "カラー漢字 LCD シールド" の回転＆拡大/縮小のスケッチ例です。PNG 画像を背景表示します。
  コンパイルすると 20kB 強になるので、Arduino Uno (ATmega328P) 等のフラッシュメモリが 32kB 以上でないと動作しません。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 1/aug/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// カラー漢字 LCD 用ライブラリ
#include <ColorKanjiLCD.h>


// カラー漢字 LCD シールドの TWI アドレスを 0x3C 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、カラー漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define CKLCD_TWI_ADDR  0x3C


// カラー漢字 LCD のインスタンス生成
ColorKanjiLCD cklcd (CKLCD_TWI_ADDR); // カラー漢字 LCD シールドの TWI アドレスを指定する


// サイコロ PNG パターン : サイコロ４つ : 120×90 画素 : Full RGBA non-interlaced
//   Title: PNG transparency demonstration 1.png の Thumbnail
//   Description: English: PNG transparency demonstration, 24bit PNG with 8bit alpha layer
//   Source: Own work: Rendered in POV-Ray by user:ed_g2s.
//   Author: POV-Ray source code   ←←← 本当は ed_g2s 氏だと思うが、下記 url ではこうなってます
//   This file is licensed under the Creative Commons Attribution-Share Alike 3.0 Unported license.
//   このファイルはクリエイティブ・コモンズ 表示-継承 3.0 非移植ライセンスのもとに利用を許諾されています。
//   from http://en.wikipedia.org/wiki/File:PNG_transparency_demonstration_1.png
// ※巨大なので、プログラムメモリに格納します。PROGMEM 指定が必須です。
PROGMEM const uint8_t DicesPng[] = {
	0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A, 0x00, 0x00, 0x00, 0x0D, 0x49, 0x48, 0x44, 0x52, 
	0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 0x00, 0x5A, 0x08, 0x06, 0x00, 0x00, 0x00, 0x73, 0x00, 0x92, 
	0xEF, 0x00, 0x00, 0x00, 0x06, 0x62, 0x4B, 0x47, 0x44, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0xA0, 
	0xBD, 0xA7, 0x93, 0x00, 0x00, 0x00, 0x09, 0x70, 0x48, 0x59, 0x73, 0x00, 0x00, 0x00, 0x48, 0x00, 
	0x00, 0x00, 0x48, 0x00, 0x46, 0xC9, 0x6B, 0x3E, 0x00, 0x00, 0x2E, 0xF2, 0x49, 0x44, 0x41, 0x54, 
	0x78, 0xDA, 0xED, 0xBD, 0x79, 0x90, 0x5D, 0xF7, 0x75, 0xDF, 0xF9, 0x3D, 0xBF, 0xE5, 0x6E, 0xEF, 
	0xDD, 0xF7, 0x7A, 0x6F, 0x00, 0xDD, 0xD8, 0x48, 0x80, 0x3B, 0x41, 0x8A, 0x14, 0x45, 0xC9, 0x14, 
	0x29, 0x51, 0x8A, 0x62, 0x9B, 0xA2, 0x6C, 0x49, 0x89, 0x54, 0xA2, 0xA2, 0xD8, 0xF2, 0x56, 0x4A, 
	0xA2, 0x8C, 0x62, 0x8D, 0xB7, 0x1A, 0xDB, 0x93, 0xD4, 0xD4, 0x24, 0x8E, 0xC6, 0x19, 0xD9, 0x7F, 
	0x4C, 0xEC, 0x4A, 0x3C, 0x89, 0x1D, 0x4B, 0x13, 0x53, 0xB2, 0x2D, 0x4B, 0xF2, 0x48, 0xA6, 0x16, 
	0x93, 0xDA, 0xB8, 0x81, 0x1B, 0x40, 0x12, 0x20, 0x40, 0xEC, 0x68, 0xF4, 0xBE, 0xBE, 0xE5, 0xBE, 
	0xBB, 0xFD, 0x96, 0x33, 0x7F, 0x34, 0x40, 0x51, 0x92, 0x13, 0xD9, 0x8E, 0x28, 0x01, 0x1E, 0x7C, 
	0xAB, 0xBA, 0xD0, 0xD5, 0xA8, 0x7A, 0x7D, 0xEF, 0xEF, 0xD3, 0xE7, 0x9C, 0xDF, 0x59, 0x7E, 0xF7, 
	0x02, 0x97, 0x75, 0x59, 0x97, 0x75, 0xE9, 0x8A, 0x2E, 0x9E, 0x4B, 0xE1, 0x8B, 0xFB, 0xF2, 0x2E, 
	0x51, 0xA9, 0x1F, 0xD4, 0x2F, 0x96, 0xF2, 0x51, 0x38, 0x77, 0x50, 0x0C, 0x0F, 0xBF, 0xA5, 0xD1, 
	0x6E, 0xEF, 0x1E, 0x21, 0xC2, 0x18, 0x80, 0x21, 0xA5, 0x10, 0xA7, 0x69, 0x6E, 0x99, 0xAB, 0x2A, 
	0xCB, 0x8A, 0x81, 0xB5, 0xAE, 0xE7, 0x7D, 0x77, 0xC3, 0xDA, 0xAA, 0x33, 0x3B, 0xBB, 0xBB, 0x1A, 
	0x1D, 0x3D, 0x86, 0xF5, 0xF5, 0xAB, 0x2F, 0x93, 0xBB, 0x18, 0x01, 0x37, 0x1A, 0x27, 0x20, 0xE5, 
	0x95, 0xA8, 0xEB, 0xC3, 0x21, 0xD1, 0xFA, 0x4E, 0xAD, 0xF7, 0xDC, 0x14, 0x86, 0xB8, 0x99, 0x08, 
	0xD7, 0x49, 0x89, 0x69, 0x21, 0x7C, 0x33, 0x08, 0x8C, 0x12, 0xC2, 0xFA, 0xB2, 0xD4, 0xC6, 0x39, 
	0x2E, 0x99, 0x8B, 0x1E, 0x50, 0x9F, 0x25, 0xAA, 0x0F, 0x6C, 0xDB, 0xF6, 0xD0, 0x63, 0xC0, 0xFE, 
	0x23, 0x00, 0x32, 0xA5, 0xDE, 0x03, 0x6B, 0x3F, 0xF1, 0x03, 0x59, 0xB4, 0xE4, 0x1D, 0x09, 0xA2, 
	0x24, 0xC2, 0xC6, 0xD2, 0x86, 0x62, 0xCD, 0x09, 0x22, 0x24, 0xD1, 0x44, 0x14, 0xA5, 0x57, 0xA4, 
	0x42, 0x8E, 0xC9, 0x9C, 0x52, 0xEA, 0x05, 0x5B, 0x83, 0xF2, 0xEC, 0x9D, 0x67, 0xFD, 0x95, 0x47, 
	0xAE, 0xC4, 0xC9, 0x6B, 0x4F, 0xFE, 0xDD, 0x77, 0xD1, 0x5B, 0xB6, 0x9C, 0x83, 0xF7, 0x5A, 0x13, 
	0x55, 0x57, 0xB6, 0x5A, 0xFE, 0x4D, 0xD3, 0xD3, 0xE6, 0xEF, 0xB5, 0xDB, 0x9D, 0x7D, 0xAB, 0xAB, 
	0xE9, 0xE4, 0xC6, 0xC6, 0xEE, 0x44, 0x4A, 0x12, 0xCC, 0x19, 0x98, 0x0B, 0x38, 0x27, 0x51, 0xD7, 
	0x09, 0xBC, 0x2F, 0x00, 0x0C, 0x40, 0x64, 0x9D, 0xF7, 0x55, 0xA7, 0x28, 0xE6, 0x0F, 0x15, 0xC5, 
	0x91, 0x4F, 0xE7, 0xF9, 0xCC, 0x9F, 0x5B, 0xBB, 0x7A, 0x96, 0xA8, 0xE1, 0x99, 0xFF, 0xC3, 0xF7, 
	0x6F, 0xB5, 0x6E, 0x00, 0x70, 0x08, 0x10, 0x3F, 0x22, 0x92, 0xA0, 0xA9, 0x77, 0xC8, 0x50, 0x5D, 
	0x2B, 0x22, 0x71, 0x95, 0x6C, 0xC8, 0x6D, 0xBA, 0xA5, 0x5B, 0xC1, 0x48, 0xA0, 0xB8, 0xC5, 0x7D, 
	0x8C, 0xE0, 0x64, 0x30, 0x11, 0x1C, 0x94, 0xB1, 0x3C, 0x02, 0x8B, 0x65, 0x08, 0x58, 0x78, 0xE0, 
	0xE4, 0xED, 0x27, 0xFF, 0xEE, 0x01, 0xDE, 0xBA, 0xF5, 0x2C, 0x80, 0x18, 0x40, 0x7F, 0x52, 0x88, 
	0xFA, 0xCD, 0xED, 0x36, 0xFE, 0xC1, 0x1D, 0x77, 0xB4, 0x5E, 0x7B, 0xE7, 0x9D, 0x6A, 0xA2, 0x28, 
	0x96, 0xD5, 0x97, 0xBE, 0x54, 0xE3, 0xF8, 0xF1, 0x2D, 0x60, 0x8E, 0x51, 0x55, 0x15, 0xAA, 0x2A, 
	0x07, 0xB3, 0x84, 0x10, 0x0C, 0xA5, 0xE6, 0x20, 0x65, 0x0E, 0xAD, 0x5B, 0xA8, 0xAA, 0x08, 0x83, 
	0x81, 0xF7, 0xD6, 0xAE, 0xCC, 0x66, 0xD9, 0xB1, 0xCF, 0x16, 0xC5, 0x99, 0x3F, 0x60, 0x5E, 0x7E, 
	0x0E, 0x68, 0x38, 0xE0, 0x95, 0x85, 0x4C, 0x3F, 0x42, 0x68, 0x5E, 0xD3, 0x44, 0xDD, 0xA9, 0x43, 
	0x1D, 0xE9, 0xBD, 0x21, 0xC2, 0xBB, 0xB4, 0xD0, 0xAF, 0x97, 0xB1, 0xBC, 0x56, 0x36, 0xE4, 0x96, 
	0x20, 0x08, 0x1A, 0x32, 0x90, 0x9A, 0x1B, 0x2C, 0x8A, 0xA8, 0xF0, 0xDC, 0xE2, 0xAE, 0x9A, 0x50, 
	0xA7, 0x44, 0x22, 0x1E, 0x27, 0xA2, 0xAF, 0x40, 0xE2, 0x19, 0x0E, 0x78, 0x91, 0x1C, 0xB9, 0x93, 
	0x37, 0x9E, 0xFC, 0xBE, 0x6E, 0x2D, 0xE4, 0x2B, 0xF9, 0xE1, 0xE3, 0xE3, 0xCF, 0xC1, 0xB9, 0x81, 
	0x22, 0xF2, 0xFB, 0xA4, 0x2C, 0xFF, 0x49, 0x18, 0x6E, 0x7C, 0x60, 0x6C, 0xAC, 0xBE, 0xFD, 0x55, 
	0xAF, 0x6A, 0x0E, 0x69, 0x2D, 0xC4, 0xE3, 0x8F, 0x33, 0x0E, 0x1E, 0x2C, 0xB1, 0xBE, 0xBE, 0x80, 
	0x3C, 0xAF, 0xD9, 0x5A, 0x22, 0x40, 0x43, 0xCA, 0x00, 0x44, 0x15, 0x88, 0x32, 0x08, 0x51, 0x02, 
	0x68, 0xA0, 0xAE, 0x23, 0x78, 0xAF, 0x48, 0xA9, 0x56, 0x4B, 0x88, 0x68, 0x2F, 0xB3, 0x6F, 0x3B, 
	0xE7, 0x4F, 0x31, 0x7F, 0x6A, 0x19, 0x78, 0x17, 0x80, 0x03, 0xAF, 0xC8, 0x3D, 0xB4, 0x3E, 0xDC, 
	0x82, 0x8A, 0x15, 0xE9, 0xB6, 0x9E, 0xD4, 0xC3, 0xFA, 0xDE, 0xA8, 0x15, 0xFD, 0x93, 0x66, 0xDC, 
	0x7C, 0x77, 0x94, 0x44, 0xB7, 0xA2, 0x8D, 0x69, 0x6E, 0x72, 0xAA, 0x03, 0x1D, 0x86, 0x2A, 0x54, 
	0x2A, 0x54, 0xB2, 0x15, 0xB7, 0xD4, 0x9E, 0xA1, 0x3D, 0xCD, 0x28, 0x89, 0xA6, 0x73, 0x91, 0xDF, 
	0x80, 0x12, 0x37, 0xCA, 0xBE, 0x9C, 0xA4, 0x92, 0x4A, 0x96, 0x58, 0xA3, 0xFF, 0x99, 0xEB, 0xED, 
	0x7F, 0xBC, 0x03, 0x6B, 0xBF, 0xBB, 0x76, 0xE9, 0x02, 0x9E, 0x9E, 0x3E, 0x88, 0x66, 0xF3, 0x03, 
	0x50, 0x8A, 0x9A, 0x44, 0xEE, 0x2D, 0x80, 0xF9, 0x05, 0xAD, 0xF3, 0x77, 0x04, 0x41, 0x7F, 0xAA, 
	0x2C, 0xA5, 0x3C, 0x75, 0x4A, 0xE2, 0xF1, 0xC7, 0x0D, 0x9E, 0x7F, 0x3E, 0xC7, 0xFA, 0xFA, 0x19, 
	0xCE, 0xF3, 0xFD, 0xA6, 0x2C, 0x67, 0x2B, 0xE7, 0x8C, 0xF7, 0x5E, 0x12, 0x11, 0x11, 0x33, 0x93, 
	0xF7, 0x01, 0x98, 0x5B, 0x60, 0x1E, 0x05, 0xB3, 0x02, 0x20, 0x00, 0x28, 0x52, 0xAA, 0x15, 0x4B, 
	0x99, 0xEC, 0x04, 0x58, 0x01, 0x57, 0x1D, 0xD9, 0xB5, 0xEB, 0xA3, 0x9D, 0x6E, 0x77, 0x19, 0xCC, 
	0x4F, 0x7D, 0x4F, 0xEF, 0x63, 0xF4, 0x5F, 0x8E, 0x42, 0x48, 0xA1, 0x82, 0xE1, 0xE0, 0x06, 0x19, 
	0xC9, 0x9F, 0x91, 0xB1, 0xFC, 0x59, 0x19, 0xC9, 0xD7, 0x05, 0x41, 0x30, 0x1A, 0xC6, 0xA1, 0xF6, 
	0x81, 0xA7, 0x9A, 0x6A, 0x90, 0x23, 0x28, 0xAF, 0xE0, 0xB4, 0x43, 0xAB, 0xD1, 0xC2, 0xBD, 0x57, 
	0xDC, 0x8B, 0x1B, 0xC6, 0x6E, 0xA0, 0xA5, 0xFE, 0x52, 0x38, 0x58, 0x1E, 0x6C, 0x11, 0x7D, 0x71, 
	0x2D, 0x24, 0x76, 0xBA, 0xD0, 0x19, 0xF9, 0x6A, 0xB9, 0x30, 0x74, 0xE3, 0x70, 0xDE, 0x6D, 0x6C, 
	0xC0, 0x3E, 0x68, 0x2F, 0x3D, 0xC0, 0xD3, 0xD3, 0x87, 0x00, 0x10, 0x88, 0xE4, 0x04, 0x11, 0xBD, 
	0x87, 0x48, 0x7C, 0x98, 0x88, 0xEE, 0x20, 0x42, 0xC3, 0xFB, 0x00, 0x65, 0x19, 0xA1, 0xD7, 0xAB, 
	0x90, 0x65, 0x05, 0x8A, 0x62, 0x9D, 0xCB, 0xF2, 0x68, 0x69, 0xCC, 0xB1, 0xCC, 0xDA, 0xA5, 0xC2, 
	0x98, 0xE5, 0xDA, 0x98, 0x25, 0x5B, 0x55, 0xCB, 0xCE, 0xB9, 0xCC, 0x09, 0xE1, 0x1D, 0x11, 0x39, 
	0xE6, 0x50, 0x30, 0x4B, 0xDA, 0xF4, 0x6D, 0x02, 0x80, 0x84, 0x52, 0x8D, 0x58, 0xCA, 0xC6, 0xB4, 
	0x94, 0x51, 0xD6, 0xED, 0x7E, 0xE5, 0x70, 0x18, 0xEE, 0x2A, 0xEB, 0xFA, 0xB3, 0xDF, 0x93, 0x7B, 
	0x18, 0xFE, 0xD5, 0x61, 0x4C, 0xFD, 0xC2, 0x14, 0x8A, 0xF9, 0xA2, 0xA1, 0x22, 0x75, 0x97, 0x0E, 
	0xF4, 0x3F, 0x0F, 0x55, 0xF8, 0x2E, 0x92, 0xB4, 0x13, 0x1A, 0xEA, 0xAA, 0xB1, 0xAB, 0xF0, 0xFA, 
	0xF1, 0xD7, 0x63, 0xC8, 0x0D, 0xA1, 0x5B, 0x74, 0xE1, 0x4B, 0x0F, 0x67, 0x1D, 0x17, 0xAA, 0xF0, 
	0xB9, 0xCA, 0xB1, 0xB5, 0xB9, 0x95, 0xEE, 0x18, 0xBF, 0x03, 0x65, 0xA7, 0xC4, 0xD9, 0xA5, 0xB3, 
	0xE4, 0xE0, 0x12, 0x4A, 0x69, 0xA7, 0x15, 0x66, 0x57, 0xB9, 0x5E, 0x54, 0xCB, 0x8F, 0x2D, 0x9E, 
	0x6D, 0x6C, 0x8F, 0x72, 0xBA, 0x43, 0xA0, 0x7E, 0xC0, 0x5C, 0x3A, 0x80, 0x37, 0xE1, 0x82, 0x00, 
	0xDA, 0x03, 0xE0, 0x03, 0x00, 0x7D, 0x80, 0x88, 0xAE, 0x05, 0x84, 0x62, 0x56, 0xF0, 0x5E, 0x43, 
	0x4A, 0x46, 0x14, 0xF7, 0x21, 0xC5, 0x00, 0x65, 0xD9, 0xAF, 0xEB, 0xFA, 0x6C, 0xDF, 0xFB, 0xAC, 
	0x62, 0xB6, 0xCC, 0xDC, 0xF3, 0xCE, 0x2D, 0x19, 0xEF, 0x97, 0x6A, 0x29, 0xB3, 0xDA, 0xFB, 0x6E, 
	0x59, 0x55, 0xCB, 0x06, 0x48, 0x94, 0x94, 0xA9, 0x0A, 0x43, 0x8F, 0xDD, 0xBB, 0x0D, 0xC6, 0xC6, 
	0x18, 0x79, 0xAE, 0xC0, 0x9C, 0xA4, 0x52, 0x26, 0xE3, 0x52, 0x36, 0x8E, 0x55, 0xD5, 0xB9, 0x53, 
	0xCD, 0xE6, 0xCF, 0x71, 0x59, 0xFE, 0xF1, 0xFF, 0xD0, 0x3D, 0xB4, 0x7F, 0xB9, 0x8D, 0xE4, 0x8A, 
	0x04, 0x83, 0x93, 0x83, 0x31, 0x19, 0xCA, 0x1F, 0xD7, 0x4A, 0xFF, 0x4F, 0x31, 0xC5, 0x6F, 0x8E, 
	0x38, 0x1A, 0x72, 0xC2, 0xD1, 0x68, 0x3A, 0x8A, 0xF7, 0x5D, 0xF7, 0x3E, 0xFC, 0xFD, 0x2D, 0x7F, 
	0x1F, 0x34, 0x20, 0x9C, 0xE9, 0x9F, 0xC1, 0xB2, 0x5D, 0xF6, 0x59, 0x96, 0x15, 0x79, 0x27, 0xEF, 
	0x1A, 0x36, 0x76, 0x69, 0xB0, 0x24, 0xAA, 0xAC, 0xA2, 0x8D, 0xEE, 0x06, 0x4E, 0xE6, 0x27, 0xA9, 
	0x8A, 0x2B, 0x88, 0x90, 0x94, 0x27, 0x3F, 0xC1, 0xC4, 0x53, 0xBA, 0x11, 0x64, 0x3A, 0x08, 0x4F, 
	0xAB, 0xD1, 0xA0, 0xDC, 0x72, 0xDF, 0x56, 0xAC, 0xFF, 0xD1, 0xFA, 0xC5, 0x9F, 0x26, 0x4D, 0x4F, 
	0x1F, 0x06, 0x33, 0x09, 0x22, 0xBF, 0x0F, 0xC0, 0x07, 0x01, 0xBC, 0x1D, 0xC0, 0x18, 0x40, 0x10, 
	0xC2, 0x43, 0xA9, 0x1A, 0x42, 0xE4, 0x50, 0xAA, 0x04, 0x51, 0x8D, 0x41, 0xE6, 0xAC, 0x31, 0x8B, 
	0x03, 0xA0, 0x57, 0x13, 0x31, 0xA4, 0x94, 0x02, 0x10, 0x24, 0x84, 0xF4, 0x71, 0x3C, 0x06, 0x20, 
	0xB0, 0x75, 0xDD, 0x03, 0x73, 0xDF, 0x11, 0xCD, 0x55, 0x5A, 0x8F, 0x05, 0x37, 0xDF, 0xAC, 0xC5, 
	0x7B, 0xDF, 0x2B, 0x21, 0x84, 0xC2, 0xC7, 0x3F, 0x0E, 0x3C, 0xF9, 0xA4, 0x20, 0xA0, 0x7D, 0x1D, 
	0x20, 0xFE, 0x61, 0xA3, 0xB1, 0xEF, 0x90, 0x10, 0xC1, 0xB9, 0xE1, 0xE1, 0x4F, 0x60, 0x63, 0xE3, 
	0x3D, 0x7F, 0xBB, 0x78, 0xFB, 0x4B, 0x2D, 0x14, 0xF3, 0x05, 0xC9, 0x58, 0x6E, 0x13, 0x89, 0x78, 
	0x37, 0x49, 0xFA, 0x49, 0xE1, 0xC5, 0xF5, 0xC2, 0x0A, 0x55, 0x89, 0x0A, 0xB5, 0xA8, 0x31, 0x12, 
	0x8F, 0xE0, 0xAA, 0xB1, 0xAB, 0x30, 0x19, 0x4C, 0x42, 0x47, 0x1A, 0x15, 0x2A, 0x97, 0x65, 0xD9, 
	0xA0, 0x58, 0x2C, 0x36, 0xDC, 0xC0, 0xE5, 0x41, 0x3F, 0x50, 0xA7, 0x57, 0x4F, 0xC7, 0x9F, 0x3C, 
	0xF3, 0xC9, 0x48, 0x4B, 0xAD, 0xFA, 0xB2, 0x27, 0xC4, 0x90, 0x94, 0xA9, 0x48, 0xC3, 0x60, 0x6B, 
	0xA0, 0xC3, 0x38, 0xBC, 0x09, 0x3D, 0xFC, 0x84, 0xDC, 0x90, 0xAB, 0xC1, 0x7A, 0xF0, 0x05, 0x1F, 
	0x72, 0x75, 0xD1, 0xE7, 0xC1, 0xDB, 0xB6, 0x1D, 0x85, 0xB5, 0x92, 0x94, 0xAA, 0x6F, 0x04, 0xF0, 
	0x4B, 0x00, 0xDE, 0x06, 0x20, 0x05, 0x00, 0x22, 0x8F, 0x30, 0xCC, 0xA1, 0x54, 0x17, 0xCE, 0x95, 
	0x30, 0x46, 0x03, 0x88, 0x5D, 0xBB, 0xBD, 0x51, 0xC4, 0xF1, 0xBA, 0x5D, 0x59, 0x29, 0xA4, 0x52, 
	0xCD, 0x20, 0x4D, 0xC7, 0x43, 0x21, 0x02, 0x2B, 0x44, 0x68, 0x8D, 0x11, 0xDC, 0xE9, 0x74, 0x2B, 
	0x66, 0x30, 0xE0, 0xB9, 0xAE, 0x17, 0x2A, 0xA5, 0xA6, 0xA3, 0xDB, 0x6F, 0xDF, 0x1D, 0xDE, 0x7D, 
	0xF7, 0x10, 0x00, 0x81, 0x17, 0x5E, 0x58, 0xC7, 0x73, 0xCF, 0x95, 0x28, 0x4B, 0x11, 0x48, 0x99, 
	0xBC, 0x09, 0xC0, 0x97, 0xAB, 0x6A, 0xF1, 0x4F, 0xB5, 0xDE, 0xFA, 0xB7, 0x0A, 0x6C, 0xED, 0xFF, 
	0xBD, 0x8D, 0xED, 0xC9, 0x76, 0x2C, 0x34, 0x16, 0xA6, 0x74, 0xA4, 0x7F, 0x52, 0x48, 0xF1, 0x7E, 
	0x16, 0x7C, 0x85, 0x65, 0x2B, 0x06, 0x7A, 0x00, 0x2F, 0x3D, 0x3C, 0x79, 0x6C, 0xF4, 0x36, 0x70, 
	0x70, 0xE6, 0x20, 0xD6, 0x92, 0x35, 0x3C, 0xB1, 0xF6, 0x84, 0x39, 0xB7, 0x74, 0xAE, 0x9F, 0xCF, 
	0xE7, 0x1D, 0xDB, 0xB7, 0x05, 0x18, 0xEC, 0x3A, 0xCE, 0x0B, 0x2B, 0xCC, 0x2A, 0xAF, 0xF6, 0xE1, 
	0x21, 0x40, 0x10, 0x32, 0x95, 0x5A, 0x37, 0xF5, 0xA8, 0x9E, 0xD4, 0x4D, 0x59, 0x4B, 0x2D, 0x7A, 
	0xE2, 0x56, 0x61, 0xE9, 0x3D, 0x6E, 0xD2, 0x9F, 0x2B, 0xDE, 0x54, 0x1C, 0xD8, 0x73, 0xC3, 0x1E, 
	0x3E, 0xF1, 0x9A, 0x13, 0x17, 0x27, 0xE0, 0x1D, 0x3B, 0x8E, 0x40, 0xCA, 0x3E, 0xBC, 0x0F, 0x76, 
	0x4A, 0x59, 0xFF, 0x73, 0x22, 0xF7, 0x36, 0x63, 0xA2, 0xD4, 0xFB, 0xCD, 0x8F, 0x26, 0x62, 0x30, 
	0x7B, 0x54, 0x95, 0x86, 0x73, 0x11, 0xC6, 0xC6, 0x1A, 0xFC, 0x86, 0x37, 0xC4, 0x7C, 0xCD, 0x35, 
	0x51, 0xF0, 0xEC, 0xB3, 0xF3, 0xE2, 0x4B, 0x5F, 0xEA, 0x79, 0xA2, 0x2D, 0x61, 0xA3, 0x31, 0xEE, 
	0x98, 0x15, 0x97, 0xA5, 0xD1, 0x59, 0xB6, 0x56, 0x39, 0xE7, 0xF8, 0x42, 0xF9, 0xD2, 0xB9, 0xBE, 
	0xA9, 0xEB, 0x85, 0x4A, 0x88, 0x2B, 0x74, 0x14, 0x35, 0x84, 0x31, 0x0E, 0xDE, 0x67, 0x08, 0xC3, 
	0x0C, 0x44, 0xC3, 0x60, 0x96, 0xD3, 0x55, 0xA5, 0xDE, 0xEA, 0x7D, 0xEB, 0x51, 0x29, 0xA3, 0x99, 
	0x6D, 0xDB, 0x1E, 0xC3, 0xFC, 0xFC, 0xEB, 0xFE, 0xFA, 0x70, 0xD1, 0x46, 0xA8, 0x43, 0x2C, 0x17, 
	0xCB, 0xC3, 0x71, 0x14, 0xBF, 0x3B, 0x14, 0xE1, 0x4F, 0x0A, 0x2F, 0xAE, 0x2C, 0xA9, 0xA4, 0x5A, 
	0xD5, 0x70, 0xE4, 0x00, 0x06, 0x88, 0x09, 0x6B, 0xD9, 0x1A, 0x3E, 0xF1, 0xFC, 0x27, 0x38, 0xF0, 
	0x41, 0x75, 0xF6, 0xDC, 0xD9, 0xEE, 0xF2, 0xDC, 0x72, 0xD7, 0x16, 0xB6, 0x7E, 0x69, 0x41, 0x85, 
	0x0A, 0x92, 0x30, 0x91, 0x03, 0x3B, 0xA8, 0xEA, 0xBA, 0x36, 0x00, 0x58, 0x5B, 0xED, 0x77, 0xF3, 
	0x6E, 0x3B, 0xC2, 0x23, 0x98, 0x2B, 0xE6, 0x50, 0x55, 0x55, 0x83, 0x84, 0xBC, 0x8B, 0x9B, 0x7C, 
	0x54, 0x9F, 0xD1, 0xB3, 0xC4, 0xB4, 0xBC, 0xE7, 0xD0, 0x1E, 0x9C, 0xB8, 0xE1, 0xC4, 0xC5, 0x07, 
	0x38, 0x08, 0x4F, 0xC1, 0x7B, 0x17, 0x46, 0x61, 0x72, 0xAF, 0x92, 0xE6, 0x5E, 0xEF, 0x45, 0x6A, 
	0x6D, 0xF0, 0xD2, 0xFF, 0x7B, 0x2F, 0x51, 0x55, 0x4D, 0x30, 0x87, 0xD0, 0x3A, 0xB3, 0x77, 0xDF, 
	0x2D, 0xFD, 0x8F, 0xFD, 0x58, 0x28, 0x4E, 0x9C, 0xE8, 0xF0, 0x89, 0x13, 0xEB, 0xA6, 0x28, 0x22, 
	0x1E, 0x1A, 0x6A, 0x1A, 0x22, 0x59, 0xAF, 0xAE, 0xAE, 0xF1, 0x60, 0x90, 0x53, 0x5D, 0x57, 0xEE, 
	0x5B, 0x6B, 0xD3, 0x96, 0x7B, 0xBD, 0xB9, 0xF2, 0xAB, 0x5F, 0x5D, 0x88, 0x76, 0xED, 0x1A, 0x09, 
	0xB2, 0xCC, 0x62, 0xFF, 0xFE, 0x1E, 0x94, 0xCA, 0x10, 0x45, 0x29, 0xBC, 0x27, 0xC5, 0x6C, 0xEF, 
	0x30, 0xC6, 0xDF, 0x9E, 0xE7, 0x33, 0x73, 0x41, 0xD0, 0x70, 0x7F, 0xA3, 0x74, 0xEE, 0x37, 0xC7, 
	0x41, 0x4C, 0xCA, 0x45, 0xEE, 0x2E, 0x02, 0xBD, 0x47, 0x58, 0x71, 0x85, 0x85, 0x25, 0xA7, 0x1D, 
	0x88, 0x00, 0x61, 0x24, 0xA8, 0x26, 0x38, 0xED, 0x50, 0x72, 0xE9, 0x4F, 0x2C, 0x9D, 0xC8, 0xCB, 
	0x95, 0xB2, 0x53, 0xAF, 0xD7, 0x19, 0x1B, 0xFE, 0x16, 0x8F, 0x11, 0xEA, 0x30, 0x1A, 0x4A, 0xDB, 
	0x8D, 0x86, 0x6E, 0xE4, 0xBD, 0x41, 0xAF, 0xCE, 0xEB, 0xBC, 0x94, 0x91, 0xE4, 0xEB, 0xD2, 0xEB, 
	0xFC, 0xEB, 0xB7, 0xBC, 0x9E, 0xBF, 0xBE, 0xFE, 0x75, 0xDA, 0xDF, 0x7F, 0x02, 0xCE, 0xBA, 0x2D, 
	0xE4, 0xE8, 0x47, 0x61, 0x70, 0xD0, 0x36, 0xED, 0x5F, 0x90, 0xA1, 0x57, 0x64, 0xB7, 0xF5, 0x3F, 
	0xB6, 0xC9, 0xD2, 0xEF, 0x42, 0xA8, 0xDB, 0x18, 0x14, 0x0B, 0xBB, 0xB4, 0xAA, 0xFF, 0x99, 0x10, 
	0xE2, 0x55, 0x75, 0xDD, 0x26, 0x6B, 0x93, 0xF3, 0x35, 0x94, 0xCD, 0x8C, 0x9E, 0x99, 0xB9, 0xAE, 
	0x97, 0xEB, 0x3C, 0x7F, 0x3A, 0x73, 0xEE, 0x58, 0xF9, 0xFC, 0xF3, 0xA7, 0xCC, 0x03, 0x0F, 0xCC, 
	0xD8, 0xE3, 0xC7, 0xDB, 0x32, 0x0C, 0xB7, 0xB9, 0xE1, 0xE1, 0x66, 0xD9, 0xED, 0xF6, 0xFC, 0xCA, 
	0xCA, 0x2A, 0x1B, 0x63, 0x3C, 0xB3, 0xFF, 0x8E, 0x26, 0x84, 0xF7, 0x35, 0xAF, 0xAC, 0x48, 0x71, 
	0xF0, 0x60, 0xAC, 0x1F, 0x7D, 0xD4, 0xD0, 0xFC, 0x3C, 0x43, 0x4A, 0x46, 0x10, 0x04, 0x60, 0x76, 
	0xA8, 0xEB, 0xB2, 0x69, 0x8C, 0x1F, 0x10, 0xD1, 0x93, 0x42, 0x84, 0x7D, 0x21, 0x6E, 0x47, 0x55, 
	0x7D, 0xFE, 0xBB, 0x5B, 0xEF, 0xBF, 0x6A, 0xA3, 0xB1, 0xBD, 0x81, 0xDC, 0xE7, 0x3B, 0x45, 0x22, 
	0x7E, 0x86, 0x24, 0xBD, 0x29, 0x12, 0x51, 0x18, 0x45, 0x11, 0x6A, 0x51, 0x83, 0x8C, 0x40, 0xD0, 
	0x0F, 0xE0, 0x73, 0xCF, 0xA5, 0x29, 0x4D, 0xBE, 0x91, 0x77, 0x8B, 0xC5, 0x62, 0xDD, 0x6C, 0x98, 
	0x01, 0x2C, 0xDC, 0xB7, 0x17, 0x2D, 0x94, 0x56, 0xBA, 0xD1, 0x68, 0x44, 0x43, 0xE3, 0x43, 0x22, 
	0x6E, 0xC6, 0xB1, 0xA9, 0x0D, 0x2A, 0x57, 0x99, 0x38, 0x8D, 0x83, 0x3B, 0x76, 0xDF, 0x11, 0x4F, 
	0x36, 0xB6, 0x88, 0x63, 0x9D, 0x17, 0xD1, 0x19, 0x74, 0xC8, 0xB3, 0x6F, 0x5B, 0x67, 0x07, 0xB6, 
	0x6B, 0x0F, 0xA9, 0x54, 0x75, 0x82, 0x3B, 0x03, 0x0C, 0xFE, 0x6C, 0x70, 0xF1, 0x58, 0x70, 0x12, 
	0xDE, 0x86, 0xF9, 0xB9, 0x0F, 0x22, 0x6E, 0x7C, 0xE4, 0xAA, 0xBA, 0xEC, 0x5F, 0x13, 0x06, 0x4D, 
	0xA1, 0x94, 0x87, 0x10, 0x1B, 0xE7, 0x5D, 0x73, 0x08, 0x20, 0x80, 0xB5, 0xCB, 0xA6, 0xAA, 0x9E, 
	0xCF, 0x81, 0x45, 0x77, 0xE4, 0x48, 0xA2, 0xB4, 0x4E, 0x83, 0xA2, 0xD8, 0x13, 0x28, 0x35, 0x8C, 
	0x76, 0xBB, 0xA8, 0x9C, 0x1B, 0xF0, 0xFA, 0xFA, 0x06, 0xBC, 0xF7, 0xDF, 0xD9, 0x63, 0x7A, 0xC9, 
	0x90, 0xAD, 0xCF, 0xB2, 0x73, 0xE5, 0xCC, 0xCC, 0x44, 0x18, 0x45, 0xD3, 0xA1, 0x10, 0x31, 0x84, 
	0xD0, 0x50, 0xAA, 0x82, 0x73, 0x06, 0xD6, 0x4A, 0x0D, 0xE8, 0xD7, 0x11, 0x05, 0x37, 0xF5, 0xFB, 
	0x87, 0x17, 0xE2, 0x78, 0x3B, 0xFF, 0xB5, 0xAA, 0x54, 0x44, 0x98, 0x7D, 0x64, 0x56, 0xB4, 0x5E, 
	0xD5, 0xBA, 0x89, 0x1A, 0xF4, 0xDA, 0x5D, 0xF1, 0xAE, 0xC6, 0x5D, 0xE1, 0x5D, 0x10, 0x7D, 0x81, 
	0xAF, 0x2D, 0x7C, 0x0D, 0x27, 0xFB, 0x27, 0x61, 0x32, 0x63, 0xF3, 0x32, 0x2F, 0xD9, 0x72, 0x16, 
	0x55, 0x51, 0x8E, 0x01, 0xCA, 0xC2, 0x15, 0xFE, 0xAF, 0xFA, 0xBC, 0x62, 0x90, 0x17, 0x6B, 0x2B, 
	0x6B, 0x3E, 0x1D, 0x4B, 0x69, 0x7C, 0xEB, 0x78, 0xA3, 0x2A, 0xAB, 0x38, 0x5F, 0xCA, 0xCB, 0xC3, 
	0x0B, 0x87, 0xAB, 0x3F, 0x79, 0xF1, 0x4F, 0xEC, 0x15, 0x63, 0x57, 0x28, 0x30, 0x60, 0x94, 0x01, 
	0x72, 0xA4, 0x6C, 0xF8, 0x4E, 0x28, 0x7C, 0xA3, 0xEC, 0x96, 0x73, 0x52, 0xCA, 0xFA, 0xA2, 0x72, 
	0xD1, 0x51, 0xB8, 0x0D, 0xA3, 0x63, 0x7F, 0xA4, 0x92, 0x78, 0xFB, 0x2E, 0x82, 0x1C, 0x0D, 0x82, 
	0x26, 0x82, 0x00, 0x88, 0xA2, 0x1A, 0x52, 0xD6, 0x18, 0x0C, 0x1C, 0xF2, 0x7C, 0xCD, 0xD5, 0xF5, 
	0x82, 0x8F, 0xA2, 0x66, 0x12, 0xC7, 0xFB, 0xA4, 0x73, 0x89, 0xE8, 0x76, 0x03, 0x61, 0x6D, 0x88, 
	0x76, 0xDB, 0x95, 0x44, 0x03, 0xBB, 0xB1, 0xD1, 0x41, 0x55, 0x55, 0x7F, 0x45, 0xDB, 0xF0, 0x5B, 
	0x19, 0x39, 0xD7, 0x35, 0x55, 0x75, 0xAA, 0x54, 0xAA, 0xA1, 0xB5, 0x1E, 0x12, 0x65, 0x29, 0xE1, 
	0x3D, 0xE0, 0xBD, 0x80, 0xB5, 0x01, 0x88, 0xDC, 0x2E, 0x21, 0xE8, 0xF5, 0x49, 0x72, 0xF5, 0x23, 
	0x44, 0xBA, 0xF7, 0xD7, 0x72, 0x61, 0x89, 0x44, 0xBC, 0x33, 0x6E, 0x30, 0xF1, 0xF5, 0x5A, 0xEA, 
	0x6D, 0xB7, 0x6F, 0xBB, 0x1D, 0xEF, 0x1C, 0x7F, 0x27, 0x96, 0x4F, 0x2F, 0xE3, 0xE8, 0x99, 0xA3, 
	0xEE, 0xB9, 0xE5, 0xE7, 0x8A, 0xBC, 0xC8, 0x07, 0x89, 0x48, 0x8A, 0x37, 0xEC, 0x7E, 0x83, 0xDE, 
	0xDB, 0xDA, 0xDB, 0x3E, 0x74, 0xE6, 0x50, 0xF8, 0xE8, 0xD9, 0x47, 0x7B, 0x03, 0x33, 0x78, 0x29, 
	0x14, 0x48, 0x21, 0x49, 0x90, 0x80, 0x75, 0xD6, 0xF5, 0xBB, 0xFD, 0xDC, 0x1B, 0x4F, 0xD3, 0xDB, 
	0xA6, 0xED, 0xA0, 0x3F, 0x08, 0xE6, 0x67, 0xE7, 0x39, 0xDB, 0xC8, 0xEA, 0xC7, 0x66, 0x1E, 0xAB, 
	0x8E, 0xF7, 0x8F, 0x87, 0xD6, 0x5A, 0x52, 0x52, 0xC1, 0x48, 0x03, 0x38, 0xEC, 0x21, 0xA2, 0xBB, 
	0x21, 0xF0, 0x04, 0x80, 0x53, 0xDB, 0xEE, 0xDF, 0x86, 0xF9, 0xFB, 0xE6, 0x2F, 0x0E, 0xC0, 0xED, 
	0xF6, 0x16, 0x58, 0xE3, 0x65, 0xB3, 0xA9, 0x86, 0xC2, 0xB0, 0x11, 0x30, 0xC7, 0x20, 0x22, 0x84, 
	0x61, 0x8D, 0x38, 0x2E, 0x50, 0x14, 0x4D, 0x64, 0x59, 0xC7, 0x65, 0xD9, 0x5C, 0x59, 0x55, 0x52, 
	0xAD, 0xAD, 0x91, 0xCC, 0x73, 0x2B, 0x00, 0x07, 0xAD, 0x19, 0xCE, 0x19, 0x57, 0x14, 0x5D, 0xDF, 
	0xEF, 0xF7, 0xE1, 0x3D, 0x7F, 0x07, 0xDC, 0x0B, 0xD6, 0xCB, 0xCC, 0x90, 0x12, 0x50, 0xCA, 0xC3, 
	0xDA, 0xD9, 0xD2, 0x98, 0xD1, 0x50, 0xEB, 0x46, 0xC8, 0x1C, 0x51, 0x59, 0x06, 0x60, 0xB6, 0x20, 
	0xD2, 0x20, 0x0A, 0x12, 0x40, 0xDC, 0xC6, 0x6C, 0xB6, 0x2B, 0x95, 0x1E, 0x6E, 0x36, 0x3F, 0x8D, 
	0x2C, 0x7B, 0xC7, 0x7F, 0xF7, 0x1E, 0x44, 0x53, 0x00, 0x25, 0xDA, 0xA4, 0x68, 0xA7, 0x80, 0x48, 
	0x52, 0x9D, 0xA2, 0x19, 0x37, 0x31, 0x2F, 0xE7, 0x79, 0x90, 0x0D, 0x06, 0xE5, 0x4A, 0xB9, 0x66, 
	0xBD, 0x2D, 0xAE, 0xBD, 0xE6, 0xDA, 0xE0, 0xFD, 0x6F, 0x78, 0xFF, 0xE8, 0x16, 0xBD, 0xA5, 0x91, 
	0x52, 0x9A, 0x1F, 0x5F, 0x39, 0x5E, 0x0E, 0xBA, 0x83, 0x02, 0x0C, 0x8C, 0xB5, 0xC6, 0xE4, 0xED, 
	0xBB, 0x6F, 0x6F, 0x0A, 0x2B, 0xE4, 0x33, 0xE7, 0x9E, 0x19, 0xCC, 0x67, 0xF3, 0x95, 0x56, 0x9A, 
	0xF7, 0x5C, 0xB9, 0xA7, 0x86, 0x04, 0xCF, 0x9C, 0x9B, 0xA1, 0xB5, 0xD5, 0x35, 0x33, 0xD8, 0x18, 
	0x14, 0x22, 0x11, 0x8D, 0x94, 0x52, 0x15, 0xCB, 0x18, 0x08, 0x00, 0xC7, 0x2E, 0xF6, 0xF0, 0xAF, 
	0x23, 0xD0, 0x2D, 0x20, 0x9C, 0x03, 0xC1, 0x5C, 0x34, 0x16, 0x3C, 0x31, 0xDE, 0x82, 0x31, 0x84, 
	0x34, 0xDD, 0x04, 0x66, 0x0C, 0x50, 0x55, 0x0A, 0xCE, 0x09, 0x54, 0x95, 0x86, 0x10, 0x11, 0x1A, 
	0x8D, 0x50, 0x6A, 0xBD, 0xCA, 0x75, 0x3D, 0xD7, 0x8B, 0x22, 0x57, 0xAE, 0xAF, 0x47, 0x8D, 0xC1, 
	0x40, 0x44, 0xDE, 0x13, 0xD5, 0x75, 0xE6, 0xFA, 0xFD, 0x1E, 0xAA, 0xAA, 0xFE, 0x6F, 0xC0, 0x25, 
	0x28, 0xA5, 0x44, 0x92, 0x68, 0x71, 0xDB, 0x6D, 0xC3, 0xC1, 0x9E, 0x3D, 0x4D, 0xF5, 0xD4, 0x53, 
	0x8B, 0xF5, 0x0B, 0x2F, 0xCC, 0x15, 0xDE, 0x6F, 0x51, 0x4A, 0x85, 0xCA, 0x7B, 0x01, 0x40, 0xC1, 
	0x7B, 0x80, 0x99, 0x00, 0xA8, 0xDD, 0x5A, 0x07, 0xBB, 0x8D, 0xD9, 0x77, 0x38, 0x4D, 0xEF, 0x47, 
	0x96, 0xFD, 0xF7, 0xEF, 0x41, 0x44, 0x02, 0xAE, 0x70, 0x21, 0x29, 0x4A, 0x4B, 0x5F, 0xAA, 0xFD, 
	0x73, 0xFB, 0x21, 0x73, 0x89, 0x99, 0xF9, 0x19, 0x7F, 0xA4, 0x77, 0xA4, 0xB0, 0xCE, 0x16, 0x2A, 
	0x56, 0x7E, 0xFB, 0xE4, 0x76, 0x75, 0xE5, 0xD4, 0x95, 0x3A, 0x32, 0x91, 0xD0, 0x5A, 0xB3, 0x75, 
	0xD6, 0xC3, 0x03, 0x81, 0x0E, 0xE8, 0xAD, 0xB7, 0xBC, 0x35, 0xBD, 0xEF, 0xF6, 0xFB, 0xC6, 0x56, 
	0x16, 0x56, 0xA4, 0xAB, 0x1D, 0xAD, 0x16, 0xAB, 0x26, 0x0C, 0x42, 0x3F, 0x3C, 0x34, 0x2C, 0x76, 
	0x5D, 0xBF, 0x2B, 0xDE, 0x39, 0xBF, 0x33, 0xC8, 0x1E, 0xCD, 0x3A, 0xF5, 0x7A, 0x9D, 0x9B, 0x31, 
	0x53, 0xD7, 0x69, 0xAD, 0x42, 0x11, 0x62, 0x6B, 0xB4, 0x15, 0xE3, 0xF1, 0x38, 0x16, 0xAB, 0xC5, 
	0x2B, 0x16, 0xF3, 0xC5, 0x37, 0x7A, 0xEF, 0x9F, 0x80, 0xC0, 0xCC, 0x8E, 0x3F, 0xDA, 0x81, 0x99, 
	0xF7, 0xCE, 0xFC, 0xE0, 0x01, 0x4B, 0x19, 0xC3, 0x7B, 0xED, 0x81, 0xAC, 0xAF, 0x54, 0x61, 0x76, 
	0xEF, 0xEE, 0x62, 0x68, 0xA8, 0x89, 0x17, 0x5F, 0x0C, 0xB1, 0xB2, 0xA2, 0xB1, 0x6D, 0x1B, 0x43, 
	0x88, 0x40, 0xCE, 0xCD, 0x4D, 0x45, 0x42, 0x74, 0xAA, 0xB1, 0x31, 0x5B, 0x0F, 0x0F, 0x57, 0x6E, 
	0x66, 0xC6, 0xF9, 0x7E, 0x5F, 0xC4, 0xD6, 0x56, 0xC8, 0xF3, 0x01, 0x3B, 0x47, 0x7F, 0xA5, 0x5B, 
	0x8E, 0xE3, 0xA6, 0x1E, 0x19, 0x99, 0x6E, 0x5C, 0x7F, 0x7D, 0x5B, 0x7D, 0xE0, 0x03, 0xD3, 0xEA, 
	0xDA, 0x6B, 0x87, 0xE5, 0x17, 0xBE, 0xF0, 0x62, 0xFE, 0x91, 0x8F, 0x3C, 0x9E, 0xF5, 0x7A, 0xC7, 
	0xF3, 0x28, 0xA2, 0xA6, 0x31, 0x4A, 0x10, 0x49, 0x78, 0x2F, 0xA0, 0x54, 0x8C, 0x46, 0x43, 0x0E, 
	0x07, 0x01, 0xED, 0x28, 0x8A, 0x8F, 0x07, 0x42, 0x44, 0xDF, 0x3D, 0xA6, 0x75, 0x00, 0xE4, 0xF0, 
	0x4A, 0x2B, 0x0B, 0x05, 0x7F, 0x70, 0xED, 0x00, 0x8E, 0x2E, 0x1C, 0x45, 0x5D, 0xD7, 0xDC, 0xD5, 
	0x1D, 0x2F, 0x53, 0xC9, 0xA4, 0x09, 0xA7, 0x97, 0x4F, 0x9B, 0xC7, 0x8E, 0x3E, 0x36, 0x48, 0x38, 
	0x31, 0xFB, 0xCF, 0xEC, 0xEF, 0xAE, 0xF6, 0x57, 0x0D, 0x2C, 0x30, 0xDC, 0x1A, 0x16, 0x3F, 0x74, 
	0xED, 0x0F, 0xC5, 0xFB, 0xF6, 0xEC, 0x0B, 0x8F, 0xD8, 0x23, 0x9C, 0xC6, 0xA9, 0x94, 0x42, 0x82, 
	0x24, 0xA1, 0x97, 0xF5, 0xB8, 0x88, 0x8A, 0xA8, 0xBD, 0xAB, 0xDD, 0x1E, 0x9A, 0x1B, 0xC2, 0xDA, 
	0xEC, 0x5A, 0xBF, 0xEE, 0xD6, 0x79, 0x95, 0x56, 0x91, 0x23, 0x27, 0xA6, 0xF5, 0x34, 0xEE, 0xD9, 
	0x71, 0x0F, 0x56, 0xCC, 0x4A, 0xE3, 0x81, 0x99, 0x07, 0x5E, 0x33, 0xD3, 0x9D, 0xB9, 0xC1, 0x17, 
	0x7E, 0xCE, 0x49, 0xE7, 0x2E, 0x0A, 0x0B, 0xAE, 0x2A, 0x8D, 0x67, 0x9F, 0xBD, 0xDA, 0xDC, 0x72, 
	0xCB, 0x0B, 0x67, 0x00, 0x5E, 0x25, 0x5A, 0x19, 0xD9, 0xB9, 0x73, 0x11, 0x8D, 0xC6, 0x04, 0x9E, 
	0x7B, 0x6E, 0x18, 0x7B, 0xF7, 0xAE, 0x63, 0x6A, 0x8A, 0x70, 0xE0, 0x40, 0x1A, 0x2C, 0x2C, 0x6C, 
	0x69, 0xDC, 0x7A, 0x6B, 0x4E, 0xD6, 0x7A, 0xF7, 0xF0, 0xC3, 0x73, 0x83, 0xD3, 0xA7, 0x73, 0x66, 
	0x8E, 0x44, 0x51, 0x28, 0xCD, 0xEC, 0xCC, 0xB7, 0x6F, 0xAC, 0x88, 0x88, 0xD2, 0x74, 0x24, 0x1C, 
	0x1E, 0x9E, 0x4C, 0xF6, 0xEC, 0x19, 0x15, 0x37, 0xDD, 0xB4, 0x1B, 0x53, 0x53, 0x4D, 0x5C, 0x7D, 
	0x75, 0x2E, 0x9B, 0xCD, 0xFD, 0xBC, 0xB8, 0x78, 0x3C, 0xF7, 0x5E, 0x4B, 0x60, 0x3C, 0xB1, 0xD6, 
	0x13, 0x51, 0x0C, 0xAD, 0xDB, 0x50, 0x2A, 0x0C, 0x98, 0x7D, 0xDB, 0xFB, 0x2D, 0x0A, 0x08, 0xBF, 
	0x2B, 0x60, 0xEE, 0x32, 0x60, 0x91, 0x69, 0xA9, 0x97, 0xB5, 0xD6, 0x55, 0xCF, 0xF7, 0x82, 0x55, 
	0xB3, 0x02, 0x11, 0x48, 0x21, 0xDA, 0x32, 0x90, 0x95, 0x57, 0xF0, 0x30, 0x2F, 0xCC, 0xBC, 0x50, 
	0xFF, 0xD6, 0xCA, 0x6F, 0xAD, 0x05, 0x08, 0x68, 0x66, 0x69, 0xC6, 0x94, 0x75, 0xE9, 0x01, 0xA0, 
	0xA8, 0x0A, 0x9E, 0x5B, 0x9B, 0x33, 0xF3, 0x1B, 0xF3, 0xF5, 0xE9, 0x8D, 0xD3, 0xE6, 0xE4, 0xC6, 
	0xC9, 0xDC, 0x09, 0xE7, 0xAD, 0xB3, 0x38, 0x73, 0xEE, 0x8C, 0xEA, 0xB7, 0xFA, 0xD1, 0xC0, 0x0F, 
	0x82, 0x60, 0x5B, 0x30, 0xA2, 0x2B, 0x2D, 0xCC, 0xC0, 0x58, 0x5B, 0xD8, 0x1A, 0x29, 0xA2, 0xB9, 
	0x6A, 0x0E, 0xAB, 0xF5, 0x2A, 0x6E, 0x1A, 0xBB, 0x09, 0xA7, 0xBB, 0xA7, 0xAF, 0x58, 0xEA, 0x2D, 
	0xBD, 0x3A, 0xF3, 0xD9, 0xE3, 0x44, 0xB4, 0x7E, 0x51, 0x00, 0x76, 0x4E, 0xE2, 0xA6, 0x9B, 0x8E, 
	0xC2, 0x5A, 0x79, 0x6C, 0x30, 0x68, 0x1C, 0x3B, 0x79, 0xD2, 0xEE, 0xD5, 0x7A, 0x8D, 0xB6, 0x6C, 
	0x39, 0x8D, 0x76, 0x3B, 0xC0, 0xFC, 0x7C, 0x8D, 0xA1, 0xA1, 0x33, 0xD8, 0xB5, 0x6B, 0xAB, 0xCC, 
	0xF3, 0x24, 0x32, 0x66, 0x85, 0x76, 0xEE, 0x64, 0x7A, 0xF5, 0xAB, 0x1B, 0x70, 0xCE, 0xE4, 0x83, 
	0x41, 0x1A, 0x0F, 0x06, 0x53, 0x8D, 0xB5, 0xB5, 0xE5, 0xAC, 0xAA, 0x0A, 0xFB, 0xF2, 0x5D, 0x33, 
	0x11, 0x40, 0x04, 0xC1, 0x6C, 0x68, 0x63, 0xA3, 0xC0, 0xC6, 0x86, 0xE3, 0xA1, 0x21, 0xC7, 0xC7, 
	0x8E, 0x75, 0xCD, 0xDA, 0x5A, 0xED, 0xBD, 0xEF, 0x7B, 0xE6, 0xA5, 0x2A, 0x49, 0x26, 0xA3, 0xAA, 
	0xD2, 0x92, 0xB9, 0x09, 0xAD, 0x47, 0x61, 0x6D, 0x20, 0xBC, 0x97, 0xC2, 0xB9, 0x0C, 0xDE, 0x87, 
	0xDF, 0xF5, 0x1E, 0x8C, 0x31, 0xA8, 0x8A, 0xAA, 0x9B, 0xB4, 0x92, 0x63, 0x61, 0x10, 0xAE, 0x37, 
	0x64, 0x23, 0x45, 0xC4, 0xD0, 0xD0, 0x82, 0x9A, 0xA2, 0x99, 0x95, 0xFD, 0xA2, 0xEC, 0x57, 0xBD, 
	0xDA, 0xD4, 0xFE, 0xC4, 0xC6, 0x89, 0x1A, 0x1E, 0x80, 0xFB, 0x66, 0x37, 0xBD, 0x5F, 0xF6, 0xFD, 
	0xFD, 0xDF, 0xB8, 0xBF, 0x7B, 0xE8, 0xEC, 0xA1, 0x7A, 0x7E, 0x6D, 0xDE, 0x1E, 0x5E, 0x3A, 0x5C, 
	0xCA, 0x48, 0x22, 0x2F, 0x72, 0xCC, 0xAD, 0xCC, 0x45, 0xB5, 0xAD, 0xC3, 0x8D, 0xD5, 0x0D, 0x18, 
	0x67, 0xB4, 0x6C, 0xCB, 0xE1, 0xBA, 0x5F, 0x17, 0x75, 0x51, 0xB3, 0x4C, 0x24, 0xF7, 0xD0, 0xA3, 
	0x07, 0xCE, 0x3D, 0x80, 0xF5, 0x72, 0x1D, 0x9D, 0xB2, 0xD3, 0x66, 0xE2, 0xEB, 0x85, 0x12, 0x13, 
	0xA4, 0x68, 0x7D, 0xEB, 0x7F, 0xD9, 0x8A, 0x85, 0xF7, 0x2F, 0xFC, 0x60, 0x01, 0x1F, 0x3C, 0xB8, 
	0x17, 0xB7, 0xDC, 0x72, 0x18, 0x00, 0xCD, 0x32, 0xFB, 0x47, 0xBA, 0xDD, 0xE4, 0x8E, 0xA3, 0x47, 
	0x77, 0x0F, 0x77, 0x3A, 0x5D, 0x64, 0x59, 0x80, 0x7E, 0x7F, 0x1A, 0x44, 0x39, 0xC6, 0xC6, 0x32, 
	0x54, 0x75, 0x2C, 0x8F, 0x1C, 0x45, 0x0D, 0x2C, 0xAB, 0x56, 0xAB, 0x15, 0x8E, 0x8E, 0xC6, 0x05, 
	0x33, 0x4A, 0x29, 0x5B, 0xCD, 0xC1, 0x20, 0xD3, 0x17, 0x00, 0x5F, 0x70, 0xD3, 0xDE, 0x3B, 0x1E, 
	0x0C, 0x16, 0xEB, 0x20, 0xF0, 0xD5, 0x81, 0x03, 0x11, 0x7E, 0xEB, 0xB7, 0x56, 0xEA, 0xC9, 0xC9, 
	0xD8, 0x7F, 0xF9, 0xCB, 0x2F, 0x96, 0x9D, 0x4E, 0xE5, 0x89, 0x04, 0xB4, 0x0E, 0xA9, 0xDD, 0x0E, 
	0xA9, 0x2C, 0x15, 0xEA, 0xBA, 0x05, 0x29, 0x03, 0x00, 0xBE, 0x16, 0xC2, 0x76, 0x99, 0x3B, 0x4E, 
	0x88, 0xD6, 0x77, 0xF7, 0x42, 0xBE, 0xC2, 0x17, 0x7F, 0xEB, 0x8B, 0xF5, 0x3B, 0x7E, 0xF3, 0x1D, 
	0x87, 0x88, 0xE8, 0xA4, 0x20, 0xDA, 0x2E, 0x59, 0x09, 0x25, 0x35, 0xE2, 0x30, 0x0A, 0x5C, 0xC3, 
	0xB6, 0xCA, 0xA2, 0xAA, 0x98, 0xB9, 0x78, 0xA9, 0x99, 0xF5, 0xB2, 0x71, 0x09, 0x26, 0xC6, 0xF1, 
	0xC5, 0xE3, 0xE6, 0xE4, 0xD2, 0x49, 0xC3, 0xE7, 0xC3, 0x4B, 0x42, 0x09, 0xBA, 0x83, 0xAE, 0x8A, 
	0xFA, 0x51, 0x4A, 0x39, 0x09, 0x33, 0x6B, 0x72, 0xF4, 0x51, 0xA9, 0x4A, 0x79, 0x63, 0x0C, 0xD5, 
	0xA6, 0x16, 0x92, 0x24, 0x07, 0xCD, 0x80, 0x66, 0xDC, 0x0C, 0x56, 0x8B, 0x55, 0x78, 0xEF, 0x95, 
	0xF1, 0x66, 0x9B, 0x54, 0x72, 0x54, 0x40, 0xC0, 0x07, 0xFE, 0xE2, 0xA8, 0x64, 0x11, 0x11, 0xA4, 
	0x5C, 0xC9, 0xBD, 0x1F, 0xF9, 0x1A, 0xE0, 0xDE, 0xD6, 0xEF, 0xA7, 0x3F, 0x54, 0x14, 0xA3, 0x00, 
	0x42, 0x00, 0x1A, 0x33, 0x33, 0x37, 0x60, 0x79, 0x25, 0x47, 0x36, 0xE8, 0x8B, 0xAA, 0x8A, 0xE1, 
	0x5D, 0x92, 0xB5, 0xDB, 0xCC, 0x65, 0x29, 0x58, 0x29, 0x57, 0x03, 0xBE, 0x8C, 0xA2, 0x50, 0xF6, 
	0x7A, 0x82, 0xBC, 0xF7, 0xE7, 0xCB, 0x93, 0x0C, 0xE6, 0x0A, 0x59, 0xD6, 0xA9, 0xAA, 0x6A, 0xCD, 
	0x01, 0x09, 0x8E, 0x1F, 0x17, 0x16, 0x20, 0x76, 0xCE, 0x9F, 0x77, 0xE1, 0x4D, 0x49, 0x34, 0x16, 
	0xE4, 0xB9, 0x25, 0x63, 0x1C, 0x84, 0xD0, 0x20, 0xF2, 0x50, 0x2A, 0x5B, 0x67, 0xEE, 0x9E, 0x5B, 
	0x5B, 0xFB, 0x44, 0xBD, 0x65, 0xCB, 0x7B, 0xBF, 0xEB, 0xF5, 0xE7, 0x1F, 0xC9, 0xF1, 0x96, 0xD1, 
	0xB7, 0xC3, 0x85, 0xEE, 0x88, 0x43, 0xF5, 0x84, 0x66, 0xBA, 0x95, 0x4A, 0xB4, 0x6B, 0x55, 0x43, 
	0x08, 0x41, 0x14, 0x8B, 0xA4, 0xD1, 0x68, 0xA4, 0x49, 0x23, 0x76, 0xFD, 0x2A, 0x33, 0xB5, 0xAB, 
	0xF9, 0x3B, 0xCA, 0x43, 0x0C, 0x78, 0x6C, 0x02, 0x11, 0x42, 0x00, 0x1E, 0xD2, 0xE4, 0xA6, 0x9D, 
	0x9D, 0xCE, 0x9A, 0xB6, 0x6B, 0xCB, 0xB5, 0x33, 0x6B, 0xAB, 0x79, 0x37, 0x2F, 0x21, 0x41, 0x6A, 
	0x4C, 0x35, 0xB5, 0xD0, 0xA1, 0x26, 0x4D, 0xBA, 0xAB, 0xE1, 0x9A, 0x16, 0xDD, 0xA4, 0x0B, 0x16, 
	0x0C, 0x58, 0x10, 0x11, 0xD1, 0xD6, 0x1B, 0xB6, 0x62, 0xF9, 0xD8, 0xF2, 0xC5, 0x01, 0xF8, 0xE9, 
	0xA7, 0xAF, 0xC3, 0x6D, 0xB7, 0x1D, 0x01, 0x11, 0x5E, 0x00, 0xE4, 0x97, 0x01, 0x71, 0xBD, 0x73, 
	0xB2, 0x2D, 0x04, 0x20, 0x04, 0x60, 0x4C, 0x02, 0xE7, 0x22, 0x78, 0xAE, 0xE0, 0x58, 0xAB, 0xC5, 
	0xA5, 0xA6, 0x5B, 0x5F, 0xF7, 0xA5, 0x10, 0x82, 0x89, 0x08, 0x41, 0xE0, 0xCA, 0x34, 0x8D, 0xA2, 
	0x8D, 0x0D, 0x2D, 0xAB, 0xAA, 0xB4, 0xDF, 0xDC, 0x68, 0xD5, 0xF0, 0xDE, 0x70, 0x59, 0x1A, 0x43, 
	0xE4, 0x41, 0x94, 0x00, 0x50, 0x20, 0xD2, 0x22, 0x08, 0xC6, 0x74, 0x10, 0xEC, 0x88, 0xAD, 0x1D, 
	0x8D, 0xD6, 0xD6, 0x6A, 0x02, 0x24, 0x5A, 0x2D, 0x09, 0x22, 0xC1, 0x40, 0x75, 0x8A, 0x68, 0xE3, 
	0xC4, 0xDE, 0xBD, 0xBF, 0xCA, 0xEB, 0xEB, 0x8F, 0x61, 0x46, 0x29, 0x10, 0x20, 0x2A, 0xE7, 0x1A, 
	0x4E, 0xA9, 0x91, 0x35, 0x21, 0xB6, 0x2E, 0x32, 0x4F, 0xEF, 0x4E, 0xD3, 0x89, 0x3D, 0x49, 0x92, 
	0x54, 0x42, 0x90, 0xFB, 0xF7, 0x54, 0x08, 0xE1, 0x97, 0x7F, 0xE3, 0xAE, 0x3A, 0xFF, 0xCA, 0x55, 
	0xBC, 0xBC, 0xDE, 0xA0, 0x76, 0x21, 0x3D, 0x1C, 0x15, 0x18, 0x4E, 0xDB, 0xF2, 0xEE, 0x89, 0xBB, 
	0xDB, 0xC3, 0x13, 0xC3, 0xC1, 0xE3, 0x47, 0x1E, 0xEF, 0x1E, 0x58, 0x38, 0x90, 0x59, 0x6F, 0xFF, 
	0xEA, 0x22, 0x8A, 0x07, 0xC8, 0x13, 0x29, 0xA3, 0x62, 0xDD, 0xD7, 0x49, 0xDE, 0xCD, 0xCB, 0xDE, 
	0x5A, 0xB7, 0xD3, 0xEF, 0xF4, 0x73, 0x04, 0x10, 0xD1, 0x58, 0x94, 0x46, 0x5B, 0xA3, 0x21, 0x1D, 
	0xEB, 0x28, 0xAE, 0x62, 0x92, 0x46, 0xA2, 0xAA, 0x18, 0x9C, 0xD5, 0x28, 0xA9, 0x62, 0x67, 0x5D, 
	0x97, 0x98, 0xBA, 0x87, 0x5E, 0x3C, 0x84, 0xD1, 0x2B, 0x46, 0x2F, 0x9E, 0x6E, 0x12, 0x51, 0x00, 
	0x29, 0x75, 0xD7, 0x7B, 0xF7, 0x45, 0x80, 0xDE, 0x42, 0x84, 0x1F, 0x22, 0xFA, 0x66, 0x2C, 0x95, 
	0x52, 0x20, 0x6D, 0x24, 0xC8, 0x8B, 0x54, 0x66, 0x59, 0x1D, 0x78, 0x9F, 0x57, 0x42, 0x10, 0x0B, 
	0x21, 0x20, 0x04, 0xB9, 0x66, 0x53, 0xDB, 0x46, 0x23, 0x7E, 0x19, 0xE0, 0x4D, 0xFF, 0x77, 0x21, 
	0x55, 0xDA, 0x4C, 0x7F, 0x24, 0x94, 0x6A, 0xAB, 0x38, 0x9E, 0x8C, 0xDB, 0xED, 0xB1, 0x58, 0xA9, 
	0x71, 0xBD, 0xB1, 0x11, 0x90, 0x31, 0x3D, 0x28, 0x55, 0x20, 0x0C, 0xD7, 0x21, 0x84, 0xEA, 0x14, 
	0x75, 0xFD, 0xD5, 0x0D, 0x56, 0xA7, 0xB3, 0x7C, 0xAF, 0x5E, 0xA4, 0xF1, 0x5D, 0x7E, 0x64, 0xE4, 
	0x66, 0x0A, 0x82, 0xDB, 0x1A, 0x53, 0x53, 0x37, 0xFB, 0xED, 0xDB, 0x77, 0x19, 0x60, 0xD4, 0x1E, 
	0x3E, 0x9C, 0x44, 0x6B, 0x6B, 0x01, 0x11, 0x89, 0x33, 0x5A, 0xE3, 0xB8, 0x10, 0x3C, 0xC5, 0x5C, 
	0x7F, 0xF8, 0x11, 0x59, 0xFF, 0xF4, 0x01, 0x13, 0x1C, 0x99, 0x60, 0x1C, 0x9C, 0x62, 0x3C, 0xBE, 
	0x0B, 0x98, 0x7C, 0xD5, 0x4D, 0xF4, 0xBE, 0x3D, 0xFF, 0x38, 0x14, 0x4B, 0x14, 0x54, 0xDD, 0xCA, 
	0x1F, 0x5F, 0x3D, 0x5E, 0x74, 0xAA, 0xCE, 0x77, 0x74, 0xAC, 0x04, 0x0B, 0x21, 0xBD, 0x94, 0x5C, 
	0x31, 0x97, 0x45, 0x69, 0x5C, 0xDF, 0x6D, 0x58, 0x63, 0x6D, 0x6D, 0xEA, 0x5A, 0xA4, 0x22, 0x88, 
	0xB6, 0x45, 0x43, 0xD1, 0x78, 0xD4, 0x52, 0xA1, 0x0A, 0x34, 0x34, 0x48, 0x00, 0xA5, 0x28, 0x61, 
	0x61, 0x41, 0x56, 0x40, 0x39, 0x55, 0xC1, 0xE2, 0x8C, 0xF7, 0x7E, 0x35, 0x0E, 0x62, 0x2C, 0xFF, 
	0xB3, 0xE5, 0x8B, 0x07, 0xF0, 0x13, 0x4F, 0x5C, 0x89, 0xD7, 0xBE, 0xF6, 0x2C, 0x88, 0xC4, 0x61, 
	0x80, 0xBE, 0x40, 0x84, 0xEB, 0x88, 0xC4, 0x10, 0x11, 0x81, 0xCE, 0x93, 0xD6, 0x4A, 0xA3, 0x95, 
	0xA6, 0x54, 0x55, 0xB5, 0xF6, 0xB6, 0x92, 0x44, 0xDE, 0x13, 0x29, 0x10, 0x09, 0xC4, 0xB1, 0x34, 
	0xED, 0x76, 0xA4, 0xBB, 0x5D, 0x21, 0x9C, 0x73, 0x7E, 0xB3, 0x86, 0x1D, 0x82, 0x88, 0x11, 0x04, 
	0x8A, 0xC6, 0xC7, 0x87, 0xA5, 0xF7, 0x93, 0xBA, 0xAE, 0xF7, 0xC4, 0x69, 0x9A, 0x44, 0x71, 0x4C, 
	0xC2, 0x98, 0x11, 0x44, 0x51, 0x00, 0xA5, 0x2C, 0xA2, 0xA4, 0x40, 0x14, 0x77, 0x0D, 0x8A, 0xC1, 
	0xB3, 0x77, 0x15, 0x9F, 0x3A, 0xF7, 0x93, 0xF5, 0xEF, 0xDD, 0x77, 0x3C, 0x1C, 0x79, 0x4B, 0x44, 
	0x7C, 0xAB, 0x9E, 0x98, 0xD8, 0xDE, 0xFA, 0xB9, 0x9F, 0x0B, 0xE5, 0x5D, 0x77, 0x61, 0xCD, 0x5A, 
	0x94, 0x8F, 0x3F, 0x8E, 0xA1, 0x99, 0x19, 0xF0, 0xDA, 0x1A, 0x5C, 0x92, 0x60, 0x2C, 0x0C, 0x51, 
	0x18, 0x43, 0xA2, 0xAE, 0x43, 0x5D, 0xBB, 0x70, 0x6B, 0xCE, 0xD8, 0xB2, 0x4A, 0x78, 0xFD, 0x11, 
	0xC2, 0x3D, 0x2D, 0xE0, 0x74, 0xC7, 0x9A, 0x9D, 0x7B, 0x47, 0x45, 0x7F, 0x98, 0x85, 0x0A, 0x15, 
	0x98, 0xBE, 0xD5, 0x78, 0x09, 0x84, 0x50, 0x87, 0x61, 0x33, 0x68, 0xB6, 0x34, 0xEB, 0x30, 0xA7, 
	0x3C, 0xEB, 0xF7, 0xFA, 0xBD, 0xAA, 0xAC, 0x2A, 0x52, 0x44, 0x7A, 0x4C, 0x37, 0xE2, 0xA9, 0x78, 
	0x38, 0x18, 0x0A, 0x9A, 0x52, 0x4A, 0x19, 0xD9, 0x10, 0x01, 0x85, 0xE0, 0x90, 0xC1, 0xA8, 0xE1, 
	0xDD, 0xA6, 0x6B, 0x0F, 0x64, 0xB0, 0x14, 0xC8, 0xE0, 0x69, 0x76, 0xBC, 0x4E, 0x44, 0xC8, 0x90, 
	0x5D, 0x5C, 0x0D, 0xFF, 0xCD, 0x58, 0xAC, 0x7A, 0xDE, 0xBB, 0x07, 0x00, 0xBA, 0x5B, 0x08, 0xBC, 
	0x31, 0x0C, 0x1D, 0x39, 0x27, 0xC0, 0x4C, 0xC8, 0x8B, 0x1A, 0x41, 0x00, 0x28, 0x39, 0xAC, 0xAC, 
	0x37, 0x12, 0xE8, 0x18, 0xC0, 0x83, 0x48, 0x41, 0x4A, 0xE9, 0xD3, 0x34, 0x34, 0x41, 0x40, 0x5C, 
	0x14, 0x17, 0x76, 0xD2, 0x1A, 0x5A, 0x6B, 0x7A, 0xD3, 0x9B, 0xB6, 0xC7, 0x3F, 0xFE, 0xE3, 0x7B, 
	0x1B, 0x4B, 0x4B, 0xA3, 0xF2, 0xF3, 0x9F, 0x9F, 0x90, 0xAB, 0xAB, 0x8E, 0xAA, 0x4A, 0xC2, 0xDA, 
	0x14, 0xA4, 0x0C, 0x92, 0x80, 0x31, 0x15, 0x64, 0x78, 0x95, 0x79, 0xD6, 0xFD, 0xB0, 0xFF, 0xDA, 
	0xCE, 0x2B, 0xA3, 0x17, 0xFE, 0x6D, 0xAE, 0x92, 0x89, 0x92, 0x59, 0x5B, 0xE7, 0x10, 0x2B, 0x05, 
	0x79, 0xEE, 0x1C, 0xF2, 0x8F, 0x7D, 0x0C, 0x1B, 0xA7, 0x4F, 0xA3, 0xF1, 0xC2, 0x0B, 0x88, 0xBB, 
	0x5D, 0xCC, 0x1A, 0xE3, 0x5D, 0xA7, 0x43, 0x43, 0x49, 0x42, 0xD7, 0x4E, 0x4C, 0x20, 0xBA, 0xE6, 
	0x1A, 0x0C, 0x76, 0xEC, 0xC0, 0xB9, 0xD5, 0x55, 0xF8, 0x87, 0x1E, 0x82, 0xE8, 0xF5, 0x20, 0x3A, 
	0x70, 0x2B, 0xFF, 0xE9, 0x8B, 0x2B, 0x0F, 0x1D, 0xDA, 0x60, 0x7F, 0xDF, 0x5D, 0x23, 0xCF, 0xD7, 
	0x87, 0x7C, 0xC1, 0xC5, 0xB7, 0x10, 0x56, 0xA4, 0x64, 0xBB, 0xD1, 0x6E, 0x35, 0x82, 0xC6, 0x90, 
	0x2D, 0xAD, 0x65, 0x66, 0xCF, 0xC4, 0x4C, 0x21, 0xC9, 0x60, 0x22, 0x48, 0xE3, 0xA9, 0x78, 0x58, 
	0xB7, 0x74, 0x42, 0x44, 0x44, 0x8E, 0x40, 0x20, 0x58, 0xB2, 0x30, 0xDE, 0xC0, 0x7A, 0x7B, 0xE1, 
	0xAF, 0xC4, 0x80, 0xB0, 0x9F, 0x88, 0x1E, 0x2B, 0xB8, 0xA8, 0x46, 0xF4, 0xC8, 0xC5, 0x37, 0xD1, 
	0xF1, 0xD8, 0x63, 0x3B, 0x70, 0xC7, 0x1D, 0x73, 0x20, 0xA2, 0xC3, 0x80, 0xF8, 0xB4, 0x52, 0x7E, 
	0x6F, 0xA3, 0x51, 0x4E, 0x33, 0x7B, 0x54, 0x75, 0x0C, 0x6B, 0x23, 0x14, 0x65, 0x00, 0xE6, 0x50, 
	0x48, 0x29, 0xA5, 0x94, 0x96, 0x80, 0x01, 0x33, 0xD7, 0x20, 0x8A, 0x91, 0xA6, 0xA9, 0x6F, 0xB7, 
	0x63, 0x14, 0x45, 0xEF, 0x25, 0xF7, 0x3C, 0x3E, 0x3E, 0x24, 0xDF, 0xF9, 0xCE, 0x9B, 0x93, 0x1F, 
	0xFB, 0xB1, 0xAB, 0xC2, 0x95, 0x15, 0x81, 0x53, 0xA7, 0x2A, 0x7C, 0xE3, 0x61, 0x86, 0x63, 0x85, 
	0x88, 0x32, 0xEC, 0xA3, 0x63, 0xB8, 0x83, 0xBF, 0x88, 0x3B, 0xEA, 0xC7, 0xB1, 0x8B, 0x17, 0xA2, 
	0x90, 0xAA, 0xDD, 0xB9, 0x60, 0x64, 0x44, 0xD0, 0x00, 0xD2, 0x28, 0x82, 0xCC, 0x73, 0xAC, 0x7C, 
	0xEE, 0x73, 0xC8, 0x07, 0x03, 0x88, 0x2C, 0x43, 0x0E, 0xE0, 0x85, 0xBA, 0x1E, 0x3C, 0x5B, 0xD7, 
	0x9D, 0xA1, 0x5E, 0x2F, 0x7E, 0xE3, 0xFB, 0xDE, 0x37, 0x74, 0xED, 0x2F, 0xFE, 0xA2, 0xA8, 0x86, 
	0x86, 0xB0, 0x7A, 0xE8, 0x10, 0x06, 0x9F, 0xF9, 0x0C, 0x54, 0x5D, 0x43, 0x10, 0x21, 0x20, 0x70, 
	0x9B, 0x3D, 0x1D, 0x7B, 0xF0, 0xA1, 0x41, 0xF7, 0xEC, 0xD3, 0x6E, 0xF1, 0x87, 0x55, 0xE2, 0x1A, 
	0x5E, 0xA3, 0x86, 0x23, 0x10, 0x60, 0x01, 0x6F, 0x3D, 0x97, 0xA2, 0x2C, 0x2D, 0xD9, 0xB5, 0xB2, 
	0x2C, 0xAB, 0xB2, 0x2A, 0x0B, 0x8A, 0x48, 0xA4, 0xE3, 0x69, 0x7B, 0x62, 0xDB, 0xF8, 0x70, 0x94, 
	0x46, 0x51, 0xC6, 0x03, 0xAA, 0x6C, 0x05, 0x2F, 0x3C, 0x0A, 0x55, 0x6E, 0x86, 0xEB, 0x0B, 0x8E, 
	0x4A, 0xC0, 0x83, 0xF0, 0x2C, 0x13, 0x7F, 0x92, 0xC1, 0xA7, 0x22, 0x19, 0xE1, 0xCC, 0xAF, 0x9E, 
	0xB9, 0xF8, 0x00, 0x6F, 0xEE, 0x20, 0x15, 0xBC, 0x8F, 0x0A, 0x66, 0xF3, 0x79, 0xEF, 0xE9, 0x56, 
	0xEF, 0xCD, 0xBB, 0xA6, 0xA6, 0xBA, 0xC9, 0xD8, 0x58, 0x89, 0xD9, 0xD9, 0x36, 0x16, 0x16, 0x1B, 
	0x48, 0x47, 0x4A, 0x0A, 0xC3, 0x48, 0x76, 0xBB, 0x31, 0xAA, 0xAA, 0x84, 0x52, 0x86, 0x88, 0x00, 
	0xAD, 0x43, 0x1E, 0x1D, 0x6D, 0x63, 0x7D, 0xBD, 0x87, 0xAA, 0x62, 0x48, 0x39, 0xA2, 0x86, 0x86, 
	0xF6, 0x35, 0xB6, 0x6E, 0xBD, 0x2A, 0x18, 0x1E, 0x1E, 0x41, 0x59, 0xD5, 0x88, 0xE2, 0x05, 0x4C, 
	0x84, 0x67, 0x71, 0xC5, 0xE0, 0x08, 0xEE, 0xAE, 0xF6, 0xE3, 0x2E, 0x79, 0x0C, 0xA3, 0xB4, 0x01, 
	0x02, 0xE0, 0x41, 0xF0, 0x20, 0xC4, 0x04, 0x8C, 0x2B, 0x85, 0x72, 0xF7, 0x6E, 0x44, 0xF7, 0xDE, 
	0x8B, 0xF8, 0xE6, 0x9B, 0x91, 0x97, 0x25, 0xBA, 0x5F, 0xFA, 0x12, 0xDC, 0xFD, 0xF7, 0xF3, 0x99, 
	0xC1, 0x60, 0xF0, 0x8D, 0xA2, 0x58, 0xED, 0x32, 0xD7, 0x11, 0x50, 0xD8, 0x03, 0x07, 0xAA, 0x73, 
	0xBF, 0xF6, 0x6B, 0xC3, 0x75, 0x9E, 0x87, 0xD1, 0xC2, 0x02, 0x0D, 0x77, 0x3A, 0xF5, 0x32, 0x90, 
	0xC5, 0xCE, 0x05, 0xA3, 0xDE, 0x37, 0xAF, 0x0B, 0xC3, 0x09, 0xEF, 0x9C, 0x38, 0x75, 0xA2, 0xDB, 
	0xBB, 0x99, 0x94, 0xEF, 0xDC, 0x19, 0xCA, 0x5E, 0x43, 0x09, 0x97, 0x5B, 0xB8, 0xDA, 0x79, 0x57, 
	0x3B, 0xDF, 0x1B, 0xF4, 0x32, 0x12, 0x04, 0x8A, 0x88, 0x65, 0x4B, 0x8A, 0x89, 0xD1, 0x89, 0xF6, 
	0x4D, 0x53, 0x37, 0x8D, 0x5C, 0xD1, 0xBA, 0x22, 0x2C, 0x79, 0x40, 0xCF, 0xF5, 0x0E, 0x61, 0xC9, 
	0x2E, 0xA3, 0x0A, 0x2B, 0x38, 0xE1, 0x5E, 0xEE, 0xDF, 0x1D, 0x08, 0xCF, 0x03, 0xF8, 0xBF, 0x01, 
	0x7C, 0x05, 0xC0, 0xF7, 0x7C, 0x7C, 0xE7, 0x7B, 0x3A, 0x74, 0x37, 0x33, 0xF3, 0x7F, 0x62, 0xE7, 
	0xCE, 0xFF, 0x0D, 0xCE, 0x05, 0x3D, 0x29, 0x93, 0x42, 0x08, 0x79, 0xD5, 0xD0, 0xD0, 0xFA, 0xD4, 
	0xEE, 0xDD, 0xA7, 0xC5, 0xD0, 0x90, 0x45, 0x9E, 0x37, 0x31, 0x35, 0xB5, 0x42, 0x37, 0xDE, 0xB8, 
	0x4C, 0x44, 0xE0, 0xAA, 0x52, 0xB4, 0x77, 0x6F, 0x10, 0x6F, 0xDD, 0x1A, 0x06, 0x79, 0x9E, 0x28, 
	0xEF, 0x5B, 0x2A, 0xCF, 0x73, 0x54, 0x55, 0x2C, 0xD3, 0xF4, 0x35, 0x69, 0x14, 0x5D, 0x9B, 0x8C, 
	0x8D, 0xB5, 0xC4, 0xD0, 0xB0, 0xC4, 0x8B, 0x4F, 0x9C, 0x41, 0xF6, 0x17, 0x9F, 0xC4, 0xDD, 0x8B, 
	0x1F, 0xC3, 0x3D, 0x63, 0xCF, 0xE1, 0x96, 0xDB, 0xC6, 0x20, 0x5E, 0xB3, 0x0F, 0xEB, 0x8D, 0x26, 
	0xEA, 0xD5, 0x35, 0x2C, 0xE8, 0x00, 0x4B, 0x4A, 0x01, 0x42, 0xA0, 0x4C, 0x12, 0xC4, 0xEF, 0x79, 
	0x0F, 0x1A, 0xB7, 0xDE, 0x0A, 0x73, 0xFC, 0x38, 0xEC, 0x5F, 0xFE, 0x25, 0xFA, 0x5F, 0xFC, 0xA2, 
	0x39, 0xB4, 0xBC, 0xB2, 0xF1, 0x9C, 0xA9, 0xD7, 0x72, 0x66, 0x27, 0xCE, 0x67, 0xB4, 0xCB, 0x4B, 
	0x4B, 0xF5, 0xCC, 0xB1, 0x63, 0x03, 0xB1, 0xB8, 0x58, 0x27, 0x42, 0xE4, 0xF3, 0x41, 0xB0, 0xDE, 
	0xAF, 0x6B, 0x84, 0x49, 0xD2, 0xB4, 0x49, 0xA2, 0x52, 0x66, 0xB1, 0x23, 0x08, 0x62, 0x21, 0x84, 
	0xAD, 0x16, 0xCB, 0xDC, 0xB7, 0x23, 0xB1, 0x7E, 0x5D, 0x9C, 0xB2, 0xE2, 0x88, 0x3D, 0x33, 0x31, 
	0x79, 0x30, 0x58, 0xC4, 0x02, 0x6A, 0x58, 0x51, 0x38, 0x12, 0xB6, 0x6E, 0xD8, 0x75, 0xC3, 0xE8, 
	0x1B, 0xAF, 0x7C, 0x63, 0x34, 0xD2, 0x1E, 0xA1, 0xF9, 0x7A, 0x01, 0xE7, 0xCA, 0x59, 0xE4, 0x32, 
	0x87, 0x53, 0x6E, 0xF3, 0xB7, 0x6E, 0xB6, 0xCA, 0x4B, 0x10, 0xF6, 0x63, 0x73, 0x6A, 0xFF, 0x33, 
	0x00, 0xD6, 0x01, 0x60, 0xF9, 0xD7, 0x97, 0xBF, 0xA7, 0x80, 0xBF, 0xE7, 0x67, 0x93, 0x1E, 0x7E, 
	0x38, 0xC4, 0x1B, 0xDF, 0xC8, 0x8E, 0x19, 0x0F, 0x1B, 0xD3, 0xFA, 0xF8, 0xE2, 0xE2, 0xF4, 0xA8, 
	0xD6, 0x6B, 0x57, 0x4F, 0x4D, 0x9D, 0xA1, 0xD1, 0xD1, 0x61, 0x0C, 0x06, 0x40, 0x59, 0x2E, 0x04, 
	0x3B, 0x76, 0x34, 0x5B, 0x75, 0x3D, 0xE5, 0xD3, 0x74, 0x49, 0x4D, 0x4F, 0x97, 0xD0, 0x7A, 0x9A, 
	0x4F, 0x9E, 0x9C, 0xE2, 0x7E, 0x3F, 0x2A, 0xF2, 0xFC, 0xA4, 0x09, 0xC3, 0x76, 0x50, 0x55, 0x91, 
	0xF8, 0xF4, 0x67, 0x2C, 0x06, 0x4F, 0xEC, 0xC7, 0xDB, 0x96, 0x7F, 0x07, 0xF7, 0xAD, 0x3C, 0x8B, 
	0xE1, 0x9F, 0x7C, 0x37, 0x1A, 0xEF, 0xB9, 0x0F, 0x95, 0x94, 0x58, 0x78, 0xEA, 0x29, 0x9C, 0x98, 
	0x9B, 0x87, 0xD5, 0x1A, 0xA1, 0xB5, 0x08, 0x88, 0x50, 0x08, 0x01, 0x59, 0x96, 0x28, 0x7F, 0xEF, 
	0xF7, 0xA0, 0x00, 0xE4, 0x59, 0x66, 0x4F, 0xF7, 0xFB, 0x83, 0x43, 0x65, 0x31, 0x58, 0x80, 0x05, 
	0x49, 0x6E, 0xB5, 0x15, 0x05, 0x20, 0x92, 0x4A, 0xEA, 0xA2, 0x99, 0xA4, 0xFD, 0x24, 0x0C, 0x1D, 
	0x98, 0x7B, 0x2F, 0xE6, 0x39, 0x36, 0xAA, 0x8A, 0x76, 0x32, 0xB7, 0xAA, 0x66, 0x33, 0xEC, 0x34, 
	0x1A, 0x68, 0x32, 0x23, 0xAA, 0x6B, 0x79, 0x75, 0x18, 0x8E, 0xAD, 0x90, 0xE0, 0xDE, 0xA1, 0xBC, 
	0x9C, 0xB9, 0x25, 0x1C, 0xEA, 0x6C, 0x4B, 0x15, 0x37, 0x7C, 0x5E, 0xF7, 0xEA, 0x5E, 0x9D, 0xD7, 
	0x03, 0x48, 0x58, 0x0A, 0x29, 0xA4, 0x84, 0xDA, 0xEB, 0x6A, 0x3D, 0x7A, 0xA2, 0x78, 0x82, 0x32, 
	0x93, 0x61, 0x36, 0x9F, 0x45, 0x5F, 0xF5, 0xE0, 0xE0, 0xC1, 0xDF, 0xAC, 0xB3, 0xAF, 0x01, 0x78, 
	0x08, 0xC0, 0x1F, 0x02, 0xF8, 0x06, 0x80, 0xDE, 0x2B, 0x01, 0xF7, 0x15, 0x01, 0xFC, 0xCD, 0x46, 
	0x04, 0x32, 0xEF, 0xE9, 0x53, 0x83, 0xC1, 0xD8, 0xD0, 0xEC, 0xEC, 0xBE, 0x9F, 0x2D, 0x8A, 0xD9, 
	0x2B, 0x9C, 0x4B, 0x44, 0xAF, 0xD7, 0x06, 0x73, 0x0F, 0x93, 0x93, 0xAB, 0x0A, 0x18, 0xC2, 0xDC, 
	0x5C, 0x08, 0x29, 0x0F, 0x62, 0x64, 0x64, 0x11, 0xFD, 0xFE, 0x15, 0xE8, 0xF5, 0x76, 0x87, 0xE7, 
	0xCE, 0x1D, 0xA8, 0xF2, 0xFC, 0x99, 0x02, 0x22, 0xA1, 0x68, 0xB9, 0x2B, 0x7F, 0x78, 0xF9, 0x37, 
	0xE8, 0x6A, 0x3A, 0x0A, 0xAF, 0x43, 0xF0, 0xD1, 0x17, 0x91, 0x7F, 0xF4, 0xA3, 0xE0, 0x85, 0x05, 
	0x0C, 0xCF, 0xCC, 0xE0, 0xCA, 0x3C, 0x87, 0x04, 0xD0, 0xDE, 0x9C, 0xD0, 0x03, 0x8F, 0x8F, 0x23, 
	0xBC, 0xEA, 0x2A, 0xD8, 0xA9, 0x29, 0x2C, 0xD4, 0xB5, 0x3F, 0xF8, 0xF9, 0xCF, 0x15, 0x1B, 0xDD, 
	0x75, 0x69, 0xA3, 0x60, 0xAC, 0x29, 0x43, 0x1D, 0xC6, 0x09, 0xA7, 0x49, 0xA3, 0x82, 0xF7, 0x99, 
	0xA9, 0xCD, 0xA0, 0xAA, 0x8C, 0x2B, 0xFB, 0x7D, 0x38, 0xE7, 0x88, 0x01, 0x48, 0x66, 0x54, 0x80, 
	0x89, 0xBD, 0x47, 0x97, 0x19, 0x3D, 0x29, 0x91, 0x32, 0x43, 0x4A, 0xA9, 0xC6, 0xB6, 0x6D, 0x9D, 
	0xDC, 0xB1, 0xB6, 0xBE, 0xBA, 0xFB, 0xB9, 0x22, 0x3F, 0xFA, 0x96, 0x91, 0xD1, 0x20, 0x0E, 0x5A, 
	0x83, 0x66, 0x16, 0xF8, 0xBE, 0x0B, 0xBC, 0xE5, 0xCA, 0x91, 0x4B, 0x5C, 0xEC, 0x92, 0x05, 0x2C, 
	0xD0, 0xEA, 0xC6, 0x2A, 0x9C, 0x71, 0x20, 0x4F, 0x90, 0x90, 0x9B, 0x03, 0x10, 0x60, 0x07, 0xE0, 
	0x34, 0x80, 0x4F, 0x03, 0xF8, 0x63, 0x00, 0xCF, 0x03, 0xA8, 0x5E, 0x09, 0xB0, 0xAF, 0x28, 0xE0, 
	0xAF, 0x7E, 0x95, 0xF0, 0xE6, 0x37, 0x33, 0x00, 0x5E, 0xF5, 0x9E, 0xFE, 0x4B, 0xBF, 0x3F, 0x52, 
	0x95, 0x65, 0xFA, 0x13, 0x42, 0xE8, 0xEB, 0x89, 0x54, 0xD0, 0xE9, 0xDC, 0x08, 0x63, 0xFA, 0x28, 
	0x8A, 0x61, 0x38, 0x27, 0x31, 0x3F, 0xAF, 0x31, 0x3A, 0xBA, 0x88, 0x28, 0xEA, 0x63, 0x6C, 0xAC, 
	0x29, 0xD3, 0x74, 0x9B, 0x9A, 0x9F, 0x3F, 0x5E, 0xB9, 0x6A, 0xCD, 0xEE, 0xE5, 0xBF, 0x6C, 0xB5, 
	0x9A, 0x07, 0x95, 0x8F, 0x13, 0xC0, 0x5A, 0x60, 0xFF, 0x7E, 0x5C, 0xE8, 0x1C, 0x1B, 0x00, 0xA1, 
	0xF7, 0x50, 0x44, 0x50, 0x42, 0x40, 0xDF, 0xF5, 0x06, 0x8E, 0xFE, 0xC5, 0x87, 0xA8, 0x18, 0x1F, 
	0xC7, 0xD9, 0x13, 0xC7, 0x70, 0xF0, 0x53, 0x7F, 0x22, 0xFA, 0x65, 0x9E, 0x8E, 0x44, 0x89, 0x37, 
	0xCD, 0xB4, 0x56, 0x43, 0x43, 0xEB, 0x00, 0x8A, 0x95, 0x8D, 0x8D, 0xB2, 0xE8, 0xF5, 0xD8, 0xD6, 
	0x35, 0x3C, 0xF3, 0x4B, 0x53, 0x37, 0x04, 0x20, 0x20, 0xE2, 0x8A, 0xD9, 0xA4, 0x55, 0xE5, 0x33, 
	0x22, 0x51, 0xA6, 0x29, 0xAC, 0x73, 0x10, 0x55, 0x85, 0x50, 0x08, 0x31, 0x34, 0x31, 0x31, 0x7A, 
	0xD3, 0xD9, 0x85, 0xF5, 0x33, 0xCE, 0x98, 0x7A, 0xAB, 0xD4, 0x1C, 0x70, 0x4C, 0x81, 0xD0, 0xD2, 
	0xC3, 0x09, 0x12, 0x52, 0x86, 0x52, 0x89, 0x58, 0x40, 0x92, 0x84, 0xF2, 0x0A, 0x41, 0x1D, 0x00, 
	0x12, 0xC8, 0x91, 0x67, 0x1E, 0xFE, 0x69, 0x00, 0x9F, 0x04, 0xE1, 0x2F, 0x00, 0x9C, 0x03, 0xE0, 
	0x5F, 0x49, 0xB8, 0xAF, 0xA8, 0x05, 0x3F, 0xF8, 0x20, 0xE1, 0xF5, 0xAF, 0xCF, 0x10, 0x04, 0x62, 
	0xA9, 0xAA, 0xDC, 0x1F, 0x32, 0x87, 0x73, 0x5A, 0xAB, 0xFB, 0xC2, 0xD0, 0xBF, 0x4E, 0xEB, 0x7A, 
	0x84, 0x28, 0x0C, 0x94, 0xD2, 0x70, 0x4E, 0xA2, 0xD7, 0xBB, 0x0A, 0xC6, 0x4C, 0x23, 0x0C, 0x05, 
	0x98, 0x07, 0x00, 0x32, 0xAE, 0xCC, 0x5C, 0xBD, 0xD3, 0xCC, 0x84, 0xBB, 0xC5, 0x23, 0xE2, 0xE9, 
	0x9E, 0xC7, 0xA2, 0xF3, 0xD8, 0x32, 0x34, 0x84, 0x61, 0x00, 0xD2, 0x5A, 0xAC, 0x58, 0x8B, 0xAE, 
	0x73, 0x30, 0xCC, 0x4C, 0x80, 0xD7, 0x82, 0xFD, 0xDA, 0xB1, 0x43, 0x70, 0x1F, 0xFD, 0x0D, 0xB5, 
	0x71, 0xEE, 0x1C, 0x2F, 0x9E, 0x9D, 0x31, 0xB6, 0xD3, 0xAD, 0x26, 0x92, 0xB4, 0x56, 0x13, 0x93, 
	0x79, 0xD2, 0x6C, 0x9A, 0xD9, 0x8D, 0x0D, 0x3F, 0xBB, 0xBC, 0x0C, 0x69, 0x0C, 0xC2, 0x97, 0x41, 
	0xE5, 0x97, 0x35, 0x2A, 0x99, 0x99, 0x2C, 0xB3, 0x33, 0x71, 0xEC, 0xC6, 0xEE, 0xB9, 0x47, 0xE4, 
	0x57, 0x5D, 0x85, 0xA5, 0x53, 0xA7, 0x30, 0xFE, 0xE9, 0x4F, 0x63, 0x7C, 0x30, 0x40, 0xB7, 0xD9, 
	0x94, 0x37, 0x0F, 0x1A, 0xCD, 0x67, 0xE6, 0x8A, 0xFA, 0x85, 0x29, 0xD2, 0x14, 0x12, 0x62, 0x11, 
	0x2B, 0xEF, 0xBD, 0x62, 0xC1, 0xD0, 0x4A, 0x23, 0xF6, 0x31, 0x04, 0xC4, 0xA6, 0x4B, 0x96, 0xA8, 
	0x6C, 0x68, 0xCF, 0x80, 0xF1, 0x10, 0x2C, 0x3E, 0x03, 0xC6, 0x7E, 0x78, 0x74, 0x41, 0xC0, 0xF2, 
	0xFF, 0xFA, 0xCA, 0xC2, 0x7D, 0x45, 0x01, 0x6F, 0xC6, 0xE3, 0x26, 0x5E, 0xFD, 0xEA, 0x39, 0x6C, 
	0xDB, 0xB6, 0x6D, 0x23, 0xCF, 0xF9, 0x33, 0x44, 0x38, 0x44, 0x44, 0x77, 0x79, 0x1F, 0xBD, 0xC6, 
	0x7B, 0x77, 0x75, 0x14, 0x99, 0x2D, 0xCC, 0x72, 0xC4, 0x18, 0x99, 0xD4, 0x75, 0x4A, 0x75, 0x6D, 
	0x69, 0x65, 0x65, 0xD1, 0x96, 0x65, 0x58, 0x85, 0x2A, 0x5D, 0x7B, 0x73, 0xBC, 0xBA, 0xED, 0xA6, 
	0xB0, 0x29, 0xA4, 0x90, 0x58, 0x0F, 0x43, 0xCC, 0x37, 0x1A, 0x68, 0x0F, 0x06, 0xC0, 0xEA, 0x2A, 
	0x8A, 0xBA, 0xC6, 0x64, 0x9A, 0x22, 0x88, 0x63, 0x77, 0xA6, 0xD7, 0xE1, 0xF5, 0x3C, 0x93, 0xBA, 
	0xD7, 0xAB, 0x4F, 0x3D, 0xF1, 0xCC, 0x0A, 0x04, 0xB9, 0x28, 0x8C, 0x4C, 0xBB, 0x35, 0xE4, 0x92, 
	0xB1, 0x31, 0x20, 0x0C, 0x71, 0x76, 0x76, 0x16, 0x2B, 0x9D, 0x0E, 0x42, 0x00, 0x92, 0x08, 0x38, 
	0xEF, 0xD2, 0xC1, 0x0C, 0x66, 0x06, 0x98, 0xD9, 0x12, 0x51, 0x09, 0x80, 0x98, 0x6D, 0x79, 0xCF, 
	0x3D, 0x66, 0xF7, 0x7D, 0xF7, 0x69, 0x06, 0x30, 0xB8, 0xF6, 0x5A, 0xD8, 0xD5, 0x55, 0xB4, 0xFF, 
	0xFC, 0xCF, 0xB1, 0xB3, 0x2C, 0x91, 0xB0, 0x8C, 0x6E, 0x7F, 0x8E, 0x7B, 0x47, 0xAF, 0x41, 0x19, 
	0xD9, 0x66, 0xA8, 0x20, 0xA9, 0x0C, 0x4A, 0x94, 0x28, 0xC1, 0x82, 0xE1, 0x84, 0x63, 0xF6, 0x9C, 
	0xBB, 0xC0, 0xCD, 0xFB, 0xC0, 0x3F, 0xCD, 0x9A, 0xBF, 0xC4, 0x9E, 0xBF, 0x4E, 0x19, 0xCD, 0x90, 
	0x26, 0xB3, 0xF8, 0x2B, 0x8B, 0xF8, 0x7E, 0xE9, 0x15, 0x3F, 0x00, 0xFE, 0xD4, 0x53, 0x53, 0x00, 
	0x80, 0x1F, 0xFD, 0x51, 0x36, 0x65, 0x89, 0x23, 0x44, 0x74, 0x92, 0x39, 0xF9, 0x73, 0xEF, 0xED, 
	0xB6, 0x56, 0xAB, 0x9A, 0x52, 0xCA, 0x6F, 0xAF, 0x2A, 0xB1, 0xB5, 0xAE, 0x5D, 0x92, 0xE7, 0x99, 
	0x9C, 0x9F, 0xEF, 0x96, 0x79, 0x39, 0xB2, 0x74, 0x55, 0xA4, 0xFA, 0xB7, 0x47, 0x1B, 0xBF, 0xAA, 
	0x85, 0x1A, 0x23, 0x00, 0x0D, 0xEF, 0x61, 0x01, 0xE8, 0x24, 0x81, 0x6A, 0xB5, 0x50, 0xE7, 0xB9, 
	0x6F, 0xB7, 0x5A, 0x6E, 0x4F, 0x9A, 0xEA, 0xBA, 0xDF, 0x87, 0xAF, 0x3D, 0x9F, 0x36, 0xF5, 0x46, 
	0x41, 0x94, 0x45, 0x52, 0x93, 0x0A, 0x42, 0xC4, 0x71, 0x0C, 0x66, 0xC6, 0xD9, 0xB3, 0x67, 0x91, 
	0xE7, 0x39, 0x82, 0x0B, 0x60, 0xBF, 0x39, 0x0B, 0x04, 0x11, 0xC7, 0x22, 0xBD, 0xE5, 0x96, 0x86, 
	0x48, 0x53, 0xB9, 0xFA, 0xD4, 0x53, 0x99, 0x59, 0x5A, 0xB2, 0x1C, 0x04, 0x3C, 0xFA, 0xDA, 0xD7, 
	0xFA, 0x9D, 0xBB, 0x76, 0xC1, 0x59, 0x8B, 0x41, 0x59, 0xA2, 0xBB, 0x6B, 0x17, 0x2C, 0x33, 0xC6, 
	0x8A, 0x02, 0x02, 0xA0, 0xDB, 0x17, 0x34, 0x7F, 0xE9, 0x1C, 0x7D, 0x2E, 0x1B, 0xD7, 0x6D, 0x62, 
	0x6C, 0x85, 0x46, 0xE4, 0xD9, 0xC3, 0x1B, 0x9F, 0x73, 0xC0, 0x4B, 0x42, 0x89, 0x17, 0x21, 0xF0, 
	0xA4, 0x87, 0x7F, 0x1A, 0x0A, 0xA7, 0x31, 0x86, 0x1C, 0x39, 0xB0, 0xF8, 0x8F, 0xBE, 0x7F, 0x70, 
	0xBF, 0x2F, 0x80, 0x2F, 0xE8, 0x81, 0x07, 0x5E, 0x9A, 0xDA, 0xA8, 0x01, 0x2C, 0x7C, 0xF8, 0xC3, 
	0x66, 0x61, 0x30, 0xC8, 0x9E, 0x61, 0x8E, 0xA4, 0xB5, 0xD0, 0x45, 0x61, 0xD4, 0xEA, 0x6A, 0x9F, 
	0x96, 0x96, 0x2A, 0xBF, 0xBE, 0xFA, 0xB6, 0xF2, 0xFF, 0x9A, 0xFA, 0x17, 0xB7, 0x44, 0x64, 0x1A, 
	0x7C, 0xDE, 0x8D, 0x86, 0xCE, 0xA1, 0x3D, 0x18, 0x40, 0x2B, 0x05, 0x29, 0x25, 0xE2, 0x24, 0xE1, 
	0x87, 0xFA, 0xFD, 0xB5, 0x95, 0xB2, 0x6C, 0xAD, 0xF7, 0xFB, 0x49, 0x17, 0x28, 0x96, 0x80, 0x5C, 
	0x10, 0x11, 0x11, 0x41, 0x48, 0x09, 0x0F, 0x60, 0x69, 0x69, 0x09, 0x45, 0x9E, 0x83, 0x88, 0xBE, 
	0x09, 0xF6, 0x82, 0x7B, 0xD6, 0x9A, 0xF6, 0xFE, 0xF4, 0x4F, 0x8F, 0x4C, 0xFD, 0xC8, 0x8F, 0x0C, 
	0x59, 0x66, 0x9A, 0x7F, 0xEA, 0xA9, 0xE4, 0xE9, 0x8F, 0x7E, 0x74, 0xD1, 0x67, 0x99, 0x0F, 0xBC, 
	0x77, 0xCD, 0x46, 0x63, 0x73, 0xB8, 0x50, 0x29, 0x14, 0xA7, 0x4F, 0xBF, 0x34, 0x2C, 0xE5, 0x01, 
	0x6C, 0x29, 0x45, 0xF2, 0x9E, 0x2F, 0x98, 0xAF, 0xFC, 0xF6, 0xCF, 0xF2, 0x63, 0xA1, 0xA9, 0xB7, 
	0x19, 0x32, 0x6D, 0xEF, 0x3D, 0x4C, 0x69, 0xBA, 0xDE, 0xFB, 0x45, 0x95, 0xA8, 0x45, 0x48, 0xAC, 
	0x32, 0x71, 0x0D, 0x80, 0x57, 0xDE, 0xB1, 0x82, 0x1F, 0x84, 0x7E, 0x60, 0xCF, 0xE8, 0xF8, 0xED, 
	0xDF, 0xD6, 0x17, 0xC2, 0x9E, 0x3D, 0xFF, 0xF5, 0x92, 0xFE, 0x64, 0x7A, 0x1A, 0x00, 0x52, 0x02, 
	0x22, 0xBA, 0xD0, 0xB1, 0x00, 0x10, 0x95, 0x25, 0x40, 0x04, 0x06, 0x10, 0x0B, 0x21, 0xAD, 0x73, 
	0xF2, 0xCF, 0xFA, 0xFD, 0xE5, 0x1B, 0xBD, 0xDF, 0xBA, 0x0E, 0xE4, 0x35, 0xE0, 0x42, 0x80, 0x3C, 
	0x00, 0xCF, 0x8C, 0xAC, 0xD7, 0x43, 0x5D, 0x9E, 0xAF, 0x1C, 0x79, 0x0F, 0x11, 0x04, 0xC4, 0xD6, 
	0x32, 0x7B, 0x0F, 0xF6, 0x1E, 0xE3, 0xD7, 0x5F, 0x1F, 0xDC, 0xF2, 0xDE, 0xF7, 0xB6, 0xD2, 0xB1, 
	0x31, 0xAA, 0xEA, 0x1A, 0xAD, 0x38, 0x8E, 0x4F, 0x7E, 0xEA, 0x53, 0x41, 0x7E, 0xE0, 0x40, 0xF1, 
	0xEC, 0xC7, 0x3E, 0x96, 0xED, 0xB9, 0xFA, 0xEA, 0xB4, 0x31, 0x39, 0x29, 0x7A, 0x9F, 0xFF, 0x3C, 
	0x8A, 0x87, 0x1E, 0xDA, 0xBC, 0x8E, 0xF3, 0x7F, 0x24, 0x01, 0x89, 0xE0, 0xE6, 0x25, 0xDA, 0x72, 
	0xEE, 0x4B, 0xD3, 0x07, 0x82, 0xF7, 0x3F, 0x75, 0xB0, 0xBD, 0x21, 0xA5, 0x2F, 0x3D, 0x55, 0x45, 
	0xE5, 0xCA, 0x63, 0xA5, 0x57, 0x93, 0x0A, 0xF6, 0xDF, 0x5A, 0xFC, 0xA0, 0xA5, 0x70, 0x11, 0x2A, 
	0xDA, 0xFC, 0xA7, 0x20, 0xC0, 0x5C, 0x68, 0x56, 0xBC, 0x1C, 0x34, 0x9D, 0xAF, 0x7D, 0x8F, 0x69, 
	0x1D, 0x1F, 0x15, 0xA2, 0x73, 0x3C, 0x0C, 0x97, 0xEA, 0xB2, 0x74, 0xCC, 0x4C, 0x9E, 0x19, 0x8E, 
	0x19, 0xB5, 0x31, 0xE0, 0xCD, 0x99, 0x5A, 0x88, 0x28, 0xA2, 0x2D, 0xF7, 0xDC, 0xD3, 0x6E, 0x5C, 
	0x7F, 0x7D, 0x23, 0x9F, 0x9F, 0x37, 0x4B, 0x9F, 0xFD, 0xEC, 0x7A, 0xB5, 0xB0, 0x60, 0x1A, 0xA3, 
	0xA3, 0x62, 0xFB, 0xEE, 0xDD, 0x22, 0x4C, 0x12, 0x18, 0x63, 0xA0, 0xAC, 0x65, 0x09, 0x30, 0x84, 
	0xC0, 0xCC, 0xC3, 0x0F, 0x57, 0x87, 0x7F, 0xEA, 0xA7, 0xEA, 0xD1, 0x34, 0x8D, 0xEA, 0xA5, 0x25, 
	0xB0, 0x31, 0xDF, 0xDE, 0x02, 0x06, 0x04, 0xDD, 0xF1, 0x9B, 0x0F, 0x3E, 0x9C, 0x7E, 0xF4, 0x2B, 
	0xA2, 0xFF, 0xA1, 0xC1, 0xE0, 0x5B, 0x68, 0x5A, 0xD8, 0x8B, 0x62, 0x2D, 0x2F, 0x4A, 0xC0, 0x7A, 
	0x13, 0xE4, 0x1A, 0x80, 0x3E, 0x80, 0x11, 0x77, 0xA1, 0x99, 0xF1, 0xCD, 0xCE, 0x06, 0x18, 0xC0, 
	0x48, 0xB3, 0x19, 0xED, 0x9C, 0x9E, 0x6E, 0xCC, 0x9F, 0x3E, 0x3D, 0xB0, 0xDE, 0x83, 0xAA, 0x0A, 
	0x0E, 0x80, 0xF3, 0x1E, 0xB9, 0xB5, 0x68, 0x10, 0x41, 0x32, 0x63, 0xE7, 0xBD, 0xF7, 0xA6, 0x37, 
	0x7E, 0xF0, 0x83, 0x63, 0x42, 0x6B, 0x1A, 0xE4, 0x39, 0xB4, 0xB5, 0xEE, 0xE4, 0xEF, 0xFF, 0xFE, 
	0x5A, 0xF7, 0xEC, 0x59, 0x33, 0x98, 0x9D, 0xAD, 0x9B, 0xD7, 0x5F, 0x1F, 0x6A, 0xAD, 0xD1, 0x3D, 
	0x74, 0x28, 0x2F, 0x4F, 0x9F, 0x36, 0x52, 0x08, 0xAA, 0x99, 0xDD, 0xD2, 0xF2, 0x72, 0x95, 0x76, 
	0xBB, 0x11, 0x7F, 0x67, 0x7F, 0x1F, 0x0C, 0x20, 0x12, 0x62, 0xCF, 0x15, 0x49, 0x32, 0x21, 0x88, 
	0xFA, 0x18, 0x0C, 0x2E, 0xC6, 0xA5, 0x7C, 0x65, 0x1F, 0xE1, 0xF0, 0xB7, 0xD5, 0xFB, 0xDA, 0xED, 
	0x0B, 0xDF, 0xFE, 0x03, 0x00, 0xDB, 0x6A, 0x00, 0x9E, 0x08, 0xEA, 0x3C, 0x68, 0x6C, 0xEE, 0x74, 
	0x91, 0x6F, 0xDF, 0x2E, 0xF8, 0x6D, 0x6F, 0x6B, 0x90, 0xF7, 0x2E, 0x9B, 0x9B, 0xAB, 0xD8, 0x39, 
	0xD0, 0xF9, 0xBC, 0x96, 0x84, 0x40, 0x48, 0x84, 0x46, 0xB3, 0xA9, 0xEE, 0xFC, 0xE5, 0x5F, 0x1E, 
	0xDB, 0x7D, 0xEB, 0xAD, 0x3A, 0x6D, 0x36, 0x91, 0x34, 0x1A, 0xB0, 0x79, 0xEE, 0xCF, 0x3D, 0xF8, 
	0x60, 0x96, 0xAF, 0xAE, 0xFA, 0xE5, 0x23, 0x47, 0x2A, 0x15, 0x04, 0x38, 0xF1, 0xE0, 0x83, 0xF9, 
	0x57, 0x3F, 0xF2, 0x91, 0x8D, 0x72, 0x79, 0xD9, 0x09, 0x21, 0xE0, 0x99, 0x39, 0x56, 0x4A, 0x4F, 
	0x05, 0x41, 0xE3, 0x3B, 0x2C, 0xF7, 0xFC, 0x58, 0x16, 0x33, 0xAB, 0xC2, 0xB9, 0x07, 0xDB, 0x4A, 
	0x9D, 0xF8, 0x58, 0x96, 0x5D, 0x94, 0x80, 0x2F, 0x4A, 0x0B, 0x3E, 0x6F, 0x21, 0x7D, 0x01, 0xCC, 
	0x80, 0xE8, 0xD6, 0x97, 0x7E, 0x26, 0xE5, 0xE6, 0x24, 0x9E, 0x73, 0x20, 0x22, 0x90, 0x31, 0xE8, 
	0xE6, 0xB9, 0x90, 0x57, 0x5F, 0x3D, 0x1C, 0x1C, 0x3A, 0x94, 0xD7, 0xC6, 0x58, 0x57, 0x55, 0x10, 
	0xC0, 0xA6, 0xF5, 0x12, 0x61, 0xCB, 0xC8, 0x48, 0x7B, 0xA2, 0xD9, 0x8C, 0x46, 0x86, 0x86, 0xE0, 
	0xBD, 0x47, 0xD3, 0x18, 0x1C, 0x5D, 0x59, 0xA9, 0x9C, 0x31, 0x4C, 0x42, 0xE0, 0xD4, 0xD7, 0xBF, 
	0x5E, 0x9E, 0x79, 0xE4, 0x91, 0x92, 0xBD, 0x07, 0x33, 0x83, 0x84, 0xC0, 0xF9, 0x8D, 0x1D, 0xAD, 
	0x1B, 0x53, 0x55, 0xCC, 0x2E, 0x02, 0x24, 0x7F, 0x67, 0xBE, 0x0C, 0x49, 0xD4, 0x68, 0x48, 0x79, 
	0xDB, 0x4D, 0x69, 0xFA, 0x85, 0x9F, 0x5A, 0x5C, 0xC4, 0x1F, 0x5C, 0x84, 0x6B, 0x29, 0x2E, 0x46, 
	0xC0, 0x8E, 0x08, 0x21, 0x51, 0xED, 0x88, 0x9E, 0xBB, 0xE0, 0x66, 0x04, 0x80, 0xB2, 0xD1, 0x40, 
	0xD9, 0x6A, 0x6D, 0xCE, 0x02, 0x11, 0xA1, 0xD1, 0xE9, 0x20, 0xEA, 0x76, 0x21, 0x5A, 0x2D, 0x1D, 
	0xEF, 0xDC, 0x19, 0x0B, 0xAD, 0x99, 0x85, 0x00, 0x01, 0x88, 0x88, 0x20, 0x85, 0x10, 0x81, 0x94, 
	0xF1, 0xEC, 0xE7, 0x3E, 0x87, 0xC1, 0xD2, 0x12, 0xE0, 0x1C, 0x56, 0x9E, 0x7B, 0xAE, 0x3C, 0xFA, 
	0xA7, 0x7F, 0xDA, 0xFF, 0x16, 0xA0, 0x9B, 0x43, 0x5E, 0xC0, 0x79, 0xB8, 0x38, 0xEF, 0x05, 0x3A, 
	0xC6, 0xD4, 0x7D, 0xE6, 0xDA, 0x11, 0xA1, 0x14, 0xE2, 0x65, 0xF1, 0x75, 0xB3, 0xED, 0x23, 0x37, 
	0x37, 0x7B, 0x57, 0xBD, 0xFD, 0xF8, 0x71, 0xB9, 0x3D, 0x08, 0x2E, 0xBB, 0xE8, 0xBF, 0xAE, 0xFE, 
	0xA8, 0xDB, 0xC5, 0xBB, 0x87, 0x86, 0x00, 0xA2, 0x36, 0x01, 0xEF, 0x90, 0x44, 0x4A, 0x12, 0xA1, 
	0x0C, 0x43, 0x14, 0x71, 0x8C, 0xB0, 0xAA, 0x20, 0x99, 0xA1, 0xAC, 0x05, 0x88, 0xD0, 0x9D, 0x9E, 
	0x26, 0xD7, 0xED, 0xD6, 0xD5, 0xCC, 0x4C, 0xC1, 0xCC, 0x10, 0xCE, 0xA1, 0x21, 0x37, 0x6F, 0x2D, 
	0x6E, 0x34, 0xE2, 0xEE, 0xD2, 0x52, 0x78, 0xFC, 0xD1, 0x47, 0x71, 0xEC, 0xC1, 0x07, 0xFD, 0x43, 
	0xBF, 0xFB, 0xBB, 0x2B, 0x6B, 0x47, 0x8F, 0x16, 0x42, 0x08, 0xFA, 0x76, 0xB7, 0x7B, 0x21, 0x37, 
	0x7E, 0x29, 0x9F, 0x63, 0xE6, 0x96, 0x52, 0x41, 0x94, 0x24, 0xF1, 0x6C, 0x92, 0x20, 0x75, 0x0E, 
	0xDA, 0x6F, 0x8E, 0xD7, 0x15, 0xE7, 0xDD, 0x9F, 0xF1, 0xBE, 0x88, 0x9D, 0xFB, 0x74, 0x2C, 0x65, 
	0xFE, 0x25, 0x63, 0x2E, 0xBB, 0xE8, 0xBF, 0xAE, 0xCA, 0x66, 0x13, 0x4C, 0x74, 0x2C, 0x2E, 0xCB, 
	0xC5, 0xC0, 0x98, 0x9D, 0x04, 0x40, 0x1A, 0x83, 0x55, 0x6B, 0xA1, 0x89, 0x30, 0x7A, 0xDE, 0xEA, 
	0x26, 0x4F, 0x9D, 0xC2, 0xAC, 0xB5, 0x59, 0xB1, 0xBC, 0x3C, 0x20, 0x21, 0x20, 0x83, 0x00, 0xDE, 
	0x98, 0x0B, 0x6E, 0x94, 0x17, 0xD6, 0xD6, 0x3A, 0x11, 0xB3, 0xC4, 0xB3, 0xCF, 0xAA, 0x4E, 0xB7, 
	0xDB, 0xCB, 0x7A, 0xBD, 0xBC, 0x75, 0x1E, 0xEE, 0x7F, 0x0B, 0x2C, 0xBF, 0x2C, 0xDF, 0x9D, 0x2D, 
	0xCB, 0x7C, 0x38, 0x4D, 0xDB, 0x5D, 0x29, 0xC5, 0x46, 0x10, 0x60, 0x8B, 0xB5, 0x90, 0xCC, 0x28, 
	0xB4, 0x46, 0x2E, 0x25, 0xD2, 0xC1, 0x60, 0xFB, 0x88, 0x52, 0x13, 0x44, 0xB4, 0x7A, 0x39, 0x06, 
	0xFF, 0x4D, 0x14, 0xC7, 0xF0, 0x5A, 0x9F, 0x23, 0x29, 0x0F, 0xA3, 0xDF, 0xDF, 0x09, 0xEF, 0x11, 
	0x19, 0x03, 0x5B, 0xD7, 0x28, 0x83, 0x00, 0x7E, 0xDB, 0x36, 0xC8, 0xA5, 0x25, 0xE8, 0xAA, 0x42, 
	0x70, 0xE8, 0x50, 0x51, 0x54, 0x55, 0x21, 0x85, 0x20, 0xA9, 0x35, 0x8C, 0x94, 0xF0, 0x9B, 0x31, 
	0x12, 0xDD, 0xA2, 0xA8, 0x96, 0xE6, 0xE7, 0x17, 0x89, 0x88, 0x9C, 0xB5, 0xAE, 0x71, 0x7E, 0x93, 
	0xC6, 0x17, 0x66, 0xEB, 0x69, 0x73, 0x4F, 0x76, 0x3E, 0xC6, 0xF2, 0x85, 0xA6, 0x14, 0x00, 0x26, 
	0x00, 0xCB, 0x75, 0x5D, 0x55, 0x75, 0x5D, 0x07, 0x40, 0xD4, 0x15, 0x02, 0x13, 0xE7, 0xC3, 0x45, 
	0xA5, 0x14, 0xBA, 0x51, 0x84, 0xD0, 0x98, 0x66, 0x53, 0xCA, 0x21, 0x21, 0x2E, 0xCA, 0x68, 0x77, 
	0x71, 0xC6, 0x60, 0x00, 0x88, 0xA5, 0xC4, 0xAE, 0xF5, 0xF5, 0x4C, 0x05, 0xC1, 0x01, 0xDA, 0x1C, 
	0x7D, 0x85, 0x62, 0xC6, 0xF6, 0xC1, 0x00, 0x48, 0x53, 0x1C, 0x7B, 0xD3, 0x9B, 0xB0, 0xB1, 0x6F, 
	0x1F, 0x48, 0x08, 0x0C, 0x29, 0x15, 0x4A, 0x21, 0x36, 0x2B, 0x58, 0x42, 0x80, 0xB4, 0x7E, 0xE9, 
	0xF0, 0x01, 0x03, 0x70, 0xCE, 0x31, 0x3B, 0xCB, 0x4D, 0x41, 0xBA, 0x21, 0x29, 0xD2, 0x82, 0x9B, 
	0x81, 0xE4, 0x76, 0x20, 0x79, 0x38, 0x92, 0x3C, 0x12, 0x2B, 0x1E, 0x55, 0x8A, 0xC7, 0x84, 0xE4, 
	0x31, 0x25, 0x79, 0x44, 0x0A, 0x6E, 0x11, 0x71, 0x0C, 0x30, 0x95, 0xCC, 0x6E, 0xB6, 0xDB, 0x1D, 
	0x34, 0xB3, 0x6C, 0xF3, 0xFC, 0xF2, 0x66, 0xDD, 0x1A, 0x11, 0x33, 0x72, 0x00, 0x7D, 0xAD, 0x03, 
	0x06, 0x5A, 0x9A, 0x2E, 0xCE, 0x27, 0xE3, 0x5E, 0xB4, 0x16, 0xFC, 0xD6, 0xFD, 0xFB, 0xF1, 0x97, 
	0x37, 0xDD, 0x04, 0x38, 0xF7, 0x04, 0x31, 0x0F, 0x40, 0xD4, 0x20, 0x00, 0x4D, 0x66, 0x78, 0x63, 
	0xB0, 0x36, 0x18, 0xA0, 0xB7, 0x63, 0x07, 0x6E, 0x3A, 0x71, 0x02, 0xC3, 0xC6, 0x04, 0x01, 0x91, 
	0xB4, 0x44, 0x4E, 0x08, 0x41, 0x41, 0x14, 0x49, 0x37, 0xC8, 0x1C, 0x81, 0x45, 0x53, 0x20, 0x88, 
	0x25, 0x05, 0x9A, 0x10, 0x68, 0x82, 0x12, 0x60, 0x45, 0xC4, 0x12, 0x80, 0xA0, 0x0B, 0x35, 0x93, 
	0xCD, 0xE1, 0x48, 0x08, 0x66, 0x0E, 0x40, 0xE0, 0xF3, 0xA3, 0x56, 0x03, 0x8F, 0x65, 0x0B, 0x98, 
	0x73, 0x65, 0x99, 0xB5, 0x86, 0x87, 0x5B, 0xD1, 0xDD, 0x77, 0xEB, 0xB5, 0xA2, 0xC0, 0x96, 0x47, 
	0x1E, 0xC1, 0xF8, 0xFA, 0x3A, 0x16, 0xC2, 0x10, 0x05, 0x33, 0x55, 0xDE, 0x87, 0x91, 0x94, 0x97, 
	0x01, 0xFF, 0x8D, 0x77, 0xD3, 0x9B, 0xB9, 0xE5, 0x73, 0xF0, 0x7E, 0x0E, 0xC0, 0x55, 0x74, 0x7E, 
	0x98, 0xAE, 0xDD, 0xEF, 0x23, 0x5D, 0x5B, 0xC3, 0xC6, 0xF4, 0x34, 0xBA, 0x13, 0x13, 0x18, 0x5A, 
	0x5F, 0x57, 0x01, 0x91, 0xB0, 0x80, 0x8D, 0xC2, 0x30, 0x1C, 0x9A, 0x9C, 0x1C, 0xED, 0x9E, 0x39, 
	0xDD, 0x0D, 0xD9, 0x46, 0x81, 0x44, 0x2C, 0x89, 0xB4, 0x03, 0x4B, 0xE6, 0xF3, 0x69, 0x34, 0x6F, 
	0xC6, 0x57, 0x66, 0x40, 0x10, 0x21, 0x12, 0x12, 0x0A, 0x02, 0x35, 0x3B, 0x12, 0xDE, 0xC1, 0x7B, 
	0x27, 0x1D, 0x13, 0x3B, 0x28, 0x59, 0x79, 0x8E, 0xC4, 0xAE, 0xE9, 0x7A, 0xEB, 0x2F, 0xFD, 0x12, 
	0xD2, 0x2B, 0xAF, 0x44, 0x5D, 0xD7, 0x58, 0x6B, 0x36, 0xA1, 0xFE, 0xE3, 0x7F, 0xC4, 0x36, 0x6B, 
	0xD1, 0x71, 0xCE, 0x6E, 0xD4, 0x75, 0x59, 0x5A, 0x7B, 0x19, 0xF0, 0xDF, 0xF8, 0xE2, 0xBC, 0x07, 
	0x03, 0x8B, 0x04, 0x1C, 0x00, 0xD1, 0x55, 0x84, 0xCD, 0x56, 0x9F, 0xB2, 0x16, 0xD3, 0xC7, 0x8F, 
	0x63, 0x30, 0x3E, 0x8E, 0xB2, 0xD5, 0x82, 0x12, 0x82, 0x22, 0x41, 0xB2, 0x60, 0xD6, 0x5A, 0x0A, 
	0x9D, 0x26, 0x49, 0xD8, 0x8F, 0xC2, 0x28, 0xAA, 0x6D, 0x2B, 0x92, 0x42, 0x79, 0x30, 0x04, 0x6F, 
	0x9A, 0x6C, 0x20, 0x08, 0x8E, 0x19, 0x85, 0x63, 0x38, 0x30, 0x9A, 0x52, 0x63, 0x32, 0x88, 0x51, 
	0x41, 0x60, 0xC3, 0x54, 0x28, 0x03, 0x86, 0x1F, 0x1E, 0xE6, 0xC1, 0x5A, 0xDF, 0x72, 0xBF, 0x1E, 
	0x23, 0x72, 0xB4, 0xEB, 0x8D, 0x77, 0x0F, 0xAE, 0xBB, 0xF3, 0x4E, 0xAD, 0x94, 0x42, 0x65, 0x0C, 
	0x56, 0xF6, 0xED, 0xC3, 0x2C, 0x00, 0xAC, 0xAE, 0xC2, 0x87, 0x61, 0x36, 0xEF, 0xDC, 0xAA, 0xF8, 
	0xDE, 0x9D, 0xF8, 0xFC, 0xFF, 0x0F, 0x60, 0x6C, 0xEE, 0x80, 0x4A, 0x00, 0x5F, 0x21, 0xE0, 0x1F, 
	0x82, 0x48, 0xD2, 0xF9, 0xDD, 0xF3, 0xD8, 0xC2, 0x02, 0xC4, 0x63, 0x8F, 0x41, 0x57, 0x15, 0x24, 
	0x91, 0x68, 0x90, 0x90, 0x3D, 0x72, 0x52, 0xB8, 0xBA, 0x1D, 0x12, 0x44, 0x33, 0x6D, 0xE9, 0xA4, 
	0x63, 0x38, 0x10, 0x84, 0xBE, 0xAB, 0xC1, 0x0C, 0x04, 0x92, 0x10, 0x0A, 0x82, 0xF1, 0x40, 0x49, 
	0x0C, 0xC2, 0xA6, 0xF5, 0x26, 0x52, 0xC1, 0x3A, 0x0F, 0x31, 0x3A, 0x86, 0xA9, 0x77, 0xBD, 0x1B, 
	0xC1, 0x35, 0xD7, 0x50, 0xE7, 0xC4, 0xE9, 0xD6, 0x0B, 0x7F, 0xF0, 0xFF, 0x20, 0x3B, 0x77, 0xAE, 
	0x1A, 0x69, 0x36, 0xD5, 0xC4, 0xC4, 0x04, 0x84, 0x10, 0xB0, 0xCC, 0xC8, 0xFB, 0x7D, 0xE8, 0xBA, 
	0xC6, 0x74, 0xDA, 0xC4, 0xAC, 0x90, 0x27, 0xE7, 0xF3, 0xFC, 0x5C, 0xFB, 0x22, 0x8D, 0xC1, 0xE2, 
	0x62, 0xE6, 0xFB, 0xE6, 0x53, 0xA7, 0x20, 0x88, 0x20, 0x88, 0x1E, 0x26, 0xA2, 0x73, 0x50, 0x0A, 
	0xE5, 0x0D, 0x37, 0x00, 0x93, 0x93, 0x10, 0xCC, 0x18, 0x9F, 0x9B, 0x43, 0x73, 0x6D, 0x0D, 0x5A, 
	0x4A, 0x8A, 0xA5, 0x14, 0x0C, 0x51, 0x83, 0xAD, 0x2A, 0x07, 0x5D, 0x91, 0xA6, 0xA9, 0x0C, 0x64, 
	0x68, 0x52, 0x15, 0x42, 0x40, 0xC0, 0x61, 0xB3, 0x09, 0x51, 0x3A, 0x46, 0xE1, 0x3C, 0x8C, 0x75, 
	0xB0, 0xD6, 0xA2, 0x6F, 0x6B, 0x2C, 0x56, 0x05, 0x56, 0xAA, 0x1C, 0x63, 0xF7, 0xDE, 0x8B, 0x7D, 
	0xF7, 0xFD, 0x23, 0x5C, 0x73, 0xF3, 0xAB, 0xB0, 0xEF, 0xAD, 0x3F, 0x2A, 0xAE, 0xFC, 0x7B, 0x77, 
	0x63, 0x57, 0x20, 0xFD, 0xDE, 0xA7, 0x9E, 0x8C, 0xAB, 0x67, 0x9E, 0x01, 0x39, 0x07, 0x77, 0xF4, 
	0x28, 0xBA, 0xF7, 0xDF, 0x8F, 0x2D, 0x5A, 0x63, 0x24, 0x8A, 0xB1, 0x58, 0xD7, 0x8F, 0xDC, 0xEF, 
	0xDC, 0x8A, 0x65, 0xBE, 0x6C, 0xC1, 0x7F, 0xAB, 0x0B, 0x24, 0x82, 0x71, 0xEE, 0x78, 0x22, 0xC4, 
	0xC3, 0x1C, 0xC7, 0xBB, 0x5E, 0xB8, 0xF6, 0x5A, 0x44, 0x45, 0x81, 0xDD, 0x5F, 0xF9, 0x0A, 0x74, 
	0x55, 0x81, 0x84, 0x78, 0xE9, 0xC4, 0x78, 0xCE, 0xA8, 0x23, 0xA6, 0x6E, 0x3F, 0xEB, 0x8E, 0x4B, 
	0x19, 0x48, 0xF2, 0xAE, 0x0A, 0x89, 0x60, 0x99, 0x61, 0x3D, 0x63, 0xC0, 0x9B, 0x87, 0x17, 0x39, 
	0x08, 0x10, 0x5C, 0x73, 0x03, 0x28, 0x8E, 0x51, 0x9E, 0x38, 0x86, 0xB2, 0xB3, 0x01, 0xAF, 0x03, 
	0xDC, 0x76, 0xEB, 0xAD, 0xD8, 0xB9, 0x6B, 0x27, 0x9C, 0x31, 0xC8, 0xAB, 0x12, 0x9D, 0xE9, 0xAD, 
	0x6E, 0x8F, 0x26, 0x15, 0x1E, 0x7B, 0x31, 0x9C, 0xFB, 0xA7, 0xFF, 0x14, 0x6E, 0xC7, 0x0E, 0xAC, 
	0x1E, 0x3F, 0x0E, 0x31, 0x3F, 0x8F, 0xAD, 0x69, 0x8A, 0xD3, 0xC6, 0x2D, 0x9F, 0xA8, 0xED, 0x67, 
	0xFF, 0x8F, 0x66, 0x9B, 0x7F, 0x25, 0xEB, 0x5E, 0x06, 0xFC, 0xB7, 0x51, 0x91, 0x65, 0xD8, 0xDD, 
	0x6A, 0xD5, 0x83, 0xBA, 0xFE, 0x33, 0x59, 0x55, 0xEF, 0xD4, 0x6B, 0x6B, 0xC9, 0xB9, 0x6D, 0xDB, 
	0xB0, 0x75, 0x7A, 0x1A, 0x13, 0xA7, 0x4F, 0xC3, 0x13, 0xA1, 0xE3, 0x1C, 0x47, 0x52, 0x3A, 0xB2, 
	0x16, 0x99, 0xA3, 0x5E, 0xE4, 0x38, 0x09, 0xB9, 0x8C, 0x17, 0x6D, 0x35, 0x70, 0xD6, 0x3B, 0xC3, 
	0x5E, 0x32, 0x00, 0xC7, 0x00, 0x49, 0x89, 0xA9, 0x77, 0xDD, 0x87, 0xAD, 0xF7, 0xBE, 0x1D, 0x8E, 
	0x08, 0xAB, 0xFB, 0x1F, 0xC5, 0xCC, 0x7F, 0xF8, 0x1D, 0xB6, 0x99, 0xE1, 0x60, 0x50, 0x51, 0x33, 
	0x49, 0x08, 0x00, 0xF4, 0x86, 0xC7, 0xD8, 0xEC, 0x39, 0xBD, 0x42, 0x84, 0x23, 0x1B, 0x1D, 0xA4, 
	0x79, 0x81, 0xD6, 0xE9, 0xD3, 0x58, 0xC8, 0x32, 0x5C, 0x95, 0xA6, 0xB0, 0x0C, 0x1C, 0x2A, 0xCD, 
	0x9F, 0x7E, 0xA6, 0x18, 0x3C, 0xF9, 0xDA, 0x20, 0xBE, 0x68, 0xD7, 0x4F, 0x5E, 0xEC, 0x80, 0x3F, 
	0x9E, 0x65, 0xF8, 0xF9, 0xD1, 0x51, 0x84, 0x42, 0xAC, 0x04, 0xDE, 0xBF, 0xCE, 0x69, 0xBD, 0x7B, 
	0x76, 0x7A, 0x1A, 0x49, 0x55, 0x61, 0xD7, 0xFC, 0x3C, 0x4C, 0x5D, 0x63, 0xE0, 0xBD, 0xEF, 0x09, 
	0xD1, 0xCD, 0x98, 0x5D, 0xC6, 0xCC, 0x04, 0x51, 0x37, 0xB5, 0x48, 0x06, 0x75, 0x5D, 0xB9, 0xDA, 
	0x40, 0x09, 0x48, 0x30, 0x98, 0xBD, 0xE3, 0xF6, 0xDE, 0xAB, 0xF1, 0xBA, 0x5F, 0xFB, 0x57, 0xB4, 
	0x75, 0xD7, 0x6E, 0xB4, 0xDA, 0x6D, 0xC4, 0x23, 0x63, 0x28, 0x1F, 0x79, 0x12, 0x53, 0xAB, 0xEB, 
	0xD4, 0x5A, 0x59, 0x22, 0xB5, 0x73, 0x27, 0xB8, 0x2C, 0xB1, 0xF2, 0x89, 0x4F, 0xC0, 0x7E, 0xF9, 
	0xCB, 0x90, 0x59, 0x86, 0xC2, 0x5A, 0x94, 0xD6, 0x42, 0x29, 0x05, 0xCF, 0x8C, 0xA9, 0x46, 0x03, 
	0xCF, 0xD4, 0xF6, 0xC9, 0xFD, 0x45, 0xF9, 0x2B, 0x37, 0x86, 0xC9, 0xF2, 0xBF, 0x2F, 0xB3, 0x8B, 
	0xD7, 0x03, 0xE2, 0x12, 0x11, 0x09, 0xB1, 0x46, 0xCC, 0xBF, 0xBF, 0x73, 0x6E, 0xEE, 0xB5, 0x8B, 
	0x33, 0x33, 0x49, 0xAE, 0x35, 0xD6, 0xE2, 0x18, 0x2D, 0x6B, 0xD1, 0x04, 0x38, 0x04, 0xD0, 0x56, 
	0x8A, 0x0D, 0x40, 0x06, 0x28, 0x3B, 0x8E, 0x96, 0x9D, 0xD0, 0xBA, 0xE7, 0x8A, 0x2C, 0x91, 0x52, 
	0x31, 0x00, 0xF6, 0xCC, 0xC9, 0xE8, 0x98, 0xD8, 0xB1, 0x67, 0x6F, 0x10, 0x26, 0x0D, 0xD4, 0xD6, 
	0xA0, 0x59, 0xD7, 0x98, 0x6C, 0xC6, 0x14, 0x8D, 0xB6, 0x31, 0x38, 0x7B, 0x06, 0x27, 0x3E, 0xF4, 
	0x21, 0x8C, 0xA5, 0x29, 0xA2, 0x4E, 0x07, 0x5B, 0xAA, 0x0A, 0x36, 0x49, 0xD0, 0xD0, 0x1A, 0x0B, 
	0x59, 0xB6, 0x39, 0x3E, 0x2B, 0x04, 0x9E, 0xAA, 0xEC, 0x81, 0x27, 0x6B, 0xF3, 0x4B, 0xEF, 0x6C, 
	0x0F, 0x1F, 0xFB, 0xFD, 0x8D, 0xE5, 0x8B, 0x7A, 0xDD, 0xC4, 0xA5, 0x00, 0xF7, 0xCA, 0x63, 0xC7, 
	0x36, 0x13, 0x58, 0x21, 0x3E, 0x17, 0x56, 0xD5, 0x17, 0x6E, 0x7B, 0xE6, 0x19, 0xBC, 0xFA, 0xF0, 
	0x61, 0x8C, 0x38, 0x07, 0xA1, 0x14, 0x42, 0xAD, 0xD5, 0x8D, 0x61, 0x38, 0x3E, 0xC1, 0x9C, 0x4E, 
	0x08, 0x11, 0x8F, 0x2A, 0x15, 0x91, 0x27, 0x53, 0x93, 0x1A, 0x74, 0x2C, 0x17, 0xB5, 0x67, 0xEB, 
	0x98, 0xE0, 0x45, 0x28, 0xD4, 0x6A, 0x4F, 0x62, 0x65, 0x15, 0x61, 0x18, 0xA0, 0x19, 0xC7, 0x48, 
	0x0F, 0x1E, 0x44, 0x63, 0x6D, 0x0D, 0x4A, 0x6B, 0x68, 0x21, 0xA0, 0x3A, 0x1D, 0xA8, 0xF9, 0x79, 
	0x44, 0x55, 0x05, 0x03, 0xC0, 0x28, 0x05, 0xD2, 0x01, 0x9A, 0x8D, 0x26, 0xD6, 0x41, 0xBD, 0x17, 
	0x1C, 0xFD, 0xE1, 0xE3, 0x75, 0xFD, 0x8F, 0x3F, 0xFA, 0xFE, 0x5F, 0xFC, 0xDA, 0xBF, 0x59, 0x9D, 
	0xC7, 0xFD, 0x75, 0x79, 0x51, 0xAF, 0x9D, 0xBC, 0x54, 0x2C, 0xF8, 0x17, 0xC7, 0xC6, 0x90, 0x0B, 
	0xAE, 0x48, 0xE0, 0x0C, 0xAC, 0x7D, 0x13, 0xEA, 0x7A, 0xC4, 0x78, 0x0F, 0xC3, 0x0C, 0x63, 0x2D, 
	0x6A, 0x6B, 0xF5, 0xB8, 0xD6, 0x4D, 0xD9, 0x6E, 0xA7, 0x13, 0x93, 0x93, 0xE9, 0x34, 0x51, 0x1A, 
	0x97, 0x65, 0x70, 0x76, 0x90, 0x67, 0x0A, 0x4A, 0x6C, 0x6F, 0x8C, 0x34, 0x27, 0x83, 0x86, 0xC2, 
	0x46, 0x4F, 0xAC, 0xBD, 0x70, 0x14, 0xAD, 0xBA, 0x02, 0x3D, 0xF4, 0x20, 0xCA, 0xFF, 0xFC, 0x9F, 
	0xC1, 0xBD, 0xDE, 0x85, 0x94, 0x0C, 0x99, 0xF7, 0xA8, 0x01, 0x44, 0x42, 0x80, 0x18, 0x10, 0xCC, 
	0xB0, 0x02, 0x83, 0x59, 0x6F, 0xBE, 0x70, 0xDC, 0xF2, 0xFF, 0xF2, 0x94, 0x93, 0xBF, 0xA3, 0x41, 
	0xF3, 0x6F, 0x7B, 0xF4, 0x01, 0x9C, 0x61, 0x7F, 0xF1, 0x7B, 0x3E, 0x5C, 0x42, 0xFA, 0x37, 0xEF, 
	0x1D, 0xC7, 0x6A, 0xD3, 0xC6, 0xF7, 0x9E, 0x1A, 0xF9, 0xF5, 0xF6, 0x82, 0xFA, 0x70, 0xC2, 0x32, 
	0x8E, 0x94, 0x82, 0x38, 0xDF, 0xD3, 0x75, 0x42, 0x60, 0x23, 0x8A, 0x90, 0x37, 0x9B, 0x68, 0x02, 
	0x58, 0x99, 0x9B, 0x33, 0x1F, 0x3F, 0x71, 0xE2, 0x9C, 0xF5, 0x5E, 0xBE, 0x26, 0x1D, 0x9E, 0x1E, 
	0xF1, 0x52, 0x3A, 0xF6, 0x18, 0x95, 0x84, 0x6B, 0x1B, 0x21, 0x86, 0x95, 0x78, 0xE9, 0x71, 0x3E, 
	0x17, 0x50, 0x6D, 0x58, 0x8B, 0xBE, 0xF3, 0x98, 0x90, 0x92, 0x4D, 0x4B, 0xFA, 0x8D, 0x6B, 0xD2, 
	0x41, 0xFF, 0xEA, 0xF8, 0xFF, 0x9D, 0x0D, 0x67, 0xFF, 0x75, 0x0B, 0xC9, 0xD1, 0x77, 0xFE, 0xCB, 
	0x63, 0x97, 0xD2, 0x92, 0x5D, 0x3A, 0x31, 0x18, 0x00, 0x8E, 0x06, 0x03, 0xF4, 0x32, 0xCB, 0x5F, 
	0x7B, 0x4D, 0x7C, 0xF6, 0xE6, 0xA5, 0x64, 0xE9, 0x86, 0x23, 0x7A, 0x97, 0x2E, 0x14, 0x7C, 0xBB, 
	0x05, 0x55, 0x96, 0xD0, 0x65, 0x89, 0xA1, 0xAA, 0x42, 0xE8, 0x1C, 0x94, 0x52, 0x50, 0x4A, 0xB1, 
	0x07, 0xD0, 0xF3, 0x56, 0x3C, 0x5C, 0x6C, 0xD0, 0x2D, 0xAA, 0xC9, 0x37, 0x86, 0x31, 0xED, 0x8D, 
	0x03, 0xB4, 0x03, 0x09, 0x47, 0x04, 0xFF, 0xB2, 0xFC, 0x95, 0x3C, 0x43, 0x4B, 0xE1, 0x57, 0x9A, 
	0xCE, 0x9C, 0xBA, 0xAE, 0xE0, 0xF8, 0xAE, 0x96, 0x6A, 0xDF, 0xD0, 0x94, 0x3A, 0xE4, 0x7C, 0xBA, 
	0x0A, 0x98, 0x44, 0x88, 0xAF, 0x7E, 0x63, 0x07, 0xDE, 0x78, 0xE7, 0xCC, 0x65, 0xC0, 0xAF, 0x84, 
	0x06, 0xBD, 0x0A, 0x75, 0xE1, 0x68, 0x6D, 0x24, 0xAB, 0x5F, 0xDC, 0xE3, 0x33, 0x9E, 0xF6, 0xFE, 
	0x8A, 0x43, 0x42, 0x34, 0x32, 0x89, 0xA6, 0x4A, 0x21, 0x9D, 0x83, 0xF1, 0x1E, 0x82, 0x19, 0xC6, 
	0x18, 0x64, 0xCE, 0x19, 0x27, 0x84, 0x97, 0x10, 0xB5, 0xF1, 0xDC, 0x39, 0x82, 0xA2, 0xA5, 0x3C, 
	0x0B, 0x45, 0x52, 0xCC, 0x47, 0x09, 0x2B, 0x15, 0xF0, 0x70, 0x5E, 0x70, 0x58, 0x57, 0xBE, 0xA3, 
	0x9D, 0x19, 0xEC, 0x6A, 0x9B, 0xC1, 0x2D, 0x7B, 0xAA, 0xB5, 0x1D, 0x1B, 0x41, 0x15, 0x9D, 0x6E, 
	0xEA, 0x89, 0x4A, 0xD5, 0x66, 0x96, 0x85, 0xA3, 0x26, 0xE0, 0x03, 0x5B, 0xF7, 0x01, 0x8D, 0xCB, 
	0x16, 0xFC, 0x4A, 0xC9, 0x6D, 0x3E, 0xCF, 0xC2, 0xAF, 0xAD, 0xE6, 0x59, 0x18, 0xF8, 0xB5, 0x68, 
	0xA7, 0xCF, 0xF2, 0xB7, 0xD4, 0xCD, 0xE6, 0xD9, 0xCA, 0x4E, 0x9E, 0x6C, 0x9B, 0xAD, 0x25, 0x05, 
	0x49, 0x49, 0xDA, 0x93, 0x40, 0x26, 0x08, 0x73, 0xCE, 0x0F, 0xC2, 0x20, 0xF4, 0xDE, 0x7B, 0xF2, 
	0xCE, 0xAD, 0x57, 0xB5, 0xAB, 0x9F, 0xB0, 0x79, 0xF3, 0xB9, 0xBA, 0x42, 0xA3, 0xDB, 0xCD, 0x86, 
	0xD3, 0xC0, 0x8D, 0x6C, 0xA1, 0x60, 0xF2, 0xE6, 0x38, 0xD4, 0xD7, 0x68, 0xB1, 0x26, 0xE4, 0xA0, 
	0x35, 0x16, 0xD6, 0x8D, 0x78, 0x2A, 0x0C, 0x8D, 0x14, 0xCE, 0x9C, 0x81, 0x73, 0x06, 0x52, 0x92, 
	0x02, 0x48, 0x09, 0xE1, 0x71, 0xA9, 0xE9, 0x92, 0x02, 0x4C, 0x04, 0x04, 0x01, 0xD9, 0xAA, 0xF2, 
	0xFD, 0x95, 0x95, 0x6A, 0x35, 0x0C, 0xC5, 0x02, 0x4F, 0xF8, 0xD6, 0xFC, 0x70, 0xC6, 0xFB, 0xA7, 
	0xCE, 0xAD, 0x0F, 0xA9, 0x10, 0x7B, 0xFA, 0x8D, 0xE1, 0x66, 0xBF, 0xD1, 0x86, 0x1C, 0x0A, 0xC0, 
	0x81, 0x6E, 0x77, 0xA4, 0x14, 0x10, 0xCE, 0x32, 0x7B, 0x2F, 0xD0, 0x45, 0x88, 0xAE, 0x9E, 0x60, 
	0xA6, 0x09, 0x23, 0xDD, 0x0E, 0xC4, 0xFE, 0x8A, 0x20, 0x2C, 0x27, 0x5C, 0xC3, 0x07, 0x4C, 0xD9, 
	0xC2, 0x5C, 0xDE, 0x3D, 0x93, 0xE5, 0xAF, 0x7A, 0xD5, 0x5B, 0x39, 0x19, 0x19, 0xA3, 0xD5, 0xD5, 
	0x25, 0x54, 0x55, 0x05, 0x29, 0x25, 0x29, 0xB5, 0xF9, 0x7A, 0x82, 0x4B, 0xED, 0x8D, 0xA8, 0x97, 
	0x14, 0xE0, 0x38, 0x96, 0xC8, 0x32, 0xEB, 0x5B, 0xAD, 0x60, 0x10, 0x45, 0x3A, 0x37, 0x46, 0x76, 
	0xD7, 0xD7, 0xB8, 0xCF, 0x0C, 0x55, 0x13, 0x99, 0x33, 0xC3, 0xB9, 0x3D, 0x99, 0x66, 0x25, 0x95, 
	0xB4, 0xAE, 0x6A, 0x2D, 0xC4, 0xB0, 0x44, 0x3E, 0x66, 0x8C, 0x71, 0xDE, 0x7A, 0x22, 0x66, 0xC9, 
	0x4C, 0x5A, 0x40, 0xB4, 0x5A, 0x5A, 0x36, 0xD2, 0xA0, 0x12, 0xD6, 0xCC, 0x2E, 0xF4, 0xCA, 0x8D, 
	0x5E, 0x3E, 0x34, 0x35, 0xD5, 0x6A, 0x6A, 0xA5, 0x02, 0x6B, 0xB2, 0x48, 0x6B, 0x96, 0xCD, 0xE6, 
	0x28, 0xAD, 0xAF, 0xA7, 0x28, 0x8A, 0x0E, 0x0B, 0xE1, 0x5D, 0x14, 0x09, 0x37, 0x18, 0x38, 0x0C, 
	0x0D, 0xA9, 0xCB, 0x80, 0x5F, 0x29, 0x4D, 0x4C, 0x28, 0xBC, 0xED, 0x6D, 0xAF, 0xE1, 0xE7, 0x9F, 
	0x7F, 0xA1, 0xDF, 0x6C, 0x36, 0xAB, 0x28, 0x8A, 0x85, 0x73, 0x0E, 0x55, 0x95, 0xC7, 0xD6, 0x72, 
	0x62, 0xBA, 0xBE, 0x9F, 0xE7, 0x5C, 0x3B, 0xCF, 0xC6, 0xFB, 0x1A, 0x4C, 0x60, 0x4A, 0xCE, 0xCF, 
	0x58, 0x6D, 0x0E, 0x69, 0xA1, 0xD9, 0x9C, 0x8C, 0x26, 0xB7, 0xDC, 0x36, 0x9A, 0xA6, 0xDB, 0x12, 
	0xAD, 0x7D, 0xBD, 0xB4, 0x74, 0xB0, 0xB3, 0xB4, 0x74, 0x78, 0xD1, 0xDA, 0xA2, 0x91, 0xA6, 0x14, 
	0x05, 0x81, 0x8B, 0x7A, 0xBD, 0x63, 0x5A, 0xA9, 0xDD, 0x90, 0x72, 0x12, 0xD6, 0x16, 0x7E, 0x30, 
	0x58, 0x33, 0x75, 0x6D, 0x6D, 0x59, 0x32, 0x1A, 0x0D, 0xBE, 0x0C, 0xF8, 0x95, 0xD2, 0xF0, 0xB0, 
	0x44, 0xBF, 0xBF, 0x82, 0x2D, 0x5B, 0x9A, 0x2B, 0x23, 0x23, 0x13, 0xF3, 0x61, 0xD8, 0xB8, 0xB1, 
	0xD7, 0xCB, 0xB6, 0xF6, 0xFB, 0x18, 0x65, 0x4E, 0x9C, 0x31, 0xCD, 0x20, 0x8E, 0x55, 0x3F, 0xCF, 
	0x57, 0xAB, 0x7E, 0x7F, 0xB9, 0xDE, 0x3C, 0x50, 0x8F, 0x97, 0x11, 0x06, 0x92, 0x64, 0x48, 0x6D, 
	0xD9, 0x72, 0x65, 0xD4, 0x6A, 0x6D, 0x6F, 0x32, 0xD7, 0x75, 0x96, 0xCD, 0x66, 0xB3, 0xB3, 0x6E, 
	0xFD, 0xDC, 0xB9, 0x41, 0xD6, 0x6E, 0xEB, 0xE6, 0x96, 0x2D, 0x6A, 0x7C, 0x7D, 0xFD, 0xA8, 0x04, 
	0x2C, 0xDA, 0xED, 0x7D, 0x88, 0xA2, 0x84, 0xD7, 0xD7, 0x9F, 0xAA, 0xAB, 0xAA, 0x67, 0x99, 0x05, 
	0xF8, 0xD2, 0xE2, 0x7B, 0xE9, 0x14, 0x3A, 0x00, 0xE0, 0xAB, 0x5F, 0x35, 0xF8, 0x89, 0x9F, 0xD8, 
	0x8D, 0x28, 0x0A, 0xED, 0xD4, 0xD4, 0xF4, 0x88, 0x10, 0xE2, 0xC6, 0x2C, 0x2B, 0xF7, 0x1A, 0x13, 
	0x35, 0xE3, 0xF8, 0x6A, 0xDD, 0x6A, 0xBD, 0x26, 0x1E, 0x1B, 0xBB, 0xB1, 0x99, 0xA6, 0x43, 0x22, 
	0xCF, 0x97, 0xAB, 0xAA, 0xCA, 0x5E, 0x46, 0x78, 0xF3, 0xD8, 0xA8, 0x73, 0xB5, 0x53, 0xCA, 0x39, 
	0xEF, 0xBB, 0xD5, 0xC6, 0xC6, 0xF1, 0x7C, 0x6D, 0xED, 0x48, 0xED, 0x5C, 0x47, 0x44, 0x11, 0xC2, 
	0x20, 0xA0, 0x20, 0x8A, 0x44, 0x1C, 0x86, 0x36, 0x0E, 0x82, 0x50, 0x4E, 0x4E, 0xEE, 0x43, 0x14, 
	0x25, 0xF5, 0x60, 0x30, 0xFF, 0x74, 0x51, 0xAC, 0x3F, 0x42, 0xA4, 0x36, 0xDE, 0xFD, 0xEE, 0xB9, 
	0xCB, 0x16, 0xFC, 0x4A, 0x6A, 0x74, 0x74, 0x12, 0x6F, 0x7E, 0xB3, 0x2D, 0x9E, 0x7C, 0xB2, 0x71, 
	0xB4, 0xD3, 0x59, 0x9F, 0x37, 0xA6, 0x00, 0x51, 0x80, 0xE1, 0xE1, 0x86, 0x08, 0x82, 0x76, 0x58, 
	0x14, 0x01, 0xBC, 0x8F, 0x65, 0xAB, 0x15, 0x11, 0xB3, 0x86, 0xB5, 0x1E, 0xCE, 0x49, 0x22, 0x8A, 
	0x95, 0x52, 0x81, 0x12, 0xA2, 0xD6, 0x9D, 0xCE, 0x01, 0x65, 0xED, 0x61, 0x52, 0x0A, 0x41, 0xB3, 
	0x69, 0x1B, 0xED, 0x76, 0xAC, 0x94, 0x22, 0x25, 0x25, 0x84, 0x94, 0xA4, 0x9D, 0x83, 0x2C, 0x8A, 
	0x0E, 0xFA, 0xFD, 0xD3, 0x10, 0x42, 0x58, 0xEF, 0x31, 0xC0, 0xE6, 0xB1, 0xD7, 0xCB, 0xBB, 0xE8, 
	0x57, 0x5A, 0x6F, 0x7E, 0xF3, 0x5F, 0xE0, 0x0B, 0x5F, 0xB8, 0x1B, 0x45, 0x91, 0x9F, 0x2D, 0x8A, 
	0xEC, 0x59, 0x29, 0xAB, 0x3B, 0xE3, 0xB8, 0x6C, 0x06, 0xC1, 0x11, 0x00, 0x4B, 0xBE, 0xDF, 0xE7, 
	0x6C, 0x75, 0x75, 0xB9, 0x2B, 0x65, 0xA7, 0x1E, 0x1F, 0x0F, 0x04, 0xB3, 0x0C, 0xA5, 0xDC, 0xD5, 
	0xD4, 0xFA, 0xAA, 0x46, 0x1C, 0xB7, 0x82, 0x20, 0x58, 0xD5, 0x75, 0x7D, 0x58, 0x03, 0xEB, 0x4A, 
	0x4A, 0x08, 0x66, 0x2D, 0x00, 0x90, 0x94, 0x44, 0x44, 0x80, 0x73, 0x02, 0xDE, 0x13, 0xF2, 0xBC, 
	0xC4, 0xDC, 0xDC, 0x33, 0x20, 0x0A, 0x4B, 0x6B, 0xFB, 0x6B, 0x52, 0xCA, 0xCC, 0xFB, 0xCB, 0x80, 
	0xBF, 0x2F, 0x2A, 0xCB, 0x02, 0x49, 0x32, 0xD9, 0x25, 0xB2, 0x8F, 0x36, 0x1A, 0xB8, 0x0B, 0xE4, 
	0xB7, 0x58, 0xBB, 0x14, 0x15, 0xC5, 0x1C, 0x19, 0xE3, 0x94, 0x94, 0x36, 0xD0, 0xDA, 0x25, 0x81, 
	0xA6, 0x46, 0x18, 0xC5, 0xAD, 0x34, 0xDD, 0xD9, 0x4C, 0x92, 0x7D, 0xA1, 0xD6, 0xA9, 0x10, 0xE2, 
	0x2C, 0xF5, 0xFB, 0xB3, 0x28, 0xCB, 0x2E, 0xBE, 0xF9, 0x6E, 0x08, 0x01, 0x22, 0x81, 0xCD, 0xF7, 
	0x35, 0x49, 0x6C, 0xBE, 0x25, 0x35, 0x87, 0x31, 0x6B, 0x4C, 0x14, 0x2C, 0x4A, 0x49, 0xA7, 0x87, 
	0x86, 0x64, 0x6E, 0x2D, 0x5D, 0x72, 0x6B, 0x25, 0x2E, 0x45, 0xC0, 0x6F, 0x7F, 0xFB, 0xE3, 0x58, 
	0x59, 0x79, 0x86, 0xC3, 0x90, 0x0F, 0x2B, 0x85, 0x2F, 0x13, 0xF8, 0xAC, 0xB1, 0x8E, 0xBD, 0xB7, 
	0x22, 0x0C, 0x5D, 0x9A, 0xA6, 0xBC, 0x55, 0x2B, 0xBB, 0xDD, 0xB3, 0xDD, 0x06, 0xB8, 0x51, 0xEF, 
	0x97, 0x12, 0x21, 0x8E, 0x4A, 0xE0, 0x00, 0x15, 0xC5, 0xF3, 0x30, 0x66, 0x03, 0xCE, 0x29, 0x78, 
	0x1F, 0x03, 0x18, 0x07, 0xD1, 0xD5, 0xD0, 0xFA, 0x16, 0x24, 0xC9, 0x35, 0x08, 0x82, 0x06, 0x94, 
	0xDA, 0x3C, 0xDC, 0x56, 0xD7, 0xB6, 0x53, 0x96, 0xD5, 0xE3, 0x55, 0xE5, 0x9F, 0xFD, 0x99, 0x9F, 
	0x59, 0xAC, 0xAD, 0x15, 0x97, 0x2D, 0xF8, 0xFB, 0x25, 0xEF, 0x0D, 0x3A, 0x1D, 0xD7, 0x4B, 0x12, 
	0xF1, 0x20, 0x33, 0xDD, 0xA8, 0x95, 0x9C, 0x24, 0xF0, 0x10, 0xB3, 0x95, 0x52, 0x72, 0x53, 0xE9, 
	0x10, 0x40, 0x42, 0x3A, 0x48, 0xE0, 0xDC, 0x12, 0xBA, 0xDD, 0x25, 0x08, 0x01, 0x38, 0x57, 0xC1, 
	0x18, 0xC0, 0xB9, 0x18, 0x52, 0x86, 0x88, 0xE3, 0x5D, 0x08, 0x82, 0x1B, 0x10, 0x86, 0xE3, 0x50, 
	0x6A, 0x15, 0xD6, 0x3E, 0x8A, 0xB2, 0x3C, 0x09, 0x21, 0x78, 0x40, 0x84, 0x87, 0x8D, 0xA9, 0x3F, 
	0x5D, 0xD7, 0x7C, 0xE6, 0xD7, 0x7F, 0x7D, 0x14, 0x1F, 0xFC, 0xE0, 0xEC, 0x65, 0x0B, 0xFE, 0x7E, 
	0xE9, 0x5D, 0xEF, 0x3A, 0x8D, 0x20, 0xD0, 0x30, 0x26, 0x38, 0x29, 0x84, 0xFC, 0xAC, 0x94, 0xE2, 
	0x71, 0xA5, 0x64, 0xA9, 0xB5, 0x42, 0x1C, 0xB7, 0xA8, 0xD5, 0xBA, 0x81, 0xD2, 0xD6, 0xDD, 0x68, 
	0xB7, 0xEE, 0x44, 0x9A, 0x5E, 0x01, 0x22, 0x8B, 0xBA, 0xEE, 0xC3, 0x18, 0x03, 0x66, 0x0F, 0x21, 
	0x36, 0x5F, 0x21, 0x4F, 0xB4, 0x8E, 0x28, 0x5A, 0x87, 0x10, 0x2B, 0x28, 0xCB, 0x0D, 0x94, 0xA5, 
	0xF7, 0x75, 0xAD, 0x97, 0x8C, 0xA1, 0xBF, 0x10, 0x42, 0xFC, 0x27, 0xAD, 0xE5, 0x23, 0x63, 0x63, 
	0x61, 0x55, 0x55, 0xE6, 0x92, 0x5C, 0xA7, 0x4B, 0xD6, 0x82, 0x01, 0x40, 0xEB, 0x08, 0x4A, 0xAD, 
	0xD5, 0xCC, 0xFA, 0x11, 0x00, 0x43, 0x44, 0x60, 0x22, 0xBC, 0x36, 0x8E, 0xE3, 0xA1, 0x56, 0x7B, 
	0x17, 0x81, 0xAF, 0x83, 0x94, 0x1A, 0xC6, 0x38, 0x54, 0xD5, 0x59, 0x18, 0x53, 0x60, 0xF3, 0x08, 
	0x91, 0x83, 0x38, 0xFF, 0x50, 0xD0, 0xBA, 0x3E, 0x8B, 0x7E, 0x7F, 0x1D, 0x44, 0x8D, 0xBA, 0xAE, 
	0xFD, 0xBA, 0x73, 0xC5, 0x09, 0x29, 0xF1, 0x75, 0xE7, 0xC4, 0x03, 0xCE, 0xD1, 0x01, 0xE7, 0x68, 
	0x90, 0xE7, 0x8C, 0x7F, 0xF7, 0xEF, 0x7A, 0x97, 0xE4, 0x1A, 0x11, 0x2E, 0x71, 0xFD, 0xF1, 0x1F, 
	0xEF, 0x84, 0xB5, 0x35, 0x94, 0x52, 0x6D, 0x63, 0xDC, 0x0F, 0x19, 0xE3, 0xEE, 0x91, 0x32, 0xBE, 
	0x3D, 0x49, 0x76, 0xEF, 0x94, 0x72, 0xAA, 0x55, 0xD7, 0x4E, 0xF7, 0xFB, 0x33, 0x34, 0x18, 0xCC, 
	0x90, 0xF7, 0xE6, 0xC2, 0xC1, 0x32, 0xCB, 0x8C, 0x1A, 0x40, 0x8F, 0x19, 0x7D, 0x80, 0xD7, 0x00, 
	0x3A, 0xC5, 0x2C, 0x8E, 0x00, 0xE2, 0x20, 0x80, 0xE7, 0xBD, 0xE7, 0x45, 0x40, 0x58, 0x21, 0x0C, 
	0x7E, 0xE1, 0x17, 0x16, 0x2F, 0xD9, 0xF5, 0xB9, 0xE4, 0x01, 0x03, 0xC0, 0x27, 0x3F, 0x39, 0x8D, 
	0x4E, 0x67, 0x80, 0x24, 0x09, 0x03, 0x6B, 0xFD, 0x2E, 0x29, 0xC5, 0x8D, 0x5A, 0x47, 0xFB, 0x84, 
	0x88, 0x76, 0xD4, 0xB5, 0x1B, 0x19, 0x0C, 0x8A, 0xA8, 0xAA, 0xAC, 0x38, 0x0F, 0x35, 0x67, 0x46, 
	0x87, 0x19, 0x2B, 0xCC, 0x58, 0x06, 0xB0, 0x02, 0x60, 0x41, 0x08, 0x2C, 0x10, 0xF1, 0x32, 0xE0, 
	0xBA, 0x4A, 0x75, 0xAD, 0xF7, 0x21, 0x3E, 0xF4, 0xA1, 0xEC, 0x92, 0x5F, 0x9B, 0xBF, 0x13, 0x80, 
	0x2F, 0xE8, 0xBF, 0xFE, 0xD7, 0xF1, 0xF3, 0xA9, 0x8F, 0x0C, 0x00, 0xB4, 0x99, 0x79, 0xC4, 0x18, 
	0xA4, 0x55, 0x25, 0xA2, 0xBA, 0x06, 0xCE, 0x03, 0x2E, 0x99, 0xD1, 0x67, 0xC6, 0xC0, 0x39, 0x14, 
	0xCC, 0xA8, 0x99, 0x61, 0xEA, 0x1A, 0x3E, 0x08, 0x80, 0x5F, 0xF9, 0x95, 0xC5, 0xBF, 0x4B, 0x4B, 
	0xF2, 0x77, 0x0B, 0xF0, 0xB7, 0xEB, 0xFE, 0xFB, 0xAF, 0x45, 0x9A, 0xE6, 0x28, 0x0A, 0x8D, 0x7E, 
	0xBF, 0x46, 0xBF, 0x5F, 0xA3, 0xAE, 0x19, 0x5A, 0x13, 0x7E, 0xFE, 0xE7, 0x17, 0x71, 0x59, 0x97, 
	0x75, 0x59, 0x97, 0x75, 0x59, 0x97, 0x75, 0x59, 0x97, 0x75, 0x59, 0x97, 0x75, 0x59, 0x97, 0x75, 
	0x59, 0x97, 0x75, 0x59, 0x97, 0x75, 0x59, 0x97, 0x75, 0x59, 0x17, 0x8B, 0xFE, 0x3F, 0x73, 0x6E, 
	0x8B, 0xE1, 0x8C, 0x28, 0x4B, 0x59, 0x00, 0x00, 0x00, 0x25, 0x74, 0x45, 0x58, 0x74, 0x64, 0x61, 
	0x74, 0x65, 0x3A, 0x63, 0x72, 0x65, 0x61, 0x74, 0x65, 0x00, 0x32, 0x30, 0x31, 0x32, 0x2D, 0x31, 
	0x32, 0x2D, 0x30, 0x36, 0x54, 0x30, 0x35, 0x3A, 0x34, 0x32, 0x3A, 0x35, 0x30, 0x2B, 0x30, 0x30, 
	0x3A, 0x30, 0x30, 0xF7, 0x88, 0xDC, 0xD4, 0x00, 0x00, 0x00, 0x25, 0x74, 0x45, 0x58, 0x74, 0x64, 
	0x61, 0x74, 0x65, 0x3A, 0x6D, 0x6F, 0x64, 0x69, 0x66, 0x79, 0x00, 0x32, 0x30, 0x31, 0x32, 0x2D, 
	0x31, 0x32, 0x2D, 0x30, 0x36, 0x54, 0x30, 0x35, 0x3A, 0x34, 0x32, 0x3A, 0x35, 0x30, 0x2B, 0x30, 
	0x30, 0x3A, 0x30, 0x30, 0x86, 0xD5, 0x64, 0x68, 0x00, 0x00, 0x00, 0x46, 0x74, 0x45, 0x58, 0x74, 
	0x73, 0x6F, 0x66, 0x74, 0x77, 0x61, 0x72, 0x65, 0x00, 0x49, 0x6D, 0x61, 0x67, 0x65, 0x4D, 0x61, 
	0x67, 0x69, 0x63, 0x6B, 0x20, 0x36, 0x2E, 0x36, 0x2E, 0x39, 0x2D, 0x37, 0x20, 0x32, 0x30, 0x31, 
	0x32, 0x2D, 0x30, 0x38, 0x2D, 0x31, 0x37, 0x20, 0x51, 0x31, 0x36, 0x20, 0x68, 0x74, 0x74, 0x70, 
	0x3A, 0x2F, 0x2F, 0x77, 0x77, 0x77, 0x2E, 0x69, 0x6D, 0x61, 0x67, 0x65, 0x6D, 0x61, 0x67, 0x69, 
	0x63, 0x6B, 0x2E, 0x6F, 0x72, 0x67, 0x9C, 0xBD, 0xB9, 0x48, 0x00, 0x00, 0x00, 0x18, 0x74, 0x45, 
	0x58, 0x74, 0x54, 0x68, 0x75, 0x6D, 0x62, 0x3A, 0x3A, 0x44, 0x6F, 0x63, 0x75, 0x6D, 0x65, 0x6E, 
	0x74, 0x3A, 0x3A, 0x50, 0x61, 0x67, 0x65, 0x73, 0x00, 0x31, 0xA7, 0xFF, 0xBB, 0x2F, 0x00, 0x00, 
	0x00, 0x18, 0x74, 0x45, 0x58, 0x74, 0x54, 0x68, 0x75, 0x6D, 0x62, 0x3A, 0x3A, 0x49, 0x6D, 0x61, 
	0x67, 0x65, 0x3A, 0x3A, 0x68, 0x65, 0x69, 0x67, 0x68, 0x74, 0x00, 0x36, 0x30, 0x30, 0x35, 0x83, 
	0xBE, 0x65, 0x00, 0x00, 0x00, 0x17, 0x74, 0x45, 0x58, 0x74, 0x54, 0x68, 0x75, 0x6D, 0x62, 0x3A, 
	0x3A, 0x49, 0x6D, 0x61, 0x67, 0x65, 0x3A, 0x3A, 0x57, 0x69, 0x64, 0x74, 0x68, 0x00, 0x38, 0x30, 
	0x30, 0xE3, 0xB1, 0xC0, 0xE2, 0x00, 0x00, 0x00, 0x19, 0x74, 0x45, 0x58, 0x74, 0x54, 0x68, 0x75, 
	0x6D, 0x62, 0x3A, 0x3A, 0x4D, 0x69, 0x6D, 0x65, 0x74, 0x79, 0x70, 0x65, 0x00, 0x69, 0x6D, 0x61, 
	0x67, 0x65, 0x2F, 0x70, 0x6E, 0x67, 0x3F, 0xB2, 0x56, 0x4E, 0x00, 0x00, 0x00, 0x17, 0x74, 0x45, 
	0x58, 0x74, 0x54, 0x68, 0x75, 0x6D, 0x62, 0x3A, 0x3A, 0x4D, 0x54, 0x69, 0x6D, 0x65, 0x00, 0x31, 
	0x33, 0x35, 0x34, 0x37, 0x37, 0x32, 0x35, 0x37, 0x30, 0xB8, 0x91, 0x60, 0x54, 0x00, 0x00, 0x00, 
	0x12, 0x74, 0x45, 0x58, 0x74, 0x54, 0x68, 0x75, 0x6D, 0x62, 0x3A, 0x3A, 0x53, 0x69, 0x7A, 0x65, 
	0x00, 0x32, 0x32, 0x31, 0x4B, 0x42, 0x42, 0xCD, 0x66, 0x08, 0x57, 0x00, 0x00, 0x00, 0x33, 0x74, 
	0x45, 0x58, 0x74, 0x54, 0x68, 0x75, 0x6D, 0x62, 0x3A, 0x3A, 0x55, 0x52, 0x49, 0x00, 0x66, 0x69, 
	0x6C, 0x65, 0x3A, 0x2F, 0x2F, 0x2F, 0x74, 0x6D, 0x70, 0x2F, 0x6C, 0x6F, 0x63, 0x61, 0x6C, 0x63, 
	0x6F, 0x70, 0x79, 0x5F, 0x35, 0x30, 0x36, 0x31, 0x64, 0x66, 0x32, 0x64, 0x32, 0x63, 0x30, 0x63, 
	0x2D, 0x31, 0x2E, 0x70, 0x6E, 0x67, 0x7A, 0x10, 0xF8, 0x91, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 
	0x4E, 0x44, 0xAE, 0x42, 0x60, 0x82, 
};


// ユーザー定義データ構造体：ColorKanjiLCDLoadImgSize 構造体の派生構造体にしてください
// ユーザーが自由に使えるデータです。通常は画像データに関する構造体にします。
struct UserDefData : public ColorKanjiLCDLoadImgSize {
  uint16_t nPos;  // サイコロ PNG 画像データへの位置
};


// loadPng 関数から呼び出されるユーザー定義 callback 関数
// buff に buff_size 分の画像データを転送してください。正常に転送できた時は CKLCD_ERR_NONE を返してください。
// もし転送できなかったら CKLCD_ERR_NONE 以外（例えば、CKLCD_ERR_UNKNOWN）を返してください。その時は loadPng 関数の処理を中断します。
errCode_t UserDefLoadPngCallback (uint8_t * buff, uint8_t buff_size, ColorKanjiLCDLoadImgSize * user_def_data)
{
  UserDefData * udd = (UserDefData *)user_def_data; // ユーザー定義データ構造体にキャストする：ダウンキャストなので要注意！

  // buff ポインタが指すメモリに buff_size バイト分の Test PNG パターン画像データを転送します。
  while (buff_size) { // 0バイトになるまで繰り返す
    uint8_t dat = pgm_read_byte(&DicesPng[udd->nPos]); // プログラムメモリから読み出す
    *buff++ = dat;
    udd->nPos++; // サイコロ PNG 画像データへの位置を1バイト進める
    --buff_size;
  }

  // 正常に転送できた
  return CKLCD_ERR_NONE;
}


// 初期化関数
void setup()
{
  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  // 注意："CKLCD_Wire" は Arduino Due では "Wire1" を、それ以外では "Wire" になります。
  CKLCD_Wire.begin();

#if 1
  // 注意：ここで、強制的に TWI 通信を高速化させています。カラー漢字 LCD シールドは 400kHz の速度に対応していますが、
  //       標準 100kHz の速度にしか対応していない別の TWI 機器を同時に使用する場合、そちらの TWI 機器が正常に TWI 通信できなくなる可能性があります。
  //       その場合は、ここの「#if 1」の "1" を "0" に書き換えて、TWI 通信の高速化を行わない(つまり標準 100kHz のまま)ようにしてください。
  //       当然ですが、そうした場合は、カラー漢字 LCD シールドとの TWI 通信速度も遅くなってしまいます。
  CKLCD_Wire_setHiSpeed();
#endif

  // 続いて、カラー漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功すると、画面はクリアされ、LCD 表示方向はランドスケープ(横向き) に、カーソル位置は (0,0) になります。
  // カラー漢字 LCD は、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが、上記の CKLCD_TWI_ADDR と一致しているか確認してください。
  // 仮想画面１(160×256) を前景面(ARGB4444)に、仮想画面２(160×128) を背景面(RGB565)に設定します。
  cklcd.begin (CKLCD_ORIENT_LANDSCAPE + CKLCD_DISP_SCREEN_F1_B2);


  // ～～～ 仮想画面１＆２の使い方の解説 ～～～
  //
  // 仮想画面１(160×256) は前景面(ARGB4444)に設定し、上半分を表示用に、下半分を回転前の画像に使用しています。
  // 仮想画面２(160×128) は背景面(RGB565)に設定し、背景用の PNG 画像を転送させます。
  //
  // 最初に、仮想画面２に PNG 画像を転送(LoadPng 関数)させます。これは、PNG 画像展開するのに、仮想画面１を破壊してしまうからです。(仕様です)
  // 続いて、仮想画面１をクリア(clear 関数)します。PNG 画像展開時に発生したゴミを消す為です。
  // そうしておいてから、一旦、仮想画面１の下半分に、回転前の画像を描きます(neonPrint 関数とprint 関数)。
  // その後で、仮想画面１の下半分を元画像にして(setSrcRect 関数)、仮想画面１の上半分に回転＆拡大後の画像を作成(copyWithRotate 関数)させます。
  // 最後に、flip 関数で、仮想画面１の上半分と仮想画面２を合成表示させます。
  //
  // 実質、仮想画面が３枚有る様な感じで、処理させています。(４枚あれば更に背景スクロールも可能なのですが、流石に４枚分のメモリは確保できませんでした…)


  // "読込み中"メッセージを表示させます（仮想画面２を使用）。
  cklcd.v2.print ("PNG画像読込み中...");

  // png 画像の透明部分の背景色を設定します
  cklcd.v2.setBackColor (CKLCD_COL32_GRAY); // 灰色

  // 最初に仮想画面２を背景色でクリアしておきます
  cklcd.v2.clearRect (0, 0, CKLCD_VPLANE2_MAX_L_WIDTH - 1, CKLCD_VPLANE2_MAX_L_HEIGHT - 1);

  // ユーザー定義データ構造体
  UserDefData udd;
  udd.nPos = 0;  // サイコロ PNG 画像データへの位置を0にし、パターンの先頭から開始させます

  // 仮想画面１の(20,19)に、PNG 画像データを読み込ませます。エラー発生時はメッセージを表示させます。
  // PNG 画像展開の作業用メモリとして、仮想画面１の画像は破壊されますので、注意して下さい。
  errCode_t res = cklcd.v2.loadPng (20, 19, CKLCD_LOAD_IMG_DISP_ERR_MSG, sizeof(DicesPng), UserDefLoadPngCallback, &udd);

  if (res != CKLCD_ERR_NONE) {
    cklcd.v2.print ("\nErrCode=0x");
    cklcd.v2.print (res, HEX);
    return;
  }

  // PNG 画像展開により、仮想画面１にゴミがあるので、クリアさせます。
  cklcd.v1.clear ();

  // フォントタイプをプロポーショナル(可変幅)にします。
  cklcd.v1.setFontType (CKLCD_FONT_TYPE_PROPORTIONAL);

  // ネオン効果の文字列の描画
  // colGlow2 色のグロー効果に、さらに colGlow1 色のグロー効果が付いた colBody 色の文字列を描画します。激しく遅いです。
  // 注意1：文字列は１行以内に収めてください。複数行にまたがると、既存の描画部分も dilate & blur 処理がかかり、おかしくなってしまいます。
  // 注意2：colGlow1 色と colGlow2 色の組み合わせによっては、２回目の dilate 処理の時に破綻して、変なネオン効果になる場合があります。
  // 内部では、str を colGlow2 色で描画、dilate 処理、blur 処理、str を colGlow1 色で再描画、再 dilate 処理、再 blur 処理、str を colBody 色で再々描画しています。
  cklcd.v1.neonPrint (18, CKLCD_VPLANE1_MAX_L_HEIGHT / 2 + 53, CKLCD_COL32_BLACK, CKLCD_COL32_WHITE, CKLCD_COL32_BLUE, "回転・拡大・縮小");

  // コピー元の矩形領域を指定：仮想画面１の下半分
  cklcd.v1.setSrcRect (0, CKLCD_VPLANE1_MAX_L_HEIGHT / 2, CKLCD_VPLANE1_MAX_L_WIDTH - 1, CKLCD_VPLANE1_MAX_L_HEIGHT - 1);

  // 文字色：白
  cklcd.v1.setDrawColor (CKLCD_COL32_WHITE);

  // 背景色：透明
  // 回転＆拡大/縮小の時に、コピー元に無い部分は、この背景色で代用されます。
  cklcd.v1.setBackColor (CKLCD_COL32_TRANS);
}


cklcdInvRatio10_t ratio = CKLCD_FLOAT_TO_INVRATIO10(1.2); // 1.2倍でコピー
cklcdBAM10_t      bam10 = 0; // 回転角度 ("BAM10"単位)


// ループ関数
void loop()
{
  // 現在の回転角度の値を描画：仮想画面１の下半分に描画する
  cklcd.v1.setCursorMin (60, CKLCD_VPLANE1_MAX_L_HEIGHT / 2 + 80);
  cklcd.v1.print (bam10, DEC);

  // 回転＆拡大/縮小させながらコピー：劇遅!!
  // setSrcRect()で指定されている矩形領域を回転("BAM10"単位)させながら、(x0,y0)-(x1,y1)の範囲内にコピーします。
  // invRatio10 倍の拡大/縮小もできますが、単なる近傍点補間しかしてないので、結果画像は汚いです。大昔の家庭用ゲーム機みたいな効果しかありません。
  // 回転の中心はそれぞれ、コピー元の矩形領域の中心座標と、コピー先の矩形領域の中心座標、になります。
  // 回転＆拡大/縮小の時、コピー元の矩形領域に存在し無い部分は setBackColor()で設定している色になります。
  // コピー元の矩形領域と、コピー先の矩形領域は、部分的にも重なっていてはいけません。重なってた場合、おかしな結果画像になってしまいます。
  //
  // 仮想画面１の下半分を元画像にして、仮想画面１の上半分に回転コピーさせます。
  errCode_t res = cklcd.v1.copyWithRotate (0, 0, CKLCD_MAX_L_WIDTH - 1, CKLCD_MAX_L_HEIGHT - 1, ratio, bam10);

  // 仮想画面２をスクロールさせます。
  cklcd.v2.setDispStartPos (0, bam10 & 0x7F);

  // LCD 全領域を描画させます
  cklcd.flip();

  bam10++; // 次の回転角度
}

